/* dbsavail.ec -Report on space usage by dbspace.
 */
/*
  RCS Header:
  -----------
  $Header: /home/art/GoogleDrive/Latest/RCS/dbsavail.ec,v 1.30 2023/12/08 15:10:42 art Exp $

  RCS Log:
  --------
  $Log: dbsavail.ec,v $
  Revision 1.30  2023/12/08 15:10:42  art
  Made changes needed to run successfully on v15.0.

  Revision 1.29  2021/12/21 17:12:40  art
  Updated Copyright notice.

  Revision 1.28  2020/12/21 18:14:13  art
  Fixed free space calculations for sbspaces. Thanks to Roland Wintgen
  for pointing out the issue.

  Revision 1.27  2019/10/24 14:04:40  art
  Fixed headings when -u specified and removed "total" line entry for the 
  Used column when -u & -p are specified together.

  Revision 1.26  2018/04/08 16:52:33  art
  New options: -H specify server to connect to, -R reverse sort order
    -F explicit options for default sort order (by free space)
    Hidden option, -E, to SET EXPLAIN for debugging.

  Revision 1.25  2018/02/28 17:25:44  art
  Added new options: -K, -M, -G to display total, used, & free space in KB, MB, or GB
  respectively.

  Revision 1.24  2018/01/09 11:59:42  art
  Made dbspace name column width dynamic based on the longest 
  dbspace name.

  Revision 1.23  2017/12/26 16:52:30  art
  Fixed a couple of printf's

  Revision 1.22  2016/11/16 15:51:51  art
  Improved sort order display text. Added -d option to the Usage output.

  Revision 1.21  2016/10/14 17:47:57  art
  Added options (-P, -N, -S) to sort by pct free, dbspace name, size in KB rather
  than the default of free KB.

  Revision 1.20  2016/01/12 21:14:02  art
  Added new option, -f, to use the _fast table and avoid updating
  blob chunk statistics. The report will be faster, but may be
  less accurate on servers with active blobspaces. Feature
  is only supported on v11.70 and later (ignored with a warning
  on earlier releases.

  Revision 1.19  2015/12/30 13:06:22  art
  Added new option, -d, to display chunk detail for each dbspace showing
  number of pages and free pages in the chunk and its path.

  Revision 1.18  2015/10/20 01:37:23  art
  Improved backwards compatibility with v10.00.

  Revision 1.17  2015/04/22 14:01:30  art
  FINALLY, I think I have the calculations for dumb blobs running correctly.

  Revision 1.16  2015/04/21 14:11:51  art
  Fixed BLOB calculations again!

  Revision 1.15  2014/09/19 14:50:58  art
  Added new option -u to report used KB instead of total KB.  Will work reasonably with -p
  as well reporting %used and %free.

  Revision 1.14  2014/09/14 16:58:49  art
  Expanded dbspace name from 32 to 128 characters.

  Revision 1.13  2014/09/02 00:46:51  art
  Fixed free and KB calculations once and for all!

  Revision 1.12  2013/12/24 17:12:57  art
  Fixed compile time warnings on HPUX.  Updated copyright notice.
  Added new option -p to print percent free instead of KB Free.

  Revision 1.11  2013/10/07 20:19:25  art
  Added include of string.h to eliminate compile time warning.

  Revision 1.10  2013/03/10 23:31:14  art
  Fixed RCS Tags.

  Revision 1.9  2013/03/10 23:29:43  art
  Fixed KB calculations.

  Revision 1.8  2013/02/18 20:25:49  art
  Added copyright statement.

  Revision 1.7  2012/12/19 17:38:27  art
  Fixed intermittent segv problem.  Uninitialized DECIMAL host vars used in calculations.

  Revision 1.6  2012/12/19 17:06:42  art
  Added support for temp sbspaces (ie correctly reporting space type).

  Revision 1.5  2012/12/18 16:59:37  art
  *** empty log message ***

  Revision 1.3  2000/12/04 22:39:19  kagel
  Fixed no stored procedure detection.

  Revision 1.2  1998/11/18 15:07:50  kagel
  Added rcs headers.


*/

static char RCSHeader[] = "$Header: /home/art/GoogleDrive/Latest/RCS/dbsavail.ec,v 1.30 2023/12/08 15:10:42 art Exp $";
static char RCSCompile[] = "$cc: " __FILE__ " " __DATE__ " " __TIME__ " $";

#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include <string.h>

#include "getopt.h"  /* Not <getopt.h> need the delivered header. */

EXEC SQL INCLUDE sqlca;


int main( int argc, char **argv )
{
    EXEC SQL BEGIN DECLARE SECTION;
    string dbsname[129];
    dec_t total_size, free_size, used_size;
    dec_t used_pct, free_pct;
    dec_t bused_pct, bfree_pct;
    dec_t total_k_size, free_k_size, used_k_size;
    char flag;
    int pagesize, bpagesz, basepage;
    int vers, name_len, name_width;
    dec_t dbsavers;
    string procstr[65536], callstr[2049];
    string l_path[257], servername[129];
    int l_pages, l_free, dbsn, fast_ok;
    string dtl[10];
    int scale = 1;
    EXEC SQL END DECLARE SECTION;
    char tot_str[25], free_str[25], tmp[25];
    char tot_k_str[25], free_k_str[25];
    char return_str[2000];
    dec_t sum_size, sum_free, dec_const;
    dec_t sum_k_size, sum_k_free, sum_k_used;
    char ch;
    int ret = 1, reverse = 0, sortcol = 8;
    char version_str[4] = "7.0";
    dec_t version;
    char Copyright[1024];
    int do_percents = 0, do_used = 0, do_details = 0;
    int first = 1, fast = 0, explain = 0;
    char sortby = 'F', sortstr[21] = "Free KB";
    char units[3][3] = { "KB", "MB", "GB" };
    char sizestr[3][20] = { "Total KB", "Total MB", "Total GB" };
    char usedstr[3][20] = { "Used KB", "Used MB", "Used GB" };
    char freestr[3][20] = { "Free KB", "Free MB", "Free GB" };
    int unitix = 0;

    servername[0] = (char)0;

    strcpy( Copyright, "Copyright 1998-2022 Art S. Kagel.\n" );

    deccvasc( version_str, strlen( version_str ), &version );

    while ((ch = getopt( argc, argv, "dfhpuEFGKMNPRSVH:?" )) != EOF) {
	switch (ch) {
	case 'H':
	    snprintf( servername, sizeof servername, "@%s", optarg );
	    break;
	case 'E':
	    explain = 1;
	    break;
	case 'R':
	    reverse = 1;
	    break;
	case 'K':
	    unitix = 0;
	    scale = 1;
	    break;
	case 'M':
	    unitix = 1;
	    scale = 1024;
	    break;
	case 'G':
	    unitix = 2;
	    scale = 1048576;
	    break;
	case 'F':
	    sortby = 'F';
	    break;
	case 'P':
	    sortby = 'P';
	    break;
	case 'N':
	    sortby = 'N';
	    break;
	case 'S':
	    sortby = 'S';
	    break;
	case 'f':
	    fast = 1;
	    break;
	case 'p':
	    do_percents = 1;
	    break;
	case 'u':
	    do_used = 1;
	    break;
	case 'd':
	    do_details = 1;
	    break;
	case 'V':
	    fprintf( stderr, "\n%s: %s\n", argv[0], Copyright );
	    fprintf( stderr, "%s\n\n", RCSHeader );
	    fprintf( stderr, "Procedure version: %s.\n\n", version_str );
	    exit( 0 );
	case 'h':
	    ret = 0;
	default:
	    fprintf( stderr,
		     "Usage: %s [-H server] [-f] [-p] [-u] [-R] [-F|P|S|N]\n"
		     "       %s [-V|-h]\n"
		     "\t-H - Connect to server.\n"
		     "\t-p - print %%free (Default: KB/MB/GB Free).\n"
		     "\t-u - print KB/MB/GB Used or Percent Used instead of Total KB.\n"
		     "\t-f - do not update blob chunk statistics (may be less accurate)\n"
		     "\t-d - include chunk detail by dbspace.\n"
		     "\t-F - sort the report by free size in KB/MB/GB (default).\n"
		     "\t-P - sort the report by percent free.\n"
		     "\t-S - sort the report by total size in KB/MB/GB.\n"
		     "\t-N - sort the report by dbspace name.\n"
		     "\t-R - Reverse sort order.\n"
		     "\t-V - print copyright notice and exit.\n"
		     "\t-K - Display sizes in KB (default).\n"
		     "\t-M - Display sizes in MB.\n"
		     "\t-G - Display sizes in GB.\n"
		     "\t-h - print usage and exit.\n\n"
		     "\tBy default the report is ordered by free space in KB/MB/GB.\n\n",
		     argv[0],
		     argv[0] );
	    exit( ret );
	}
    }

    switch (sortby) {
    case 'S':
      snprintf( sortstr, sizeof sortstr, "Total Size(%s)", units[unitix] );
      break;
    case 'F':
      snprintf( sortstr, sizeof sortstr, "Free %s", units[unitix] );
      break;
    case 'P':
      strcpy( sortstr, "Percent Free" );
      break;
    case 'N':
      strcpy( sortstr, "Dbspace Name" );
      break;
    }
    fprintf( stderr, "Sort by: %s.\n", sortstr );

    if (strlen(servername) > 0) {
	EXEC SQL CONNECT TO :servername;
    if (sqlca.sqlcode) {
        fprintf( stderr,
		 "Error connecting to server (%s): sqlcode: %d, ISAM: %d.\n",
		 servername,
		 sqlca.sqlcode,
		 sqlca.sqlerrd[1] );

        return sqlca.sqlcode;
    }
    }

    EXEC SQL DATABASE sysmaster;
    if (sqlca.sqlcode) {
        fprintf( stderr,
		 "Error opening database: sqlcode: %d, ISAM: %d.\n",
		 sqlca.sqlcode,
		 sqlca.sqlerrd[1] );

        return sqlca.sqlcode;
    }

    if (explain == 1)
	EXEC SQL SET EXPLAIN ON;

    EXEC SQL SELECT dbinfo( "version", "major" ) INTO :vers FROM systables WHERE tabid = 1;
    if (sqlca.sqlcode) {
        fprintf( stderr,
		 "Error fetching server version: sqlcode: %d, ISAM: %d.\n",
		 sqlca.sqlcode,
		 sqlca.sqlerrd[1] );

        return sqlca.sqlcode;
    }

    EXEC SQL SELECT count(*) INTO :fast_ok FROM systables WHERE tabname = 'syschunks_fast';
    if (sqlca.sqlcode < 0) {
	fprintf( stderr,
		 "Error verifying fast availability. SQL: %d, ISAM: %d.\n",
		 sqlca.sqlcode,
		 sqlca.sqlerrd[1] );
	return sqlca.sqlcode;
    } else if (fast == 1 && fast_ok == 0) {
	fprintf( stderr, "Fast mode not supported by your server. Ignorning.\n" );
	fast = 0;
    }

    EXEC SQL EXECUTE FUNCTION dbsavail_vers() INTO :dbsavers;
    if ((sqlca.sqlcode == -674) || deccmp( &dbsavers, &version ) != 0) {
	/* Stored procedure not there or older version, create latest and try again. */
	EXEC SQL BEGIN WORK;
	EXEC SQL DROP FUNCTION IF EXISTS dbsavail_vers();
	if (sqlca.sqlcode < 0 && sqlca.sqlcode != -674) {
	    EXEC SQL ROLLBACK WORK;
	    fprintf( stderr,
		     "Error dropping version procedure. SQL: %d, ISAM: %d.\n",
		     sqlca.sqlcode,
		     sqlca.sqlerrd[1] );
	    exit( 1 );
	}
	EXEC SQL DROP FUNCTION IF EXISTS dbsavail_vers;
	if (sqlca.sqlcode < 0 && sqlca.sqlcode != -674) {
	    EXEC SQL ROLLBACK WORK;
	    fprintf( stderr,
		     "Error dropping version procedure(2). SQL: %d, ISAM: %d.\n",
		     sqlca.sqlcode,
		     sqlca.sqlerrd[1] );
	    exit( 1 );
	}
	EXEC SQL DROP FUNCTION IF EXISTS dbsavail_detail( int );
	if (sqlca.sqlcode < 0 && sqlca.sqlcode != -674) {
	    EXEC SQL ROLLBACK WORK;
	    fprintf( stderr,
		     "Error dropping detail procedure. SQL: %d, ISAM: %d.\n",
		     sqlca.sqlcode,
		     sqlca.sqlerrd[1] );
	    exit( 1 );
	}
	EXEC SQL DROP PROCEDURE IF EXISTS dbsavail();
	if (sqlca.sqlcode < 0 && sqlca.sqlcode != -674) {
	    EXEC SQL ROLLBACK WORK;
	    fprintf( stderr,
		     "Error dropping dbsavail procedure. SQL: %d, ISAM: %d.\n",
		     sqlca.sqlcode,
		     sqlca.sqlerrd[1] );
	    exit( 1 );
	}
	EXEC SQL DROP PROCEDURE IF EXISTS dbsavail( int );
	if (sqlca.sqlcode < 0 && sqlca.sqlcode != -674) {
	    EXEC SQL ROLLBACK WORK;
	    fprintf( stderr,
		     "Error dropping dbsavail procedure. SQL: %d, ISAM: %d.\n",
		     sqlca.sqlcode,
		     sqlca.sqlerrd[1] );
	    exit( 1 );
	}
	EXEC SQL DROP PROCEDURE IF EXISTS dbsavail_fast();
	if (sqlca.sqlcode < 0 && sqlca.sqlcode != -674) {
	    EXEC SQL ROLLBACK WORK;
	    fprintf( stderr,
		     "Error dropping fast procedure. SQL: %d, ISAM: %d.\n",
		     sqlca.sqlcode,
		     sqlca.sqlerrd[1] );
	    exit( 1 );
	}
	EXEC SQL DROP PROCEDURE IF EXISTS dbsavail_fast( int );
	if (sqlca.sqlcode < 0 && sqlca.sqlcode != -674) {
	    EXEC SQL ROLLBACK WORK;
	    fprintf( stderr,
		     "Error dropping fast procedure. SQL: %d, ISAM: %d.\n",
		     sqlca.sqlcode,
		     sqlca.sqlerrd[1] );
	    exit( 1 );
	}
	EXEC SQL COMMIT WORK;

	EXEC SQL BEGIN WORK;

	if (vers >= 11) {
	    snprintf( return_str,
		      sizeof return_str,
		      "    returning int as dbsn, char(128) as dbname, char(1) as aflag, int as blob_page_kb, decimal(32,0) as total_size_pgs, \n"
		      "    decimal(32,0) as free_size_pgs, decimal(32, 0) as total_size_kb, decimal(32, 0)as free_size_kb, \n"
		      "    decimal(32, 0) as pagesize_sz, decimal(16,3) as free_pct, int as basepage, \n"
		      "    decimal(32, 0) as used_pgs, decimal(32, 0) as used_kb, decimal(16,3) as used_pct, \n"
		      "    decimal(16,3) as bfree_pct, decimal(16,3) as bused_pct; \n" );
	} else {
	    snprintf( return_str,
		      sizeof return_str,
		      "    returning int, char(128), char(1), int, decimal(32,0), \n"
		      "    decimal(32,0), decimal(32, 0), decimal(32, 0), decimal(32, 0), \n"
		      "    decimal(16,3), int, decimal(32, 0), decimal(32, 0), decimal(16, 3), \n"
		      "    decimal(16.3), decimal(16,3); \n" );
	}

	/*
	  The semantics of syschunks and page sizes changes from version to version:
	           < v10        V10       v11.10/11.50        v11.70+
	           -------  ------------  ----------------  ---------------
	  chksize  # pages  # pages       # default pages   # default pages
	  pagesize   DNA    # def pgs/pg  # bytes/page      # bytes/page
	  nfree    # pages  # def pages   # def pages       dbspace: # def pages
	                                                    blbspc:  # blb pages
	                                                    sbspace: #sbpages
	*/

	snprintf( procstr,
		  sizeof procstr,
		  "create procedure dbsavail( scale int default 1 ) \n"
		  "    %s\n"
		  "    \n"
		  "    define dbname char(128); \n"
		  "    define aflag char(1); \n"
		  "    define total_size, free_size decimal(32,0); \n"
		  "    define total_k_size, free_k_size decimal(32,0); \n"
		  "    define free_pct, bfree_pct decimal(16,3); \n"
		  "    define temp, blob, sbspc smallint; \n"
		  "    define basepage, pagsizeK, blbpagesz, vers, dbsn int; \n"
		  "    \n"
		  "    select pagesize into basepage from %s where chknum = 1; \n"
		  "    \n"
		  "    foreach \n"
		  "        select d.name, d.is_temp, d.is_blobspace, d.is_sbspace, %s blobpageK, \n"
		  "               sum(chksize) total_size, "
		  "               case when is_blobspace = 1 then sum(nfree * (c.pagesize/2048)) when is_sbspace = 1 then sum(udfree) else sum(nfree) end as nfree, %s pagsizeK, \n"
		  "               ((sum(nfree)::decimal(32,10) / sum(chksize)::decimal(32,10)) * 100.00)::decimal(16,3) free_pct, \n"
		  "               ((sum(nfree*(%s/%s))::decimal(32,10) / sum(chksize)::decimal(32,10)) * 100.00)::decimal(16,3) bfree_pct, d.dbsnum \n"
		  "        into dbname, temp, blob, sbspc, blbpagesz, total_size, free_size, pagsizeK, free_pct, bfree_pct, dbsn \n"
		  "        from sysmaster:sysdbspaces d, sysmaster:%s c, \n"
		  "                sysmaster:sysdbstab t \n"
		  "        where d.dbsnum = c.dbsnum \n"
		  "          and c.dbsnum = t.dbsnum \n"
		  "        group by 1,2,3,4,5,8,11 \n"
		  "        order by 7 \n"
		  "        \n"
		  "        if (sbspc == 1 and temp == 1) then \n"
		  "            let aflag = 'U'; \n"
		  "        elif (sbspc == 1) then \n"
		  "            let aflag = 'S'; \n"
		  "        elif (temp == 1) then \n"
		  "            let aflag = 'T'; \n"
		  "        elif (blob == 1) then \n"
		  "            let aflag = 'B'; \n"
		  "            let free_pct = bfree_pct;\n"
		  "        else \n"
		  "            let aflag = 'N'; \n"
		  "        end if \n"
		  "        \n"
		  "        let total_k_size = total_size * (basepage/1024); \n"
		  "        let free_k_size = free_size * (basepage/1024); \n"
		  "       \n"
		  "        return dbsn, dbname, aflag, blbpagesz, total_size, free_size, total_k_size/scale, free_k_size/scale, \n"
		  "               pagsizeK, free_pct, basepage, (total_size - free_size), (total_k_size - free_k_size)/scale, \n"
		  "               (100.00 - free_pct), bfree_pct, (100.00 - bfree_pct) with resume; \n"
		  "    end foreach; \n"
		  "end procedure; \n",
		  return_str,
		  ((fast_ok > 0) ? "syschunks_fast" : "syschunks"),
		  ((vers >= 10) ? "c.pagesize/1024" : "t.prtpage/1024"), /* Blob page size calculation. */
		  ((vers > 10) ? "d.pagesize/1024" : "(select (sh_pagesize * c.pagesize)/1024 from sysshmvals)"), /* Pagesize calculation */
		  ((vers > 10) ? "c.pagesize" : "t.prtpage"),
		  ((fast_ok > 0) ? "(select pagesize from syschunks_fast where chknum = 1)" : "(select pagesize from syschunks where chknum = 1)"),
		  "syschunks" );
	EXEC SQL EXECUTE IMMEDIATE :procstr;
	if (sqlca.sqlcode) {
	    fprintf( stderr,
		     "Error creating stored procedure: sqlcode: %d, ISAM: %d @%d.\n",
		     sqlca.sqlcode,
		     sqlca.sqlerrd[1],
		     sqlca.sqlerrd[4] );
	    fprintf( stderr, "%s\n", procstr);
	    EXEC SQL ROLLBACK WORK;

	    return sqlca.sqlcode;
	}

	if (fast_ok) {
	    snprintf( procstr,
		      sizeof procstr,
		      "create procedure dbsavail_fast( scale int default 1) \n"
		      "    %s\n"
		      "    \n"
		      "    define dbname char(128); \n"
		      "    define aflag char(1); \n"
		      "    define total_size, free_size decimal(32,0); \n"
		      "    define total_k_size, free_k_size decimal(32,0); \n"
		      "    define free_pct, bfree_pct decimal(16,3); \n"
		      "    define temp, blob, sbspc smallint; \n"
		      "    define basepage, pagsizeK, blbpagesz, vers, dbsn int; \n"
		      "    \n"
		      "    select pagesize into basepage from %s where chknum = 1; \n"
		      "    \n"
		      "    foreach \n"
		      "        select d.name, d.is_temp, d.is_blobspace, d.is_sbspace, %s blobpageK, \n"
		      "               sum(chksize) total_size, "
		      "               case when is_blobspace = 1 then sum(nfree * (c.pagesize/2048)) when is_sbspace = 1 then sum(udfree) else sum(nfree) end as nfree, %s pagsizeK, \n"
		      "               ((sum(nfree)::decimal(32,10) / sum(chksize)::decimal(32,10)) * 100.00)::decimal(16,3) free_pct, \n"
		      "               ((sum(nfree*(%s/%s))::decimal(32,10) / sum(chksize)::decimal(32,10)) * 100.00)::decimal(16,3) bfree_pct, d.dbsnum \n"
		      "        into dbname, temp, blob, sbspc, blbpagesz, total_size, free_size, pagsizeK, free_pct, bfree_pct, dbsn \n"
		      "        from sysmaster:sysdbspaces d, sysmaster:%s c, \n"
		      "                sysmaster:sysdbstab t \n"
		      "        where d.dbsnum = c.dbsnum \n"
		      "          and c.dbsnum = t.dbsnum \n"
		      "        group by 1,2,3,4,5,8,11 \n"
		      "        order by 7 \n"
		      "        \n"
		      "        if (sbspc == 1 and temp == 1) then \n"
		      "            let aflag = 'U'; \n"
		      "        elif (sbspc == 1) then \n"
		      "            let aflag = 'S'; \n"
		      "        elif (temp == 1) then \n"
		      "            let aflag = 'T'; \n"
		      "        elif (blob == 1) then \n"
		      "            let aflag = 'B'; \n"
		      "            let free_pct = bfree_pct;\n"
		      "        else \n"
		      "            let aflag = 'N'; \n"
		      "        end if \n"
		      "        \n"
		      "        let total_k_size = total_size * (basepage/1024); \n"
		      "        let free_k_size = free_size * (basepage/1024); \n"
		      "       \n"
		      "        return dbsn, dbname, aflag, blbpagesz, total_size, free_size, total_k_size/scale, free_k_size/scale, \n"
		      "               pagsizeK, free_pct, basepage, (total_size - free_size), (total_k_size - free_k_size)/scale, \n"
		      "               (100.00 - free_pct), bfree_pct, (100.00 - bfree_pct) with resume; \n"
		      "    end foreach; \n"
		      "end procedure; \n",
		      return_str,
		      "syschunks_fast",
		      ((vers >= 10) ? "c.pagesize/1024" : "t.prtpage/1024"), /* Blob page size calculation. */
		      ((vers > 10) ? "d.pagesize/1024" : "(select (sh_pagesize * c.pagesize)/1024 from sysshmvals)"), /* Pagesize calculation */
		      ((vers > 10) ? "c.pagesize" : "t.prtpage"),
		      "(select pagesize from syschunks_fast where chknum = 1)",
		      "syschunks_fast" );
	    EXEC SQL EXECUTE IMMEDIATE :procstr;
	    if (sqlca.sqlcode) {
		fprintf( stderr,
			 "Error creating stored procedure: sqlcode: %d, ISAM: %d @%d.\n",
			 sqlca.sqlcode,
			 sqlca.sqlerrd[1],
			 sqlca.sqlerrd[4] );
		fprintf( stderr, "%s\n", procstr);
		EXEC SQL ROLLBACK WORK;

		return sqlca.sqlcode;
	    }
	}

	snprintf( procstr,
		  sizeof procstr,
		  "create function dbsavail_vers() returning decimal(4); "
		  "    return %s; end function;",
		  version_str );

	EXEC SQL EXECUTE IMMEDIATE :procstr;
	if (sqlca.sqlcode) {
	    fprintf( stderr,
		     "Error creating version function: sqlcode: %d, ISAM: %d @%d.\n",
		     sqlca.sqlcode,
		     sqlca.sqlerrd[1],
		     sqlca.sqlerrd[4] );
	    EXEC SQL ROLLBACK WORK;

	    return sqlca.sqlcode;
	}

	snprintf( procstr,
		  sizeof procstr,
		  "CREATE FUNCTION dbsavail_detail( p_dbsnum INT ) \n"
		  "    RETURNING CHAR(256) AS chk_path, INT AS chk_pages, INT AS chk_free_pgs;\n\n"
		  "    DEFINE l_path CHAR(256);\n "
		  "    DEFINE l_pgs, l_free INTEGER;\n\n "
		  "    FOREACH select fname, chksize, nfree \n"
		  "    INTO l_path, l_pgs, l_free\n"
		  "    FROM %s\n "
		  "    WHERE dbsnum = p_dbsnum\n"
		  "    ORDER BY chknum\n "
		  "        RETURN l_path, l_pgs, l_free WITH RESUME; \n"
		  "    END FOREACH;\n "
		  "END FUNCTION;\n ",
		  (fast_ok ? "syschunks_fast" : "syschunks") );
	EXEC SQL EXECUTE IMMEDIATE :procstr;
	if (sqlca.sqlcode) {
	    fprintf( stderr,
		     "Error creating detail function: sqlcode: %d, ISAM: %d @%d.\n",
		     sqlca.sqlcode,
		     sqlca.sqlerrd[1],
		     sqlca.sqlerrd[4] );
	    fprintf( stderr, "%s\n", procstr );
	    EXEC SQL ROLLBACK WORK;

	    return sqlca.sqlcode;
	}
    }

    EXEC SQL COMMIT WORK;

    strcpy( dtl, "dtl" );

    switch (sortby) {
    case 'F':
	sortcol = 8;
	break;
    case 'P':
	sortcol = 10;
	break;
    case 'S':
	sortcol = 7;
	break;
    case 'N':
	sortcol = 2;
	break;
    }
    snprintf( callstr,
	      sizeof callstr,
	      "select * from table (%s( %d::int ) ) ORDER BY %d %s",
	      fast == 1 ? "dbsavail_fast" : "dbsavail",
	      scale,
	      sortcol,
	      reverse == 0 ? "ASC" : "DESC" );
    EXEC SQL PREPARE fred_stmt FROM :callstr;
    if (sqlca.sqlcode != 0) {
	fprintf( stderr,
		 "Error preparing statement( fred_stmt ): sqlcde: %d, ISAM: %d.\n",
		 sqlca.sqlcode,
		 sqlca.sqlerrd[1] );
	return sqlca.sqlcode;
    }

    EXEC SQL DECLARE fred CURSOR FOR fred_stmt;
    if (sqlca.sqlcode != 0) {
	fprintf( stderr,
		 "Error declaring cursor( fred ): sqlcde: %d, ISAM: %d.\n",
		 sqlca.sqlcode,
		 sqlca.sqlerrd[1] );
	return sqlca.sqlcode;
    }

    EXEC SQL PREPARE get_len FROM "SELECT max(length(name)) from sysdbspaces";
    if (sqlca.sqlcode) {
        fprintf( stderr,
		 "Error preparing length query: sqlcode: %d, ISAM: %d.\n",
		 sqlca.sqlcode,
		 sqlca.sqlerrd[1] );

        return sqlca.sqlcode;
    }
    EXEC SQL EXECUTE get_len INTO :name_len;
    if (sqlca.sqlcode) {
        fprintf( stderr,
		 "Error executing name query: sqlcode: %d, ISAM: %d.\n",
		 sqlca.sqlcode,
		 sqlca.sqlerrd[1] );

        return sqlca.sqlcode;
    }
    if (name_len < 18) name_len = 18;
    name_width = name_len * -1;    

    EXEC SQL OPEN fred;
    if (sqlca.sqlcode) {
        fprintf( stderr,
		 "Error opening cursor: sqlcode: %d, ISAM: %d.\n",
		 sqlca.sqlcode,
		 sqlca.sqlerrd[1] );

        return sqlca.sqlcode;
    }
    deccvint( 0, &sum_size );
    deccvint( 0, &sum_free );
    deccvint( 0, &total_size );
    deccvint( 0, &free_size );
    deccvint( 0, &sum_k_size );
    deccvint( 0, &sum_k_free );
    deccvint( 0, &total_k_size );
    deccvint( 0, &free_k_size );
    do {
        EXEC SQL FETCH fred
            INTO :dbsn, :dbsname, :flag, :bpagesz, :total_size, :free_size, :total_k_size,
	    :free_k_size, :pagesize, :free_pct, :basepage, :used_size, :used_k_size, :used_pct,
	    :bfree_pct, :bused_pct;
        if (sqlca.sqlcode == 0) {
	    if (first) {
		if (do_percents && !do_used)
		    printf( "\n%*.*s   %s %dK %s   %dK %s    %-12s    %-12s\n",
			    name_width, name_len, 
			    "Dbspace", "Number", basepage/1024, "Pages", basepage/1024, "Pages Free", sizestr[unitix], "Pct Free" );
		else if (do_percents && do_used)
		    printf( "\n%*.*s   %s %dK %s   %dK %s    %-12s    %-12s\n",
			    name_width, name_len, 
			    "Dbspace", "Number", basepage/1024, "Pages", basepage/1024, "Pages Free", "Pct Used", "Pct Free" );
		else if (!do_percents && do_used)
		    printf( "\n%*.*s   %s %dK %s   %dK %s    %-12s    %-12s\n",
			    name_width, name_len, 
			    "Dbspace", "Number", basepage/1024, "Pages", basepage/1024, "Pages Free", usedstr[unitix], freestr[unitix] );
		else
		    printf( "\n%*.*s   %s %dK %s   %dK %s    %-12s    %-12s\n",
			    name_width, name_len, 
			    "Dbspace", "Number", basepage/1024, "Pages", basepage/1024, "Pages Free", sizestr[unitix], freestr[unitix] );
		printf( "%*.*s   %-14s   %-13s    %-12s    %-12s\n",
			name_width, name_len,
			"------------------",
			"---------------",
			"-------------",
			"------------",
			"------------" );
		first = 0;
	    }
            dectoasc( &total_size, tmp, 12, 0 );
            ldchar( tmp, 12, tot_str );
            if ((ret = decadd( &total_size, &sum_size, &sum_size ))) {
                fprintf( stderr, "Error adding size: %d.\n", ret );
            }
            dectoasc( &free_size, tmp, 12, 0 );
            ldchar( tmp, 12, free_str );
            if ((ret = decadd( &free_size, &sum_free, &sum_free ))) {
                fprintf( stderr, "Error adding free: %d.\n", ret );
            }
	    if ((ret = decadd( &used_k_size, &sum_k_used, &sum_k_used ))) {
	        fprintf( stderr, "Error adding size %s: %d.\n", units[unitix], ret );
	    }
	    if ((ret = decadd( &total_k_size, &sum_k_size, &sum_k_size ))) {
	        fprintf( stderr, "Error adding size %s: %d.\n", units[unitix], ret );
	    }
	    if (do_used && !do_percents) {
		dectoasc( &used_k_size, tmp, 12, 0 );
	    } else if ((do_used & do_percents) && flag == 'B') {
		dectoasc( &bused_pct, tmp, 12, 2 );
	    } else if (do_used & do_percents) {
		dectoasc( &used_pct, tmp, 12, 2 );
	    } else {
		dectoasc( &total_k_size, tmp, 12, 0 );
	    }
            ldchar( tmp, 12, tot_k_str );
	    if (do_percents && flag == 'B') {
		dectoasc( &bfree_pct, tmp, 12, 2 );
	    } else if (do_percents) {
		dectoasc( &free_pct, tmp, 12, 2 );
	    } else {
		dectoasc( &free_k_size, tmp, 12, 0 );
	    }
            ldchar( tmp, 12, free_k_str );
            if ((ret = decadd( &free_k_size, &sum_k_free, &sum_k_free ))) {
	        fprintf( stderr, "Error adding free %s: %d.\n", units[unitix], ret );
            }
            if (flag == 'T') {
                printf( "%*.*s  %14s  %14s  %14s  %14s Temp(PgSz: %dK)\n",
			name_width, name_len, 
                        dbsname,
                        tot_str,
                        free_str,
                        tot_k_str,
                        free_k_str,
                        pagesize );
            } else if (flag == 'B') {
                printf( "%*.*s  %14s  %14s  %14s  %14s Blob(BlbPg: %dK)\n",
			name_width, name_len, 
                        dbsname,
                        tot_str,
                        free_str,
                        tot_k_str,
                        free_k_str,
                        pagesize );
            } else if (flag == 'S') {
		printf( "%*.*s  %14s  %14s  %14s  %14s SBsp(PgSz: %dK)\n",
			name_width, name_len, 
                        dbsname,
                        tot_str,
                        free_str,
                        tot_k_str,
                        free_k_str ,
                        bpagesz );
            } else if (flag == 'U') {
		printf( "%*.*s  %14s  %14s  %14s  %14s TSBs(PgSz: %dK)\n",
			name_width, name_len, 
                        dbsname,
                        tot_str,
                        free_str,
                        tot_k_str,
                        free_k_str ,
                        bpagesz );
            } else {
		printf( "%*.*s  %14s  %14s  %14s  %14s     (PgSz: %dK)\n",
			name_width, name_len, 
                        dbsname,
                        tot_str,
                        free_str,
                        tot_k_str,
                        free_k_str ,
                        pagesize);
	    }
	    if (do_details) {
		EXEC SQL DECLARE :dtl CURSOR FOR
		    EXECUTE FUNCTION dbsavail_detail( :dbsn );

		if (sqlca.sqlcode != 0) {
		    fprintf( stderr,
			     "Error querying detail! SQL: %d, ISAM: %d.\n",
			     sqlca.sqlcode,
			     sqlca.sqlerrd[1] );
		    return sqlca.sqlcode;
		}
		EXEC SQL OPEN :dtl;
		if (sqlca.sqlcode != 0) {
		    fprintf( stderr,
			     "Error opening querying detail! SQL: %d, ISAM: %d.\n",
			     sqlca.sqlcode,
			     sqlca.sqlerrd[1] );
		    return sqlca.sqlcode;
		}
		while (1) {
		    EXEC SQL FETCH :dtl INTO :l_path, :l_pages, :l_free;
		    if (sqlca.sqlcode < 0) {
			fprintf( stderr,
				 "Error fetching querying detail! SQL: %d, ISAM: %d.\n",
				 sqlca.sqlcode,
				 sqlca.sqlerrd[1] );
			return sqlca.sqlcode;
		    } else if (sqlca.sqlcode == 100) {
			printf( "\n" );
			break;
		    }
		    printf( "%10s  %14d  %14d  %s\n", " ", l_pages, l_free, l_path );
		}
	    }
            dectoasc( &sum_size, tmp, 12, 0 );
            ldchar( tmp, 12, tot_str );
            dectoasc( &sum_free, tmp, 12, 0 );
            ldchar( tmp, 12, free_str );
            dectoasc( &sum_k_size, tmp, 12, 0 );
            ldchar( tmp, 12, tot_k_str );
            dectoasc( &sum_k_free, tmp, 12, 0 );
            ldchar( tmp, 12, free_k_str );
            /*   printf( "\t\t%-12s\t%-12s\n", tot_str, free_str ); */
        } else if (sqlca.sqlcode == 100)
	    break;
    } while (1);
    if (sqlca.sqlcode != 100) {
        fprintf( stderr,
		 "Error fetching cursor: sqlcode: %d, ISAM: %d Ptr: %s.\n",
		 sqlca.sqlcode,
		 sqlca.sqlerrd[1],
		 sqlca.sqlerrm);

        return sqlca.sqlcode;
    }
    EXEC SQL CLOSE fred;
    EXEC SQL FREE fred;
    EXEC SQL FREE fred_stmt;

    printf( "%*.*s   %-14s   %-13s    %-12s    %-12s\n",
	    name_width, name_len, 
	    "------------------",
	    "---------------",
	    "-------------",
	    "------------",
	    "------------" );
    
    dectoasc( &sum_size, tmp, 12, 0 );
    ldchar( tmp, 12, tot_str );
    dectoasc( &sum_free, tmp, 12, 0 );
    ldchar( tmp, 12, free_str );
    // -- calculate total %free and format into tmp instead of sum_k_size if do_used
    if (do_used && do_percents) {
	deccvint( 100, &dec_const );
	decsub( &sum_k_size, &sum_k_free, &sum_k_used );
	decmul( &sum_k_used, &dec_const, &sum_k_used );
	decdiv( &sum_k_used, &sum_k_size, &used_pct );
	dectoasc( &used_pct, tmp, 12, 2 );
    } else if (do_used) {
	dectoasc( &sum_k_used, tmp, 12, 0 );
    } else {
	dectoasc( &sum_k_size, tmp, 12, 0 );
    }
    ldchar( tmp, 12, tot_k_str );
    dectoasc( &sum_k_free, tmp, 12, 0 );
    ldchar( tmp, 12, free_k_str );
    if (do_used && do_percents)
	printf( "%*.*s  %14s  %14s\n\n",
		name_width, name_len, 
		"Totals:",
		tot_str,
		free_str );
    else if (do_percents)
	printf( "%*.*s  %14s  %14s  %14s\n\n",
		name_width, name_len, 
		"Totals:",
		tot_str,
		free_str,
		tot_k_str );
    else
	printf( "%*.*s  %14s  %14s  %14s  %14s\n\n",
		name_width, name_len, 
		"Totals:",
		tot_str,
		free_str,
		tot_k_str,
		free_k_str );

    return 0;
}
