/* listdb.ec - List databases in the current server.  Optionally list tables 
   and/or dbspaces.
*/
/*
   RCS Header:
   -----------
   $Header: /home/art/GoogleDrive/Latest/RCS/listdb7.ec,v 1.27 2021/12/21 17:12:40 art Exp $

   RCS Log:
   --------
   $Log: listdb7.ec,v $
   Revision 1.27  2021/12/21 17:12:40  art
   Updated Copyright notice.

   Revision 1.26  2019/06/17 13:50:19  art
   Fixed typo.

   Revision 1.25  2019/06/11 17:06:52  art
   Added database locale to the database level detail report.

   Revision 1.24  2016/06/27 21:22:28  art
   Updated copyright notice.

   Revision 1.23  2013/12/24 18:11:12  art
   Fixed more compiler warnings.

   Revision 1.22  2013/12/24 17:12:57  art
    Fixed compile time warnings on HPUX.  Updated copyright notice.

   Revision 1.21  2013/02/10 22:19:07  art
   Updated Copyright

   Revision 1.20  2009/07/23 16:51:12  art
   Made inclusion of my getopt.h unconditional.  Needed on all platforms except Linux now.

   Revision 1.19  2008/12/17 16:33:01  art
   Removed BB Specific RCSWhat string.  Fixed revision.

   Revision 1.1  2008/04/16 18:24:04  art
   Initial revision

   Revision 1.19  2006/01/27 17:30:45  kagel
   OK, this REALLY fixed the problem.

   Revision 1.17  2003/10/13 17:06:45  kagel
   FIxed compilation on Linux.

   Revision 1.16  2003/05/08 16:09:01  kagel
   Added message if database or database(s) matching spec not found and return
   code 2 on that condition.

   Revision 1.15  2003/04/10 17:14:49  kagel
   typo

   Revision 1.14  2003/04/10 17:12:40  kagel
   Added getopt.h as in dostats.ec.

   Revision 1.13  2003/04/10 16:49:36  kagel
   Made commandline consistent.  Uses -DHAVE_GNU_GETOPT=1 to control which
   getopt version is parsing.

   Revision 1.12  2003/04/10 15:48:54  kagel
   Added extent info.

   Revision 1.11  2001/10/24 19:06:16  kagel
   Added -V option to print version and copyright info.

   Revision 1.10  2000/12/29 17:38:13  kagel
   added long name support.  reformatte d output accordingly.

   Revision 1.9  1999/11/22 18:17:50  kagel
   Forgot to fix table level calcs also.

   Revision 1.8  1999/11/22 17:54:31  kagel
   Removed dependence on bitfield.  Bitfield does not work correctly for
   dbspace mapping on Intel architecture.

   Revision 1.7  1998/04/20 22:29:09  kagel
   Removed unused code and fixed a few diagnostic messages.

# Revision 1.6  1998/04/14  14:18:07  kagel
# Typos.
#
# Revision 1.5  1998/04/14  14:16:30  kagel
# Fixed fragmented index reporting to not repeat index name after first
# fragment.
#
# Revision 1.4  1998/04/14  13:52:01  kagel
# Updated usage listing and made -? return zero.  Also modified the message
# printed when the user does not have permissions on a database to list table
# information.
#
# Revision 1.3  1998/04/14  13:21:22  kagel
# Fixed usage listing and added -? help option to force usage.
#
# Revision 1.2  1998/04/13  21:51:49  kagel
# Eureka it woiks!
#

*/

static char RCSHeader[] = "$Header: /home/art/GoogleDrive/Latest/RCS/listdb7.ec,v 1.27 2021/12/21 17:12:40 art Exp $";


#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#if defined(linux) || defined( __hpux )
#include <string.h>
#else
#include <strings.h>
#endif
#include <errno.h>

#include "getopt.h"  /* Not <getopt.h> need the delivered header. */

EXEC SQL INCLUDE sqlca;
EXEC SQL INCLUDE sqltypes;

#define DB_MAX  100
#define DB_SIZE 30
#define TAB_INCR 1000
EXEC SQL DEFINE NAME_SIZE 129;
EXEC SQL DEFINE OWNER_SIZE 33;

typedef union Partno_u {
    long partno;
    struct {
	unsigned int dbspaceno:12, pageno:20;
    } dbs;
} PartNo_t;

EXEC SQL BEGIN DECLARE SECTION;

typedef struct spaces_s {
    short dbsnum;
    string name[NAME_SIZE];
} spaces_t;

typedef struct sysdatabases_s {
    string name[NAME_SIZE];
    long partnum;
    string owner[OWNER_SIZE];
    string created[13];
    string locale[33];
    long is_logging, is_buff_log, is_ansi, is_nls;
} sysdatabases_t;

typedef struct tabinfo_s {
    long tabid;
    string tabname[NAME_SIZE];
    long partnum;
    string owner[OWNER_SIZE];
    string created[12];
    string fragtype[2];
    string indexname[NAME_SIZE];
    string dbspace[NAME_SIZE];
    long evalpos;
    string locklevel[2];
} tabinfo_t;

typedef struct tabsize_s {
    string tabname[NAME_SIZE];
    long nextents;
    long npages;
    long avgpgs;
} tabsize_t;

EXEC SQL END DECLARE SECTION;

void usage( char *prog );

int main( int argc, char *argv[] )
{
    EXEC SQL BEGIN DECLARE SECTION;
    char dbase[1024];
    char dbtable[1024];
    char dbhost[31];
    long ndbspaces;
    spaces_t *spaces;
    long ndatabases, ntabs;
    char statement[2048];
    sysdatabases_t dbrec;
    tabinfo_t tabrec;
    short tabind;
    tabsize_t *tabsizes, *ts;
    char ext_stmt[1025];
    EXEC SQL END DECLARE SECTION;

    char host[31], oldidxname[NAME_SIZE];
    long i, j, ch, count = 0, oldtabid = 0;
    int printdetail = 0, ntabsizes = 0;
    PartNo_t part;
    int TBSFACT = 0x100000;
    char *dbspname, *logstring;
    char Revision[] = "$Revision: 1.27 $";
    char oldtabname[NAME_SIZE];

    host[0] = (char)0;
    dbase[0] = (char)0;
    dbtable[0] = (char)0;
    oldidxname[0] = (char)0;

#if defined (HAVE_GNU_GETOPT)
    while ((ch = getopt( argc, argv, "h:d:t::sDV?" )) != EOF) {
#else
    while ((ch = getopt( argc, argv, "h:d:tsDV?" )) != EOF) {
#endif
	switch( ch) {
	case 'h':
	    strcpy( host, optarg );
	    break;
	case 'd':
	    strcpy( dbase, optarg );
	    break;
	case 't':
	    if (optarg)
		strcpy( dbtable, optarg );
	    else if (optind < argc-1) {
		fprintf( stderr,
			 "Option following -t %s: %s\n",
			 argv[optind],
			 argv[optind+1] );
		usage( argv[0] );
		exit(22);
	    } else if (optind == argc-1)
		strcpy( dbtable, argv[argc-1] );
	    else
		strcpy( dbtable, "*" );
	    break;
	case 'D':
	    printdetail = 1;
	    break;
	case 'V':
	    fprintf( stderr,
		     "\n%s: %s Copyright 1998-2022 by Art S. Kagel.\n",
		     argv[0],
		     strtok(Revision, "$"));
	    fprintf( stderr,
		     "\tLicense for private usage (including use by "
		     "commercial enterprise\n"
		     "\tfor its own needs) is granted.  Commercial and "
		     "exploitation rights\n"
		     "\tare reserved.\n\n" );
	    exit(0);
	    break;
	case '?':
	    usage( argv[0] );
	    exit(0);
	default:
	    fprintf( stderr, "Invalid option: %s\n", argv[optind] );
	    usage( argv[0] );
	    exit(22);
	}
    }

    if (!host[0])
	strcpy( host, getenv( "INFORMIXSERVER" ) );

    EXEC SQL WHENEVER ERROR CONTINUE;

    sprintf( dbhost, "@%s", host );

    if (dbtable[0] != (char)0) {
	EXEC SQL CONNECT TO :dbhost as 'dbase' WITH CONCURRENT TRANSACTION;
	if (sqlca.sqlcode == -27000) {
	    fprintf( stderr,
		     "Need to specify a non-shared memory host connection when "
		     "using the -t option!\n" );
	    exit(1);
	} else if (sqlca.sqlcode < 0) {
	    fprintf( stderr,
		     "Error connecting to server info (%s) for table: %d, ISAM: %d.\n",
		     host,
		     sqlca.sqlcode,
		     sqlca.sqlerrd[1] );
	    exit(1);
	}
    }

    EXEC SQL CONNECT TO :dbhost as 'master' WITH CONCURRENT TRANSACTION;
    if (sqlca.sqlcode == -27000) {
	fprintf( stderr,
		 "Need to specify a non-shared memory host connection when "
		 "using the -t option!\n" );
	exit(1);
    } else if (sqlca.sqlcode < 0) {
	fprintf( stderr,
		 "Error connecting to server (%s) for database: %d, ISAM: %d.\n",
		 host,
		 sqlca.sqlcode,
		 sqlca.sqlerrd[1] );
	exit(1);
    }

    EXEC SQL DATABASE sysmaster;
    if (sqlca.sqlcode < 0) {
	fprintf( stderr,
		 "Error opening sysmaster database: %d, ISAM: %d.\n",
		 sqlca.sqlcode,
		 sqlca.sqlerrd[1] );
	exit(1);
    }

    if (printdetail) {

	ntabsizes = TAB_INCR;
	tabsizes = (tabsize_t *)malloc( sizeof (tabsize_t) * ntabsizes );

	EXEC SQL SELECT count(*) INTO :ndbspaces FROM sysdbspaces;
	if ( sqlca.sqlcode ) {
	    fprintf( stderr,
		     "Cannot FETCH dbspace count; error=%d, isam=%d\n",
		     sqlca.sqlcode,
		     sqlca.sqlerrd[1] );
	    exit(99);
	}
	spaces = (spaces_t *)malloc( sizeof (spaces_t) * ndbspaces );
	if (!spaces) {
	    fprintf( stderr,
		     "Cannot allocate dbspace array, errno = %d\n",
		     errno );
	    exit(3);
	}
    
	EXEC SQL DECLARE dbs_curs CURSOR FOR
	    SELECT dbsnum, name
	    FROM sysdbspaces
	    ORDER BY dbsnum;
	if (sqlca.sqlcode) {
	    fprintf( stderr,
		     "Cannot DECLARE CURSOR for dbspace query. error=%d, ISAM=%d\n",
		     sqlca.sqlcode,
		     sqlca.sqlerrd[1] );
	    exit(4);
	}

	EXEC SQL OPEN dbs_curs;
	if (sqlca.sqlcode) {
	    fprintf( stderr,
		     "Cannot OPEN CURSOR for dbspace query. error=%d, ISAM=%d\n",
		     sqlca.sqlcode,
		     sqlca.sqlerrd[1] );
	    exit(4);
	}

	for (i=0; i<ndbspaces; i++) {
	    EXEC SQL FETCH dbs_curs
		INTO :spaces[i].dbsnum, :spaces[i].name;
	    if (sqlca.sqlcode < 0) {
		fprintf( stderr,
			 "Error FETCHING dbspaces cursor.  Error: %d, ISAM: %d.\n",
			 sqlca.sqlcode,
			 sqlca.sqlerrd[1] );
		exit(5);
	    } else if (sqlca.sqlcode == SQLNOTFOUND)
		break;
	}

	EXEC SQL CLOSE dbs_curs;
	if (sqlca.sqlcode < 0) {
	    fprintf( stderr,
		     "Error CLOSEing dbspaces cursor.  Error: %d, ISAM: %d.\n",
		     sqlca.sqlcode,
		     sqlca.sqlerrd[1] );
	    exit(5);
	}
	EXEC SQL FREE dbs_curs;

    }    
    
    if (dbase[0] == (char)0) {
	sprintf( statement, "SELECT COUNT(*) FROM sysdatabases" );
    } else {
	sprintf( statement,
		 "SELECT COUNT(*) FROM sysdatabases WHERE name MATCHES '%s'",
		 dbase );
    }
    EXEC SQL PREPARE st_dbc FROM :statement;
    if (sqlca.sqlcode < 0) {
	fprintf( stderr,
		 "Error PREPARing database COUNT.  Error: %d, ISAM: %d.\n",
		 sqlca.sqlcode,
		 sqlca.sqlerrd[1] );
	exit(6);
    }
    EXEC SQL EXECUTE st_dbc INTO :ndatabases;
    if (sqlca.sqlcode < 0) {
	fprintf( stderr,
		 "Error EXECUTing database COUNT.  Error: %d, ISAM: %d.\n",
		 sqlca.sqlcode,
		 sqlca.sqlerrd[1] );
	exit(6);
    }
    EXEC SQL FREE st_dbc;
    if (sqlca.sqlcode < 0) {
	fprintf( stderr,
		 "Error FREEing database COUNT.  Error: %d, ISAM: %d.\n",
		 sqlca.sqlcode,
		 sqlca.sqlerrd[1] );
	exit(6);
    }

    if (dbase[0] == (char)0)
	if (ndatabases == 0) {
	    printf( "There are no databases in this instance.\n" );
	    exit(2);
	} else if (ndatabases == 1)
	    printf( "There is currently %ld database available: \n\n",
		    ndatabases );
	else
	    printf( "There are currently %ld databases available: \n\n",
		    ndatabases );
    else
	if (ndatabases == 0) {
	    printf( "There are no databases matching: %s.\n", dbase );
	    exit(2);
	} else if (ndatabases == 1)
	    printf( "There is currently %ld matching database: \n\n",
		    ndatabases );
	else
	    printf( "There are currently %ld matching databases: \n\n",
		    ndatabases );

    if (printdetail) {
	printf( "  # Database/Table/Index/Owner          DBSpace/Created  /Log Mode      Locale/GLS?\n");
	printf( "=== ==================================  ===============  =============  ===========\n");
    } else {
	printf("  # Database/Table Name \n");
	printf("=== ======================= \n");
    }

    if (dbase[0] == (char)0) {
	sprintf( statement,
		 "SELECT name, sd.partnum, sd.owner, created, collate, is_logging, "
		 "	is_buff_log, is_ansi, is_nls "
		 "FROM sysdatabases as sd, systabnames as st "
		 "WHERE st.partnum = sd.partnum "
		 "ORDER BY name " );
    } else {
	sprintf( statement,
		 "SELECT name, sd.partnum, sd.owner, created, collate, is_logging, "
		 "	is_buff_log, is_ansi, is_nls "
		 "FROM sysdatabases as sd, systabnames as st "
		 "WHERE st.partnum = sd.partnum  "
		 "  AND name MATCHES '%s' "
		 "ORDER BY name ",
		 dbase );
    }
    if (printdetail) {
        snprintf( ext_stmt,
		  sizeof ext_stmt,
		  "SELECT tabname, count(*), sum(size), avg(size) "
		  "FROM sysextents "
		  "WHERE dbsname = ? "
		  "  AND tabname MATCHES ? "
		  "GROUP BY tabname "
		  "ORDER BY tabname " );

	EXEC SQL PREPARE s_exts FROM :ext_stmt;
	if (sqlca.sqlcode < 0) {
	    fprintf( stderr,
		     "Error PREPARing extent count statement.  Error: %d, ISAM: %d.\n",
		     sqlca.sqlcode,
		     sqlca.sqlerrd[1] );
	    exit(6);
	}

	EXEC SQL DECLARE c_exts CURSOR FOR s_exts;
	if (sqlca.sqlcode < 0) {
	    fprintf( stderr,
		     "Error DECLARing extent count statement.  Error: %d, ISAM: %d.\n",
		     sqlca.sqlcode,
		     sqlca.sqlerrd[1] );
	    exit(6);
	}
    }

    EXEC SQL PREPARE st_dbase FROM :statement;
    if (sqlca.sqlcode < 0) {
	fprintf( stderr,
		 "Error PREPARing database statement.  Error: %d, ISAM: %d.\n",
		 sqlca.sqlcode,
		 sqlca.sqlerrd[1] );
	exit(6);
    }

    EXEC SQL DECLARE c_database CURSOR WITH HOLD FOR st_dbase;
    if (sqlca.sqlcode < 0) {
	fprintf( stderr,
		 "Error DECLARing database statement.  Error: %d, ISAM: %d.\n",
		 sqlca.sqlcode,
		 sqlca.sqlerrd[1] );
	exit(7);
    }

    EXEC SQL OPEN c_database;
    if (sqlca.sqlcode < 0) {
	fprintf( stderr,
		 "Error OPENing database statement.  Error: %d, ISAM: %d.\n",
		 sqlca.sqlcode,
		 sqlca.sqlerrd[1] );
	exit(7);
    }

    for (;;) {
	EXEC SQL FETCH c_database
	    INTO :dbrec.name,
	    :dbrec.partnum,
	    :dbrec.owner,
	    :dbrec.created,
	    :dbrec.locale,
	    :dbrec.is_logging,
	    :dbrec.is_buff_log,
	    :dbrec.is_ansi,
	    :dbrec.is_nls;
	if (sqlca.sqlcode < 0) {
	    fprintf( stderr,
		     "Error FETCHing database statement.  "
		     "Error: %d, ISAM: %d.\n",
		     sqlca.sqlcode,
		     sqlca.sqlerrd[1] );
	    exit(7);
	} else if (sqlca.sqlcode == SQLNOTFOUND)
	    break;
	count++;

	if (printdetail) {
	    part.partno = dbrec.partnum;
	    if ((part.partno / TBSFACT)) {
		for (i=0; i<ndbspaces &&
			 spaces[i].dbsnum < (part.partno / TBSFACT);
		     i++)
		    ;
		if (i >= ndbspaces || spaces[i].dbsnum != (part.partno / TBSFACT))
		    dbspname = "Unknown";
		else
		    dbspname = spaces[i].name;
	    } else
		dbspname = "Shared Memory";

	    if (dbrec.is_ansi)
		logstring = "ANSI";
	    else if (dbrec.is_buff_log)
		logstring = "BUFFERED";
	    else if (dbrec.is_logging)
		logstring = "UNBUFFERED";
	    else
		logstring = "NON-LOGGED";
	    /* Database print format. */
	    printf( "%3ld %s\n",
		    count,
		    dbrec.name );
	    printf( "      %-32s  %-30s  %s\n",
		    dbrec.owner,
		    dbspname,
		    dbrec.locale );
	    printf( "        %-32s %-14s  %-13s  %-4s\n",
		    " ",
		    dbrec.created,
		    logstring,
		    dbrec.is_nls ? "Yes" : "No " );
	} else {
	    printf( "%3ld %s\n", count,  dbrec.name );
	}

	if (dbtable[0] != (char)0) {
	    /* Get extent info for tables from sysmaster before switching to DB. */
	    if (printdetail) {
		EXEC SQL SELECT COUNT(unique tabname)
		    INTO :ntabs
		    FROM SYSEXTENTS
		    WHERE dbsname = :dbrec.name
		    AND tabname MATCHES :dbtable;

		if (ntabs >= ntabsizes) {
		    while (ntabs >= ntabsizes)
			ntabsizes += TAB_INCR;
		    tabsizes = (tabsize_t *)realloc( tabsizes, sizeof (tabsize_t) * ntabsizes);
		}
		memset( tabsizes, 0, sizeof (tabsize_t) * ntabsizes );
		EXEC SQL OPEN c_exts
		  USING :dbrec.name, :dbtable;
		if (sqlca.sqlcode < 0) {
		    fprintf( stderr,
			     "Error OPENing extent count statement.  Error: %d, ISAM: %d.\n",
			     sqlca.sqlcode,
			     sqlca.sqlerrd[1] );
		    exit(6);
		}
		for (i=0; i<ntabs; i++) {
		    EXEC SQL FETCH c_exts
			INTO :tabsizes[i];
		    if (sqlca.sqlcode < 0) {
			fprintf( stderr,
				 "Error FETCHing extent count statement.  Error: %d, ISAM: %d.\n",
				 sqlca.sqlcode,
				 sqlca.sqlerrd[1] );
			exit(6);
		    } else if (sqlca.sqlcode == SQLNOTFOUND)
			break;
		}
		EXEC SQL CLOSE c_exts;
	    }

	    EXEC SQL SET CONNECTION 'dbase';
	    if (sqlca.sqlcode < 0) {
		fprintf( stderr,
			 "Error connection to dbase Err: %d, ISAM: %d.\n",
			 sqlca.sqlcode,
			 sqlca.sqlerrd[1] );
		exit(8);
	    }
	    EXEC SQL DATABASE :dbrec.name;
	    if (sqlca.sqlcode == -387) {
		fprintf( stderr, "        No connect permission!\n" );
		EXEC SQL SET CONNECTION 'master';
		if (sqlca.sqlcode < 0) {
		    fprintf( stderr,
			     "Error setting connection to master Err: %d, ISAM: %d.\n",
			     sqlca.sqlcode,
			     sqlca.sqlerrd[1] );
		    exit(8);
		}
		continue;
	    } else if (sqlca.sqlcode < 0) {
		fprintf( stderr,
			 "Error opening database (%s).  Error: %d, ISAM: %d.\n",
			 dbrec.name,
			 sqlca.sqlcode,
			 sqlca.sqlerrd[1] );
		exit(6);
	    }

	    sprintf( statement,
		     "SELECT t.tabid, tabname, partnum, owner, created, "
		     "fragtype, indexname, dbspace, evalpos, locklevel "
		     "FROM systables t, outer sysfragments f "
		     "WHERE t.tabid = f.tabid "
		     "  AND tabtype = 'T' "
		     "  AND t.tabid > 99 "
		     "  AND t.tabname MATCHES '%s' "
		     "ORDER BY t.tabname, fragtype DESC, evalpos ",
		     dbtable );
	    EXEC SQL PREPARE s_table FROM :statement;
	    if (sqlca.sqlcode < 0) {
		fprintf( stderr,
			 "Error PREPARing table info.  Error: %d, ISAM: %d.\n",
			 sqlca.sqlcode,
			 sqlca.sqlerrd[1] );
		exit(6);
	    }

	    EXEC SQL DECLARE c_table CURSOR WITH HOLD FOR s_table;
	    if (sqlca.sqlcode < 0) {
		fprintf( stderr,
			 "Error DECLARing table info cursor.  Error: %d, ISAM: %d.\n",
			 sqlca.sqlcode,
			 sqlca.sqlerrd[1] );
		exit(6);
	    }

	    EXEC SQL OPEN c_table;
	    if (sqlca.sqlcode < 0) {
		fprintf( stderr,
			 "Error OPENing table info cursor.  "
			 "Error: %d, ISAM: %d.\n",
			 sqlca.sqlcode,
			 sqlca.sqlerrd[1] );
		exit(6);
	    }

	    for (;;) {
		EXEC SQL FETCH c_table
		    INTO :tabrec.tabid,
			:tabrec.tabname,
			:tabrec.partnum,
			:tabrec.owner,
			:tabrec.created,
			:tabrec.fragtype INDICATOR tabind,
			:tabrec.indexname,
			:tabrec.dbspace,
			:tabrec.evalpos,
		        :tabrec.locklevel;
		    if (sqlca.sqlcode < 0) {
			fprintf( stderr,
				 "Error OPENing table info cursor.  "
				 "Error: %d, ISAM: %d.\n",
				 sqlca.sqlcode,
				 sqlca.sqlerrd[1] );
			exit(6);
		    } 
		    if ((printdetail && oldtabid && oldtabid != tabrec.tabid) ||
			sqlca.sqlcode == SQLNOTFOUND)
		    {
			/* Done with last table.  Output extent info. */
			ts = bsearch( oldtabname,
				      tabsizes,
				      ntabs,
				      sizeof (tabsize_t),
				      (int (*)(const void *, const void *))strcmp );
			if (!ts) {
			    fprintf(stderr, "Error seeking extents info!\n" );
			    exit(7);
			}
			printf( "                %ld Extents %ld Pages total, Average extent: %ld pages.\n",
				ts->nextents,
				ts->npages,
				ts->avgpgs );
		    }
		    if (sqlca.sqlcode == SQLNOTFOUND) {
		        oldtabid = 0;
			break;
		    }
		    if (tabrec.partnum && tabrec.tabid != oldtabid) {
			/* New unfragmented table. */
			part.partno = tabrec.partnum;
			if (printdetail) {
			    if ((part.partno / TBSFACT)) {
				for (i=0; i<ndbspaces &&
					 spaces[i].dbsnum < (part.partno / TBSFACT);
				     i++)
				    ;
				if (i >= ndbspaces || spaces[i].dbsnum != (part.partno / TBSFACT))
				    dbspname = "Unknown";
				else
				    dbspname = spaces[i].name;
			    } else
				dbspname = "Shared Memory";

			    printf( "        %s\n",
				    tabrec.tabname );
			    printf( "          %-32s %s\n",
				    tabrec.owner,
				    tabrec.dbspace );
			    printf( "          %-32s  %-10s  %-10s\n",
				    " ",
				    tabrec.created,
				    tabrec.locklevel[0] == 'P' ? "Page" : "Row" );
			    if (tabind >= 0) {
				/* Index fragments. */
				if (strcmp( tabrec.indexname, oldidxname ) == 0)
				    printf( "            %-32s  %s\n",
					    " ",
					    tabrec.dbspace );
				else {
				    printf( "          Idx: %s\n",
					    tabrec.indexname );
				    printf( "            %-32s  %s\n",
					    " ",
					    tabrec.dbspace );
				}
				strcpy( oldidxname, tabrec.indexname );
			    }
			} else {
			    printf( "        %s\n", tabrec.tabname );
			}
		    } else if (tabrec.tabid != oldtabid) {
			/* New fragmented table. */
			if (printdetail) {

			    printf( "        %s\n",
				    tabrec.tabname );
			    printf( "          %-32s %s\n",
				    tabrec.owner,
				    tabrec.dbspace );
			    printf( "          %-32s  %-10s  %-10s\n",
				    " ",
				    tabrec.created,
				    tabrec.locklevel[0] == 'P' ? "Page" : "Row" );
			}
		    } else if (tabrec.fragtype[0] == 'T') {
			/* Continuing fragmented table. */
			if (printdetail)
			    printf( "          %-32s  %s\n",
				    " ",
				    tabrec.dbspace );
		    } else if (tabrec.fragtype[0] == 'I') {
			/* Index fragments. */
			if (printdetail) {
			    printf( "          Idx: %s\n",
				    tabrec.indexname );
			    printf( "            %-32s  %s\n",
				    " ",
				    tabrec.dbspace );
			}
		    } else if (printdetail)
			printf( "Invalid fragment type: %s!\n",
				tabrec.fragtype );
		    oldtabid = tabrec.tabid;
		    strcpy( oldtabname, tabrec.tabname );
	    }
	    
	    EXEC SQL SET CONNECTION 'master';
	    if (sqlca.sqlcode < 0) {
		fprintf( stderr,
			 "Error setting connection to master(%s): %d, ISAM: %d.\n",
			 host,
			 sqlca.sqlcode,
			 sqlca.sqlerrd[1] );
		exit(8);
	    }
	}
    }
    printf( "\n" );

    return 0;
}

void usage( char *prog )
{
    fprintf( stderr,
	     "Usage:\t%s -? [-V] [-D] [-h<host>] [-d<dbase exp>] [-t [<table exp>]]\n\n",
	     prog );
    fprintf( stderr, "\t-?  Print this usage listing only.\n" );
    fprintf( stderr, "\t-V  Print Version and Copyright information only.\n" );
    fprintf( stderr,
	     "\t-D  Print full details.  Default: database names and optionally"
	     "\n\t   table names only.\n" );
    fprintf( stderr,
	     "\t-h  Specify DBSERVERNAME to report on.  Cannot be a shared "
	     "memory \n\t   connection if '-t' is specified.\n" );
    fprintf( stderr,
	     "\t-d  MATCHES clause regular expression specifying database(s) "
	     "to report\n\t   on.  Default: All databases.\n" );
    fprintf( stderr,
	     "\t-t  MATCHES clause regular expression specifying table(s) to "
	     "report on.\n\t   Enables table information reporting optionally "
	     "specifying what\n\t   table(s) to report on.  If -t is omitted "
	     "table reporting is \n\t   suppressed.  Specifying -t with no "
	     "argument reports on all tables.\n" );
#if defined (HAVE_GNU_GETOPT)
    fprintf( stderr,
	     "\t   The space between -t and the table specification is not permitted if \n"
	     "\t   other arguments follow.\n\n" );
#else
    fprintf( stderr,
	     "\t   The space between -t and the table specification is required.\n"
	     "\t   -t must be the last argument.\n\n" );
#endif
}
