/* cleancat.c - Copy strings stripping usernames.
*/
/*
   RCS Header:
   -----------
   $Header: /home/art/GoogleDrive/Latest\040Art's\040Stuff/myschema.d/fork/RCS/cleancat.c,v 2.7 2023/01/04 18:33:35 art Exp $

   RCS Log:
   --------
   $Log: cleancat.c,v $
   Revision 2.7  2023/01/04 18:33:35  art
   Fixed issue with VIEWs under user name filtering options (-O, --set-owner,
   --use-table-owner, & --use-database-owner) stripping commas from the
   SELECT statement that follow closing quotes. Thanks to Øyvind Gjerstad
   for reporting the issue.

   Revision 2.6  2010/01/06 13:55:18  art
   Added new option --set-owner to override all object owners with a specified owner name.

   Revision 2.5  2009/01/01 17:58:25  art
   Removed RCSWhat ident string which was BB specific.

   Revision 2.4  2005/08/03 13:48:20  kagel
   Fixed problem stripping owners from HUGE triggers.

   Revision 2.3  2004/10/25 21:44:50  kagel
   Found and fixed problem with VIEWS defined with quotes embedded in strings such as in
   SELECT or WHERE clauses.

   Revision 2.2  1999/09/14 18:45:50  kagel
   No change, recovery.

   Revision 2.1  1999/09/14 18:45:50  kagel
   Establishing current  source version id for initial working version of
   newly modularized source.  This is Feature Version 6.1 Source revision
   2.1.

   Revision 1.2  1999/09/14 18:45:50  kagel
   Cleanup

   Revision 1.1  1999/09/01 19:25:27  kagel
   Initial revision


*/

static const char RCSHeader[] = "@(#)$Header: /home/art/GoogleDrive/Latest\040Art's\040Stuff/myschema.d/fork/RCS/cleancat.c,v 2.7 2023/01/04 18:33:35 art Exp $";
static const char RCSCompile[] = "@(#)$cc: " __FILE__ " " __DATE__ " " __TIME__ " $";

#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include <sys/types.h>
#include <string.h>
#include <limits.h>
#include <ctype.h>
#include "myschema.h"

char *append( char *to, char *from )
{
  while (*from)
    *to++ = *from++;

  return to;
}

char *cleancat( char *to, char *from, int flag )
{
    char *t = to, *f = from, *T, *tempstr;
    int squote = 0, dquote = 0, inplace = 0;
    char *tempfrom;

    if (!flag && !replace_owners && to == from)
	return to;
    else if (!flag && !replace_owners)
	return strcat( to, from );
    else {
        tempstr = malloc( strlen( from ) + 200 );
	if (to == from) {
	  /* Can't edit in place.  Make a copy. */
	  inplace = 1;
	  tempfrom = malloc( strlen( from ) + 200 );
	  f = tempfrom;
	  strcpy( tempfrom, from );
	}
	/* OK strip user names and optionally replace them with the override. */
	T = tempstr;
	/* If to == from work in-place.  Else append from on to. */
	if (to != from)
	    t += strlen( to );
	while (*f) {
	    switch (*f) {
	    case '"':
		/* Double quote found.  Begin or end string if not in
                   single quoted string already. */
		if (!squote)
		    dquote++;
		*T = *f;
		T++;
		break;
	    case '\'':
		/* Single quote found.  Begin or end string if not in
                   double quoted string already. */
		if (!dquote)
		    squote++;
		*T = *f;
		T++;
		break;
	    case '.':
		if ((squote == 2) || (dquote == 2)) {
		    /* Dot immediately follows quoted string!  Username, eat! */
		    T = tempstr;
		    squote = dquote = 0;
		    if (replace_owners) {
		      int shift;
		      char ostr[40];

		      shift = sprintf( ostr, "\"%s\".", owner_override );
		      t = append( t, ostr );
		      /* t += shift; */
		    }
		} else if (squote || dquote) {
		    /* Dot embedded in string.  Append to tempstr. */
		    *T = *f;
		    T++;
		} else {
		    /* Dot independent of strings.  Append to output. */
		    if (t != f)
			*t = *f;
		    t++;
		}
		break;
	    default:
		if (dquote == 2 || squote == 2) {
		    /* Character other than Dot follows string.
                       Append string to output. */
		    *T = *f;
		    T++;
		    if (T > tempstr) {
			*T = (char)0;
			strcpy( t, tempstr );
			t += strlen( tempstr );
			T = tempstr;
		    }
		    squote = dquote = 0;
		} else if (dquote > 0 || squote > 0) {
		    /* Character part of string.  Append to string. */
		    *T = *f;
		    T++;
		} else {
		    /* Independent character append to output. */
		    if (t != f)
			*t = *f;
		    t++;
		}
		break;
	    }
	    f++;
	}
	if (T > tempstr) {
	    *T = (char)0;
	    strcpy( t, tempstr );
	} else
	    *t = (char)0;
	free( tempstr );
	if (inplace) {
	  free( tempfrom );
	}
   }

    return to;
}


