/* format_triggers.ec - Pretty up trigger code.
*/
/*
   RCS Header:
   -----------
   $Header: /home/art/GoogleDrive/Latest\040Art's\040Stuff/myschema.d/fork/RCS/format_triggers.ec,v 2.23 2021/03/04 18:54:08 art Exp $

   RCS Log:
   --------
   $Log: format_triggers.ec,v $
   Revision 2.23  2021/03/04 18:54:08  art
   Fixed the previous fix.

   Revision 2.22  2021/02/15 17:32:33  art
   Fixed condition where a quoted string immediately follows a comma which 
   was causing keywords within the quotes being processed instead of ignored.
   Thanks to David Grove for pointing this one out!

   Revision 2.21  2019/03/17 15:13:12  art
   Added new options --replace-views and --replace-triggers to support the CREATE OR REPLACE 
   feature in v14.10.

   Revision 2.20  2016/04/11 18:01:35  art
   COsmetic

   Revision 2.19  2016/04/11 14:58:42  art
   Fixed problem with ON clause not preceded by a space if --wrap-at -1 passed. Thanks to
   Cesar Martins for reporting the problem.

   Revision 2.18  2015/11/25 16:43:31  art
   Fixed a problem with certain triggers being formatted incorrectly.
   Thanks to Snorri Bergmann for submitting this one.

   Revision 2.17  2013/07/12 21:25:04  art
   Fixed problem with triggers line wrapping in the middle of a quoted string.

   Revision 2.16  2013/03/06 03:31:50  art
   Upcasing INSTEAD OF, WITH TRIGGER REFERENCES correctly now.
   Fixed "token ,token" strings to "token, token" for neater formatting.
   Fixed wrap calculations.

   Revision 2.15  2013/03/06 03:03:15  art
   Fixed problem with triggers with long comma lists with not white space causing problems
   when loading the schema.
   Added new feature --squish-triggers to eliminate unnecessary white space formatting
   to reduce problems loading the schema caused by IDS's trigger expansion making the
   trigger too big to load (> 65K).  This problem goes away in 12.10 anyway, but ...

   Revision 2.14  2013/03/06 02:10:11  art
   Stage I fix to triggers with long comma separated lists with no spaces.
   Some lines now have only one token in them.  Working on it.

   Revision 2.13  2012/05/30 01:28:34  art
   Removed lines commented out with C++ style comments.  Causes compile time failure on AIX.

   Revision 2.12  2010/11/08 20:40:31  art
   Added new --create_conditionally feature to optionally add " IF NOT EXISTS " clauses to c
   create object statements.  Feature supported by 11.70.

   Revision 2.11  2010/06/10 03:50:51  art
   Added new option --wrap-at to specify or disable the wrap column for
   triggers.
   Fixed problem with breaking quoted strings that were not preceded by
   a space when wrapping.

   Revision 2.10  2010/02/25 19:22:37  art
   Fixed to enable --set-owner to replace the trigger owner and triggering table's owner.

   Revision 2.9  2009/10/18 17:23:16  art
   Fixed collection type handling.  Thanks to Bruce Simms for pointing out this one.
   Fixed summary comment to include table owner if not suppressed ala dbschema.
   Removed unneeded function prototypes.

   Revision 2.8  2009/10/02 06:24:56  art
   Added new option -g to place user privileges into a separate file.
   Improved trigger parser.
   Made indexes and constraint processing recognize and report object state (ie disabled,
   filtering, etc.).
   Improved --filtering option again.

   Revision 2.4  2007/01/02 20:39:46  kagel
   Fixed problem with breaking quoted strings when parsing trigger text for
   owner names.  Fixed.  Thanks to Robert Sosnowski for finding this one also.

   Revision 2.3  2002/10/29 18:58:27  kagel
   Fix to bug Jonathan Leffler pointed out.
   Code was breaking end-of-line comments in triggers that extended past 80
   columns.  Fixed to maintain '--' comments on one line.

   Revision 2.2  1999/09/01 19:28:54  kagel
   No change, recovery.

   Revision 2.1  1999/09/01 19:28:54  kagel
   Establishing current  source version id for initial working version of
   newly modularized source.  This is Feature Version 6.1 Source revision
   2.1.

   Revision 1.1  1999/09/01 19:28:54  kagel
   Initial revision


*/

static const char RCSHeader[] = "@(#)$Header: /home/art/GoogleDrive/Latest\040Art's\040Stuff/myschema.d/fork/RCS/format_triggers.ec,v 2.23 2021/03/04 18:54:08 art Exp $";
static const char RCSCompile[] = "@(#)$cc: " __FILE__ " " __DATE__ " " __TIME__ " $";

#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include <sys/types.h>
#include <string.h>
#include <limits.h>
#include <ctype.h>
#include "tokens.h"

EXEC SQL INCLUDE sqlhdr;
EXEC SQL INCLUDE sqlca;
EXEC SQL INCLUDE sqltypes;
EXEC SQL INCLUDE datetime;

EXEC SQL INCLUDE myschema.eh;
EXEC SQL INCLUDE prototypes.h;

extern int wrap_at, squish_trig;

int in_quoted( char *str, int inquote )
{
  char qfnd;

  switch (inquote) {
  case 0:
    qfnd = (char)0;
    break;
  case 1:
    qfnd = '\'';
    break;
  case 2:
    qfnd = '"';
    break;
  }

  do {
    if (qfnd && *str == qfnd) {
      qfnd = (char)0;
      inquote = 0;
    } else if (!qfnd && (*str == '\'' || *str == '"')) {
      qfnd = *str;
      if (*str == '\'')
	inquote = 1;
      else
	inquote = 2;
    }
    str++;
  } while (*str != (char)0);

  return inquote;
}

void format_trigger( FILE *outfile, char *text, char *state )
{
    char *word, *hold_pos, delchar;
    static char *temp = (char *)NULL;
    static long tlen = 0;
    int pos = 0, wno = 0, force = 0, i, atleft=0, indent, brace = 0, inquote = 0;
    int comment = 0, qspaces = 0, have_of = 0;

    if ((i = strlen( text )) >= tlen)
        temp = (char *)realloc( temp, (tlen = (i+2)) );
    /* Strip off all owner names. */
    if (stripowner || replace_owners)
        cleancat( text, text, stripowner );
    fputs( "\n", outfile );
    tokdel( " \n," );
    word = frstok2( text, &delchar );
    /* Eat multiple delimiters. */
    while (word && *word == (char)0 && delchar != '\n' && delchar != ','){
        word = gettok2( ); /* Updates delchar */
    };
    inquote = in_quoted( word, inquote );

    strtoupper( temp, word );
    force = 0;
    indent = 0;
    atleft = 0;
    while (word) {
        wno++;
        if (wno == 4) {
            fputs( "\n", outfile );
            if (!squish_trig) indent = 2;
            atleft = 1;
	    pos = 0;
        }
        if ((int)(pos + indent + strlen( word )) >= wrap_at && !comment && !inquote) {
            fputs( "\n", outfile );
            if (!squish_trig) indent = 4;
            atleft = 1;
	    pos = 0;
        }
        if (brace && strcmp( word, "(" ) == 0) {
            force = 2;
        } else if (strcmp( word, ")" ) == 0) {
            brace = 1;
        } else {
            brace = 0;
        }
        if (strncmp( word, "--", 2 ) == 0)
            comment = 1;
	hold_pos = word;
        if (!comment && !inquote) 
            if (strcmp( temp, "CREATE" ) == 0) {
                wno = 1;
                word = temp;
            } else if (strcmp( temp, "TRIGGER" ) == 0) {
                word = temp;
            } else if (strcmp( temp, "UPDATE" ) == 0) {
		if (wno != 4) force = 4;
                word = temp;
            } else if (strcmp( temp, "OF" ) == 0) {
		have_of = 1;
                word = temp;
            } else if (strcmp( temp, "DELETE" ) == 0) {
		if (wno != 4) force = 4;
                word = temp;
            } else if (strcmp( temp, "INSERT" ) == 0) {
		if (wno != 4) force = 4;
                word = temp;
            } else if (strcmp( temp, "INTO" ) == 0) {
                word = temp;
            } else if (strcmp( temp, "ON" ) == 0) {
		if (have_of == 1) {
		    force = 4;
		    have_of = 0;
		}
                word = temp;
            } else if (strcmp( temp, "REFERENCING" ) == 0) {
                force = 4;
                word = temp;
            } else if (strcmp( temp, "OLD" ) == 0) {
                word = temp;
            } else if (strcmp( temp, "NEW" ) == 0) {
                word = temp;
            } else if (strcmp( temp, "AS" ) == 0) {
                word = temp;
            } else if (strcmp( temp, "BEFORE" ) == 0) {
                force = 4;
                word = temp;
            } else if (strcmp( temp, "AFTER" ) == 0) {
                force = 4;
                word = temp;
            } else if (strcmp( temp, "FOR" ) == 0) {
                force = 4;
                word = temp;
            } else if (strcmp( temp, "EACH" ) == 0) {
                word = temp;
            } else if (strcmp( temp, "ROW" ) == 0) {
                word = temp;
            } else if (strcmp( temp, "WHEN" ) == 0) {
                force = 2;
                word = temp;
            } else if (strcmp( temp, "AND" ) == 0) {
                word = temp;
            } else if (strcmp( temp, "OR" ) == 0) {
                word = temp;
            } else if (strcmp( temp, "IS" ) == 0) {
                word = temp;
            } else if (strcmp( temp, "NOT" ) == 0) {
                word = temp;
            } else if (strcmp( temp, "NULL" ) == 0) {
                word = temp;
            } else if (strcmp( temp, "BETWEEN" ) == 0) {
                word = temp;
            } else if (strcmp( temp, "IN" ) == 0) {
                word = temp;
            } else if (strcmp( temp, "LIKE" ) == 0) {
                word = temp;
            } else if (strcmp( temp, "MATCHES" ) == 0) {
                word = temp;
            } else if (strcmp( temp, "ESCAPE" ) == 0) {
                word = temp;
            } else if (strcmp( temp, "TODAY" ) == 0) {
                word = temp;
            } else if (strcmp( temp, "USER" ) == 0) {
                word = temp;
            } else if (strcmp( temp, "CURRENT" ) == 0) {
                word = temp;
            } else if (strcmp( temp, "SITENAME" ) == 0) {
                word = temp;
            } else if (strcmp( temp, "DBSERVERNAME" ) == 0) {
                word = temp;
            } else if (strcmp( temp, "SELECT" ) == 0) {
                force = 2;
                word = temp;
            } else if (strcmp( temp, "FROM" ) == 0) {
                force = 4;
                word = temp;
            } else if (strcmp( temp, "WHERE" ) == 0) {
                force = 4;
                word = temp;
            } else if (strcmp( temp, "ORDER" ) == 0) {
                force = 4;
                word = temp;
            } else if (strcmp( temp, "GROUP" ) == 0) {
                force = 4;
                word = temp;
            } else if (strcmp( temp, "BY" ) == 0) {
                word = temp;
            } else if (strcmp( temp, "HAVING" ) == 0) {
                force = 4;
                word = temp;
            } else if (strcmp( temp, "EXISTS" ) == 0) {
                word = temp;
            } else if (strcmp( temp, "ANY" ) == 0) {
                word = temp;
            } else if (strcmp( temp, "ALL" ) == 0) {
                word = temp;
            } else if (strcmp( temp, "SOME" ) == 0) {
                word = temp;
            } else if (strcmp( temp, "EXECUTE" ) == 0) {
                force = 2;
                word = temp;
            } else if (strcmp( temp, "PROCEDURE" ) == 0) {
                word = temp;
            } else if (strcmp( temp, "SET" ) == 0) {
                word = temp;
            } else if (strcmp( temp, "VALUES" ) == 0) {
                force = 2;
                word = temp;
            } else if (strcmp( temp, "DISTINCT" ) == 0) {
                word = temp;
            } else if (strcmp( temp, "UNIQUE" ) == 0) {
                word = temp;
            } else if (strcmp( temp, "UNION" ) == 0) {
                force = 2;
                word = temp;
            } else if (strcmp( temp, "ALL" ) == 0) {
                word = temp;
            } else if (strcmp( temp, "OUTER" ) == 0) {
                word = temp;
            } else if (strcmp( temp, "LEFT" ) == 0) {
                word = temp;
            } else if (strcmp( temp, "RIGHT" ) == 0) {
                word = temp;
            } else if (strcmp( temp, "INNER" ) == 0) {
                word = temp;
            } else if (strcmp( temp, "JOIN" ) == 0) {
                word = temp;
	    } else if (strcmp( temp, "INSTEAD" ) == 0) {
		word = temp;
	    } else if (strcmp( temp, "REFERENCES" ) == 0) {
		word = temp;
	    } else if (strcmp( temp, "WITH" ) == 0) {
		word = temp;
            } else if (wno == 3) {
                /* Strip off owner name. */
                if (stripowner && word[0] == '"')
                    word = strchr( word, '.' ) + 1;
            }
        if (force) {
            if (!atleft)
                fputs( "\n", outfile );
            atleft = 1;
            if (!squish_trig) indent = force;
            force = 0;
	    pos = 0;
        }
        if (indent && atleft) {
            for (i=0; i<indent; i++)
                fputs( " ", outfile );
            pos = indent;
            indent = 0;
        }

	/* Triggers with a comma separated list of columns or vars without
	   spaces can cause 'word' to exceed the SQL line length limit.  Need
	   to break them up here.  Follow wrap_at if not disabled.
	*/

	if (wrap_at > 0) {
	    int slen, wlen, wrap_to;

	    for (;;) {
		/* If the current word is too long but has embedded commas, break it up. */
		wlen = find_break( word, ',', (wrap_at - pos) );
		if (wlen <= 0)
		    break;
		if (wlen > (wrap_at - pos)) {
		    pos = 0;
		    atleft = 1;
		    if (indent) {
			for (i=0; i<indent; i++)
			    fputs( " ", outfile );
			pos = indent;
		    }
		}

		if ((strncmp( word, "TRIGGER", 7 ) == 0 || strncmp( word, "trigger", 7 ) == 0) && Replace_Triggers == TRUE) {
		  fputs( "OR REPLACE ", outfile );
		}

		fwrite( word, wlen, 1, outfile );
		pos += wlen;
		/* Don't add a space after a word that is followed by a space separated comma! */
		/* Look at the original string asa "word may no longer point there! */
		if (hold_pos[wlen+1] != ',') {
		    fputc( ' ', outfile );
		    pos++;
		}
		/* OK, if the line is getting too long.  Break it up. */
		if (wlen < strlen(word) && pos + strlen((word + wlen + 1)) >= wrap_at && delchar != '\n' && !comment && !inquote) {
		    fputc( '\n', outfile );
		    pos = 0;
		    atleft = 1;
		    if (indent) {
			for (i=0; i<indent; i++)
			    fputs( " ", outfile );
			pos = indent;
		    }
		}
		if (strncmp( word, "TRIGGER", wlen ) == 0 && !inquote && wno == 2 && create_conditionally == TRUE) {
		    fprintf( outfile, "IF NOT EXISTS " );
		    pos += 15;
		}
		
		if (word)
		    inquote = in_quoted( word, inquote );
		word += wlen;
		if (*word == (char)0)
		    break;
	    }
	} else {

	    if ((strncmp( word, "TRIGGER", 7 ) == 0 || strncmp( word, "trigger", 7 ) == 0) && Replace_Triggers == TRUE) {
	        fputs( "OR REPLACE ", outfile );
	    }

	    fprintf( outfile, "%s ", word );
	    if (strcmp( word, "TRIGGER" ) == 0 && !inquote && wno == 2 && create_conditionally == TRUE) {
		fprintf( outfile, "IF NOT EXISTS " );
		pos += 15;
	    }

	    if (word)
		inquote = in_quoted( word, inquote );
	    pos += strlen( word ) + 1;
	}

	if (delchar == ',')
	    fputs( ",", outfile );

        if (comment && delchar == '\n') {
            /* EOL comment ended by a newline! */
            force = 1;
            comment = 0;
        }
        atleft = 0;
        qspaces = 0;
        do { /* Eat multiple delimiters. */
            word = gettok2( ); /* Updates delchar */
            if (inquote && word && *word == (char)0 && delchar == ' ')
                qspaces++;
        } while (word && *word == (char)0 && delchar != '\n' && delchar != ',');
        while (inquote && qspaces--)
            fputs( " ", outfile );
        if (word)
            strtoupper( temp, word );
        else
            temp[0] = (char)0;
    }
    fputs( "\n", outfile );
}
