/* getsysdata.ec - Functions to implement the -a (SMICounts) option and gather 
		other SMI data.
*/
/*
   RCS Header:
   -----------
   $Header: /home/art/GoogleDrive/Latest\040Art's\040Stuff/myschema.d/fork/RCS/getsysdata.ec,v 2.28 2020/05/10 18:42:36 art Exp $

   RCS Log:
   --------
   $Log: getsysdata.ec,v $
   Revision 2.28  2020/05/10 18:42:36  art
   Fixed the dbspace information statement and structure.

   Revision 2.27  2017/12/26 16:34:10  art
   Fixed sprintf.

   Revision 2.26  2015/04/13 22:43:19  art
   Changed getTableCount() to count all of sysptnhdr since there is no longer
   any way to identify tables.  This fixed a problem causing runs with -m
   to either crash or report a spurious -1215 error.

   Revision 2.25  2014/11/25 15:31:57  art
   Performance improvement when -l is specified by reducing the dependence on sysmaster.

   Revision 2.24  2012/11/28 12:34:35  art
   Fixed version problems around serial8 and bigserial initial value handling again.

   Revision 2.23  2011/04/29 13:35:51  art
   Fixed problems handling SERIAL8 type column initialization.
   Added support for STATLEVEL and STATCHANGE in CREATE TABLE statements.
   Added support for SECURITY POLICY in CREATE TABLE statements.  Output of policy
   definitions still not supported.
   Added option --force_autostats to force the output of STATLEVEL AUTO which is
   the default and so not printed normally.

   Revision 2.22  2010/12/13 17:37:40  art
   *** empty log message ***

   Revision 2.21  2010/12/13 17:32:36  art
   *** empty log message ***

   Revision 2.20  2010/12/13 17:28:31  art
   OK, fixed this for all server types.  Default pagesize will now be calculated
   correctly no matter what!

   Revision 2.19  2010/12/13 17:19:26  art
   Added code to handle versions that do not support the sh_pagesize variable in sysshmvals.

   Revision 2.18  2010/12/13 15:31:10  art
   typo

   Revision 2.17  2010/12/13 15:08:34  art
   Added diagnostics.

   Revision 2.16  2009/07/14 18:53:35  art
   Fixed version specific problems with the pages size calcs.
   Fixed version specific problem with handling user defined system access
   methods.

   Revision 2.15  2009/07/13 15:36:57  art
   Fixed problem with auto extent adjustment.

   Revision 2.14  2009/06/24 00:49:55  art
   Fixed problem with pagesize when run against IDS v7.xx and earlier.

   Revision 2.13  2009/03/20 14:09:22  art
   Fixed extent calculations to take account for differing page sizes, even
   between fragments of the same table.

   Revision 2.12  2009/02/15 21:42:41  art
   Fixed SERIAL, SERIAL8, and BIGSERIAL handling.  BIGINT and BIGSERIAL handling added.

   Revision 2.11  2009/02/11 18:30:34  art
   Fixed problem with LVARCHARS on AIX-64bit.

   Revision 2.10  2009/01/01 17:58:25  art
   Removed RCSWhat ident string which was BB specific.

   Revision 2.9  2004/09/14 17:26:54  kagel
   Fixing ANSI database access.

   Revision 2.8  2004/06/28 15:02:11  kagel
   Recoded getTableCount() to not use a temp table for a unique count.

   Revision 2.7  2002/10/29 21:56:27  kagel
   Made npdata agree with npused when -M used.

   Revision 2.6  2002/10/29 16:26:13  kagel
   Added control of extent determination method for fragmented tables.

   Revision 2.5  2001/11/08 22:07:26  kagel
   Made temp table NON-LOGGED.

   Revision 2.4  2001/01/16 22:15:42  kagel
   Included errno.h.

   Revision 2.3  2001/01/16 21:46:18  kagel
   Added protection to calls to malloc().

   Revision 2.2  1999/09/15 20:59:37  kagel
   No change, recovery.

   Revision 2.1  1999/09/15 20:59:37  kagel
   Establishing current  source version id for initial working version of
   newly modularized source.  This is Feature Version 6.1 Source revision
   2.1.

   Revision 1.2  1999/09/15 20:59:37  kagel
   Fixed function to return table count.

   Revision 1.1  1999/09/08 18:19:49  kagel
   Initial revision


*/

static const char RCSHeader[] = "@(#)$Header: /home/art/GoogleDrive/Latest\040Art's\040Stuff/myschema.d/fork/RCS/getsysdata.ec,v 2.28 2020/05/10 18:42:36 art Exp $";
static const char RCSCompile[] = "@(#)$cc: " __FILE__ " " __DATE__ " " __TIME__ " $";

#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include <sys/types.h>
#include <string.h>
#include <limits.h>
#include <ctype.h>
#include <errno.h>

EXEC SQL INCLUDE sqlhdr;
EXEC SQL INCLUDE sqlca;
EXEC SQL INCLUDE sqltypes;
EXEC SQL INCLUDE datetime;

EXEC SQL INCLUDE myschema.eh;

EXEC SQL INCLUDE prototypes.h;

void getSMICounts()
{
    long idx;
    EXEC SQL BEGIN DECLARE SECTION;
    string statement[32768];
    string ser4str[36], ser8str[36], bigserstr[36], ptntbl[36];
    string op[5], pgstr[30];
    string ser8val[30], serbigval[30];
    EXEC SQL END DECLARE SECTION;

    if (HowExtents == ES_Average) {
      strcpy( op, "avg" );
    } else if (HowExtents == ES_Maximum) {
      strcpy( op, "max" );
    } else if (HowExtents == ES_Minimum) {
      strcpy( op, "min" );
    }

    if (major_vers < 10) {
      strcpy( ptntbl, "sysptnhdr" );
      strcpy( ser4str, "(serialv)" );
      strcpy( ser8str, "( 1 )" );
      strcpy( bigserstr, "( 1 )" );
      snprintf( pgstr, sizeof pgstr, "%d", getSystemPageSize() );
    } else {
      strcpy( ptntbl, "sysactptnhdr" );
      if (major_vers == 10)
         strcpy( ser4str, "(serialv)" );
      else
         strcpy( ser4str, "(cur_serial4)" );
      strcpy( ser8str, "(cur_serial8)" );
      if (major_vers > 11 || (major_vers == 11 && minor_vers >= 50)) {
	  strcpy( bigserstr, "(cur_bigserial)::int8" );
      } else {
	  strcpy( bigserstr, " 1::int8 " );
      }
      strcpy( pgstr, "(sp.pagesize / 1024)" );
    }

    snprintf( statement,
	      sizeof statement,
    	      "SELECT dbsname, tabname, max%s serialv, "
	      "       max%s serial8, max%s bigserial, count(*) nfrags, "
	      "       sum(npused * %s) npused, "
	      "       sum(npdata * %s) npdata, sum(nptotal * %s) nptotal, "
	      "       %s(npdata * %s) maxdata, %s(npused * %s) maxused "
	      "FROM \"informix\".systabnames st, \"informix\".%s sp "
	      "WHERE st.partnum = sp.partnum "
	      "  AND dbsname = '%s' AND tabname MATCHES '%s' "
	      "GROUP BY 1,2 "
	      "ORDER BY 1,2 ",
	      ser4str,
	      ser8str,
	      bigserstr,
	      pgstr,
	      pgstr,
	      pgstr,
	      op,
	      pgstr,
	      op,
	      pgstr,
	      ptntbl,
	      database,
	      tablename );

    if (verbose > 1) {
	fprintf( stderr,
		 "GetSMICounts: Page query is:\n%s\n",
		 statement );
	fprintf( stderr,
		 "Major_vers = %d, pgstr = '%s'\n",
		 major_vers,
		 pgstr );
    }

    EXEC SQL PREPARE page_stmt FROM :statement;
    if ( sqlca.sqlcode ) {
	fprintf( stderr,
		 "Cannot PREPARE statement for table space info; error=%d, isam=%d\n",
		 sqlca.sqlcode,
		 sqlca.sqlerrd[1] );
	fprintf( stderr,
		 "SQL: \n%s';\nNote: %s.\n",
		 statement,
		 sqlca.sqlerrm );
	rexit(99);
    }

    EXEC SQL DECLARE page_cur CURSOR FOR page_stmt;
    if ( sqlca.sqlcode ) {
	fprintf( stderr,
		 "Cannot DECLARE CURSOR for table space info; error=%d, isam=%d\n",
		 sqlca.sqlcode,
		 sqlca.sqlerrd[1] );
	rexit(99);
    }

    EXEC SQL OPEN page_cur;
    if ( sqlca.sqlcode ) {
	fprintf( stderr,
		 "Cannot OPEN CURSOR for table space; error=%d, isam=%d\n",
		 sqlca.sqlcode,
		 sqlca.sqlerrd[1] );
	rexit(99);
    }

    for (idx=0;;idx++) {
      /* Note: all 'pgs' and 'max' values are now in KB not pages -- ASK */
	EXEC SQL FETCH page_cur
	    INTO :pagecounts[idx].dbsname,
	    :pagecounts[idx].tabname,
	    :pagecounts[idx].serialv,
	    :ser8val,
	    :serbigval,
	    :pagecounts[idx].nfrags,
	    :pagecounts[idx].pgsused,
	    :pagecounts[idx].pgsdata,
	    :pagecounts[idx].pgstotal,
	    :pagecounts[idx].maxdata,
	    :pagecounts[idx].maxused
	    ;
	if (sqlca.sqlcode < 0) {
	    fprintf( stderr,
		     "Cannot FETCH table space info.  Error=%d, ISAM=%d\n",
		     sqlca.sqlcode,
		     sqlca.sqlerrd[1] );
	    rexit(99);
	} else if (sqlca.sqlcode == SQLNOTFOUND) {
	    break;
	}
	/* ASK - Note:
	   serial - Contains the current last serial inserted.
	        It goes negative in sysactptnhdr/sysptnhdr if it
	        wraps but, the value next inserted is the ones 
		complement of that value (serialv - (-1 * LONG_MAX)).
		Continues to count up.
	   serial8 - Contains the next serial8 value to be added.
		Wraps in place after 9223372036854775807 to 1
	   bigserial = Contains the next bigserial value to be added.
		Goes negative after 9223372036854775807 and inserts
		the negative value.  Continues to count up.

	   ASK - There seems to be a bug in ESQL/C handling the int8 internal type cast of
		64bit integers.  So, fetching SERIAL8 and BIGSERIAL into strings and translating
		to long long integers in C.
	*/

	pagecounts[idx].serialv8 = atoll( ser8val );
	pagecounts[idx].serialvBig = atoll( serbigval );
	if (pagecounts[idx].serialv < 0)
	  pagecounts[idx].serialv = pagecounts[idx].serialv - (-2147483647);
    }
    EXEC SQL CLOSE page_cur;
    EXEC SQL FREE page_cur;
    EXEC SQL FREE page_stmt;
}

void getIndexSummary()
{
    EXEC SQL BEGIN DECLARE SECTION;
    t_idxcounts counts;
    short keyind;
    EXEC SQL END DECLARE SECTION;
    long idx;
    const int tabincr = 10000;

    if (verbose)
	fprintf( stderr,
		 "Collecting index sizing information for table summary.\n" );

    alltables = tabincr;

    /* Get row count for each table. */
    EXEC SQL DECLARE row_cur CURSOR FOR
	SELECT dbsname, tabname, sum(nrows)
	FROM "informix".systabnames, "informix".sysptnhdr
	WHERE systabnames.partnum = sysptnhdr.partnum
	AND dbsname = :database and tabname matches :tablename
	GROUP BY 1,2
	ORDER BY 1,2;
    if ( sqlca.sqlcode ) {
	fprintf( stderr,
		 "Cannot DECLARE CURSOR for row sizing; error=%d, isam=%d\n",
		 sqlca.sqlcode,
		 sqlca.sqlerrd[1] );
	rexit(99);
    }

    /* Gather index sizes from sysmaster for table(s). */
    EXEC SQL DECLARE idx_cur CURSOR FOR
/*		SELECT dbsname, tabname, sum( ((keylen * 1.5) + 6) ) */
	SELECT dbsname, tabname, sum( ((keylen + 4)) ) * 1.5,
	((sum( ((keylen + 4)) ) * 1.5) / count(*))
	FROM "informix".systabnames, outer "informix".sysptnkey
	WHERE systabnames.partnum = sysptnkey.partnum
	AND kpartno = 0
	AND dbsname = :database /* and tabname matches :tablename */
	GROUP BY 1,2
	ORDER BY 1,2;
    if ( sqlca.sqlcode ) {
	fprintf( stderr,
		 "Cannot DECLARE CURSOR for index sizing; error=%d, isam=%d\n",
		 sqlca.sqlcode,
		 sqlca.sqlerrd[1] );
	rexit(99);
    }

    idxcounts = (t_idxcounts *)malloc( alltables * sizeof (t_idxcounts) );
    if (!idxcounts) {
	fprintf( stderr,
		 "Cannot allocate memory for index sizing.  Errno=%d.\n",
		 errno );
	rexit(99);
    }

    EXEC SQL OPEN idx_cur;
    if ( sqlca.sqlcode ) {
	fprintf( stderr,
		 "Cannot OPEN CURSOR for index sizing; error=%d, isam=%d\n",
		 sqlca.sqlcode,
		 sqlca.sqlerrd[1] );
	rexit(99);
    }

    EXEC SQL OPEN row_cur;
    if ( sqlca.sqlcode ) {
	fprintf( stderr,
		 "Cannot OPEN CURSOR for index sizing; error=%d, isam=%d\n",
		 sqlca.sqlcode,
		 sqlca.sqlerrd[1] );
	rexit(99);
    }

    counts.dbsname[0] = (char)0;
    counts.tabname[0] = (char)0;
    //    for (idx=0; idx < alltables; idx++) {
    for (idx=0;;idx++) {
	int l, m;

	EXEC SQL FETCH idx_cur
	    INTO :idxcounts[idx].dbsname,
	    :idxcounts[idx].tabname,
	    :idxcounts[idx].keylen : keyind,
	    :idxcounts[idx].detlen
	    ;
	if (sqlca.sqlcode < 0) {
	    fprintf( stderr,
		     "Cannot FETCH index sizing info.  Error=%d, ISAM=%d\n",
		     sqlca.sqlcode,
		     sqlca.sqlerrd[1] );
	} else if (sqlca.sqlcode == SQLNOTFOUND) {
	    break;
	}
		
	while ((l = strcmp( idxcounts[idx].dbsname, counts.dbsname )) > 0
	       || (l == 0 &&
		   (m = strcmp( idxcounts[idx].tabname, counts.tabname )) > 0))
	{
	    EXEC SQL FETCH row_cur
		INTO :counts.dbsname,
		:counts.tabname,
		:counts.nrows
		;
	    if (sqlca.sqlcode < 0) {
		fprintf( stderr,
			 "Cannot FETCH index sizing info.  Error=%d, ISAM=%d\n",
			 sqlca.sqlcode,
			 sqlca.sqlerrd[1] );
	    } else if (sqlca.sqlcode == SQLNOTFOUND) {
		break;
	    }
	}
	if (l != 0 || m != 0)
	    idxcounts[idx].nrows = -1;
	else {
	    idxcounts[idx].nrows = counts.nrows;
	}
	if (keyind < 0)
	    idxcounts[idx].keylen = 0;  /* No indexes.  Replace NULL. */
	if (idx >= (alltables - 1)) {
	    alltables += tabincr;
	    idxcounts = realloc( idxcounts, alltables * sizeof (t_idxcounts) );
	}
    }
    EXEC SQL CLOSE idx_cur;
}

int getSystemPageSize()
{
    EXEC SQL BEGIN DECLARE SECTION;
    static int pagesize = 0;
    EXEC SQL END DECLARE SECTION;

    if (verbose)
	fprintf( stderr, "Fetching system pagesize... " );

    if (pagesize == 0) {
	EXEC SQL SELECT sh_pagesize/1024
	    INTO :pagesize FROM "informix".sysshmvals;
	if (sqlca.sqlcode == -206 || sqlca.sqlcode == -217) {
	    EXEC SQL SELECT pagesize/1024
		INTO :pagesize FROM "informix".sysdbspaces WHERE dbsnum = 1;
	    if (sqlca.sqlcode == -206 || sqlca.sqlcode == -217) {
#if defined( _AIX )  || defined( WINDOWS )
	        /* Windows is 4K also,  but no one has gotten this to compile on windows yet, so... */
	        pagesize = 4;
#else
	        pagesize = 2;
#endif
	    }
	} else if (sqlca.sqlcode < 0) {
	    fprintf( stderr, "Cannot fetch pagesize. SQL: %d, ISAM: %d.\n", sqlca.sqlcode, sqlca.sqlerrd[1] );
	    rexit(97);
	}
	if (verbose)
	    fprintf( stderr, "System pagesize is: %dK.\n", pagesize );
    }

    return pagesize;
}

long getDbspaceInfo()
{
    EXEC SQL BEGIN DECLARE SECTION;
    long idx, nspaces;
    EXEC SQL END DECLARE SECTION;

    if (verbose)
	fprintf( stderr, "Collecting DBSPACE information.\n" );
    EXEC SQL SELECT count(*) INTO :nspaces FROM "informix".sysdbspaces;
    if (sqlca.sqlcode < 0) {
	fprintf( stderr, "Cannot fetch dbspace count. SQL: %d, ISAM: %d.\n", sqlca.sqlcode, sqlca.sqlerrd[1] );
	rexit(97);
    }
    sysdbspaces =
	(struct s_sysdbspaces *)malloc( nspaces * sizeof (struct s_sysdbspaces) );
    if (!sysdbspaces) {
	fprintf( stderr,
		 "Cannot allocate memory for index sizing.  Errno=%d.\n",
		 errno );
	rexit(99);
    }

    strcpy( statement, 
	    "SELECT dbsnum, name, owner, pagesize, fchunk, nchunks, "
	    "       is_mirrored, is_blobspace, is_sbspace, is_temp, flags "
	    "FROM sysmaster:sysdbspaces "
	    "ORDER BY dbsnum" );

    EXEC SQL PREPARE spcs FROM :statement;
    if (sqlca.sqlcode) {
	fprintf( stderr,
		 "Sysdbspaces not available.  Ignoring dbspace IN CLAUSE.\n" );
	gotdbs = 0;
    } else {
	
	EXEC SQL
	    DECLARE dbs CURSOR FOR spcs;
	if (sqlca.sqlcode) {
	    fprintf( stderr,
		     "Cannot DECLARE CURSOR for dbspace query. error=%d, ISAM=%d\n",
		     sqlca.sqlcode,
		     sqlca.sqlerrd[1] );
	    rexit(4);
	}
	EXEC SQL OPEN dbs;
	if (sqlca.sqlcode) {
	    fprintf( stderr,
		     "Cannot OPEN CURSOR for dbspace query. error=%d, ISAM=%d\n",
		     sqlca.sqlcode,
		     sqlca.sqlerrd[1] );
	    rexit(5);
	}
	for (idx=0; idx<nspaces; idx++) {
	    EXEC SQL
		FETCH dbs 
		INTO :sysdbspaces[idx].dbsnum, 
		:sysdbspaces[idx].name,
		:sysdbspaces[idx].owner,
		:sysdbspaces[idx].fchunk,
		:sysdbspaces[idx].nchunks,
		:sysdbspaces[idx].is_mirrored,
		:sysdbspaces[idx].is_blobspace,
		:sysdbspaces[idx].is_temp,
		:sysdbspaces[idx].flags;
	    if (sqlca.sqlcode == SQLNOTFOUND)
		break;
	    else if (sqlca.sqlcode) {
		fprintf( stderr,
			 "Cannot FETCH CURSOR for dbspace query. error=%d, ISAM=%d\n",
			 sqlca.sqlcode,
			 sqlca.sqlerrd[1] );
		rexit(6);
	    } else  /* Strip trailing spaces. */
		strtok( sysdbspaces[idx].name, " " );
	}
	EXEC SQL CLOSE dbs;
	EXEC SQL FREE dbs;
	EXEC SQL FREE spcs;
	gotdbs = 1;
    }

    return nspaces;
}

int getTableCount( char *dbs, char *table )
{
    EXEC SQL BEGIN DECLARE SECTION;
    int ntables;
    char *database = dbs, *tablename = table;
    string db[IDBYTES], tb[IDBYTES];
    string ltb[IDBYTES];
    string qry[2049];
    EXEC SQL END DECLARE SECTION;

    /* Yes, "table" argument is being ignored. See change log. - ASK */

    snprintf( qry,
	      sizeof qry,
	      "SELECT count(*) "
	      "FROM %s:sysptnhdr;",
	      "sysmaster" );

    EXEC SQL PREPARE qry_s FROM :qry;
    if (sqlca.sqlcode) {
	fprintf( stderr,
		 "Cannot PREPARE table count query; error=%d, isam=%d\n",
		 sqlca.sqlcode,
		 sqlca.sqlerrd[1] );
	rexit(99);
    }

    EXEC SQL EXECUTE qry_s INTO :ntables;
    if (sqlca.sqlcode) {
	fprintf( stderr,
		 "Cannot EXECUTE table count query; error=%d, isam=%d\n",
		 sqlca.sqlcode,
		 sqlca.sqlerrd[1] );
	rexit(99);
    }

    return ntables;
}
