/* myschema.eh - Informix ESQL version of myschema header file.
*/
/*
   RCS Header:
   -----------
   $Header: /home/art/GoogleDrive/Latest\040Art's\040Stuff/myschema.d/fork/RCS/myschema.eh,v 2.92 2024/05/01 15:28:49 art Exp $

   RCS Log:
   --------
   $Log: myschema.eh,v $
   Revision 2.92  2024/05/01 15:28:49  art
   Stripped trailing spaces from the end of procedure/function
   definitions. Thanks to Doug Lawry for the suggestion.
   New option --no-sequence-init to suppress the ALTER statement
   that sets a sequence to its current value. Thanks to Doug
   Lawry for the suggestion. New option --no-er-columns to 
   suppress printing ER related special columns such as ERKEY,
   REPLCHECK, VERCOLS, & CRCOLS.
   Thanks to Paul Watson for this suggestion.

   Revision 2.91  2023/12/11 14:21:26  art
   Added support for the LARGE and SMALL keywords in CREATE TABLE
   statements.
   Added support for INVISIBLE indexes.
   Added new options to suppress the SMALL and LARGE keywords in
   CREATE TABLE statements and the INVISIBLE keyword in CREATE
   INDEX statements to permit using myschema to port to pre-v15
   servers and to recreate tables as LARGE or SMALL depending on
   server capabilities and the v15 implicit table type specified
   in the ONCONFIG file (SMALL_TABLES=0|1).

   Revision 2.90  2022/11/06 18:21:05  art
   Support for new option --include-sysprocs

   Revision 2.89  2021/05/13 22:42:35  art
   Added MyexportOrder to the sort order enum.

   Revision 2.88  2021/03/09 16:19:40  art
   New option --for-er adds WITH REPLCHECK and WITH ERKEY to each table to 
   create a schema suitable for building an ER ready copy of the database.
   Also generates code to create the index on the primary key column and
   the ifx_replcheck column if a primary key exists.

   Revision 2.87  2020/12/02 18:55:16  art
   Fixed -o to sort systable objects other than tables in name order within type.
   Fixed the interaction between -o and --dependency-order  to sort tables my name
   within dependency level and to sort all non-systables objects by name (was
   reverting to id order when --dependency-order is specified.

   Revision 2.86  2020/05/10 18:42:36  art
   Fixed the dbspace information statement and structure.

   Revision 2.85  2020/05/07 14:27:28  art
   Added global variable NoPrivSyns to manage --no-private-synonyms option.

   Revision 2.84  2019/04/09 20:03:37  art
   Improved stored procedure handling. Stripping out IF NOT EXISTS from the procedure
   source (apparently the engine saves that. New options --no-proc-cleaning to disable
   the scrubbing and rewriting of the CREATE PROCEDURE/FUNCTION statement line because
   that sometimes causes issues.

   Revision 2.83  2019/03/17 15:13:12  art
   Added new options --replace-views and --replace-triggers to support the CREATE OR REPLACE 
   feature in v14.10.

   Revision 2.82  2019/03/15 14:48:19  art
   Added support for CREATE OR REPLACE routines with --replace-procs option (v14.10 feature).

   Revision 2.81  2018/12/31 21:26:36  art
   Fixed a couple of defines.

   Revision 2.80  2018/06/22 16:30:05  art
   New options: --use-database-owner, --use-table-owner, --constraint-file
   Fixed some problems with the partial schema file output.
   Expanded "owner"s to all objects.

   Revision 2.79  2018/02/26 23:39:57  art
   Changed npused to double to reflect new catalog tables.

   Revision 2.78  2016/12/13 23:51:19  art
   Added support for unencrypted storage spaces when encrypted storage is enabled (12.10.xC8+).
   Added new option --no-check to output CHECK constraints with the NOVALIDATE option for
   migrating databases (effective only on a populated table created without CHECK constraints.

   Revision 2.77  2016/11/28 00:37:38  art
   Added new option, --keep-generated-names, to create constraints with the names auto-generated
   in the existing schema. The default is to not explicitely name constraints that have generated
   names.

   Revision 2.76  2015/05/07 13:36:35  art
   Added code to print out VTI Access Method parameter lists for virtual tables.

   Revision 2.75  2014/09/21 22:35:26  art
   New option --no-validate added. When specified all FOREIGN KEY constrataints
   will be generated with the NOVALIDATE clause included.

   Revision 2.74  2013/12/29 20:03:19  art
   Added support for v12.10 ROLLING fragments DETACH and DISCARD keywords.
   Support for DETACH (ANY|INTERVAL FIRST|INTERVAL ONLY) no in there yet
   because the documented syntax doesn't work!

   Revision 2.73  2013/11/20 15:18:06  art
   Added #define of atoll() to strtoll() for HPUX users in the header file.

   Revision 2.72  2013/09/11 18:09:28  art
   Added new option, --dependency-order, to order the tables based on foreign key
   dependencies with dependent tables output after the tables that they reference.

   Revision 2.71  2013/07/08 13:04:07  art
   Opps.  Initial fix for v12.10 viewtext size change broke myschema for earlier releases.
   This fixes that.

   Revision 2.70  2013/06/20 17:47:32  art
   Looks like sysviews.viewtext was expanded from 64 bytes to 256 bytes in V12! Fixed.

   Revision 2.69  2013/03/19 23:16:30  art
   Fixed handling of attached/IN TABLE indexes. Fixed a problem when
   -O and --drop-proc are specified together causing the DROP routine
   statement to have no argument list. Added missing support for
   REFERENCES BYTE/TEXT arguments in DROP routine statements when
   --drop-proc is specified.

   Revision 2.68  2013/03/14 03:44:21  art
   Added new option --no-storage-options which suppresses "IN <dbspace>" and
   FRAGMENT BY clauses.  This can ease some migrations.

   Revision 2.67  2013/03/11 16:55:30  art
   Added new option -K to name autonamed indexes and unnamed constraints as <table>_pk
   for primary key indexes and <table>_<type>### where type is one of "uk", "fk", "ak"
   and ### is a sequential number within the type.
   Fixed -U.
   Enhanced -v to print progress after "Analyzing schema".
   reordered index printouts to group by the type of constraint each supports if any.

   Revision 2.66  2013/01/24 00:03:49  art
   Added Centaurus feature support.

   Revision 2.65  2012/11/28 12:45:02  art
   Fixed uninitialized variables. Fixed possible memory overrun in print_procedures.ec.
   Fixed handling of BIGSERIAL initialization in releases prior to 11.50, again.

   Revision 2.64  2012/08/26 18:31:03  art
   Typos.

   Revision 2.63  2012/08/22 20:54:00  art
   Fixed a couple of types in a structure in myschema.eh that did not match the ones in myschema.h
   Could be causing SEGV's in a 32bit compile.

   Revision 2.62  2012/08/08 21:35:58  art
   Fixed constraints printouts to support owners.  Full support for -O and --set-owner implemented.

   Revision 2.61  2012/03/12 00:16:51  art
   Added support for new option --index-online to add the ONLINE keyword to create index
   statements.  Added support for foreign keys created with the INDEX DISABLED option.

   Revision 2.60  2012/01/31 19:00:57  art
   Better fix for big nrows problem.

   Revision 2.59  2012/01/31 17:54:21  art
   Fixed problem with obtaining row counts for tables with more than 2^31 rows.

   Revision 2.58  2011/07/21 15:50:44  art
   Fixed a typo

   Revision 2.57  2011/07/18 21:14:02  art
   Added isproc column to sysprocedures structure.

   Revision 2.56  2011/07/01 16:32:43  art
   Added new feature --drop-proc to optionally insert a drop procedure statement before each create.
   Thanks to Bruce Simms for suggesting and funding this one.

   Revision 2.55  2011/04/29 13:35:51  art
   Fixed problems handling SERIAL8 type column initialization.
   Added support for STATLEVEL and STATCHANGE in CREATE TABLE statements.
   Added support for SECURITY POLICY in CREATE TABLE statements.  Output of policy
   definitions still not supported.
   Added option --force_autostats to force the output of STATLEVEL AUTO which is
   the default and so not printed normally.

   Revision 2.54  2011/04/07 16:33:44  art
   Fixed a bug that prevented printing the column names of the hidden columns for
   VERCOLS, REPLCHECK, and ERKEY in indexes that include those columns in their
   keys.  Thanks to Jeffrey Michell for pointing this one out.

   Revision 2.53  2011/04/06 03:32:55  art
   Added new feature --no-blade-funcs/--no-blade-procs to eliminate stored
   routines associated with IBM Datablades from the output.

   Revision 2.52  2010/12/13 17:31:46  art
   *** empty log message ***

   Revision 2.51  2010/12/13 17:28:31  art
   OK, fixed this for all server types.  Default pagesize will now be calculated
   correctly no matter what!

   Revision 2.50  2010/11/17 20:49:40  art
   Added support for WITH REPLCHECK, WITH ERKEY, and WITH AUDIT clauses in CREATE TABLE.

   Revision 2.49  2010/11/15 15:53:54  art
   Forest of Trees index and Index Extent Sizing support completed.

   Revision 2.48  2010/11/08 20:40:31  art
   Added new --create_conditionally feature to optionally add " IF NOT EXISTS " clauses to c
   create object statements.  Feature supported by 11.70.

   Revision 2.47  2010/05/04 23:23:32  art
   Fixed sequence output for IDS 10.00 and 11.10 (oops).
   Implemented new option --sequence, --sequence=matches-spec to print only sequence
   definitions.

   Revision 2.46  2010/03/24 16:10:13  art
   Modified sequence code to acquire the current/last sequence value from
   sysactptnhdr in sysmaster if the database has logging (no remote join
   to sysmaster from a non-logged database).

   Revision 2.45  2010/03/04 14:41:01  art
   A new option to ignore instrumentation routines added by AGS Server Studio
   needed to debug stored procedures in SS.

   Revision 2.44  2010/01/06 13:55:18  art
   Added new option --set-owner to override all object owners with a specified owner name.

   Revision 2.43  2009/11/23 18:47:05  art
   Added support for EXTERNAL tables (11.50xc6+).
   Improved 64bit compiler support.  Removed warnings.

   Revision 2.42  2009/06/03 18:25:25  art
   Removed stdint.h.  Not needed and not available everywhere.

   Revision 2.41  2009/03/20 14:09:22  art
   Fixed extent calculations to take account for differing page sizes, even
   between fragments of the same table.

   Revision 2.40  2009/03/15 23:04:59  art
   Fixed type of BIG fields to work better with older SDK versions.

   Revision 2.39  2009/02/15 21:42:41  art
   Fixed SERIAL, SERIAL8, and BIGSERIAL handling.  BIGINT and BIGSERIAL handling added.

   Revision 2.38  2009/01/14 17:34:34  root
   Added support for VERCOLS.

...

   Revision 1.1  1999/09/08 14:57:26  kagel
   Initial revision


*/

#if defined( __hpux )
#define atoll( t ) strtoll( (t), (char **)NULL, 10 )
#endif

#if !defined( myschemaHEADER )
#define myschemaHEADER 1
static const char myschemaRCSHeader[] = "@(#)$Header: /home/art/GoogleDrive/Latest\040Art's\040Stuff/myschema.d/fork/RCS/myschema.eh,v 2.92 2024/05/01 15:28:49 art Exp $";

#if !defined( FALSE )
#define FALSE (0)
#define TRUE (!FALSE)
#endif

EXEC SQL DEFINE IDBYTES 129;
EXEC SQL DEFINE UNAMELEN (33);
				   
typedef enum { ES_Minimum, ES_Average, ES_Maximum } ExtSchema;
typedef enum TableSortOrder { IdOrder, NameOrder, DependOrder, MyexportOrder } TableSortOrder_t;
typedef enum SerialHandling { Serial_Unset,
			      Serial_NoCurrent,
			      Serial_Current,
			      Serial_Default,
			      Serial_Value } SerialHandling_t;

#if defined( MAINCODE )
	ExtSchema HowExtents = ES_Maximum;
	TableSortOrder_t doTabOrder = IdOrder;
	TableSortOrder_t doOrder = IdOrder;
	SerialHandling_t init_Serial = Serial_Unset;
	int init_Serial_Value = 0;
	int do_filtering = 0;
	int do_drop_proc = 0;
	int do_index_online = 0;
	int NoValidateFK = 0;
	int NoValidateChk = 0;
	int lvl_num = 0;
	int NoPrivSyns = FALSE;
	int do_For_ER = FALSE;
	int include_sysprocs = FALSE;
	int suppress_invisible = FALSE;
#else
	extern ExtSchema HowExtents;
	extern TableSortOrder_t doTabOrder;
	extern TableSortOrder_t doOrder;
	extern SerialHandling_t init_Serial;
	extern int init_Serial_Value;
	extern int do_filtering;
	extern int do_drop_proc;
	extern int do_index_online;
	extern int NoValidateFK;
	extern int NoValidateChk;
	extern int lvl_num;
	extern int NoPrivSyns;
	extern int do_For_ER;
	extern int include_sysprocs;
	extern int suppress_invisible;
#endif

EXEC SQL BEGIN DECLARE SECTION;
/* DEFINITIONS SHARED BY MAINLINE AND DEPENDENT CODE. */
 
typedef struct SYSROLEAUTH_S {
        string rolename[UNAMELEN];
        string grantee[UNAMELEN];
        string is_grantable[2];
} sysroleauth_t;

typedef struct systables_s { 
    string tabname[IDBYTES], owner[UNAMELEN], dirpath[65], audpath[65];
    int tabid;
    int partnum, fextsize, nextsize;
    char locklevel[2];
    double npused;
    char tabtype[2];
    double nrows;
    int ncols;
    int rowsize;
    int flags;
    int has_violations;
    int vercols_flag;
    int replcheck_flag;
    int erkey_flag;
    int pagesize;
    int secpolicyid;
    short statchange;
    string statlevel[2];
    string am_owner[UNAMELEN], am_name[IDBYTES];
    string xtd_owner[UNAMELEN], xtd_name[IDBYTES];
    string par_owner[UNAMELEN], par_name[IDBYTES];
    short am_ind, xtd_ind, par_ind, viol_ind, statind, statlevelind;
} systables_t;

typedef struct syscolumns_s {
    int tabid;
    string colname[IDBYTES];
    short colno;
    short coltype;
    short collength;
    int colmin;
    int colmax;
    int extended_id;
    int colattr;
} syscolumns_t;

typedef struct sysindexes_s {
    string idxname[IDBYTES];
    string owner[UNAMELEN];
    int tabid;
    char idxtype[2], clustered[2];
    short part[16];
    int op_class_id[16];
    int amid, funcid;
    int nhashcols, nbuckets;
    int fextsize, nextsize;
    int indexattr;
} sysindexes_t;

typedef struct sysconstraints_s {
    int constrid;
    string constrname[IDBYTES];
    int tabid;
    string constrtype[2],
	idxname[IDBYTES];
    string owner[UNAMELEN];
} sysconstraints_t;

typedef struct SYSCOLDEPEND_S {
    int constrid;
    int tabid;
    short colno;
} syscoldepend_t;

typedef struct SYSCHECKS_S {
    int constrid;
    string type[2];
    short seqno;
    fixchar checktext[33];
} syschecks_t;
 
typedef struct sysdefaults_s {
    int tabid;
    short colno;
    string type[2];
    string deflt[257];
} sysdefaults_t;

typedef struct SYSFRAGMENTS_S {
    string fragtype[2];
    int tabid;
    string indexname[IDBYTES];
    string strategy[2];
    int evalpos;
    double nrows;
    double npused;
    int colno;
    int flags;
    loc_t exprtext;
    string dbspace[IDBYTES];
    string partition[IDBYTES];
} sysfragments_t;

typedef struct iSYSFRAGMENTS_S {
    string fragtype[2];
    int tabid;
    string indexname[IDBYTES];
    string strategy[2];
    int evalpos;
    loc_t exprtext;
    string dbspace[IDBYTES];
    string partition[IDBYTES];
} isysfragments_t;

typedef struct s_sysdbspaces {
    int dbsnum;
    char name[IDBYTES];
    char owner[UNAMELEN];
    int pagesize, fchunk, nchunks;
    int is_mirrored, is_blobspace, is_sbspace, is_temp;
    int flags;
} sysdbspaces_t;

typedef struct {
    string servername[IDBYTES], dbname[IDBYTES], owner[UNAMELEN], tabname[IDBYTES], ltabname[IDBYTES];
    int btabid;
} syssyntable_t;

typedef struct {
    char viewtext[65];
} sysviews_t;

typedef struct {
    char viewtext[257];
} sysviews12_t;

typedef struct s_pagecounts {
    string dbsname[IDBYTES];
    string tabname[IDBYTES];
    int serialv;
exec sql ifdef BIGINTNULL;
    bigint serialv8;
    bigint serialvBig;
exec sql else;
exec sql ifdef INT8NULL;
    int8 serialv8;
exec sql else;
    int serialv8;
exec sql endif;
    int serialvBig;
exec sql endif;
    int nfrags;
    int pgsused;
    int pgsdata;
    int pgstotal;
    int maxdata;
    int maxused;
} t_pagecounts;

typedef struct s_idxcounts {
    int keylen, detlen;
    double nrows;
    string dbsname[IDBYTES],
	tabname[IDBYTES];
} t_idxcounts;

typedef struct SYSPROCEDURES_S {
    string procname[IDBYTES];
    string owner[UNAMELEN];
    int procid;  /* SERIAL */
    string mode[2];
    int retsize;
    int symsize;
    int datasize;
    int codesize;
    int numargs;
    char isproc[2];
} sysprocedures_t;
 
typedef struct SYSPROCBODY_S {
    int procid;
    string datakey[2];
    int seqno;
    fixchar data[256];
    fixchar fill;
} sysprocbody_t;
 
typedef struct SYSPROCAUTH_S {
    string grantor[UNAMELEN];
    string grantee[UNAMELEN];
    int procid;
    string procauth[2];
} sysprocauth_t;

typedef struct SYSTRIGBODY_S {
    int trigid;
    string datakey[2];
    int seqno;
    char data[257];
} systrigbody_t;

typedef struct S_BLOBBUFF {
    char *buff;
    int bbsize, global_length;
} t_blobbuff;

typedef struct SYSTABAMDATA_S {
        string am_param[8193];  /* LVARCHAR */
        string am_space[129];
} systabamdata_t;

EXEC SQL IFDEF MAINCODE;
/* DEFINITIONS AND DECLARATIONS FOR MAINLINE ONLY. */

systables_t systables, prev;
syscolumns_t syscolumns;
sysindexes_t sysindexes;
sysconstraints_t sysconstraints;
syscoldepend_t syscoldepend;
syschecks_t syschecks;
sysdefaults_t sysdefaults;
sysfragments_t fragments;
sysfragments_t ifragments;
short isconstraint;
sysdbspaces_t *sysdbspaces;
syssyntable_t syssyntable;
sysviews_t sysviews;
sysviews12_t sysviews12;
systabamdata_t systabamdata;

char databasename[301], host[301], statement[8192], statement2[4096],
    statement3[1024], statement4[1024];
char database[IDBYTES], tablename[IDBYTES], dbspace[IDBYTES], alien[10];
string dbowner[UNAMELEN];
char *bigtxt = (char *)NULL;
int bigtxtsz = 0;
t_pagecounts *pagecounts;
t_idxcounts *idxcounts;
int ntables, alltables, gotdbs;
int TBSFACT = 0x100000;
int is_OL_version = 0;
int major_vers = 0;
int minor_vers = 0;
int conn5 = FALSE;
int stripowner = FALSE;
int ids_server = TRUE;
int verbose = FALSE;
int silent = FALSE;
int doDistrib = FALSE;
int NoKeyIndices = FALSE;
int LongKeyIndices = FALSE;
int KeepGenNames = FALSE;
int ncols = 0;
int ncols2 = 0;
int Dynamic = 1;
int Universal = 0;
int is_external_table = 0;
int Replace_Procs = FALSE;
int Replace_Views = FALSE;
int Replace_Triggers = FALSE;
int replace_owners = FALSE;
int no_ags_procs = FALSE;
int db_has_log = FALSE;
int sequence_only = FALSE;
int No_Sequence_Init = FALSE;
int create_conditionally = FALSE;
int no_blade_funcs = FALSE;
int no_storage_opt = FALSE;
int UseTableOwner = FALSE, UseDatabaseOwner = FALSE, UseSetOwner = FALSE;
int NoProcClean = FALSE;
char sequence_str[129];
char owner_override[33], original_owner[33];
char prev_override[33], prev_owner[33];
int has_external_tables;
t_blobbuff iblobbuff, tblobbuff, *blobbuff;
char *columns[32767],  *columns2[32767];
char dtparts[7][10] = { "YEAR",
			"MONTH",
			"DAY",
			"HOUR",
			"MINUTE",
			"SECOND",
			"FRACTION" };

EXEC SQL ELSE;
/* DEFINITIONS ONLY FOR DEPENDENT MODULES. */

extern systables_t systables, prev;
extern syscolumns_t syscolumns;
extern sysindexes_t sysindexes;
extern sysconstraints_t sysconstraints;
extern syscoldepend_t syscoldepend;
extern syschecks_t syschecks;
extern sysdefaults_t sysdefaults;
extern sysfragments_t fragments;
extern sysfragments_t ifragments;
extern short isconstraint;
extern sysdbspaces_t *sysdbspaces;
extern syssyntable_t syssyntable;
extern sysviews_t sysviews;
extern sysviews12_t sysviews12;
extern char databasename[101], host[101], statement[2048], statement2[2048],
    statement3[1024];
extern string dbowner[UNAMELEN];
extern char database[IDBYTES], tablename[IDBYTES], dbspace[IDBYTES], alien[10];
extern char sequence_str[129];
extern char *bigtxt;
extern int bigtxtsz;
extern t_pagecounts *pagecounts;
extern t_idxcounts *idxcounts;
extern int ntables, alltables, gotdbs,  TBSFACT;
extern int is_OL_version, is_external_table, has_external_tables;
extern int major_vers, minor_vers;
extern int conn5, stripowner, ids_server, verbose, silent, doDistrib;
extern int ncols, ncols2, NoKeyIndices, LongKeyIndices, KeepGenNames;
extern int Dynamic;
extern int Universal;
extern t_blobbuff iblobbuff, tblobbuff, *blobbuff;
extern int replace_owners;
extern int no_ags_procs;
extern int db_has_log;
extern int sequence_only;
extern int No_Sequence_Init;
extern int create_conditionally, no_blade_funcs, no_storage_opt;
extern int UseTableOwner, UseDatabaseOwner, UseSetOwner;
extern int NoProcClean;
extern char owner_override[33], original_owner[33];
extern char dtparts[7][10];
extern char *columns[32767],  *columns2[32767];
extern int Replace_Procs;
extern int Replace_Views;
extern int Replace_Triggers;

EXEC SQL ENDIF;

EXEC SQL END DECLARE SECTION;
#endif

