/* print_aggregates.ec - Print out user defined aggregates
*/
/*
   RCS Header:
   -----------
   $Header: /home/art/GoogleDrive/Latest\040Art's\040Stuff/myschema.d/fork/RCS/print_aggregates.ec,v 1.12 2020/11/25 12:24:06 art Exp $

   RCS Log:
   --------
   $Log: print_aggregates.ec,v $
   Revision 1.12  2020/11/25 12:24:06  art
   Extended --no-blade-procs/--no-blade-funcs to also exclude the resources added by the 
   blade management routines when any blade is registered.

   Revision 1.11  2010/11/08 20:40:31  art
   Added new --create_conditionally feature to optionally add " IF NOT EXISTS " clauses to c
   create object statements.  Feature supported by 11.70.

   Revision 1.10  2010/01/06 14:02:42  art
   Fixed include.

   Revision 1.9  2010/01/06 13:55:18  art
   Added new option --set-owner to override all object owners with a specified owner name.

   Revision 1.8  2009/10/18 17:23:16  art
   Fixed collection type handling.  Thanks to Bruce Simms for pointing out this one.
   Fixed summary comment to include table owner if not suppressed ala dbschema.
   Removed unneeded function prototypes.

   Revision 1.7  2009/02/11 18:30:34  art
   Fixed problem with LVARCHARS on AIX-64bit.

   Revision 1.6  2009/01/01 17:58:25  art
   Removed RCSWhat ident string which was BB specific.

   Revision 1.5  2009/01/01 17:38:19  art
   Fixed aggregate printout to exclude the built-in genxml* aggregates included in every
   database since v11.10

   Revision 1.4  2004/09/14 17:24:31  kagel
   Fixing ANSI database access.

   Revision 1.3  2003/09/30 21:32:03  kagel
   Added undocumented -o option to support mydbdiff.  Changed some default
   sort orders so the output will be more consistent when tables are altered.
   Increased the Features version to 6.25 to recognize the changes.

   Revision 1.2  2002/11/20 21:47:54  kagel
   Added DELIMIDENT support, RAW table support.

   Revision 1.1  2001/10/31 18:28:19  kagel
   Initial revision

   Revision 1.3  2001/09/20 21:23:29  kagel
   Filling in the body.

   Revision 1.2  2001/09/20 20:49:43  kagel
   FIxed

   Revision 1.1  2001/09/14 18:38:45  kagel
   Initial revision


*/

static const char RCSHeader[] = "@(#)$Header: /home/art/GoogleDrive/Latest\040Art's\040Stuff/myschema.d/fork/RCS/print_aggregates.ec,v 1.12 2020/11/25 12:24:06 art Exp $";
static const char RCSCompile[] = "@(#)$cc: " __FILE__ " " __DATE__ " " __TIME__ " $";

#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>

EXEC SQL INCLUDE "myschema.eh";

EXEC SQL INCLUDE prototypes.h;

EXEC SQL BEGIN DECLARE SECTION;
typedef struct SYSAGGREGATES_S {
	string name[129];
	string owner[32];
	long aggid;  /* SERIAL */
	string init_func[129];
	string iter_func[129];
	string combine_func[129];
	string final_func[129];
	char handlesnulls[2];
} sysaggregates_t;
typedef struct AGGINDICATORS_S {
	short init_func;
	short iter_func;
	short combine_func;
	short final_func;
} aggindicators_t;
EXEC SQL END DECLARE SECTION;


void print_aggregates( FILE *outfile )
{
    EXEC SQL BEGIN DECLARE SECTION;
    sysaggregates_t agg;
    aggindicators_t ind;
    string argument_typename[IDBYTES], result_typename[IDBYTES];
    short routine_i;
    EXEC SQL END DECLARE SECTION;
    char lenstr[24];
    long do_comma;

    EXEC SQL DECLARE c_aggregates CURSOR FOR
	SELECT	name,
		owner,
		aggid,
		init_func,
		iter_func,
		combine_func,
		final_func,
		handlesnulls
	FROM "informix".sysaggregates
	WHERE name NOT IN (
	      'genxml',
	      'genxmlelem',
	      'genxmlschema',
	      'genxmlclob',
	      'genxmlelemclob',
	      'genxmlschemaclob',
	      'addtoset',
	      'addtoset2',
	      'push',
	      'push2' )
	ORDER BY name;

    if (sqlca.sqlcode < 0) {
	fprintf( stderr,
		 "Error DECLARING aggregate query: %d, ISAM: %d.\n",
		 sqlca.sqlcode,
		 sqlca.sqlerrd[1] );
	exit(24);
    }

    EXEC SQL OPEN c_aggregates;

    if (sqlca.sqlcode < 0) {
	fprintf( stderr,
		 "Error OPENING aggregate query: %d, ISAM: %d.\n",
		 sqlca.sqlcode,
		 sqlca.sqlerrd[1] );
	exit(24);
    }

    for (;;) {
	EXEC SQL FETCH c_aggregates
	    INTO
		:agg.name,
		:agg.owner,
		:agg.aggid,
		:agg.init_func indicator ind.init_func,
		:agg.iter_func indicator ind.iter_func,
		:agg.combine_func indicator ind.combine_func,
		:agg.final_func indicator ind.final_func,
		:agg.handlesnulls
	    ;
        if (replace_owners) {
	    strcpy( agg.owner, owner_override );
        }
	if (sqlca.sqlcode == SQLNOTFOUND)
	    break;
	else if (sqlca.sqlcode < 0) {
	    fprintf( stderr,
		     "Error FETCHING aggregate query: %d, ISAM: %d.\n",
		     sqlca.sqlcode,
		     sqlca.sqlerrd[1] );
	    exit(24);
	}

	if (!stripowner)
	    fprintf( outfile,
		     "CREATE AGGREGATE%s '%s'.%s WITH (\n",
		     create_conditionally ? " IF NOT EXISTS" : "",
		     agg.owner,
		     delimident( agg.name ) );
	else
	    fprintf( outfile,
		     "CREATE AGGREGATE%s %s WITH (\n",
		     create_conditionally ? " IF NOT EXISTS" : "",
		     delimident( agg.name ) );
	do_comma = 0;
	if (ind.init_func >= (short)0) {
	    fprintf( outfile,
		     "\tINIT = %s",
		     agg.init_func );
	    do_comma = 1;
	}
	if (ind.iter_func >= (short)0) {
	    fprintf( outfile,
		     "%sITER = %s",
		     (do_comma ? ",\n\t" : "\t"),
		     agg.iter_func );
	    do_comma = 1;
	}
	if (ind.combine_func >= (short)0) {
	    fprintf( outfile,
		     "%sCOMBINE = %s",
		     (do_comma ? ",\n\t" : "\t"),
		     agg.combine_func );
	    do_comma = 1;
	}
	if (ind.final_func >= (short)0) {
	    fprintf( outfile,
		     "%sFINAL = %s",
		     (do_comma ? ",\n\t" : "\t"),
		     agg.final_func );
	    do_comma = 1;
	}
	if (agg.handlesnulls[0] == 't') {
	    fprintf( outfile,
		     "%sHANDLESNULLS",
		     (do_comma ? ",\n\t" : "\t") );
	}
	fprintf( outfile,
		 "\n);\n\n" );
    }
    EXEC SQL FREE c_aggregates;
}
