/* print_alters.ec - Print out type alters for inherited columns that have been
   altered since creation.
*/
/*
   RCS Header:
   -----------
   $Header: /home/art/GoogleDrive/Latest\040Art's\040Stuff/myschema.d/fork/RCS/print_alters.ec,v 1.12 2018/06/22 16:30:05 art Exp $

   RCS Log:
   --------
   $Log: print_alters.ec,v $
   Revision 1.12  2018/06/22 16:30:05  art
   New options: --use-database-owner, --use-table-owner, --constraint-file
   Fixed some problems with the partial schema file output.
   Expanded "owner"s to all objects.

   Revision 1.11  2012/11/12 14:55:38  art
   Modified code to work OK when run against versions earlier than 9.xx.

   Revision 1.10  2010/01/06 13:55:18  art
   Added new option --set-owner to override all object owners with a specified owner name.

   Revision 1.9  2009/03/11 15:04:29  art
   Modified to fix a bug causing core dumps if routines with defaulted
   parameters exist.  Thanks to Doug Lawry for pointing this one out.

   Revision 1.8  2009/02/11 18:30:34  art
   Fixed problem with LVARCHARS on AIX-64bit.

   Revision 1.7  2009/01/01 17:58:25  art
   Removed RCSWhat ident string which was BB specific.

   Revision 1.6  2004/09/14 17:24:08  kagel
   Fixing ANSI database access.

   Revision 1.5  2003/09/30 21:31:26  kagel
   Added undocumented -o option to support mydbdiff.  Changed some default
   sort orders so the output will be more consistent when tables are altered.
   Increased the Features version to 6.25 to recognize the changes.

   Revision 1.4  2002/11/20 21:47:54  kagel
   Added DELIMIDENT support, RAW table support.

   Revision 1.3  2001/11/08 22:08:13  kagel
   Made temp table NON_LOGGED>

   Revision 1.2  2001/11/01 15:37:18  kagel
   New module to output alter table statements  needed when a table created
   OF TYPE is altered later.  Dbschema misses this trick!

   Revision 1.1  2001/10/31 21:58:03  kagel
   Initial revision

   Revision 1.3  2001/09/20 21:23:29  kagel
   Filling in the body.

   Revision 1.2  2001/09/20 20:49:43  kagel
   FIxed

   Revision 1.1  2001/09/14 18:38:45  kagel
   Initial revision


*/

static const char RCSHeader[] = "@(#)$Header: /home/art/GoogleDrive/Latest\040Art's\040Stuff/myschema.d/fork/RCS/print_alters.ec,v 1.12 2018/06/22 16:30:05 art Exp $";
static const char RCSCompile[] = "@(#)$cc: " __FILE__ " " __DATE__ " " __TIME__ " $";

#include <stdlib.h>
#include <unistd.h>
#include <strings.h>
#include <stdio.h>

EXEC SQL INCLUDE "sqltypes.h";
EXEC SQL INCLUDE "myschema.eh";

EXEC SQL INCLUDE prototypes.h;

EXEC SQL BEGIN DECLARE SECTION;
typedef struct SYSATTRTYPES_S {
        long extended_id;
        short seqno;
        short levelno;
        short parent_no;
        string fieldname[129];
        short fieldno;
        short type;
        short length;
        long xtd_type_id;
} sysattrtypes_t;
EXEC SQL END DECLARE SECTION;


void print_alters( FILE *outfile, int pc )
{
    EXEC SQL BEGIN DECLARE SECTION;
    syscolumns_t syscolumns;
    sysattrtypes_t sysattrtypes;
    string argument_typename[IDBYTES], result_typename[IDBYTES];
    short routine_i;
    sysdefaults_t sysdefaults;
    EXEC SQL END DECLARE SECTION;
    char lenstr[24];
    char tmp1[150], tmp2[150];

    if (major_vers >= 9) {
	EXEC SQL
	    SELECT 
            sc.tabid,
            sc.colname,
            sc.colno,
            sc.coltype,
            sc.collength,
            sc.colmin,
            sc.colmax,
            sc.extended_id,
            sa.type row_type,
            sd.type def_type
	    FROM "informix".systables st,
	    "informix".sysattrtypes sa,
	    "informix".syscolumns sc, 
	    outer "informix".sysdefaults sd
	    WHERE st.tabid = :prev.tabid
	    AND sc.tabid = st.tabid
	    AND sa.extended_id = st.type_xid
	    AND sc.colno = sa.fieldno
	    AND sc.tabid = sd.tabid
	    AND sc.colno = sd.colno
	    INTO TEMP alters_myschema WITH NO LOG;
    } else {
	EXEC SQL
	    SELECT 
	    sc.tabid,
	    sc.colname,
	    sc.colno,
	    sc.coltype,
	    sc.collength,
	    sc.colmin,
	    sc.colmax,
	    0 as extended_id,
	    0 as row_type,
	    sd.type def_type
	    FROM "informix".systables st,
	    "informix".syscolumns sc, 
	    outer "informix".sysdefaults sd
	    WHERE st.tabid = :prev.tabid
	    AND sc.tabid = st.tabid
	    AND sc.tabid = sd.tabid
	    AND sc.colno = sd.colno
	    INTO TEMP alters_myschema WITH NO LOG;
    }
    EXEC SQL DECLARE c_alters CURSOR FOR
	SELECT 
		tabid,
		colname,
		colno,
		coltype,
		collength,
		colmin,
		colmax,
		extended_id
	FROM alters_myschema
	WHERE (row_type != coltype OR def_type IS NOT NULL)
	ORDER BY 3;
    if (sqlca.sqlcode < 0) {
	fprintf( stderr,
		 "Error DECLARING alter query: %d, ISAM: %d.\n",
		 sqlca.sqlcode,
		 sqlca.sqlerrd[1] );
	exit(24);
    }

    EXEC SQL OPEN c_alters;

    if (sqlca.sqlcode < 0) {
	fprintf( stderr,
		 "Error OPENING alter query: %d, ISAM: %d.\n",
		 sqlca.sqlcode,
		 sqlca.sqlerrd[1] );
	exit(24);
    }

    for (;;) {
	EXEC SQL FETCH c_alters
	    INTO
	    :syscolumns;
	if (sqlca.sqlcode == SQLNOTFOUND)
	    break;
	else if (sqlca.sqlcode < 0) {
	    fprintf( stderr,
		     "Error FETCHING alter query: %d, ISAM: %d.\n",
		     sqlca.sqlcode,
		     sqlca.sqlerrd[1] );
	    exit(24);
	}

	fprintf( outfile,
		 "ALTER TABLE '%s'.%s MODIFY (\n\t%s ",
		 prev.owner,
		 delimident2( prev.tabname, tmp1 ),
		 delimident2( syscolumns.colname, tmp2 ) );
	print_coltype( 0,
		       outfile,
		       systables.tabid,
		       systables.tabname,
		       systables.owner,
		       syscolumns,
		       pc );
	/* Process defaults. */
	if (major_vers >= 9) {
	    EXEC SQL SELECT type, default
	             INTO :sysdefaults.type, :sysdefaults.deflt
	             FROM "informix".sysdefaults
	             WHERE tabid = :prev.tabid
	               AND class = 'T' 
	               AND colno = :syscolumns.colno;
	} else {
	    EXEC SQL SELECT type, default
	             INTO :sysdefaults.type, :sysdefaults.deflt
	             FROM "informix".sysdefaults
	             WHERE tabid = :prev.tabid
	               AND colno = :syscolumns.colno;
	}
	if (sqlca.sqlcode == 0) {
	    sysdefaults.tabid = prev.tabid;
	    sysdefaults.colno = syscolumns.colno;

	    print_defaults( outfile, syscolumns, sysdefaults );
	}
	if (syscolumns.coltype & SQLNONULL)
	    fprintf( outfile, " NOT NULL" );
	fprintf( outfile,
		 "\n);\n\n" );

    }
    EXEC SQL CLOSE c_alters;
    EXEC SQL FREE c_alters;
    EXEC SQL DROP TABLE alters_myschema;
}
