/* print_casts.ec - Print out type casts
*/
/*
   RCS Header:
   -----------
   $Header: /home/art/GoogleDrive/Latest\040Art's\040Stuff/myschema.d/fork/RCS/print_casts.ec,v 1.18 2020/11/30 22:19:46 art Exp $

   RCS Log:
   --------
   $Log: print_casts.ec,v $
   Revision 1.18  2020/11/30 22:19:46  art
   Removed trace.

   Revision 1.17  2020/11/25 12:24:06  art
   Extended --no-blade-procs/--no-blade-funcs to also exclude the resources added by the 
   blade management routines when any blade is registered.

   Revision 1.16  2013/02/10 22:08:48  art
   Protected BIGINT and INT8 so the file will compile with SDK 3.10 and earlier.

   Revision 1.15  2012/01/25 20:03:47  art
   Shifted the implicit/explicit keyword to upper case for consistency.
   Fixed a bug that was ignoring casts from or to built-in (non-UDT) types.

   Revision 1.14  2010/11/08 20:40:31  art
   Added new --create_conditionally feature to optionally add " IF NOT EXISTS " clauses to c
   create object statements.  Feature supported by 11.70.

   Revision 1.13  2010/01/06 14:02:42  art
   Fixed include.

   Revision 1.12  2010/01/06 13:55:18  art
   Added new option --set-owner to override all object owners with a specified owner name.

   Revision 1.11  2009/10/18 17:23:16  art
   Fixed collection type handling.  Thanks to Bruce Simms for pointing out this one.
   Fixed summary comment to include table owner if not suppressed ala dbschema.
   Removed unneeded function prototypes.

   Revision 1.10  2009/02/11 18:30:34  art
   Fixed problem with LVARCHARS on AIX-64bit.

   Revision 1.9  2009/01/01 17:58:25  art
   Removed RCSWhat ident string which was BB specific.

   Revision 1.8  2004/09/14 17:23:17  kagel
   Fixing ANSI database access.

   Revision 1.7  2003/09/30 21:23:02  kagel
   Added undocumented -o option to support mydbdiff.  Changed some default
   sort orders so the output will be more consistent when tables are altered.
   Increased the Features version to 6.25 to recognize the changes.

   Revision 1.6  2002/12/03 14:42:44  kagel
   Corrected calls.

   Revision 1.5  2002/12/03 14:22:20  kagel
   Fixed declaration.

   Revision 1.4  2002/11/20 21:47:54  kagel
   Added DELIMIDENT support, RAW table support.

   Revision 1.3  2001/09/20 21:23:29  kagel
   Filling in the body.

   Revision 1.2  2001/09/20 20:49:43  kagel
   FIxed

   Revision 1.1  2001/09/14 18:38:45  kagel
   Initial revision


*/

static const char RCSHeader[] = "@(#)$Header: /home/art/GoogleDrive/Latest\040Art's\040Stuff/myschema.d/fork/RCS/print_casts.ec,v 1.18 2020/11/30 22:19:46 art Exp $";
static const char RCSCompile[] = "@(#)$cc: " __FILE__ " " __DATE__ " " __TIME__ " $";

#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>

EXEC SQL INCLUDE sqltypes.h;
EXEC SQL INCLUDE "myschema.eh";
EXEC SQL INCLUDE prototypes.h;

EXEC SQL BEGIN DECLARE SECTION;
typedef struct SYSCASTS_S {
    string owner[UNAMELEN];
    short argument_type;
    long argument_xid;
    short result_type;
    long result_xid;
    string routine_name[IDBYTES];
    string routine_owner[UNAMELEN];
    string class[2];
} syscasts_t;
typedef struct SYSXTDTYPES_S {
    string owner[UNAMELEN];
    string name[IDBYTES];
    short type;
    long maxlen, length;
    short align;
} sysxtdtypes_t;
short arg_type_ind, res_type_ind;
EXEC SQL END DECLARE SECTION;


void print_casts( FILE *outfile )
{
    EXEC SQL BEGIN DECLARE SECTION;
    syscasts_t cast;
    sysxtdtypes_t types;
    string argument_typename[IDBYTES], result_typename[IDBYTES];
    short routine_i;
    string statement[4096];
    EXEC SQL END DECLARE SECTION;
    char lenstr[24], tmp[150], tmp1[150], tmp2[150];

    snprintf( statement,
	      sizeof statement,
	      "%s%s%s",
	      "SELECT c.owner, c.routine_name, c.routine_owner,	c.class, x1.name, "
	      "       x2.name, c.argument_type, c.result_type "
	      "FROM \"informix\".syscasts c "
	      "LEFT OUTER JOIN \"informix\".sysxtdtypes x1 "
	      "   ON c.argument_xid = x1.extended_id "
	      "  AND c.argument_type = x1.type "
	      "LEFT OUTER JOIN \"informix\".sysxtdtypes x2 "
	      "   ON c.result_xid = x2.extended_id "
	      "  AND c.result_type = x2.type "
	      "JOIN \"informix\".sysprocedures p "
	      "   ON c.routine_name = p.procname "
	      "  AND p.mode IN ('D','O','P','R') ",
	      (no_blade_funcs == 1) ? "WHERE x1.name != 'sysbldsqltext' AND x2.name != 'sysbldsqltext' " : " ",
	      "ORDER BY x1.name, x2.name; " );

    EXEC SQL PREPARE s_casts FROM :statement;

    if (sqlca.sqlcode < 0) {
	fprintf( stderr,
		 "Error PREPARING cast query: %d, ISAM: %d.\n",
		 sqlca.sqlcode,
		 sqlca.sqlerrd[1] );
	fprintf( stderr,
		 "%s\n\n",
		 statement );
	exit(24);
    }

    EXEC SQL DECLARE c_casts CURSOR FOR s_casts;

    if (sqlca.sqlcode < 0) {
	fprintf( stderr,
		 "Error DECLARING cast query: %d, ISAM: %d.\n",
		 sqlca.sqlcode,
		 sqlca.sqlerrd[1] );
	exit(24);
    }

    EXEC SQL OPEN c_casts;

    if (sqlca.sqlcode < 0) {
	fprintf( stderr,
		 "Error OPENING cast query: %d, ISAM: %d.\n",
		 sqlca.sqlcode,
		 sqlca.sqlerrd[1] );
	exit(24);
    }

    for (;;) {
	EXEC SQL FETCH c_casts
	    INTO
	    :cast.owner,
	    :cast.routine_name indicator :routine_i,
	    :cast.routine_owner,
	    :cast.class,
	    :argument_typename indicator :arg_type_ind,
	    :result_typename indicator :res_type_ind,
	    :cast.argument_type,
	    :cast.result_type;
        if (replace_owners) {
	  strcpy( cast.owner, owner_override );
	  strcpy( cast.routine_owner, owner_override );
	}
	if (sqlca.sqlcode == SQLNOTFOUND)
	    break;
	else if (sqlca.sqlcode < 0) {
	    fprintf( stderr,
		     "Error FETCHING cast query: %d, ISAM: %d.\n",
		     sqlca.sqlcode,
		     sqlca.sqlerrd[1] );
	    exit(24);
	}
	if (arg_type_ind < 0) {
	    switch( cast.argument_type ) {
	    case SQLCHAR:
		snprintf( argument_typename, sizeof argument_typename, "%s", "CHAR" );
		break;
	    case SQLSMINT:
		snprintf( argument_typename, sizeof argument_typename, "%s", "SMALLINT" );
		break;
	    case SQLINT:
		snprintf( argument_typename, sizeof argument_typename, "%s", "INT" );
		break;
	    case SQLFLOAT:
		snprintf( argument_typename, sizeof argument_typename, "%s", "FLOAT" );
		break;
	    case SQLSMFLOAT:
		snprintf( argument_typename, sizeof argument_typename, "%s", "SMALLFLOAT" );
		break;
	    case SQLDECIMAL:
		snprintf( argument_typename, sizeof argument_typename, "%s", "DECIMAL" );
		break;
	    case SQLSERIAL:
		snprintf( argument_typename, sizeof argument_typename, "%s", "SERIAL" );
		break;
	    case SQLDATE:
		snprintf( argument_typename, sizeof argument_typename, "%s", "DATE" );
		break;
	    case SQLMONEY:
		snprintf( argument_typename, sizeof argument_typename, "%s", "MONEY" );
		break;
	    case SQLDTIME:
		snprintf( argument_typename, sizeof argument_typename, "%s", "DATETIME" );
		break;
	    case SQLBYTES:
		snprintf( argument_typename, sizeof argument_typename, "%s", "BYTES" );
		break;
	    case SQLTEXT:
		snprintf( argument_typename, sizeof argument_typename, "%s", "TEXT" );
		break;
	    case SQLVCHAR:
		snprintf( argument_typename, sizeof argument_typename, "%s", "VCHAR" );
		break;
	    case SQLINTERVAL:
		snprintf( argument_typename, sizeof argument_typename, "%s", "INTERVAL" );
		break;
	    case SQLNCHAR:
		snprintf( argument_typename, sizeof argument_typename, "%s", "NCHAR" );
		break;
	    case SQLNVCHAR:
		snprintf( argument_typename, sizeof argument_typename, "%s", "NVCHAR" );
		break;
#if defined( SQLINT8 )
	    case SQLINT8:
		snprintf( argument_typename, sizeof argument_typename, "%s", "INT8" );
		break;
	    case SQLSERIAL8:
		snprintf( argument_typename, sizeof argument_typename, "%s", "SERIAL8" );
		break;
#endif
	    case SQLSET:
		snprintf( argument_typename, sizeof argument_typename, "%s", "SET" );
		break;
	    case SQLMULTISET:
		snprintf( argument_typename, sizeof argument_typename, "%s", "MULTISET" );
		break;
	    case SQLLIST:
		snprintf( argument_typename, sizeof argument_typename, "%s", "LIST" );
		break;
	    case SQLROW:
		snprintf( argument_typename, sizeof argument_typename, "%s", "ROW" );
		break;
	    case SQLCOLLECTION:
		snprintf( argument_typename, sizeof argument_typename, "%s", "COLLECTION" );
		break;
	    case SQLBOOL:
		snprintf( argument_typename, sizeof argument_typename, "%s", "BOOLEAN" );
		break;
#if defined( SQLINFXBIGINT )
	    case SQLINFXBIGINT:
		snprintf( argument_typename, sizeof argument_typename, "%s", "BIGINT" );
		break;
	    case SQLBIGSERIAL:
		snprintf( argument_typename, sizeof argument_typename, "%s", "BIGSERIAL" );
		break;
#endif
	    }
	}

	if (res_type_ind < 0) {
	    switch( cast.result_type ) {
	    case SQLCHAR:
		snprintf( result_typename, sizeof result_typename, "%s", "CHAR" );
		break;
	    case SQLSMINT:
		snprintf( result_typename, sizeof result_typename, "%s", "SMALLINT" );
		break;
	    case SQLINT:
		snprintf( result_typename, sizeof result_typename, "%s", "INT" );
		break;
	    case SQLFLOAT:
		snprintf( result_typename, sizeof result_typename, "%s", "FLOAT" );
		break;
	    case SQLSMFLOAT:
		snprintf( result_typename, sizeof result_typename, "%s", "SMALLFLOAT" );
		break;
	    case SQLDECIMAL:
		snprintf( result_typename, sizeof result_typename, "%s", "DECIMAL" );
		break;
	    case SQLSERIAL:
		snprintf( result_typename, sizeof result_typename, "%s", "SERIAL" );
		break;
	    case SQLDATE:
		snprintf( result_typename, sizeof result_typename, "%s", "DATE" );
		break;
	    case SQLMONEY:
		snprintf( result_typename, sizeof result_typename, "%s", "MONEY" );
		break;
	    case SQLDTIME:
		snprintf( result_typename, sizeof result_typename, "%s", "DATETIME" );
		break;
	    case SQLBYTES:
		snprintf( result_typename, sizeof result_typename, "%s", "BYTES" );
		break;
	    case SQLTEXT:
		snprintf( result_typename, sizeof result_typename, "%s", "TEXT" );
		break;
	    case SQLVCHAR:
		snprintf( result_typename, sizeof result_typename, "%s", "VCHAR" );
		break;
	    case SQLINTERVAL:
		snprintf( result_typename, sizeof result_typename, "%s", "INTERVAL" );
		break;
	    case SQLNCHAR:
		snprintf( result_typename, sizeof result_typename, "%s", "NCHAR" );
		break;
	    case SQLNVCHAR:
		snprintf( result_typename, sizeof result_typename, "%s", "NVCHAR" );
		break;
#if defined( SQLINT8 )
	    case SQLINT8:
		snprintf( result_typename, sizeof result_typename, "%s", "INT8" );
		break;
	    case SQLSERIAL8:
		snprintf( result_typename, sizeof result_typename, "%s", "SERIAL8" );
		break;
#endif
	    case SQLSET:
		snprintf( result_typename, sizeof result_typename, "%s", "SET" );
		break;
	    case SQLMULTISET:
		snprintf( result_typename, sizeof result_typename, "%s", "MULTISET" );
		break;
	    case SQLLIST:
		snprintf( result_typename, sizeof result_typename, "%s", "LIST" );
		break;
	    case SQLROW:
		snprintf( result_typename, sizeof result_typename, "%s", "ROW" );
		break;
	    case SQLCOLLECTION:
		snprintf( result_typename, sizeof result_typename, "%s", "COLLECTION" );
		break;
	    case SQLBOOL:
		snprintf( result_typename, sizeof result_typename, "%s", "BOOLEAN" );
		break;
#if defined( SQLINFXBIGINT )
	    case SQLINFXBIGINT:
		snprintf( result_typename, sizeof result_typename, "%s", "BIGINT" );
		break;
	    case SQLBIGSERIAL:
		snprintf( result_typename, sizeof result_typename, "%s", "BIGSERIAL" );
		break;
#endif
	    }
	}

	if (routine_i >= 0)
	    if (!stripowner)
		fprintf( outfile,
			 "CREATE %s CAST%s (%s AS %s WITH \"%s\".%s);\n\n",
			 (cast.class[0] == 'E') ? "EXPLICIT" : "IMPLICIT",
			 create_conditionally ? " IF NOT EXISTS" : "",
			 delimident2( argument_typename, tmp ),
			 delimident2( result_typename, tmp1 ),
			 cast.routine_owner,
			 delimident2( cast.routine_name, tmp2 ) );
	    else
		fprintf( outfile,
			 "CREATE %s CAST%s (%s AS %s WITH %s);\n\n",
			 (cast.class[0] == 'E') ? "EXPLICIT" : "IMPLICIT",
			 create_conditionally ? " IF NOT EXISTS" : "",
			 delimident2( argument_typename, tmp ),
			 delimident2( result_typename, tmp1 ),
			 delimident2( cast.routine_name, tmp2 ) );
	else
	    fprintf( outfile,
		     "CREATE %s CAST%s (%s AS %s);\n\n",
		     (cast.class[0] == 'E') ? "EXPLICIT" : "IMPLICIT",
		     create_conditionally ? " IF NOT EXISTS" : "",
		     delimident2( argument_typename, tmp ),
		     delimident2( result_typename, tmp1 ) );
    }
}
