/* print_checks.ec - print out any check constraints associated with table.
*/
/*
   RCS Header:
   -----------
   $Header: /home/art/GoogleDrive/Latest\040Art's\040Stuff/myschema.d/fork/RCS/print_checks.ec,v 2.16 2018/06/22 16:30:05 art Exp $

   RCS Log:
   --------
   $Log: print_checks.ec,v $
   Revision 2.16  2018/06/22 16:30:05  art
   New options: --use-database-owner, --use-table-owner, --constraint-file
   Fixed some problems with the partial schema file output.
   Expanded "owner"s to all objects.

   Revision 2.15  2016/12/14 00:14:53  art
   Made check constraints obey --keep-generated-names

   Revision 2.14  2016/12/13 23:51:19  art
   Added support for unencrypted storage spaces when encrypted storage is enabled (12.10.xC8+).
   Added new option --no-check to output CHECK constraints with the NOVALIDATE option for
   migrating databases (effective only on a populated table created without CHECK constraints.

   Revision 2.13  2009/02/11 18:30:34  art
   Fixed problem with LVARCHARS on AIX-64bit.

   Revision 2.12  2009/01/01 17:58:25  art
   Removed RCSWhat ident string which was BB specific.

   Revision 2.11  2006/06/12 17:11:33  kagel
   Improved formatting.

   Revision 2.10  2006/06/12 17:08:24  kagel
   Fixed problem with losing the constraint name of the last check
   constraint for a table.

   Revision 2.9  2004/01/20 19:27:27  kagel
   Fixed return type of internal function.  HP was complaining.

   Revision 2.8  2003/09/30 21:47:54  kagel
   Added undocumented -o option to support mydbdiff.  Changed some default
   sort orders so the output will be more consistent when tables are altered.
   Increased the Features version to 6.25 to recognize the changes.

   Revision 2.7  2002/12/03 14:42:19  kagel
   Corrected calls.

   Revision 2.6  2002/11/20 21:47:54  kagel
   Added DELIMIDENT support, RAW table support.

   Revision 2.5  2001/11/01 13:31:15  kagel
   Fix to get the closing parenthesis correct by setting the return code here
   for myschema.ec to use.

   Revision 2.4  2001/10/31 18:13:49  kagel
   Fixed CHECK clause call based on Sergey E. Volkov's patch.

   Revision 2.3  2000/04/12 17:45:31  kagel
   Fixed bug affecting CHECK CONSTRAINTS longer than 32 chars which caused the
   constraint name to be embedded in the middle of the constraint text rather
   that at the end if the constraint had an explicit name.

   Revision 2.2  1999/11/03 20:30:03  kagel
   Had to make identifier and username length changes to support IDS.2000
   again.  Some strange interaction between my makefile and using emacs to
   check-out of RCS lost those changes.

   Revision 2.1  1999/09/29 16:43:06  kagel
   Establishing current  source version id for initial working version of
   newly modularized source.  This is Feature Version 6.1 Source revision
   2.1.

   Revision 1.3  1999/09/28 20:22:40  kagel
   Updates

   Revision 1.2  1999/09/22 20:45:38  kagel
   Moved statement preparation and cursor declaration in here.

   Revision 1.1  1999/09/01 19:34:08  kagel
   Initial revision


*/

static const char RCSHeader[] = "@(#)$Header: /home/art/GoogleDrive/Latest\040Art's\040Stuff/myschema.d/fork/RCS/print_checks.ec,v 2.16 2018/06/22 16:30:05 art Exp $";
static const char RCSCompile[] = "@(#)$cc: " __FILE__ " " __DATE__ " " __TIME__ " $";

#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include <sys/types.h>
#include <string.h>
#include <limits.h>
#include <ctype.h>

EXEC SQL INCLUDE sqlhdr;
EXEC SQL INCLUDE sqlca;
EXEC SQL INCLUDE sqltypes;
EXEC SQL INCLUDE datetime;

EXEC SQL INCLUDE myschema.eh;

EXEC SQL INCLUDE prototypes.h;


short print_checks( FILE *outfile, int alone )
{
    int pos, a, b;
    char constrname[IDBYTES];
    EXEC SQL BEGIN DECLARE SECTION;
    string tabname[IDBYTES], name[IDBYTES], lclstmt[2048], owner[UNAMELEN];
    EXEC SQL END DECLARE SECTION;
    static char checkstring[32768];
    static char novalidstr[12];
    static int prepped = 0;
    short ret = 0;

    /* Check Constraints. */
    if (!prepped) {
	snprintf( lclstmt,
		  sizeof lclstmt,
		  "SELECT st.tabname, st.owner, co.constrname, ch.* "
		  "FROM \"informix\".systables st, \"informix\".sysconstraints co, \"informix\".syschecks ch "
		  "WHERE co.constrtype = 'C' "
		  "%s"
		  "    AND co.tabid = st.tabid "
		  "    AND co.constrid = ch.constrid "
		  "    AND ch.type = 'T' ",
		  alone ? "" : "    AND st.tabid = ? ");
	if (doOrder == NameOrder)
	     strcat( lclstmt, "ORDER BY constrname, ch.seqno" );
	else
	     strcat( lclstmt, "ORDER BY ch.constrid, ch.seqno" );
	EXEC SQL PREPARE s_chks FROM :lclstmt;
	if (sqlca.sqlcode) {
	    fprintf( stderr,
		     "Cannot PREPARE select statement s_chks, "
		     "error: %d, ISAM=%d.\n",
		     sqlca.sqlcode,
		     sqlca.sqlerrd[1] );
	    fprintf( stderr, "Error at character %d.\n",
		     sqlca.sqlerrd[4] );
	    rexit(18);
	}
	EXEC SQL DECLARE chks CURSOR FOR s_chks;
	if (sqlca.sqlcode < 0) {
	    fprintf( stderr,
		     "Error DECLARing CURSOR chks: Err=%d, ISAM=%d\n",
		     sqlca.sqlcode,
		     sqlca.sqlerrd[1] );
	    rexit( 21 );
	}
	if (NoValidateChk == 1) {
	    strcpy( novalidstr, " NOVALIDATE" );
	} else {
	    novalidstr[0] = (char)0;
	}
	prepped = 1;
    }

    if (alone)
	EXEC SQL OPEN chks;
    else
	EXEC SQL OPEN chks USING :prev.tabid;
    if (sqlca.sqlcode < 0) {
	fprintf( stderr,
		 "Error OPENing CURSOR chks: Err=%d, ISAM=%d\n",
		 sqlca.sqlcode,
		 sqlca.sqlerrd[1] );
	rexit( 22 );
    }
    memset( checkstring, 0, 32768 );
    pos = 0;
    constrname[0] = (char)0;
    for (;;) {
      EXEC SQL FETCH chks INTO :tabname, :owner, :name, :syschecks;
	if (sqlca.sqlcode == SQLNOTFOUND) {
	    if (strlen( checkstring )) {
		if (KeepGenNames || sscanf( constrname, "c%d_%d", &a, &b ) != 2)
		    sprintf( &checkstring[pos],
			     "\n\t\tCONSTRAINT %s",
			     delimident( constrname ) );
		if (NoValidateChk) {
		    sprintf( &checkstring[pos], "%s", novalidstr );
		    pos = strlen( checkstring );
		}
		if (alone)
		    strcat( checkstring, ";\n\n" );
		check_form( outfile, checkstring );
		ret = 1; /* Looks very bad but ... works */
	    }
	    break;
	} else if (sqlca.sqlcode < 0) {
	    fprintf( stderr,
		     "Error FETCHing CURSOR chks: Err=%d, ISAM=%d\n",
		     sqlca.sqlcode,
		     sqlca.sqlerrd[1] );
	    rexit( 21 );
	}
	syschecks.checktext[32] = (char)0;
	if (syschecks.seqno == 0) {
	    if (strlen( checkstring )) {
		if (KeepGenNames || sscanf( constrname, "c%d_%d", &a, &b ) != 2) {
		    sprintf( &checkstring[pos],
			     "\n\t\tCONSTRAINT %s",
			     delimident( constrname ) );
		    pos = strlen( checkstring );
		}
		if (NoValidateChk) {
		    sprintf( &checkstring[pos], "%s", novalidstr );
		    pos = strlen( checkstring );
		}
		if (alone) {
		    strcat( checkstring, ";\n\n" );
		    pos += 2;
		}
		check_form( outfile, checkstring );
		memset( checkstring, 0, pos );
		pos = 0;
	    }
	    if (alone) {
	      if (stripowner)
		sprintf( &checkstring[pos],
			 "ALTER TABLE %s ADD CONSTRAINT CHECK %s",
			 tabname,
			 syschecks.checktext );
	      else
		sprintf( &checkstring[pos],
			 "ALTER TABLE \"%s\".%s ADD CONSTRAINT CHECK %s",
			 UseDatabaseOwner ? dbowner : owner,
			 tabname,
			 syschecks.checktext );

	    } else if (!(prev.xtd_ind < 0) && !ret) {
	      sprintf( &checkstring[pos],
		       "\n(\n\tCHECK %s%s",
		       syschecks.checktext,
		       novalidstr );
	      ret = 1; /* Looks very bad but ... works */
	    } else {
		sprintf( &checkstring[pos],
			 ",\n\tCHECK %s%s",
			 syschecks.checktext,
			 novalidstr );
	    }
	    strcpy( constrname, name );
	} else {
	    sprintf( &checkstring[pos], "%s", syschecks.checktext );
	}
	pos = strlen( checkstring );
    }
    EXEC SQL CLOSE chks;
    fflush( outfile );

    return ret;
}


void check_form( FILE *outfile, char *checkstring )
{
    int q1state = 0, q2state = 0, sawspace = 0;
    long linecnt = 0;
    static char word[32768];
    char *ch, *out = word;

    memset( word, 0, 32768 );
    ch = checkstring;
    while (*ch) {
	switch (*ch) {
	case '\n':
	    if (sawspace) {
		*out++ = ' ';
		linecnt++;
		sawspace = 0;
	    }
	    linecnt = 0;
	    fprintf( outfile, "%s\n", word );
	    ch++;
	    out = word;
	    break;
	case '\t':
	    if (sawspace) {
		*out++ = ' ';
		linecnt++;
		sawspace = 0;
	    }
	    linecnt += 8;
	    *out++ = *ch++;
	    break;
	case '\'':
	    if (sawspace) {
		*out++ = ' ';
		linecnt++;
		sawspace = 0;
	    }
	    if (q1state)
		q1state = 0;
	    else if (!q2state)
		q1state = 1;
	    linecnt++;
	    *out++ = *ch++;
	    break;
	case '"':
	    if (sawspace) {
		*out++ = ' ';
		linecnt++;
		sawspace = 0;
	    }
	    if (q2state)
		q2state = 0;
	    else if (!q1state)
		q2state = 1;
	    linecnt++;
	    *out++ = *ch++;
	    break;
	case ' ':
	    if (sawspace)
		ch++;
	    else {
		if (!q1state && !q2state) {
		    sawspace = 1;
		} else {
		    linecnt++;
		    *out++ = *ch++;
		}
	    }
	    break;
	default:
	    if (sawspace) {
		*out++ = ' ';
		linecnt++;
		sawspace = 0;
	    }
	    *out++ = *ch++;
	    break;
	}
	*out = (char)0;
    }
    if (strlen( word ) > 0)
	fprintf( outfile, "%s", word );
}
