/* print_distributions.ec - extract distributions from sysdistribs and recreate
   them.
*/
/*
   RCS Header:
   -----------
   $Header: /home/art/GoogleDrive/Latest\040Art's\040Stuff/myschema.d/fork/RCS/print_distributions.ec,v 1.15 2009/10/18 17:23:16 art Exp $

   RCS Log:
   --------
   $Log: print_distributions.ec,v $
   Revision 1.15  2009/10/18 17:23:16  art
   Fixed collection type handling.  Thanks to Bruce Simms for pointing out this one.
   Fixed summary comment to include table owner if not suppressed ala dbschema.
   Removed unneeded function prototypes.

   Revision 1.14  2009/03/11 15:09:07  art
   typo.

   Revision 1.13  2009/03/11 15:06:34  art
   Changes added to support the new SAMPLE SIZE option to medium distributions.

   Revision 1.12  2009/02/11 18:30:34  art
   Fixed problem with LVARCHARS on AIX-64bit.

   Revision 1.11  2009/01/01 17:58:25  art
   Removed RCSWhat ident string which was BB specific.

   Revision 1.10  2004/09/14 17:19:48  kagel
   Fixing ANSI database access.

   Revision 1.9  2001/10/24 18:01:01  kagel
   Protected the #define of _H_LOCALEDEF from multiple definition.  This is
   needed for some versions of AIX but apparently later versions of the
   Informix SDK already define it.

   Revision 1.8  2001/05/15 14:46:25  kagel
   It seems that if a table has 'WITH CRCOLS' clause there will be a
   distribution for colno -1 for one of the crcols.  Coded to ignore that.

   Revision 1.7  2000/10/26 00:15:24  kagel
   Added _H_LOCALEDEF for AIX

   Revision 1.6  1999/12/08 19:40:18  kagel
   Renamed a file scope variable which had the same name as a global.  This
   was harmless but unneccessary.

   Revision 1.5  1999/11/01 21:22:24  kagel
   Fixed data structures to accomodate changes to identifyier lengths and
   user name lengths in IDS.2000 also fixed the array columns which used to
   limit the number of supported columns to 1024.  Myschema now supports the
   documented maximum number of columns per table (32767).  The array now uses
   malloc() to allocate string space and is an array of char* instead of a
   static array of strings.

   Revision 1.4  1999/10/28 21:00:55  kagel
   Made function no-op for 5.xx servers.  There are NO distributions there.

   Revision 1.3  1999/10/27 17:56:28  kagel
   Added new arguments to print_distributions so a stand alone application can
   be written using that function.

   Revision 1.2  1999/10/26 22:17:52  kagel
   Fixed minor problems.

   Revision 1.1  1999/10/26 21:43:11  kagel
   Initial revision

   Revision 1.1  1999/10/26 21:09:49  kagel
   Initial revision


*/

static const char RCSHeader[] = "@(#)$Header: /home/art/GoogleDrive/Latest\040Art's\040Stuff/myschema.d/fork/RCS/print_distributions.ec,v 1.15 2009/10/18 17:23:16 art Exp $";
static const char RCSCompile[] = "@(#)$cc: " __FILE__ " " __DATE__ " " __TIME__ " $";

 
#if defined( _AIX ) /* Needed as AIX defines a locale control structure loc_t */
#if !defined( _H_LOCALEDEF )
#define _H_LOCALEDEF
#endif
#endif
 
#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include <sys/types.h>
#include <string.h>
#include <limits.h>
#include <ctype.h>
EXEC SQL INCLUDE sqlhdr;
EXEC SQL INCLUDE sqlca;
EXEC SQL INCLUDE sqltypes;
EXEC SQL INCLUDE datetime;
EXEC SQL INCLUDE locator;
EXEC SQL INCLUDE myschema.eh;

EXEC SQL INCLUDE prototypes.h;

static int prepped = 0;
EXEC SQL BEGIN DECLARE SECTION;
static string diststmt[2048];
typedef struct SYSDISTRIB_S {
    long tabid;
    short colno;
    string mode[2];
    float resolution;
    float confidence;
    float smplsize;
} sysdistrib_t;
static sysdistrib_t distrib;
EXEC SQL END DECLARE SECTION;

int print_distributions( FILE *outfile,
			 long tabid,
			 char *tabname,
			 char **columns )
{
    if (conn5)
	return 1;

    if (!prepped) {
        if (major_vers >= 11) {
	    strcpy( diststmt,
		    "SELECT UNIQUE tabid, colno, mode, resolution, confidence, smplsize "
		    "FROM \"informix\".sysdistrib "
		    "WHERE tabid = ? and colno > 0"
		    "ORDER BY 1, 5, 2" );
	} else {
	    strcpy( diststmt,
		    "SELECT UNIQUE tabid, colno, mode, resolution, confidence, 0.0 "
		    "FROM \"informix\".sysdistrib "
		    "WHERE tabid = ? and colno > 0"
		    "ORDER BY 1, 5, 2" );
	}
	EXEC SQL PREPARE s_distrib FROM :diststmt;
	if (sqlca.sqlcode < 0) {
	    fprintf( stderr,
		     "Error PREPARing Distributions statement.  "
		     "Error = %d, ISAM = %d.\n",
		     sqlca.sqlcode,
		     sqlca.sqlerrd[1] );
	    rexit(1);
	}
	EXEC SQL DECLARE c_distrib CURSOR FOR s_distrib;
	if (sqlca.sqlcode < 0) {
	    fprintf( stderr,
		     "Error DECLARing Distributions cursor.  "
		     "Error = %d, ISAM = %d.\n",
		     sqlca.sqlcode,
		     sqlca.sqlerrd[1] );
	    rexit(1);
	}
	prepped = 1;
    }
    EXEC SQL OPEN c_distrib USING :prev.tabid;
    if (sqlca.sqlcode < 0) {
	fprintf( stderr,
		 "Error OPENing Distributions cursor.  "
		 "Error = %d, ISAM = %d.\n",
		 sqlca.sqlcode,
		 sqlca.sqlerrd[1] );
	rexit(1);
    }

    for (;;) {
	EXEC SQL FETCH c_distrib INTO :distrib;
  
	if (sqlca.sqlcode < 0) {
	    fprintf( stderr,
		     "Error FETCHing distribution data.  "
		     "Error = %d, ISAM = %d.\n",
		     sqlca.sqlcode,
		     sqlca.sqlerrd[1] );
	    rexit(1);
	} else if (sqlca.sqlcode == SQLNOTFOUND) {
	    break;
	} else if (distrib.mode[0] == 'H') {
	    fprintf( outfile,
		     "UPDATE STATISTICS HIGH FOR TABLE %s( %s )\n"
		     "\tRESOLUTION %15.10f DISTRIBUTIONS ONLY;\n",
		     prev.tabname,
		     columns[distrib.colno-1],
		     distrib.resolution );
	} else if (distrib.mode[0] == 'M') {
	    if (distrib.smplsize > 0.0) {
	        fprintf( outfile,
			 "UPDATE STATISTICS MEDIUM FOR TABLE %s( %s )\n"
			 "\tSAMPLING SIZE %15.10f RESOLUTION %15.10f %15.10f DISTRIBUTIONS ONLY;\n",
			 prev.tabname,
			 columns[distrib.colno-1],
			 distrib.smplsize,
			 distrib.resolution,
			 distrib.confidence );
	    } else {
	        fprintf( outfile,
			 "UPDATE STATISTICS MEDIUM FOR TABLE %s( %s )\n"
			 "\tRESOLUTION %15.10f %15.10f DISTRIBUTIONS ONLY;\n",
			 prev.tabname,
			 columns[distrib.colno-1],
			 distrib.resolution,
			 distrib.confidence );
	    }
	} else {
	    fprintf( stderr,
		     "Unknown distributions mode: %s.\n",
		     distrib.mode );
	    rexit(1);
	}
    }

    return 0;
}
