/* print_external.ec - Print out definitions DDL for external tables
 */
/*
  RCS Header:
  -----------
  $Header: /home/art/GoogleDrive/Latest\040Art's\040Stuff/myschema.d/fork/RCS/print_external.ec,v 1.6 2014/08/20 00:15:27 art Exp $

  RCS Log:
  --------
  $Log: print_external.ec,v $
  Revision 1.6  2014/08/20 00:15:27  art
  Added RCS Headers.

*/
static const char RCSHeader[] = "@(#)$Header: /home/art/GoogleDrive/Latest\040Art's\040Stuff/myschema.d/fork/RCS/print_external.ec,v 1.6 2014/08/20 00:15:27 art Exp $";
static const char RCSCompile[] = "@(#)$cc: " __FILE__ " " __DATE__ " " __TIME__ " $";

#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include <sys/types.h>
#include <string.h>
#include <limits.h>
#include <ctype.h>
#include <errno.h>
#if !defined(__gnu_linux__)
#include <stropts.h>
#endif
#include "getopt.h"

EXEC SQL INCLUDE sqlhdr;
EXEC SQL INCLUDE sqlca;
EXEC SQL INCLUDE sqltypes;
EXEC SQL INCLUDE datetime;
EXEC SQL INCLUDE locator;

EXEC SQL INCLUDE myschema.eh;
EXEC SQL INCLUDE prototypes.h;

EXEC SQL BEGIN DECLARE SECTION;
typedef struct SYSEXTDFILES_S {
	int tabid;
	string dfentry[470];
} sysextdfiles_t;

typedef struct SYSEXTCOLS_S {
	int tabid;
	short colno;
	short exttype;
	short extstart;
	short extlength;
	string nullstr[257];
	short decprec;
	string extstype[130];
} sysextcols_t;

typedef struct SYSEXTCOLS_INDICATORS_S {
	short tabid;
	short colno;
	short exttype;
	short extstart;
	short extlength;
	short nullstr;
	short decprec;
	short extstype;
} sysextcols_indicators_t;

typedef struct SYSEXTERNAL_S {
	int tabid;
	string fmttype[2];
	string codeset[130];
	string recdelim[130];
	string flddelim[5];
	string datefmt[9];
	string moneyfmt[21];
	int maxerrors;
	string rejectfile[465];
	int flags;
	int ndfiles;
} sysexternal_t;
typedef struct SYSEXTERNAL_INDICAORS_S {
	short tabid;
	short fmttype;
	short codeset;
	short recdelim;
	short flddelim;
	short datefmt;
	short moneyfmt;
	short maxerrors;
	short rejectfile;
	short flags;
	short ndfiles;
} sysexternal_indicators_t;
EXEC SQL END DECLARE SECTION;

int probe_external(
    EXEC SQL BEGIN DECLARE SECTION;
    PARAMETER int tabid,
    PARAMETER int colno,
    EXEC SQL END DECLARE SECTION;
    FILE *outfile
)
{
    EXEC SQL BEGIN DECLARE SECTION;
    static string fmttype[2];
    sysextcols_t sysextcols;
    sysextcols_indicators_t sysextcols_indicators;
    EXEC SQL END DECLARE SECTION;
    static int prev_tabid = 0;

    if (prev_tabid == 0) {
        EXEC SQL PREPARE ext_col_s FROM
	  "SELECT * FROM sysextcols WHERE tabid = ? AND colno = ?";
	if (sqlca.sqlcode < 0) {
	    fprintf( stderr,
		     "Error %d ISAM %d preparing extended column type statement.\n",
		     sqlca.sqlcode,
		     sqlca.sqlerrd[1] );
	    exit(1);
	}
    }

    if (tabid != prev_tabid)
        EXEC SQL 
	  SELECT fmttype
	  INTO :fmttype
	  FROM sysexternal
	  WHERE tabid = :tabid;

    prev_tabid = tabid;

    if (sqlca.sqlcode == SQLNOTFOUND || fmttype[0] != 'F') {
        return 0;
    } else if (sqlca.sqlcode != 0) {
	fprintf( stderr,
		   "Error %d ISAM %d fetching extended table type.\n",
		   sqlca.sqlcode,
		   sqlca.sqlerrd[1] );
	exit(1);
    }

    EXEC SQL EXECUTE ext_col_s INTO :sysextcols INDICATOR :sysextcols_indicators USING :tabid, :colno;
    if (sqlca.sqlcode < 0) {
        fprintf( stderr,
	         "Error %d ISAM %d fetching extended column type.\n",
	         sqlca.sqlcode,
	         sqlca.sqlerrd[1] );
        exit(1);
    } else if (sqlca.sqlcode == 0) {
        fprintf( outfile,
		 " EXTERNAL CHAR(%d)",
		 sysextcols.extlength );
	if (sysextcols_indicators.nullstr == 0) {
	    fprintf( outfile,
		     " NULL '%s'",
		     sysextcols.nullstr );
	}
    }

    return 1;
}


int print_external(
    EXEC SQL BEGIN DECLARE SECTION;
    PARAMETER int tabid,
    EXEC SQL END DECLARE SECTION;
    FILE *outfile
 )
{
    EXEC SQL BEGIN DECLARE SECTION;
    sysexternal_t sysexternal;
    sysexternal_indicators_t sysexternal_indicators;
    sysextdfiles_t sysextdfiles;
    sysextcols_t sysextcols;
    EXEC SQL END DECLARE SECTION;
    int first = 1;
    static int initial = 1;

    if (initial) {
	EXEC SQL PREPARE get_ext_s FROM
	    "SELECT tabid, fmttype, codeset, recdelim, flddelim, datefmt, \
		    moneyfmt, maxerrors, rejectfile, flags, ndfiles \
	     FROM sysexternal \
	     WHERE tabid = ? ";
	if (sqlca.sqlcode != 0) {
	    fprintf( stderr,
		     "Error %d, ISAM %d PREPARING get_ext_s.\n",
		     sqlca.sqlcode,
		     sqlca.sqlerrd[1] );
	    exit(1);
	}
	EXEC SQL PREPARE files_ext_s FROM
	"SELECT tabid, dfentry FROM sysextdfiles WHERE tabid = ?";
	if (sqlca.sqlcode != 0) {
	    fprintf( stderr,
		     "Error %d, ISAM %d PREPARING files_ext_s.\n",
		     sqlca.sqlcode,
		     sqlca.sqlerrd[1] );
	    exit(1);
	}

	EXEC SQL DECLARE files_ext_c CURSOR FOR files_ext_s;
	if (sqlca.sqlcode != 0) {
	    fprintf( stderr,
		     "Error %d, ISAM %d DECLARING files_ext_c.\n",
		     sqlca.sqlcode,
		     sqlca.sqlerrd[1] );
	    exit(1);
	}
	initial = 0;
    }
    fprintf( outfile, "USING ( " );
    EXEC SQL 
        EXECUTE get_ext_s
        INTO :sysexternal INDICATOR :sysexternal_indicators
      USING :tabid;
	if (sqlca.sqlcode != 0) {
	    fprintf( stderr,
		     "Error %d, ISAM %d FETCHING get_ext_s.\n",
		     sqlca.sqlcode,
		     sqlca.sqlerrd[1] );
	    exit(1);
	}

    if (sysexternal.fmttype[0] == 'D') {
	fprintf( outfile,
		 "%s\n\tFORMAT \"DELIMITED\"",
		 first ? "" : "," );
	first = 0;
    } else if (sysexternal.fmttype[0] == 'F') {
	fprintf( outfile,
		 "%s\n\tFORMAT \"FIXED\"",
		 first ? "" : "," );
	first = 0;
    } else if (sysexternal.fmttype[0] == 'I') {
	fprintf( outfile,
		 "%s\n\tFORMAT \"INFORMIX\"",
		 first ? "" : "," );
	first = 0;
    }

    if (major_vers >= 12) {
	fprintf( outfile,
		 "%s\n\tESCAPE %s",
		 first ? "" : ",",
		 (sysexternal.flags & 2) ? "ON" : "OFF" );
	first = 0;
    } else if (sysexternal.flags & 2) {
	    fprintf( outfile,
		     "%s\n\tESCAPE",
		     first ? "" : "," );
	    first = 0;
    }

    if (sysexternal.ndfiles > 0) {
	int fil;

	EXEC SQL OPEN files_ext_c USING :tabid;
	if (sqlca.sqlcode != 0) {
	    fprintf( stderr,
		     "Error %d, ISAM %d OPENING files_ext_c.\n",
		     sqlca.sqlcode,
		     sqlca.sqlerrd[1] );
	    exit(1);
	}

	fprintf( outfile, "%s\n\tDATAFILES (", first ? "" : "," );
	first = 0;
	for (fil = 0; ; fil++) {
	    EXEC SQL FETCH files_ext_c INTO :sysextdfiles;
	    if (sqlca.sqlcode == SQLNOTFOUND)
		break;
	    else if (sqlca.sqlcode != 0) {
		fprintf( stderr,
			 "Error %d, ISAM %d FETCHING files_ext_c.\n",
			 sqlca.sqlcode,
			 sqlca.sqlerrd[1] );
		exit(1);
	    }
	    fprintf( outfile,
		     "%s\n\t\t\"%s\"", fil ? "," : "",
		     sysextdfiles.dfentry );
	}

	EXEC SQL CLOSE files_ext_c;

	fprintf( outfile, "\n\t)" );
    }

    if (sysexternal_indicators.rejectfile >= 0) {
        fprintf( outfile,
	         "%s\n\tREJECTFILE \"%s\"",
		 first ? "" : ",",
	         sysexternal.rejectfile );
        first = 0;
    }

    if (sysexternal_indicators.recdelim >= 0) {
        fprintf( outfile,
	         "%s\n\tRECORDEND \"%s\"",
		 first ? "" : ",",
	         sysexternal.recdelim );
        first = 0;
    }

    if (sysexternal_indicators.flddelim >= 0) {
        fprintf( outfile,
	         "%s\n\tDELIMITER \"%s\"",
		 first ? "" : ",",
	         sysexternal.flddelim );
        first = 0;
    }

    if (sysexternal.maxerrors >= 0) {
        fprintf( outfile,
	         "%s\n\tMAXERRORS %d",
		 first ? "" : ",",
	         sysexternal.maxerrors );
        first = 0;
    }

    fprintf( outfile, "\n);\n\n" );

    return 0;
}

