/* print_index_rename.ec - Print out commands to rename auto generated indexes.
*/
/*
   RCS Header:
   -----------
   $Header: /home/art/GoogleDrive/Latest\040Art's\040Stuff/myschema.d/fork/RCS/print_index_rename.ec,v 1.2 2019/02/06 14:16:54 art Exp $

   RCS Log:
   --------
   $Log: print_index_rename.ec,v $
   Revision 1.2  2019/02/06 14:16:54  art
   Fixed problem with RENAME INDEX statements that include the owner.

   Revision 1.1  2018/11/07 15:25:01  art
   Initial revision


*/

static const char RCSHeader[] = "@(#)$Header: /home/art/GoogleDrive/Latest\040Art's\040Stuff/myschema.d/fork/RCS/print_index_rename.ec,v 1.2 2019/02/06 14:16:54 art Exp $";
static const char RCSCompile[] = "@(#)$cc: " __FILE__ " " __DATE__ " " __TIME__ " $";

#if defined( _AIX ) /* Needed as AIX defines a locale control structure loc_t */
#if !defined( _H_LOCALEDEF )
#define _H_LOCALEDEF
#endif
#endif

#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include <sys/types.h>
#include <string.h>
#include <limits.h>
#include <ctype.h>
#include <errno.h>

EXEC SQL INCLUDE sqlhdr;
EXEC SQL INCLUDE sqlca;
EXEC SQL INCLUDE sqltypes;
EXEC SQL INCLUDE datetime;
EXEC SQL INCLUDE locator;

EXEC SQL INCLUDE myschema.eh;

void print_index_rename( FILE *outfile, char *i_tabname )
{
  EXEC SQL BEGIN DECLARE SECTION;
  string idxname[129], constrtype[2];
  string tabname[129], newname[129];
  string idxowner[33];
  string query[1024];
  EXEC SQL END DECLARE SECTION;

  int uknum=0, fknum=0, aknum=0;
  
  snprintf( query,
	    sizeof query,
	    "SELECT st.tabname, si.idxname, si.owner, constrtype "
	    "FROM systables st "
	    "JOIN sysindices si "
	    "  ON st.tabid = si.tabid "
	    "  AND si.idxname[1,1] = ' ' "
	    "  AND st.tabname matches '%s' "
	    "JOIN sysconstraints sc "
	    "  ON si.tabid = sc.tabid "
	    "  AND si.idxname = sc.idxname ",
	    i_tabname );

    EXEC SQL PREPARE s_renameidx FROM :query;
    if (sqlca.sqlcode != 0) {
      fprintf( stderr, 
	       "Error preparing renameidx. SQL: %d, ISAM: %d (%s).\n",
	       sqlca.sqlcode,
	       sqlca.sqlerrd[1],
	       sqlca.sqlerrm );
      fprintf( stderr,
	       "SQL: %s;\n",
	       query );
      exit( 1 );
    }

    EXEC SQL DECLARE c_renameidx CURSOR FOR s_renameidx;
    if (sqlca.sqlcode != 0) {
      fprintf( stderr, 
	       "Error declaring cursor renameidx. SQL: %d, ISAM: %d.\n",
	       sqlca.sqlcode,
	       sqlca.sqlerrd[1] );
      exit( 1 );
    }

    EXEC SQL OPEN c_renameidx;
    if (sqlca.sqlcode != 0) {
      fprintf( stderr, 
	       "Error opening cursor renameidx. SQL: %d, ISAM: %d.\n",
	       sqlca.sqlcode,
	       sqlca.sqlerrd[1] );
      exit( 1 );
    }

    while (sqlca.sqlcode == 0) {
      EXEC SQL FETCH c_renameidx INTO :tabname, :idxname, :idxowner, :constrtype;
      if (sqlca.sqlcode < 0) {
	fprintf( stderr, 
		 "Error fetching cursor renameidx. SQL: %d, ISAM: %d.\n",
		 sqlca.sqlcode,
		 sqlca.sqlerrd[1] );
	exit( 1 );
      } else if (sqlca.sqlcode == SQLNOTFOUND) {
	break;
      }

      if (LongKeyIndices) {
	switch (constrtype[0]) {
	case 'R':
	  snprintf( newname, sizeof newname, "%s_fk%.3d", tabname, ++fknum );
	  break;
	case 'U':
	  snprintf( newname, sizeof newname, "%s_uk%.3d", tabname, ++uknum );
	  break;
	case 'P':
	  snprintf( newname, sizeof newname, "%s_pk", tabname );
	  break;
	default:
	  snprintf( newname, sizeof newname, "%s_ak%.3d", tabname, ++aknum );
	  break;
	}
      } else {
	switch (constrtype[0]) {
	case 'R':
	  snprintf( newname, sizeof newname, "F%s", &(idxname[1]) );
	  break;
	case 'U':
	  snprintf( newname, sizeof newname, "U%s", &(idxname[1]) );
	  break;
	case 'P':
	  snprintf( newname, sizeof newname, "P%s", &(idxname[1]) );
	  break;
	default:
	  snprintf( newname, sizeof newname, "A%s", &(idxname[1]) );
	  break;
	}
      }

      if (stripowner) {
	fprintf( outfile,
		 "RENAME INDEX %s TO %s;\n",
		 idxname,
		 newname );
      } else {
	fprintf( outfile,
		 "RENAME INDEX %s.%s TO %s;\n",
		 idxowner,
		 idxname,
		 newname );
      }
    }
}
