/* print_seconstraints.ec - Print all constraints associated with a
 *table for SE servers.  
 */
/*
   RCS Header:
   -----------
   $Header: /home/art/GoogleDrive/Latest\040Art's\040Stuff/myschema.d/fork/RCS/print_seconstraints.ec,v 2.14 2009/02/11 18:30:34 art Exp $

   RCS Log:
   --------
   $Log: print_seconstraints.ec,v $
   Revision 2.14  2009/02/11 18:30:34  art
   Fixed problem with LVARCHARS on AIX-64bit.

   Revision 2.13  2009/01/01 17:58:25  art
   Removed RCSWhat ident string which was BB specific.

   Revision 2.12  2003/09/30 21:42:20  kagel
   Added undocumented -o option to support mydbdiff.  Changed some default
   sort orders so the output will be more consistent when tables are altered.
   Increased the Features version to 6.25 to recognize the changes.

   Revision 2.11  2001/01/16 22:16:09  kagel
   Included errno.h.

   Revision 2.10  2001/01/16 22:12:42  kagel
   Added protection to malloc() calls.

   Revision 2.9  2000/04/10 18:19:22  kagel
   Fixed columnname problem with UNIQUE constraints, column retrieval loop
   was missing from that block.

   Revision 2.8  2000/01/20 19:55:55  kagel
   Fixed references to cursor 'alien' to use the value of the hostvar alien
   rather than hard coded.  The var was used in the DECLARE but hard coded
   elsewhere, it worked because the var contained the same name that was
   hard coded.

   Revision 2.7  1999/11/11 14:31:06  kagel
   Added new -F option which outputs foreign key constraints for any tables
   referencing the output table for a single table schema.  Noop for database
   level schemas since this is already done.

   Revision 2.6  1999/11/04 14:53:56  kagel
   Failed to initialize columns2 when it was changed from array of array of char
   to array of pointer to char.   Also had to fix constraint code to protect
   against overrunning the already allocated pointers in columns[] and columns2.
   Now expands the list on the fly as needed.

   Revision 2.5  1999/11/03 20:28:49  kagel
   Had to make identifier and username length changes to support IDS.2000
   again.  Some strange interaction between my makefile and using emacs to
   check-out of RCS lost those changes.

   Revision 2.4  1999/10/12 17:09:59  kagel
   Removed some testing traces left around.

   Revision 2.3  1999/09/30 18:08:30  kagel
   Latest.

   Revision 2.2  1999/09/30 18:08:30  kagel
   Fixed a typo and added the tflag argument as for print_constraints.

   Revision 2.1  1999/09/29 15:27:09  kagel
   Establishing current  source version id for initial working version of
   newly modularized source.  This is Feature Version 6.1 Source revision
   2.1.

   Revision 1.1  1999/09/22 20:39:58  kagel
   Initial revision


*/

static const char RCSHeader[] = "@(#)$Header: /home/art/GoogleDrive/Latest\040Art's\040Stuff/myschema.d/fork/RCS/print_seconstraints.ec,v 2.14 2009/02/11 18:30:34 art Exp $";
static const char RCSCompile[] = "@(#)$cc: " __FILE__ " " __DATE__ " " __TIME__ " $";

#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include <sys/types.h>
#include <string.h>
#include <limits.h>
#include <ctype.h>
#include <errno.h>

EXEC SQL INCLUDE sqlhdr;
EXEC SQL INCLUDE sqlca;
EXEC SQL INCLUDE sqltypes;
EXEC SQL INCLUDE datetime;

EXEC SQL INCLUDE myschema.eh;

EXEC SQL INCLUDE prototypes.h;


void print_seconstraints( FILE *outfile, int tflag )
{
    int i, a, b;

    EXEC SQL BEGIN DECLARE SECTION;
    string reftable[129], delrule[2];
    long primconstr, ptabid;
    char localstr1[2048], localstr2[2048];
    EXEC SQL END DECLARE SECTION;

    EXEC SQL PREPARE s_cols FROM
	"SELECT colno, colname \
         FROM \"informix\".syscolumns \
         WHERE tabid = ? \
         ORDER BY colno ";
    if (sqlca.sqlcode) {
	fprintf( stderr,
		 "Cannot PREPARE select statement s_cols, error: %d.\n",
		 sqlca.sqlcode );
	fprintf( stderr, "Error at character %d.\n",
		 sqlca.sqlerrd[4] );
	rexit(18);
    }
    EXEC SQL DECLARE cols CURSOR FOR s_cols;
    if (sqlca.sqlcode < 0) {
	fprintf( stderr,
		 "Error DECLARing CURSOR cols: Err=%d, ISAM=%d\n",
		 sqlca.sqlcode,
		 sqlca.sqlerrd[1] );
	rexit( 21 );
    }

    strcpy( alien, "alien" );
    snprintf( localstr1,
	      sizeof localstr1,
	      "SELECT st.tabname, rt.tabname, sr.primary, sr.ptabid, "
	      "sr.delrule, sc.constrid, constrname, constrtype, "
	      "si.idxname, si.tabid, si.part1, si.part2, si.part3, "
	      "si.part4, si.part5, si.part6, si.part7, si.part8 "
	      "FROM \"informix\".systables st, \"informix\".sysconstraints sc, "
	      "\"informix\".sysindexes si, \"informix\".sysreferences sr, "
	      "\"informix\".systables rt "
	      "WHERE st.tabid = sc.tabid "
	      "AND rt.tabid = sr.ptabid "
	      "AND sc.constrid = sr.constrid "
	      "AND sc.tabid = si.tabid "
	      "AND sc.idxname = si.idxname "
	      "AND sc.constrtype = 'R' "
	      "AND st.tabname MATCHES ? " );
    if (doOrder == IdOrder)
	strcat( localstr1, "ORDER BY si.tabid, sc.constrid " );
    else
	strcat( localstr1, "ORDER BY si.tabid, sc.constrname " );

    EXEC SQL PREPARE psc_1 FROM :localstr1;
    if (sqlca.sqlcode < 0) {
	fprintf( stderr,
		 "Error PREPARING statement for alien: Err=%d, ISAM=%d\n",
		 sqlca.sqlcode,
		 sqlca.sqlerrd[1] );
	rexit( 21 );
    }

    EXEC SQL DECLARE :alien CURSOR FOR pcs_1;

    if (sqlca.sqlcode < 0) {
	fprintf( stderr,
		 "Error DECLARing CURSOR alien: Err=%d, ISAM=%d\n",
		 sqlca.sqlcode,
		 sqlca.sqlerrd[1] );
	rexit( 21 );
    }

    snprintf( localstr2,
	      sizeof localstr2,
	      "SELECT tabname, constrid, constrname, constrtype, "
	      "si.idxname, si.tabid, si.part1, si.part2, si.part3, si.part4, "
	      "si.part5, si.part6, si.part7, si.part8 "
	      "FROM \"informix\".systables st, \"informix\".sysconstraints sc, "
	      "\"informix\".sysindexes si "
	      "WHERE st.tabid = sc.tabid "
	      "AND sc.tabid = si.tabid "
	      "AND sc.idxname = si.idxname "
	      "AND sc.constrtype = 'U' "
	      "AND st.tabname MATCHES ? " );
    if (doOrder == IdOrder)
	strcat( localstr2, "ORDER BY si.tabid, constrid " );
    else
	strcat( localstr2, "ORDER BY si.tabid, constrname " );
    EXEC SQL PREPARE psc_2 FROM :localstr2;
    if (sqlca.sqlcode < 0) {
	fprintf( stderr,
		 "Error PREPARING statement for uniq: Err=%d, ISAM=%d\n",
		 sqlca.sqlcode,
		 sqlca.sqlerrd[1] );
	rexit( 21 );
    }

    EXEC SQL DECLARE uniq CURSOR FOR psc_2;
    if (sqlca.sqlcode < 0) {
	fprintf( stderr,
		 "Error DECLARing CURSOR uniq: Err=%d, ISAM=%d\n",
		 sqlca.sqlcode,
		 sqlca.sqlerrd[1] );
	rexit( 21 );
    }

    strcpy( statement2,
	    "SELECT tabname, constrid, constrname, constrtype, \n"
	    "     si.idxname, si.tabid, si.part1, si.part2, si.part3, \n"
	    "     si.part4, si.part5, si.part6, si.part7, si.part8 \n"
	    "FROM \"informix\".systables st, \n"
	    "     \"informix\".sysconstraints sc, \n"
	    "     \"informix\".sysindexes si \n"
	    "WHERE st.tabid = sc.tabid \n"
	    "  AND sc.tabid = si.tabid \n"
	    "  AND sc.idxname = si.idxname \n"
	    "  AND sc.constrtype = 'P' \n"
	    "  AND st.tabname MATCHES ? \n" );
    if (!tflag || strcmp( tablename, "*" ) == 0)
	strcat( statement2, "  AND st.tabid >= 100 " );
    if (doOrder == IdOrder)
	strcat( statement2, "ORDER BY si.tabid, constrid; " );
    else
	strcat( statement2, "ORDER BY si.tabid, constrname; " );

    EXEC SQL PREPARE s_prim FROM :statement2;
    if (sqlca.sqlcode) {
	fprintf( stderr,
		 "Cannot PREPARE select statement s_prim, "
		 "error: %d, ISAM=%d.\n",
		 sqlca.sqlcode,
		 sqlca.sqlerrd[1] );
	fprintf( stderr, "Error at character %d.\n",
		 sqlca.sqlerrd[4] );
	fprintf( stderr, "%s\n\n", statement2 );
	rexit(18);
    }
    EXEC SQL DECLARE prim CURSOR FOR s_prim;
	    
    if (sqlca.sqlcode < 0) {
	fprintf( stderr,
		 "Error DECLARing CURSOR prim: Err=%d, ISAM=%d\n",
		 sqlca.sqlcode,
		 sqlca.sqlerrd[1] );
	rexit( 21 );
    }

    /* Primary Key Constraints. */
    EXEC SQL OPEN prim USING :tablename;
    if (sqlca.sqlcode < 0) {
	fprintf( stderr,
		 "Error OPENing CURSOR prim: Err=%d, ISAM=%d\n",
		 sqlca.sqlcode,
		 sqlca.sqlerrd[1] );
	rexit( 22 );
    }
    for (;;) {
	EXEC SQL FETCH prim
	    INTO
	    :systables.tabname,
	    :sysconstraints.constrid,
	    :sysconstraints.constrname,
	    :sysconstraints.constrtype,
	    :sysindexes.idxname,
	    :sysindexes.tabid,
	    :sysindexes.part[0],
	    :sysindexes.part[1],
	    :sysindexes.part[2],
	    :sysindexes.part[3],
	    :sysindexes.part[4],
	    :sysindexes.part[5],
	    :sysindexes.part[6],
	    :sysindexes.part[7];
	if (sqlca.sqlcode == SQLNOTFOUND) {
	    break;
	} else if (sqlca.sqlcode < 0) {
	    fprintf( stderr,
		     "Error FETCHing CURSOR prim: Err=%d, ISAM=%d\n",
		     sqlca.sqlcode,
		     sqlca.sqlerrd[1] );
	    rexit( 21 );
	}
	EXEC SQL OPEN cols USING :sysindexes.tabid;
	if (sqlca.sqlcode < 0) {
	    fprintf( stderr,
		     "Error OPENing CURSOR cols: Err=%d, ISAM=%d\n",
		     sqlca.sqlcode,
		     sqlca.sqlerrd[1] );
	    rexit( 22 );
	}
	for (i=0;;i++) {
	    EXEC SQL FETCH cols INTO :syscolumns.colno, :syscolumns.colname;
	    if (sqlca.sqlcode == SQLNOTFOUND)
		break;
	    else if (sqlca.sqlcode < 0) {
		fprintf( stderr,
			 "Error FETCHing CURSOR cols: Err=%d, ISAM=%d\n",
			 sqlca.sqlcode,
			 sqlca.sqlerrd[1] );
	    }
	    if (ncols <= i) {
		columns[i] = (char *)malloc( IDBYTES );
		if (!columns[i]) {
		    fprintf( stderr,
			     "Cannot allocate memory for column constraint "
			     "data(1).  Errno=%d.\n",
			     errno );
		    rexit(99);
		}
		ncols++;
	    }
	    strcpy( columns[i], syscolumns.colname );
	}
	EXEC SQL CLOSE cols;

	fprintf( outfile,
		 "\nALTER TABLE %s ADD CONSTRAINT PRIMARY KEY (\n",
		 systables.tabname );
	for (i=0; i<8 && sysindexes.part[i]; i++) {
	    if (i)
		fprintf( outfile, ",\n" );
	    fprintf( outfile,
		     "\t%s",
		     columns[abs(sysindexes.part[i]) - 1] );
	}
	fprintf( outfile, "\n)" );
	if (sscanf( sysconstraints.constrname, "u%d_%d", &a, &b ) == 2)
	    fprintf( outfile, ";\n" );
	else
	    fprintf( outfile, " CONSTRAINT %s;\n", sysconstraints.constrname );
    }
    EXEC SQL CLOSE prim;

    /* Unique Constraints. */
    EXEC SQL OPEN uniq USING :tablename;
    if (sqlca.sqlcode < 0) {
	fprintf( stderr,
		 "Error OPENing CURSOR uniq: Err=%d, ISAM=%d\n",
		 sqlca.sqlcode,
		 sqlca.sqlerrd[1] );
	rexit( 22 );
    }
    for (;;) {
	EXEC SQL FETCH uniq
	    INTO
	    :systables.tabname,
	    :sysconstraints.constrid,
	    :sysconstraints.constrname,
	    :sysconstraints.constrtype,
	    :sysindexes.idxname,
	    :sysindexes.tabid,
	    :sysindexes.part[0],
	    :sysindexes.part[1],
	    :sysindexes.part[2],
	    :sysindexes.part[3],
	    :sysindexes.part[4],
	    :sysindexes.part[5],
	    :sysindexes.part[6],
	    :sysindexes.part[7];
	if (sqlca.sqlcode == SQLNOTFOUND) {
	    break;
	} else if (sqlca.sqlcode < 0) {
	    fprintf( stderr,
		     "Error FETCHing CURSOR uniq: Err=%d, ISAM=%d\n",
		     sqlca.sqlcode,
		     sqlca.sqlerrd[1] );
	    rexit( 21 );
	}
	EXEC SQL OPEN cols USING :sysindexes.tabid;
	if (sqlca.sqlcode < 0) {
	    fprintf( stderr,
		     "Error OPENing CURSOR cols: Err=%d, ISAM=%d\n",
		     sqlca.sqlcode,
		     sqlca.sqlerrd[1] );
	    rexit( 22 );
	}
	for (i=0;;i++) {
	    EXEC SQL FETCH cols INTO :syscolumns.colno, :syscolumns.colname;
	    if (sqlca.sqlcode == SQLNOTFOUND)
		break;
	    else if (sqlca.sqlcode < 0) {
		fprintf( stderr,
			 "Error FETCHing CURSOR cols: Err=%d, ISAM=%d\n",
			 sqlca.sqlcode,
			 sqlca.sqlerrd[1] );
	    }
	    if (ncols <= i) {
		columns[i] = (char *)malloc( IDBYTES );
		if (!columns[i]) {
		    fprintf( stderr,
			     "Cannot allocate memory for column constraint "
			     "data(2).  Errno=%d.\n",
			     errno );
		    rexit(99);
		}
		ncols++;
	    }
	    strcpy( columns[i], syscolumns.colname );
	}
	EXEC SQL CLOSE cols;
	fprintf( outfile,
		 "\nALTER TABLE %s ADD CONSTRAINT UNIQUE (\n",
		 systables.tabname );
	for (i=0; i<8 && sysindexes.part[i]; i++) {
	    if (i)
		fprintf( outfile, ",\n" );
	    fprintf( outfile,
		     "\t%s",
		     columns[abs(sysindexes.part[i]) - 1] );
	}
	fprintf( outfile, "\n)" );
	if (sscanf( sysconstraints.constrname, "c%d_%d", &a, &b ) == 2)
	    fprintf( outfile, ";\n" );
	else
	    fprintf( outfile, " CONSTRAINT %s;\n", sysconstraints.constrname );
    }
    EXEC SQL CLOSE uniq;

    /* Foreign Key Constraints. */
    EXEC SQL OPEN :alien USING :tablename;
    if (sqlca.sqlcode < 0) {
	fprintf( stderr,
		 "Error OPENing CURSOR alien: Err=%d, ISAM=%d\n",
		 sqlca.sqlcode,
		 sqlca.sqlerrd[1] );
	rexit( 22 );
    }
    for (;;) {
	EXEC SQL FETCH :alien
	    INTO
	    :systables.tabname,
	    :reftable,
	    :primconstr,
	    :ptabid,
	    :delrule,
	    :sysconstraints.constrid,
	    :sysconstraints.constrname,
	    :sysconstraints.constrtype,
	    :sysindexes.idxname,
	    :sysindexes.tabid,
	    :sysindexes.part[0],
	    :sysindexes.part[1],
	    :sysindexes.part[2],
	    :sysindexes.part[3],
	    :sysindexes.part[4],
	    :sysindexes.part[5],
	    :sysindexes.part[6],
	    :sysindexes.part[7];
	if (sqlca.sqlcode == SQLNOTFOUND) {
	    break;
	} else if (sqlca.sqlcode < 0) {
	    fprintf( stderr,
		     "Error FETCHing CURSOR alien: Err=%d, ISAM=%d\n",
		     sqlca.sqlcode,
		     sqlca.sqlerrd[1] );
	    rexit( 21 );
	}
	EXEC SQL OPEN cols USING :sysindexes.tabid;
	if (sqlca.sqlcode < 0) {
	    fprintf( stderr,
		     "Error OPENing CURSOR cols: Err=%d, ISAM=%d\n",
		     sqlca.sqlcode,
		     sqlca.sqlerrd[1] );
	    rexit( 22 );
	}
	for (i=0;;i++) {
	    EXEC SQL FETCH cols INTO :syscolumns.colno, :syscolumns.colname;
	    if (sqlca.sqlcode == SQLNOTFOUND)
		break;
	    else if (sqlca.sqlcode < 0) {
		fprintf( stderr,
			 "Error FETCHing CURSOR cols: Err=%d, ISAM=%d\n",
			 sqlca.sqlcode,
			 sqlca.sqlerrd[1] );
	    }
	    strcpy( columns[i], syscolumns.colname );
	}
	EXEC SQL CLOSE cols;

	fprintf( outfile,
		 "\nALTER TABLE %s ADD CONSTRAINT FOREIGN KEY (\n",
		 systables.tabname );
	for (i=0; i<8 && sysindexes.part[i]; i++) {
	    if (i)
		fprintf( outfile, ",\n" );
	    fprintf( outfile,
		     "\t%s",
		     columns[abs(sysindexes.part[i]) - 1] );
	}
	fprintf( outfile, "\n) REFERENCES %s (\n", reftable );
	EXEC SQL
	    SELECT part1, part2, part3, part4, part5, part6, part7, part8
	    INTO
	    :sysindexes.part[0],
	    :sysindexes.part[1],
	    :sysindexes.part[2],
	    :sysindexes.part[3],
	    :sysindexes.part[4],
	    :sysindexes.part[5],
	    :sysindexes.part[6],
	    :sysindexes.part[7]
	    FROM "informix".sysindexes si, "informix".sysconstraints sc
	    WHERE si.tabid = :ptabid
	    AND si.idxname = sc.idxname
	    AND sc.constrid = :primconstr;

	EXEC SQL OPEN cols USING :ptabid;
	if (sqlca.sqlcode < 0) {
	    fprintf( stderr,
		     "Error OPENing CURSOR cols2: Err=%d, ISAM=%d\n",
		     sqlca.sqlcode,
		     sqlca.sqlerrd[1] );
	    rexit( 22 );
	}
	for (i=0;;i++) {
	    EXEC SQL FETCH cols INTO :syscolumns.colno, :syscolumns.colname;
	    if (sqlca.sqlcode == SQLNOTFOUND)
		break;
	    else if (sqlca.sqlcode < 0) {
		fprintf( stderr,
			 "Error FETCHing CURSOR cols2: Err=%d, ISAM=%d\n",
			 sqlca.sqlcode,
			 sqlca.sqlerrd[1] );
	    }
	    if (ncols2 <= i) {
		columns2[i] = (char *)malloc( IDBYTES );
		if (!columns2[i]) {
		    fprintf( stderr,
			     "Cannot allocate memory for column constraint "
			     "data(3).  Errno=%d.\n",
			     errno );
		    rexit(99);
		}
		ncols2++;
	    }
	    strcpy( columns2[i], syscolumns.colname );
	}
	EXEC SQL CLOSE cols;
	
	for (i=0; i<8 && sysindexes.part[i]; i++) {
	    if (i)
		fprintf( outfile, ",\n" );
	    fprintf( outfile,
		     "\t%s",
		     columns2[abs(sysindexes.part[i]) - 1] );
	}
	if (delrule[0] == 'C')
	    fprintf( outfile, "\n) ON DELETE CASCADE" );
	else
	    fprintf( outfile, "\n)" );
	if (sscanf( sysconstraints.constrname, "r%d_%d", &a, &b ) == 2)
	    fprintf( outfile, ";\n" );
	else
	    fprintf( outfile, " CONSTRAINT %s;\n", sysconstraints.constrname );
    }
    EXEC SQL CLOSE :alien;
}


void print_sereferences( FILE *outfile, int tflag ) 
{
    int i, a, b;

    EXEC SQL BEGIN DECLARE SECTION;
    string reftable[129], delrule[2];
    long primconstr, ptabid;
    EXEC SQL END DECLARE SECTION;


    EXEC SQL DECLARE c_refs CURSOR FOR
	SELECT st.tabname, rt.tabname, sr.primary, sr.ptabid, 
	sr.delrule, sc.constrid, constrname, constrtype,
	si.idxname, si.tabid, si.part1, si.part2, si.part3, 
	si.part4, si.part5, si.part6, si.part7, si.part8 
	FROM "informix".systables st, "informix".sysconstraints sc,
	"informix".sysindexes si, "informix".sysreferences sr,
	"informix".systables rt
	WHERE st.tabid = sc.tabid
	AND rt.tabid = sr.ptabid
	AND sc.constrid = sr.constrid
	AND sc.tabid = si.tabid
	AND sc.idxname = si.idxname
	AND sc.constrtype = 'R'
	AND rt.tabname MATCHES :tablename
	ORDER BY si.tabid, constrid;
    if (sqlca.sqlcode < 0) {
	fprintf( stderr,
		 "Error DECLARing CURSOR c_refs: Err=%d, ISAM=%d\n",
		 sqlca.sqlcode,
		 sqlca.sqlerrd[1] );
	rexit( 21 );
    }

    /* Foreign Key Constraints. */
    EXEC SQL OPEN c_refs;
    if (sqlca.sqlcode < 0) {
	fprintf( stderr,
		 "Error OPENing CURSOR c_refs: Err=%d, ISAM=%d\n",
		 sqlca.sqlcode,
		 sqlca.sqlerrd[1] );
	rexit( 22 );
    }
    for (;;) {
	EXEC SQL FETCH c_refs
	    INTO
	    :systables.tabname,
	    :reftable,
	    :primconstr,
	    :ptabid,
	    :delrule,
	    :sysconstraints.constrid,
	    :sysconstraints.constrname,
	    :sysconstraints.constrtype,
	    :sysindexes.idxname,
	    :sysindexes.tabid,
	    :sysindexes.part[0],
	    :sysindexes.part[1],
	    :sysindexes.part[2],
	    :sysindexes.part[3],
	    :sysindexes.part[4],
	    :sysindexes.part[5],
	    :sysindexes.part[6],
	    :sysindexes.part[7];
	if (sqlca.sqlcode == SQLNOTFOUND) {
	    break;
	} else if (sqlca.sqlcode < 0) {
	    fprintf( stderr,
		     "Error FETCHing CURSOR c_refs: Err=%d, ISAM=%d\n",
		     sqlca.sqlcode,
		     sqlca.sqlerrd[1] );
	    rexit( 21 );
	}
	EXEC SQL OPEN cols USING :sysindexes.tabid;
	if (sqlca.sqlcode < 0) {
	    fprintf( stderr,
		     "Error OPENing CURSOR cols: Err=%d, ISAM=%d\n",
		     sqlca.sqlcode,
		     sqlca.sqlerrd[1] );
	    rexit( 22 );
	}
	for (i=0;;i++) {
	    EXEC SQL FETCH cols INTO :syscolumns.colno, :syscolumns.colname;
	    if (sqlca.sqlcode == SQLNOTFOUND)
		break;
	    else if (sqlca.sqlcode < 0) {
		fprintf( stderr,
			 "Error FETCHing CURSOR cols: Err=%d, ISAM=%d\n",
			 sqlca.sqlcode,
			 sqlca.sqlerrd[1] );
	    }
	    if (ncols2 <= i) {
		columns2[i] = (char *)malloc( IDBYTES );
		if (!columns2[i]) {
		    fprintf( stderr,
			     "Cannot allocate memory for column constraint "
			     "data(4).  Errno=%d.\n",
			     errno );
		    rexit(99);
		}
		ncols2++;
	    }
	    strcpy( columns2[i], syscolumns.colname );
	}
	EXEC SQL CLOSE cols;

	fprintf( outfile,
		 "\nALTER TABLE %s ADD CONSTRAINT FOREIGN KEY (\n",
		 systables.tabname );
	for (i=0; i<16 && sysindexes.part[i]; i++) {
	    if (i)
		fprintf( outfile, ",\n" );
	    fprintf( outfile,
		     "\t%s",
		     columns2[abs(sysindexes.part[i]) - 1] );
	}
	fprintf( outfile, "\n) REFERENCES %s (\n", reftable );
	EXEC SQL
	    SELECT part1, part2, part3, part4, part5, part6, part7, part8
	    INTO
	    :sysindexes.part[0],
	    :sysindexes.part[1],
	    :sysindexes.part[2],
	    :sysindexes.part[3],
	    :sysindexes.part[4],
	    :sysindexes.part[5],
	    :sysindexes.part[6],
	    :sysindexes.part[7]
	    FROM "informix".sysindexes si, "informix".sysconstraints sc
	    WHERE si.tabid = :ptabid
	    AND si.idxname = sc.idxname
	    AND sc.constrid = :primconstr;

	EXEC SQL OPEN cols USING :ptabid;
	if (sqlca.sqlcode < 0) {
	    fprintf( stderr,
		     "Error OPENing CURSOR cols2: Err=%d, ISAM=%d\n",
		     sqlca.sqlcode,
		     sqlca.sqlerrd[1] );
	    rexit( 22 );
	}
	for (i=0;;i++) {
	    EXEC SQL FETCH cols INTO :syscolumns.colno, :syscolumns.colname;
	    if (sqlca.sqlcode == SQLNOTFOUND)
		break;
	    else if (sqlca.sqlcode < 0) {
		fprintf( stderr,
			 "Error FETCHing CURSOR cols2: Err=%d, ISAM=%d\n",
			 sqlca.sqlcode,
			 sqlca.sqlerrd[1] );
	    }
	    if (ncols <= i) {
		columns[i] = (char *)malloc( IDBYTES );
		if (!columns[i]) {
		    fprintf( stderr,
			     "Cannot allocate memory for column constraint "
			     "data(5).  Errno=%d.\n",
			     errno );
		    rexit(99);
		}
		ncols++;
	    }
	    strcpy( columns[i], syscolumns.colname );
	}
	EXEC SQL CLOSE cols;
	
	for (i=0; i<16 && sysindexes.part[i]; i++) {
	    if (i)
		fprintf( outfile, ",\n" );
	    fprintf( outfile,
		     "\t%s",
		     columns[abs(sysindexes.part[i]) - 1] );
	}
	if (delrule[0] == 'C')
	    fprintf( outfile, "\n) ON DELETE CASCADE" );
	else
	    fprintf( outfile, "\n)" );
	if (sscanf( sysconstraints.constrname, "r%d_%d", &a, &b ) == 2)
	    fprintf( outfile, ";\n" );
	else
	    fprintf( outfile, " CONSTRAINT %s;\n", sysconstraints.constrname );
    }
    EXEC SQL CLOSE c_refs;
}
