/* print_types.ec - Print out user-defined types and row-types
*/
/*
   RCS Header:
   -----------
   $Header: /home/art/GoogleDrive/Latest\040Art's\040Stuff/myschema.d/fork/RCS/print_types.ec,v 1.17 2020/11/25 12:24:06 art Exp $

   RCS Log:
   --------
   $Log: print_types.ec,v $
   Revision 1.17  2020/11/25 12:24:06  art
   Extended --no-blade-procs/--no-blade-funcs to also exclude the resources added by the 
   blade management routines when any blade is registered.

   Revision 1.16  2019/09/10 16:42:03  art
   Moved UDT CREATE TYPE statements to the main file, GRANTS on TYPES to the grant file.
   Cleaned format for DISTINCT TYPEs.

   Revision 1.15  2015/07/07 15:10:00  art
   Added ORDER BY clauses for more consistent output for sequences, types, and casts.

   Revision 1.14  2013/08/26 21:45:29  art
   Added support for DISTINCT types (UDTs) - thought I'd done that long ago!  ;-(

   Revision 1.13  2010/11/08 20:40:31  art
   Added new --create_conditionally feature to optionally add " IF NOT EXISTS " clauses to c
   create object statements.  Feature supported by 11.70.

   Revision 1.12  2010/01/06 13:55:18  art
   Added new option --set-owner to override all object owners with a specified owner name.

   Revision 1.11  2009/11/23 18:47:05  art
   Added support for EXTERNAL tables (11.50xc6+).
   Improved 64bit compiler support.  Removed warnings.

   Revision 1.10  2009/10/18 17:23:16  art
   Fixed collection type handling.  Thanks to Bruce Simms for pointing out this one.
   Fixed summary comment to include table owner if not suppressed ala dbschema.
   Removed unneeded function prototypes.

   Revision 1.9  2009/02/11 18:30:34  art
   Fixed problem with LVARCHARS on AIX-64bit.

   Revision 1.8  2009/01/01 17:58:25  art
   Removed RCSWhat ident string which was BB specific.

   Revision 1.7  2004/10/27 15:23:26  kagel
   Fixed problem with missing/incorrect permissions for UD ROW Types.
   Added new feature to print just UDTs.
   Added new feature to suppress UDTs in full schema, table, and procedure
   reports.
   Added feature to suppress UD Type Casts in full schema, table, and procedure
   reports.
   1out1

   Revision 1.6  2004/09/14 17:33:13  kagel
   Fixing ANSI database access.

   Revision 1.5  2003/09/30 20:56:01  kagel
   Added undocumented -o option to support mydbdiff.  Changed some default
   sort orders so the output will be more consistent when tables are altered.
   Increased the Features version to 6.25 to recognize the changes.

   Revision 1.4  2001/11/01 16:09:30  kagel
   Fixed print_types to output a MAXLEN clause only
   for variable length types.

   Revision 1.3  2001/10/31 21:42:23  kagel
   Added prototype for print_coltype() and adjusted call to include the
   new syscolumns parameter.

   Revision 1.2  2001/09/26 16:53:56  kagel
   Added inheritence authorization for ROWE TYPES.
   Added NOT NULL support to ROW TYPE definitions.
   Finished OPAQUE/DISTINCT type support.

   Revision 1.1  2001/09/25 14:13:51  kagel
   Initial revision

   Revision 1.2  2001/09/20 20:49:43  kagel
   FIxed

   Revision 1.1  2001/09/14 18:38:45  kagel
   Initial revision


*/

static const char RCSHeader[] = "@(#)$Header: /home/art/GoogleDrive/Latest\040Art's\040Stuff/myschema.d/fork/RCS/print_types.ec,v 1.17 2020/11/25 12:24:06 art Exp $";
static const char RCSCompile[] = "@(#)$cc: " __FILE__ " " __DATE__ " " __TIME__ " $";

#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>

EXEC SQL INCLUDE sqlca;
EXEC SQL INCLUDE sqlhdr;
EXEC SQL INCLUDE sqltypes;

EXEC SQL INCLUDE "myschema.eh";

EXEC SQL INCLUDE prototypes.h;

EXEC SQL DEFINE InHerited 0;
EXEC SQL DEFINE NotInHerited 1;

EXEC SQL BEGIN DECLARE SECTION;
typedef struct SYSCASTS_S {
    string owner[UNAMELEN];
    short argument_type;
    int argument_xid;
    short result_type;
    int result_xid;
    string routine_name[IDBYTES];
    string routine_owner[UNAMELEN];
    string class[2];
} syscasts_t;
typedef struct SYSXTDTYPES_S {
    int extended_id;
    string owner[UNAMELEN];
    string name[IDBYTES];
    short type;
    int maxlen, length;
    short align;
} sysxtdtypes_t;
typedef struct SYSINHERITS_S {
    int child, parent;
    string class[2];
} sysinherits_t;
typedef struct SYSATTRTYPES_S {
    short fieldno;
    string fieldname[IDBYTES];
    short type, length;
    int xtd_type_id;
} sysattrtypes_t;
typedef struct SYSXTDTYPEAUTH_S {
    string typename[IDBYTES];
    string owner[UNAMELEN];
    string grantor[UNAMELEN];
    string grantee[UNAMELEN];
    int type;
    string auth[3];
} sysxtdtypeauth_t;
typedef struct DISTINCT_TYPE_S {
    int extended_id;
    string owner[UNAMELEN];
    string name[IDBYTES];
    string description[257];
} distinct_type_t;
EXEC SQL END DECLARE SECTION;


/* Externs */
extern int grantas;

void print_types( FILE *outfile, FILE *grantfile )
{
    EXEC SQL BEGIN DECLARE SECTION;
    syscasts_t cast;
    sysxtdtypes_t types, parent;
    distinct_type_t dtypes;
    sysinherits_t inherits;
    sysattrtypes_t attrtypes;
    syscolumns_t syscols;
    sysxtdtypeauth_t sysxtdtypeauth;
    string argument_typename[IDBYTES], result_typename[IDBYTES];
    short routine_i, parent_i;
    char *field_curs;
    string auth1[3], auth2[3], auth3[3], auth4[3], mode[2];
    string statement[4096], a_stmt[4096];
    EXEC SQL END DECLARE SECTION;
    char lenstr[24];
    int first;
    static char field_cursors[2][33]= {
	"inherited_fields_c",
	"non_inherited_fields_c" };

    /* First discrete types. */
    
    snprintf( statement,
	      sizeof statement,
	      "%s%s%s",
	      "SELECT t.extended_id, t.owner, t.name, t.type, t.maxlen, "
	      "       t.length, t.align "
	      "FROM \"informix\".sysxtdtypes t "
	      "WHERE mode = 'B' AND extended_id >= 2048 ",
	      (no_blade_funcs == 1) ? "  AND t.name != 'sysbldsqltext' " : "",
	      "ORDER BY 2, 3" );
    EXEC SQL PREPARE s_types FROM :statement;

    if (sqlca.sqlcode < 0) {
	fprintf( stderr,
		 "Error PREPARING xtd type query: %d, ISAM: %d.\n",
		 sqlca.sqlcode,
		 sqlca.sqlerrd[1] );
	exit(24);
    }

    EXEC SQL DECLARE c_types CURSOR FOR s_types;

    if (sqlca.sqlcode < 0) {
	fprintf( stderr,
		 "Error DECLARING xtd type query: %d, ISAM: %d.\n",
		 sqlca.sqlcode,
		 sqlca.sqlerrd[1] );
	exit(24);
    }

    EXEC SQL OPEN c_types;

    if (sqlca.sqlcode < 0) {
	fprintf( stderr,
		 "Error OPENING xtd type query: %d, ISAM: %d.\n",
		 sqlca.sqlcode,
		 sqlca.sqlerrd[1] );
	exit(24);
    }

    for (;;) {
	EXEC SQL FETCH c_types INTO :types;
        if (replace_owners) {
	  strcpy( types.owner, owner_override );
	}
	if (sqlca.sqlcode < 0) {
	    fprintf( stderr,
		     "Error FETCHING xtd types: sqlcode=%d, ISAM=%d.\n",
		     sqlca.sqlcode,
		     sqlca.sqlerrd[1] );
	    exit(26);
	} else if (sqlca.sqlcode == SQLNOTFOUND) {
	    break;
	}
	sprintf( lenstr, "%d", types.length );
	if (stripowner)
	    if (types.length)
		fprintf( outfile,
			 "CREATE OPAQUE TYPE%s %s (\n"
			 "\tinternallength=%s,\n"
			 "\talignment=%d\n);\n\n",
			 create_conditionally ? " IF NOT EXISTS" : "",
			 types.name,
			 lenstr,
			 types.align );
	    else
		fprintf( outfile,
			 "CREATE OPAQUE TYPE%s %s (\n"
			 "\tinternallength=%s,\n"
			 "\tmaxlen=%d,\n"
			 "\talignment=%d\n);\n\n",
			 create_conditionally ? " IF NOT EXISTS" : "",
			 types.name,
			 "variable",
			 types.maxlen,
			 types.align );
	else
	    if (types.length)
		fprintf( outfile,
			 "CREATE OPAQUE TYPE%s '%s'.%s (\n"
			 "\tinternallength=%s,\n"
			 "\talignment=%d\n);\n\n",
			 create_conditionally ? " IF NOT EXISTS" : "",
			 types.owner,
			 types.name,
			 lenstr,
			 types.align );
	    else
		fprintf( outfile,
			 "CREATE OPAQUE TYPE%s '%s'.%s (\n"
			 "\tinternallength=%s,\n"
			 "\tmaxlen=%d,\n"
			 "\talignment=%d\n);\n\n",
			 create_conditionally ? " IF NOT EXISTS" : "",
			 types.owner,
			 types.name,
			 "variable",
			 types.maxlen,
			 types.align );
    }

    EXEC SQL CLOSE c_types;

    /* Now authorizations on opaque types. */

    snprintf( a_stmt,
	      sizeof a_stmt,
	      "%s%s%s",
	      "SELECT t.name, t.owner, a.* "
	      "FROM \"informix\".sysxtdtypeauth a, \"informix\".sysxtdtypes t "
	      "WHERE auth IN (?,?,?,?) AND t.mode = ? "
	      "  AND a.type = t.extended_id ",
	      (no_blade_funcs == 1) ? "AND t.name != 'sysbldsqltext' " : "",
	      "ORDER BY 3, 4, 2, 1" );

    EXEC SQL PREPARE s_typeauths FROM :a_stmt;

    if (sqlca.sqlcode < 0) {
	fprintf( stderr,
		 "Error PREPARING type authorization query: %d, ISAM: %d.\n",
		 sqlca.sqlcode,
		 sqlca.sqlerrd[1] );
	fprintf( stderr, "%s\n\n", a_stmt );
	exit(24);
    }

    EXEC SQL DECLARE c_typeauths CURSOR FOR s_typeauths;

    if (sqlca.sqlcode < 0) {
	fprintf( stderr,
		 "Error DECLARING type authorization query: %d, ISAM: %d.\n",
		 sqlca.sqlcode,
		 sqlca.sqlerrd[1] );
	exit(24);
    }

    strcpy( auth1, "-u" );
    strcpy( auth2, "-U" );
    strcpy( auth3, "-u" );
    strcpy( auth4, "-U" );
    strcpy( mode, "B" );
    EXEC SQL OPEN c_typeauths USING :auth1, :auth2, :auth3, :auth4, :mode;

    if (sqlca.sqlcode < 0) {
	fprintf( stderr,
		 "Error OPENING type authorization query: %d, ISAM: %d.\n",
		 sqlca.sqlcode,
		 sqlca.sqlerrd[1] );
	exit(24);
    }

    for (;;) {
	EXEC SQL FETCH c_typeauths INTO :sysxtdtypeauth;
        if (replace_owners) {
	  strcpy( sysxtdtypeauth.owner, owner_override );
	}
	if (sqlca.sqlcode == SQLNOTFOUND)
	    break;
	else if (sqlca.sqlcode < 0) {
	    fprintf( stderr,
		     "Error FETCHING type auth. query: %d, ISAM: %d.\n",
		     sqlca.sqlcode,
		     sqlca.sqlerrd[1] );
	    exit(24);
	}
	if (stripowner)
	    if (!grantas)
		fprintf( grantfile,
			 "GRANT USAGE ON TYPE %s TO '%s'%s;\n\n",
			 sysxtdtypeauth.typename,
			 sysxtdtypeauth.grantee,
			 sysxtdtypeauth.auth[1] == 'U' ? " WITH GRANT OPTION" : "");
	    else
		fprintf( grantfile,
			 "GRANT USAGE ON TYPE %s TO '%s' AS '%s'%s;\n\n",
			 sysxtdtypeauth.typename,
			 sysxtdtypeauth.grantee,
			 sysxtdtypeauth.grantor,
			 sysxtdtypeauth.auth[1] == 'U' ? " WITH GRANT OPTION" : "" );
	else
	    if (!grantas)
		fprintf( grantfile,
			 "GRANT USAGE ON TYPE '%s'.%s TO '%s'%s;\n\n",
			 sysxtdtypeauth.owner,
			 sysxtdtypeauth.typename,
			 sysxtdtypeauth.grantee,
			 sysxtdtypeauth.auth[1] == 'U' ? " WITH GRANT OPTION" : "" );
	    else
		fprintf( grantfile,
			 "GRANT USAGE ON TYPE '%s'.%s TO '%s' AS '%s'%s;\n\n",
			 sysxtdtypeauth.owner,
			 sysxtdtypeauth.typename,
			 sysxtdtypeauth.grantee,
			 sysxtdtypeauth.grantor,
			 sysxtdtypeauth.auth[1] == 'U' ? " WITH GRANT OPTION" : "" );
    }
    EXEC SQL CLOSE c_typeauths;

    /* Now named row types. */

    EXEC SQL DECLARE c_rowtypes CURSOR FOR
	SELECT t.extended_id, t.owner, t.name, t.type, t.maxlen, t.length,
		 t.align, h.parent, h.class, p.owner, p.name
	FROM "informix".sysxtdtypes t, outer ("informix".sysinherits h, "informix".sysxtdtypes p)
	WHERE t.mode = 'R' AND t.extended_id >= 2048
	  AND t.extended_id = h.child
	  AND h.parent = p.extended_id
	  AND h.class = 't'
	ORDER BY h.parent, t.extended_id;

    if (sqlca.sqlcode < 0) {
	fprintf( stderr,
		 "Error DECLARING row type query: %d, ISAM: %d.\n",
		 sqlca.sqlcode,
		 sqlca.sqlerrd[1] );
	exit(24);
    }

    /* Query to retrieve non-inherited fields for an inherited row type. */
    EXEC SQL PREPARE ifields_s FROM
	"SELECT fieldno, fieldname, type, length, xtd_type_id \
	 FROM \"informix\".sysattrtypes \
	 WHERE extended_id = ? \
	   AND fieldno > ( \
		SELECT MAX(fieldno) \
		FROM \"informix\".sysattrtypes \
		WHERE extended_id = ? ) \
	 ORDER BY 1";

    field_curs = field_cursors[InHerited];
    EXEC SQL DECLARE :field_curs CURSOR FOR ifields_s;

    /* Query to retrieve fields for non-inherited row type. */
    EXEC SQL PREPARE nifields_s FROM
	"SELECT fieldno, fieldname, type, length, xtd_type_id \
	 FROM \"informix\".sysattrtypes \
	 WHERE extended_id = ? \
	   AND fieldno > 0 \
	 ORDER BY 1";

    field_curs = field_cursors[NotInHerited];
    EXEC SQL DECLARE :field_curs CURSOR FOR nifields_s;

    EXEC SQL OPEN c_rowtypes;

    if (sqlca.sqlcode < 0) {
	fprintf( stderr,
		 "Error OPENING row type query: %d, ISAM: %d.\n",
		 sqlca.sqlcode,
		 sqlca.sqlerrd[1] );
	exit(24);
    }
    
    for (;;) {
	first = 1;
	EXEC SQL FETCH c_rowtypes
	    INTO :types, :inherits.parent : parent_i, :inherits.class,
		:parent.owner, :parent.name;
	if (replace_owners) {
	  strcpy( types.owner, owner_override );
	  strcpy( parent.owner, owner_override );
	}
	if (sqlca.sqlcode < 0) {
	    fprintf( stderr,
		     "Error FETCHING row types: sqlcode=%d, ISAM=%d.\n",
		     sqlca.sqlcode,
		     sqlca.sqlerrd[1] );
	    exit(26);
	} else if (sqlca.sqlcode == SQLNOTFOUND) {
	    break;
	}
	if (stripowner)
	    fprintf( outfile,
		     "CREATE ROW TYPE%s %s (\n",
		     create_conditionally ? " IF NOT EXISTS" : "",
		     types.name );
	else
	    fprintf( outfile,
		     "CREATE ROW TYPE%s '%s'.%s (\n",
		     create_conditionally ? " IF NOT EXISTS" : "",
		     types.owner,
		     types.name );
	if (parent_i >= 0) {
	    field_curs = field_cursors[InHerited];
	    EXEC SQL OPEN :field_curs
		USING :types.extended_id, :inherits.parent;
	} else {
	    field_curs = field_cursors[NotInHerited];
	    EXEC SQL OPEN :field_curs
		USING :types.extended_id;
	}
	for (;;) {
	    EXEC SQL FETCH :field_curs
		INTO :attrtypes;
	    if (sqlca.sqlcode == SQLNOTFOUND)
		break;
	    else if (sqlca.sqlcode < 0) {
		fprintf( stderr,
			 "Error FETCHing %s ROW TYPE field definitions!\n",
			 (parent_i >= 0) ? "Inherited" : "NonInherited" );
		exit(29);
	    }
	    /* Format type line. */
	    if (!first)
		fprintf( outfile, ",\n\t%s ", attrtypes.fieldname );
	    else {
		fprintf( outfile, "\t%s ", attrtypes.fieldname );
		first = 0;
	    }
	    syscolumns.colno = attrtypes.fieldno;
	    strcpy( syscolumns.colname, attrtypes.fieldname );
	    syscolumns.coltype = attrtypes.type;
	    syscolumns.collength = attrtypes.length;
	    syscolumns.extended_id = attrtypes.xtd_type_id;
	    syscolumns.colmin = 0;
	    print_coltype( 1, outfile, 0, "", "", syscolumns, 0 );
	    if (attrtypes.type & SQLNONULL)
		fprintf( outfile, " NOT NULL" );
	}
	if (parent_i >= 0)
	    if (stripowner)
		fprintf( outfile,
			 "\n) UNDER %s;\n\n",
			 parent.name );
	    else
		fprintf( outfile,
			 "\n) UNDER '%s'.%s;\n\n",
			 parent.owner,
			 parent.name );
	else
	    fprintf( outfile, "\n);\n\n" );

	EXEC SQL CLOSE :field_curs;
    }

    EXEC SQL CLOSE c_rowtypes;
    EXEC SQL FREE c_rowtypes;

    field_curs = field_cursors[InHerited];
    EXEC SQL FREE :field_curs;

    field_curs = field_cursors[NotInHerited];
    EXEC SQL FREE :field_curs;

    /* Now authorizations on named row types. */

    strcpy( auth1, "-u" );
    strcpy( auth2, "-U" );
    strcpy( auth3, "n-" );
    strcpy( auth4, "N-" );
    strcpy( mode, "R" );
    EXEC SQL OPEN c_typeauths USING :auth1, :auth2, :auth3, :auth4, :mode;

    if (sqlca.sqlcode < 0) {
	fprintf( stderr,
		 "Error OPENING type authorization query: %d, ISAM: %d.\n",
		 sqlca.sqlcode,
		 sqlca.sqlerrd[1] );
	exit(24);
    }

    for (;;) {
	EXEC SQL FETCH c_typeauths INTO :sysxtdtypeauth;
        if (replace_owners) {
	  strcpy( sysxtdtypeauth.owner, owner_override );
	}
	if (sqlca.sqlcode == SQLNOTFOUND)
	    break;
	else if (sqlca.sqlcode < 0) {
	    fprintf( stderr,
		     "Error FETCHING type auth. query: %d, ISAM: %d.\n",
		     sqlca.sqlcode,
		     sqlca.sqlerrd[1] );
	    exit(24);
	}
	if (sysxtdtypeauth.auth[1] == 'u' || sysxtdtypeauth.auth[1] == 'U') {
	    if (stripowner)
		if (!grantas)
		    fprintf( grantfile,
			     "GRANT USAGE ON TYPE %s TO '%s'%s;\n\n",
			     sysxtdtypeauth.typename,
			     sysxtdtypeauth.grantee,
			     sysxtdtypeauth.auth[1] == 'U' ? " WITH GRANT OPTION" : "");
		else
		    fprintf( grantfile,
			     "GRANT USAGE ON TYPE %s TO '%s' AS '%s'%s;\n\n",
			     sysxtdtypeauth.typename,
			     sysxtdtypeauth.grantee,
			     sysxtdtypeauth.grantor,
			     sysxtdtypeauth.auth[1] == 'U' ? " WITH GRANT OPTION" : "" );
	    else
		if (!grantas)
		    fprintf( grantfile,
			     "GRANT USAGE ON TYPE '%s'.%s TO '%s'%s;\n\n",
			     sysxtdtypeauth.owner,
			     sysxtdtypeauth.typename,
			     sysxtdtypeauth.grantee,
			     sysxtdtypeauth.auth[1] == 'U' ? " WITH GRANT OPTION" : "" );
		else
		    fprintf( grantfile,
			     "GRANT USAGE ON TYPE '%s'.%s TO '%s' AS '%s'%s;\n\n",
			     sysxtdtypeauth.owner,
			     sysxtdtypeauth.typename,
			     sysxtdtypeauth.grantee,
			     sysxtdtypeauth.grantor,
			     sysxtdtypeauth.auth[1] == 'U' ? " WITH GRANT OPTION" : "" );
	}
	if (sysxtdtypeauth.auth[0] == 'n' || sysxtdtypeauth.auth[0] == 'N') {
	    if (stripowner)
		if (!grantas)
		    fprintf( grantfile,
			     "GRANT UNDER ON TYPE %s TO '%s'%s;\n\n",
			     sysxtdtypeauth.typename,
			     sysxtdtypeauth.grantee,
			     sysxtdtypeauth.auth[0] == 'N' ? " WITH GRANT OPTION" : "" );
		else
		    fprintf( grantfile,
			     "GRANT UNDER ON TYPE %s TO '%s' AS '%s'%s;\n\n",
			     sysxtdtypeauth.typename,
			     sysxtdtypeauth.grantee,
			     sysxtdtypeauth.grantor,
			     sysxtdtypeauth.auth[0] == 'N' ? " WITH GRANT OPTION" : "" );
	    else
		if (!grantas)
		    fprintf( grantfile,
			     "GRANT UNDER ON TYPE '%s'.%s TO '%s'%s;\n\n",
			     sysxtdtypeauth.owner,
			     sysxtdtypeauth.typename,
			     sysxtdtypeauth.grantee,
			     sysxtdtypeauth.auth[0] == 'N' ? " WITH GRANT OPTION" : "" );
		else
		    fprintf( grantfile,
			     "GRANT UNDER ON TYPE '%s'.%s TO '%s' AS '%s'%s;\n\n",
			     sysxtdtypeauth.owner,
			     sysxtdtypeauth.typename,
			     sysxtdtypeauth.grantee,
			     sysxtdtypeauth.grantor,
			     sysxtdtypeauth.auth[0] == 'N' ? " WITH GRANT OPTION" : "" );
	}
    }
    EXEC SQL CLOSE c_typeauths;

    /* Now DISTINCT types. */
        EXEC SQL DECLARE c_dtypes CURSOR FOR
	SELECT t.extended_id, t.owner, t.name, d.description
	FROM "informix".sysxtdtypes t, "informix".sysxtddesc d
	WHERE t.mode = 'D' AND t.extended_id >= 2048
	  AND t.extended_id = d.extended_id
	ORDER BY 2, 3;

    if (sqlca.sqlcode < 0) {
	fprintf( stderr,
		 "Error DECLARING xtd type query (distinct): %d, ISAM: %d.\n",
		 sqlca.sqlcode,
		 sqlca.sqlerrd[1] );
	exit(24);
    }

    EXEC SQL OPEN c_dtypes;

    if (sqlca.sqlcode < 0) {
	fprintf( stderr,
		 "Error OPENING xtd type (distinct) query: %d, ISAM: %d.\n",
		 sqlca.sqlcode,
		 sqlca.sqlerrd[1] );
	exit(24);
    }

    for (;;) {
	EXEC SQL FETCH c_dtypes INTO :dtypes;
        if (replace_owners) {
	  strcpy( types.owner, owner_override );
	}
	if (sqlca.sqlcode < 0) {
	    fprintf( stderr,
		     "Error FETCHING xtd (DISTINCT) types: sqlcode=%d, ISAM=%d.\n",
		     sqlca.sqlcode,
		     sqlca.sqlerrd[1] );
	    exit(26);
	} else if (sqlca.sqlcode == SQLNOTFOUND) {
	    break;
	}
	if (stripowner)
	    fprintf( outfile,
		     "CREATE DISTINCT TYPE%s %s AS %s;\n\n",
		     create_conditionally ? " IF NOT EXISTS" : "",
		     dtypes.name,
		     dtypes.description );
	else
	    fprintf( outfile,
		     "CREATE DISTINCT TYPE%s '%s'.%s AS %s;\n\n",
		     create_conditionally ? " IF NOT EXISTS" : "",
		     dtypes.owner,
		     dtypes.name,
		     dtypes.description );
    }

    EXEC SQL CLOSE c_dtypes;

    /* Now authorizations on opaque types. */

    EXEC SQL PREPARE s_dtypeauths FROM
	"SELECT t.name, t.owner, a.* \
	 FROM \"informix\".sysxtdtypeauth a, \"informix\".sysxtdtypes t \
	 WHERE auth IN (?,?,?,?) AND t.mode = ? \
	   AND a.type = t.extended_id \
	 ORDER BY 3, 4, 1, 2";

    if (sqlca.sqlcode < 0) {
	fprintf( stderr,
		 "Error PREPARING type authorization (distinct) query: %d, ISAM: %d.\n",
		 sqlca.sqlcode,
		 sqlca.sqlerrd[1] );
	exit(24);
    }

    EXEC SQL DECLARE c_dtypeauths CURSOR FOR s_dtypeauths;

    if (sqlca.sqlcode < 0) {
	fprintf( stderr,
		 "Error DECLARING type authorization (distinct) query: %d, ISAM: %d.\n",
		 sqlca.sqlcode,
		 sqlca.sqlerrd[1] );
	exit(24);
    }

    strcpy( auth1, "-u" );
    strcpy( auth2, "-U" );
    strcpy( auth3, "-u" );
    strcpy( auth4, "-U" );
    strcpy( mode, "D" );
    EXEC SQL OPEN c_dtypeauths USING :auth1, :auth2, :auth3, :auth4, :mode;

    if (sqlca.sqlcode < 0) {
	fprintf( stderr,
		 "Error OPENING type authorization (distinct) query: %d, ISAM: %d.\n",
		 sqlca.sqlcode,
		 sqlca.sqlerrd[1] );
	exit(24);
    }

    for (;;) {
	EXEC SQL FETCH c_dtypeauths INTO :sysxtdtypeauth;
        if (replace_owners) {
	  strcpy( sysxtdtypeauth.owner, owner_override );
	}
	if (sqlca.sqlcode == SQLNOTFOUND)
	    break;
	else if (sqlca.sqlcode < 0) {
	    fprintf( stderr,
		     "Error FETCHING type auth. query: %d, ISAM: %d.\n",
		     sqlca.sqlcode,
		     sqlca.sqlerrd[1] );
	    exit(24);
	}
	if (stripowner)
	    if (!grantas)
		fprintf( grantfile,
			 "GRANT USAGE ON TYPE %s TO '%s'%s;\n\n",
			 sysxtdtypeauth.typename,
			 sysxtdtypeauth.grantee,
			 sysxtdtypeauth.auth[1] == 'U' ? " WITH GRANT OPTION" : "");
	    else
		fprintf( grantfile,
			 "GRANT USAGE ON TYPE %s TO '%s' AS '%s'%s;\n\n",
			 sysxtdtypeauth.typename,
			 sysxtdtypeauth.grantee,
			 sysxtdtypeauth.grantor,
			 sysxtdtypeauth.auth[1] == 'U' ? " WITH GRANT OPTION" : "" );
	else
	    if (!grantas)
		fprintf( grantfile,
			 "GRANT USAGE ON TYPE '%s'.%s TO '%s'%s;\n\n",
			 sysxtdtypeauth.owner,
			 sysxtdtypeauth.typename,
			 sysxtdtypeauth.grantee,
			 sysxtdtypeauth.auth[1] == 'U' ? " WITH GRANT OPTION" : "" );
	    else
		fprintf( grantfile,
			 "GRANT USAGE ON TYPE '%s'.%s TO '%s' AS '%s'%s;\n\n",
			 sysxtdtypeauth.owner,
			 sysxtdtypeauth.typename,
			 sysxtdtypeauth.grantee,
			 sysxtdtypeauth.grantor,
			 sysxtdtypeauth.auth[1] == 'U' ? " WITH GRANT OPTION" : "" );
    }
    EXEC SQL CLOSE c_dtypeauths;
}







