/* print_views.ec - Print view definition.
*/
/*
   RCS Header:
   -----------
   $Header: /home/art/GoogleDrive/Latest\040Art's\040Stuff/myschema.d/fork/RCS/print_views.ec,v 2.20 2023/01/04 18:33:35 art Exp $

   RCS Log:
   --------
   $Log: print_views.ec,v $
   Revision 2.20  2023/01/04 18:33:35  art
   Fixed issue with VIEWs under user name filtering options (-O, --set-owner,
   --use-table-owner, & --use-database-owner) stripping commas from the
   SELECT statement that follow closing quotes. Thanks to Øyvind Gjerstad
   for reporting the issue.

   Revision 2.19  2019/03/17 15:13:12  art
   Added new options --replace-views and --replace-triggers to support the CREATE OR REPLACE 
   feature in v14.10.

   Revision 2.18  2014/08/31 01:23:13  art
   Removed privilege GRANT statements for views.  Not really needed.
   Removed printing of SETSESSIONAUTH privilege for the owner of the database.
   Not needed and it will fail.  If target db has a different owner, likely
   the intent is not to allow the privilege to the owner of the source
   schema anyway.

   Revision 2.17  2013/07/08 13:04:07  art
   Opps.  Initial fix for v12.10 viewtext size change broke myschema for earlier releases.
   This fixes that.

   Revision 2.16  2013/06/20 17:49:30  art
   Looks like sysviews.viewtext was expanded from 64 bytes to 256 bytes in V12! Fixed.

   Revision 2.15  2010/11/08 20:40:31  art
   Added new --create_conditionally feature to optionally add " IF NOT EXISTS " clauses to c
   create object statements.  Feature supported by 11.70.

   Revision 2.14  2010/01/06 13:55:18  art
   Added new option --set-owner to override all object owners with a specified owner name.

   Revision 2.13  2009/10/18 17:23:16  art
   Fixed collection type handling.  Thanks to Bruce Simms for pointing out this one.
   Fixed summary comment to include table owner if not suppressed ala dbschema.
   Removed unneeded function prototypes.

   Revision 2.12  2009/10/02 06:24:56  art
   Added new option -g to place user privileges into a separate file.
   Improved trigger parser.
   Made indexes and constraint processing recognize and report object state (ie disabled,
   filtering, etc.).
   Improved --filtering option again.

   Revision 2.11  2009/02/11 18:30:34  art
   Fixed problem with LVARCHARS on AIX-64bit.

   Revision 2.10  2009/01/01 17:58:25  art
   Removed RCSWhat ident string which was BB specific.

   Revision 2.9  2005/02/24 15:28:57  kagel
   Fixed problems with views containing pseudo column names which are keywords.

   Revision 2.8  2004/10/25 18:53:10  kagel
   Fixed problem with breaking quoted strings in VIEW definitions that contain quote
   characters.

   Revision 2.7  2004/09/21 15:39:16  kagel
   Fixed new -F referring VIEWS feature to eliminate duplicated views when
   a view references multiple tables requested in -t argument.  Also fixed a
   bug in print_procedures that oddly only caused trouble in mode ANSI DBs.
   Made some formatting changes to the output to tweak the looks a bit, minor.

   Revision 2.6  2004/09/15 15:14:03  kagel
   Extended referencing view code to recurse to catch indirect references.
   through other views.
   Fixed missing view owners if owner names not being stripped.

   Revision 2.5  2004/09/14 17:13:49  kagel
   Fixing ANSI database access.

   Revision 2.4  2004/09/14 17:02:07  kagel
   Added reporting of dependent VIEWS when -F option is passed.

   Revision 2.3  2000/04/10 17:20:20  kagel
   Fixed parsing of VIEW definition to correct compressing out of spaces in
   quoted strings and breaking quoted strings across line boundaries.  Main
   fix involved including my tokens.c tokenizing code from asksort.

   Revision 2.2  1999/12/08 19:18:55  kagel
   Added a call to print_auths() at the end of each view to print GRANT
   statements for the view.

   Revision 2.1  1999/09/29 16:43:52  kagel
   Establishing current  source version id for initial working version of
   newly modularized source.  This is Feature Version 6.1 Source revision
   2.1.

   Revision 1.3  1999/09/22 20:25:08  kagel
   Moved statement preparation and cursor declaration in here.

   Revision 1.2  1999/09/14 19:14:09  kagel
   Completed module disconnection.

   Revision 1.1  1999/09/01 20:50:47  kagel
   Initial revision


*/

static const char RCSHeader[] = "@(#)$Header: /home/art/GoogleDrive/Latest\040Art's\040Stuff/myschema.d/fork/RCS/print_views.ec,v 2.20 2023/01/04 18:33:35 art Exp $";
static const char RCSCompile[] = "@(#)$cc: " __FILE__ " " __DATE__ " " __TIME__ " $";

#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include <sys/types.h>
#include <string.h>
#include <limits.h>
#include <ctype.h>

EXEC SQL INCLUDE sqlhdr;
EXEC SQL INCLUDE sqlca;
EXEC SQL INCLUDE sqltypes;
EXEC SQL INCLUDE datetime;

EXEC SQL INCLUDE myschema.eh;
EXEC SQL INCLUDE tokens.h;

EXEC SQL INCLUDE prototypes.h;

extern char delp[1], *pos;

void print_views( int tabid,
		  char *tabname,
		  char *owner,
		  FILE *outfile,
		  FILE *outfileG,
		  int tabauths,
		  int colauths,
		  int grantas )
{
    EXEC SQL BEGIN DECLARE SECTION;
    int seqno, vsize, tbid;
    EXEC SQL END DECLARE SECTION;
    char *token, *p, delim[3], relim[3];
    int online, inquote;
    static int first = 1;

    if (first) {
	EXEC SQL CREATE TEMP TABLE views_viewed( viewid int )
	    WITH NO LOG;
	if (sqlca.sqlcode) {
	    fprintf( stderr,
		     "Cannot CREATE TEMP TABLE for view list, error: %d, ISAM=%d.\n",
		     sqlca.sqlcode,
		     sqlca.sqlerrd[1] );
	}
	EXEC SQL PREPARE s_cnt1 FROM
	    "SELECT count(*) FROM \"informix\".sysviews WHERE tabid = ? AND tabid NOT IN ( \
		SELECT viewid FROM views_viewed )";
	if (sqlca.sqlcode) {
	    fprintf( stderr,
		     "Cannot PREPARE select statement s_cnt1, error: %d.\n",
		     sqlca.sqlcode );
	    fprintf( stderr, "Error at character %d.\n",
		     sqlca.sqlerrd[4] );
	    rexit(18);
	}
	EXEC SQL PREPARE s_v1 FROM
	    "SELECT seqno, viewtext FROM \"informix\".sysviews \
	     WHERE tabid = ? AND tabid NOT IN ( \
		SELECT viewid FROM views_viewed ) ORDER BY seqno";
	if (sqlca.sqlcode) {
	    fprintf( stderr,
		     "Cannot PREPARE select statement s_v1, error: %d.\n",
		     sqlca.sqlcode );
	    fprintf( stderr, "Error at character %d.\n",
		     sqlca.sqlerrd[4] );
	    rexit(18);
	}
	EXEC SQL DECLARE v1 CURSOR FOR s_v1;
	if (sqlca.sqlcode < 0) {
	    fprintf( stderr,
		     "Error DECLARing CURSOR v1: Err=%d, ISAM=%d\n",
		     sqlca.sqlcode,
		     sqlca.sqlerrd[1] );
	    rexit( 21 );
	}
	first = 0;
    }
    tbid = tabid;
    EXEC SQL EXECUTE s_cnt1 INTO :vsize USING :tbid;
    if (vsize == 0) {
	/* View has already been printed, skip. */
	return;
    }
    EXEC SQL OPEN v1 USING :tbid;
    if (sqlca.sqlcode < 0) {
	fprintf( stderr,
		 "Cannot OPEN view text cursor.  Error = %d.\n",
		 sqlca.sqlcode );
	rexit(1);
    }
    if (major_vers >= 12) {
	/* The viewtext column was increased from 64 bytes to 256 bytes in
	   version 12.10+.  Cannot fetch v12.10 and earlier versions' data
	   into the same structure because of space padding in CHAR and
	   trailing space stripping in STRING type host vars, so need two
	   code paths and two structures sized correctly.
	*/
	vsize *= sizeof sysviews12.viewtext;
	if (vsize >= bigtxtsz) {
	    bigtxtsz = vsize + 2;
	    bigtxt = (char *)realloc( bigtxt, bigtxtsz );
	}
	bigtxt[0] = (char)0;
		    
	EXEC SQL FETCH v1 INTO :seqno, :sysviews12.viewtext;
	while (sqlca.sqlcode == 0) {
	    sysviews12.viewtext[sizeof sysviews12.viewtext-1] = (char)0;
	    strcat( bigtxt, sysviews12.viewtext );
	    EXEC SQL FETCH v1 INTO :seqno, :sysviews12.viewtext;
	}
    } else {
	vsize *= sizeof sysviews.viewtext;
	if (vsize >= bigtxtsz) {
	    bigtxtsz = vsize + 2;
	    bigtxt = (char *)realloc( bigtxt, bigtxtsz );
	}
	bigtxt[0] = (char)0;
		    
	EXEC SQL FETCH v1 INTO :seqno, :sysviews.viewtext;
	while (sqlca.sqlcode == 0) {
	    sysviews.viewtext[sizeof sysviews.viewtext-1] = (char)0;
	    strcat( bigtxt, sysviews.viewtext );
	    EXEC SQL FETCH v1 INTO :seqno, :sysviews.viewtext;
	}
    }
    if (sqlca.sqlcode < 0) {
	fprintf( stderr,
		 "Cannot FETCH view text.  Error = %d.\n",
		 sqlca.sqlcode );
	rexit(1);
    }
    EXEC SQL CLOSE v1;


    /* Strip off all owner names. */
    cleancat( bigtxt, bigtxt, stripowner );

    tokdel( " " );
    token = frstok2( bigtxt, delp );
    token = gettok2( );
    tokdel( "(" );
    token = gettok2( );
    if (!stripowner)
	fprintf( outfile,
		 "CREATE %sVIEW%s \"%s\".%s (\n\t",
		 Replace_Views ? "OR REPLACE " : "",
		 create_conditionally ? " IF NOT EXISTS" : "",
		 owner,
		 tabname );
    else
	fprintf( outfile,
		 "CREATE %sVIEW%s %s (\n\t",
		 Replace_Views ? "OR REPLACE " : "",
		 create_conditionally ? " IF NOT EXISTS" : "",
		 tabname );
    online = 8;
    inquote = 0;
    strcpy( delim, ", " );
    strcpy( relim, ", " );
    tokdel( delim );
    token = gettok2( );
    /* Yeah, I know that this is silly, but I have to parse the SQL anyway to
     * eliminate owner names.  Besides, I LIKE my SQL in upper case!
     * */
    while (token) {

	if ((*token == (char)0) && lasttok == ' ') {
	    /* Don't compress strings of spaces. */
	    fprintf( outfile, " " );
	    online++;
	    token = gettok2( );
	    continue;
	}

	if ((int)(online + strlen( token )) > (int)77 && !inquote) {
	    fprintf( outfile, "\n\t" );
	    online = 8;
	}
	online += (strlen( token ) + strlen( relim ));

	if (token[strlen(token) - 1] == ')') {
	    strcpy( relim, " " );
	    strcpy( delim, " " );
	    tokdel( delim ); // Reset token delimiter string.
	}

	if (*delp == ' ' && inquote == 0 && strcmp( token, "as" ) == 0) {
	    fprintf( outfile, "\nAS\nSELECT\n\t" );
	    token = gettok2( );
	    online = 8;
	} else if (*delp == ' ' && inquote == 0 && strcmp( token, "from" ) == 0) {
	    fprintf( outfile, "\n    FROM\n\t" );
	    online = 8;
	} else if (*delp == ' ' && inquote == 0 && strcmp( token, "where" ) == 0) {
	    fprintf( outfile, "\n    WHERE\n\t" );
	    online = 8;
	} else if (*delp == ' ' && inquote == 0 && strcmp( token, "and" ) == 0) {
	    fprintf( outfile, "\n      AND\n\t" );
	    online = 8;
	} else if (*delp == ' ' && inquote == 0 && strcmp( token, "or" ) == 0) {
	    fprintf( outfile, "\n      OR\n\t" );
	    online = 8;
	} else if (*delp == ' ' && inquote == 0 && strcmp( token, "between" ) == 0) {
	    fprintf( outfile, "BETWEEN " );
	    online += 8;
	} else if (*delp == ' ' && inquote == 0 && strcmp( token, "in" ) == 0) {
	    fprintf( outfile, "IN " );
	    online += 3;
	} else if (*delp == ' ' && inquote == 0 && strcmp( token, "matches" ) == 0) {
	    fprintf( outfile, "MATCHES " );
	    online += 8;
	} else if (*delp == ' ' && inquote == 0 && strcmp( token, "group" ) == 0) {
	    fprintf( outfile, "GROUP " );
	    online += 6;
	} else if (*delp == ' ' && inquote == 0 && strcmp( token, "by" ) == 0) {
	    fprintf( outfile, "BY " );
	    online += 3;
	} else if (*delp == ' ' && inquote == 0 && strcmp( token, "having" ) == 0) {
	    fprintf( outfile, "HAVING " );
	    online += 7;
	} else if (*delp == ' ' && inquote == 0 && strcmp( token, "order" ) == 0) {
	    fprintf( outfile, "ORDER " );
	    online += 6;
	} else if (*delp == ' ' && inquote == 0 && strcmp( token, "outer" ) == 0) {
	    fprintf( outfile, "OUTER " );
	    online += 6;
	} else if (*delp == ' ' && inquote == 0 && strcmp( token, "with" ) == 0) {
	    fprintf( outfile, "WITH " );
	    online += 5;
	} else if (*delp == ' ' && inquote == 0 && strcmp( token, "check" ) == 0) {
	    fprintf( outfile, "CHECK " );
	    online += 6;
	} else if (inquote == 0 && strcmp( token, "option;" ) == 0) {
	    fprintf( outfile, "OPTION;" );
	    online += 7;
	} else if ((strcmp( token, "'" ) == 0)) {
	    fprintf( outfile, "'%c", lasttok );
	    if (inquote == 1) {
		inquote = 0;
	    } else if  (inquote == 0) {
		inquote = 1;
	    }
	} else if ((strcmp( token, "\"" ) == 0)) {
	    fprintf( outfile, "\"%c", lasttok );
	    if (inquote == 2) {
		inquote = 0;
	    } else if  (inquote == 0) {
		inquote = 2;
	    }
	} else {
	    fprintf( outfile, "%s%c", token, lasttok );
	    // fprintf( outfile, "%s%s", token, relim );
	    p = token;
	    while (*p) {
		if (*p == '\'' && inquote != 2) {
		    if (!inquote)
			inquote = 1;
		    else if (inquote == 1)
			inquote = 0;
		} else if (*p == '"' && inquote != 1) {
		    if (!inquote)
			inquote = 2;
		    else if (inquote == 2)
			inquote = 0;
		}
		p++;
	    }
	}

	token = gettok2( );
    }
    fprintf( outfile, "\n\n" );

    /* print_auths( tabid, tabname, outfileG, tabauths, colauths, grantas ); */
    fprintf( outfile, "\n" );
    EXEC SQL INSERT INTO views_viewed VALUES ( :tbid );
}

void print_viewref( int tabid,
		    FILE *outfile,
		    FILE *outfileG,
		    int tabauths,
		    int colauths,
		    int grantas )
{
    EXEC SQL BEGIN DECLARE SECTION;
    int tbid, ref_tabid;
    string tabname[IDBYTES], owner[UNAMELEN], vref[20];
    static char vfstatement[1024];
    static int first = 1, level = 0, highest = 0;
    EXEC SQL END DECLARE SECTION;

    level++;
    tbid = tabid;
    if (first) {
	strncpy( vfstatement,
		 "SELECT dtabid, tabname, owner "
		 "FROM \"informix\".sysdepend sd, \"informix\".systables st "
		 "WHERE sd.btabid = ? "
		 "  AND sd.dtabid = st.tabid",
		 sizeof vfstatement );
	vfstatement[sizeof vfstatement - 1] = (char)0;
	EXEC SQL PREPARE vref_stmt FROM :vfstatement;
	if (sqlca.sqlcode) {
	    fprintf( stderr,
		     "Cannot PREPARE vref_stmt, error: %d.\n",
		     sqlca.sqlcode );
	    fprintf( stderr, "Error at character %d.\nStatement is:\n",
		     sqlca.sqlerrd[4] );
	    fprintf( stderr, "%s\n\n", vfstatement );
	    fprintf( stderr,
		     "errp=%s, errm=%s\n\n",
		     sqlca.sqlerrp,
		     sqlca.sqlerrm );
	    rexit(2);
	}
    }
    snprintf( vref, sizeof vref, "vref_%d", level );
    if (highest < level) {
	EXEC SQL DECLARE :vref CURSOR FOR vref_stmt;
	if (sqlca.sqlcode ) {
	    fprintf( stderr,
		     "Cannot declare :vref cursor; Error = %d\n",
		     sqlca.sqlcode );
	    rexit(1);
	}
	highest++;
    }

    EXEC SQL OPEN :vref USING :tbid;
    if ( sqlca.sqlcode ) {
	fprintf( stderr,
		 "Cannot open vref cursor; Error = %d\n",
		 sqlca.sqlcode );
	rexit(1);
    }
    for (;;) {
	EXEC SQL FETCH :vref INTO :ref_tabid, :tabname, :owner;
	if (sqlca.sqlcode != 0)
	    break;
	print_views( ref_tabid,
		     tabname,
		     owner,
		     outfile,
		     outfileG,
		     tabauths,
		     colauths,
		     grantas );
	/* Recurse to get all cascaded references. */
	print_viewref( ref_tabid, outfile, outfileG, tabauths, colauths, grantas );
    }
    if (sqlca.sqlcode && sqlca.sqlcode != SQLNOTFOUND) {
	fprintf( stderr,
		 "Error fetching vref cursor;Error=%d\n",
		 sqlca.sqlcode);
	rexit(1);
    }

    EXEC SQL CLOSE :vref;
    level--;
}

void print_allviewrefs( FILE *outfile,
			FILE *outfileG,
			int tabauths,
			int colauths,
			int grantas )
{
    EXEC SQL BEGIN DECLARE SECTION;
    int tabid;
    EXEC SQL END DECLARE SECTION;

    EXEC SQL DECLARE avr_1 CURSOR FOR
	SELECT tabid
	FROM "informix".systables
	WHERE tabname matches :tablename
	  AND tabtype in ('T', 'V')
	  AND tabid >= 100
	AND tabid NOT IN (
	    SELECT viotid FROM "informix".sysviolations )
	AND tabid NOT IN (
	    SELECT diatid FROM "informix".sysviolations )
	ORDER BY tabid;
    if (sqlca.sqlcode ) {
	fprintf( stderr,
		 "Cannot declare avr_1 cursor; Error = %d, ISAM = %d\n",
		 sqlca.sqlcode,
		 sqlca.sqlerrd[1] );
	rexit(1);
    }

    EXEC SQL OPEN avr_1;
    if (sqlca.sqlcode ) {
	fprintf( stderr,
		 "Cannot OPEN avr_1 cursor; Error = %d, ISAM = %d\n",
		 sqlca.sqlcode,
		 sqlca.sqlerrd[1] );
	rexit(1);
    }

    for (;;) {
	EXEC SQL FETCH avr_1 INTO :tabid;
	if (sqlca.sqlcode != 0)
	    break;
	print_viewref( tabid,
			outfile,
			outfileG,
			tabauths,
			colauths,
			grantas );
    }
    if (sqlca.sqlcode < 0) {
	fprintf( stderr,
		 "Cannot FETCH avr_1 cursor; Error = %d, ISAM = %d\n",
		 sqlca.sqlcode,
		 sqlca.sqlerrd[1] );
	rexit(1);
    }
    EXEC SQL FREE avr_1;
}
