/* return_coltypes.ec - 
*/
/*
   RCS Header:
   -----------
   $Header: /home/art/GoogleDrive/Latest\040Art's\040Stuff/myschema.d/fork/RCS/return_coltypes.ec,v 1.11 2021/03/09 14:54:21 art Exp $

   RCS Log:
   --------
   $Log: return_coltypes.ec,v $
   Revision 1.11  2021/03/09 14:54:21  art
   Improved fix to multibyte issue from HHrvoje Zokovic with a
   couple of tweaks from me.

   Revision 1.9  2017/04/05 14:14:22  art
   Fixed timeseries printout to eliminate the 'size' clause.

   Revision 1.8  2015/11/24 17:09:46  art
   Fixed routine owner handling where there are "--" comments embedded in the parameter list.
   Fixed the column list included in UPDATE STATISTICS LOW when -u is passed.
   Fixed an issue with the DROP statement not printing ROW type parameters when --drop-proc is passed.
   Thanks to David Link and Snorri Bergmann for pointing these out.

   Revision 1.7  2014/09/19 16:21:34  art
   Fixed ROW type column printouts (was including a length).

   Revision 1.6  2013/03/19 23:12:54  art
   Fixed handling of attached/IN TABLE indexes. Fixed a problem when
   -O and --drop-proc are specified together causing the DROP routine
   statement to have no argument list. Added missing support for
   REFERENCES BYTE/TEXT arguments in DROP routine statements when
   --drop-proc is specified.

   Revision 1.5  2011/07/21 18:53:29  art
   Fixed arglists for DROP routine for SET/MULTISET/LIST of generics and not a specific type.

   Revision 1.4  2011/07/21 16:08:38  art
   Fixed warning.

   Revision 1.3  2011/07/21 16:04:54  art
   Fixed problems with embedded comments again from TALX.
   Imporved --drop-proc processing.
   Added REVOKE EXECUTE... FROM "PUBLIC"; to remove default priveleges from the routine added at creation.
   Fixed SET/LIST/MULTISET argument handling in DROP...

   Revision 1.2  2011/07/01 21:10:38  art
   fixes to make overloaded functions work with --drop-proc.


*/

#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <limits.h>

EXEC SQL INCLUDE sqlhdr;
EXEC SQL INCLUDE sqlca;
EXEC SQL INCLUDE sqltypes;
EXEC SQL INCLUDE datetime;
EXEC SQL INCLUDE int8;
EXEC SQL INCLUDE ifxtypes.h;
EXEC SQL INCLUDE myschema.eh;

EXEC SQL INCLUDE prototypes.h;
static const char RCSHeader[] = "@(#)$Header $";
static const char RCSCompile[] = "@(#)$cc: " __FILE__ " " __DATE__ " " __TIME__ " $";

/* EXTERNS */
extern int SMICounts, NoCurrent;

void get_extended_desc ( int xid, char *xtddescription )
{
    EXEC SQL BEGIN DECLARE SECTION;
    int paramxid;
    string xtddesc[257];
    EXEC SQL END DECLARE SECTION;

    paramxid = xid;
    xtddescription[0] = (char)0;

    EXEC SQL DECLARE c_set_desc CURSOR FOR
	SELECT description
	FROM sysxtddesc
	WHERE extended_id = :paramxid
	ORDER BY seqno;
    if (sqlca.sqlcode < 0) {
	fprintf( stderr, 
		 "Error declaring extended description. SQL=%d, ISAM=%d.\n",
		 sqlca.sqlcode,
		 sqlca.sqlerrd[1] );
	exit(1);
    }
    EXEC SQL OPEN c_set_desc;
    if (sqlca.sqlcode < 0) {
	fprintf( stderr, 
		 "Error opening extended description. SQL=%d, ISAM=%d.\n",
		 sqlca.sqlcode,
		 sqlca.sqlerrd[1] );
	exit(1);
    }
    for (;;) {
	EXEC SQL FETCH c_set_desc INTO :xtddesc;
	if (sqlca.sqlcode == 100)
	    break;
	else if (sqlca.sqlcode < 0) {
	    fprintf( stderr, 
		     "Error fetching extended description. SQL=%d, ISAM=%d.\n",
		     sqlca.sqlcode,
		     sqlca.sqlerrd[1] );
	    exit(1);
	}
	strcat( xtddescription, xtddesc );
    }
}

void return_coltypes( int prcid, char *procstr )
{
    static int first = 1;
    char part1[30], part2[30], typeowner[UNAMELEN], typename[65536];
    int p1, p2, pt1, pt2, fr1, fr2;
    int ext, col, attr;
    char typestr[100], *p;
    char xtddescription[65536], mode[2];

    EXEC SQL BEGIN DECLARE SECTION;
    static char statement[2048], stmt2[2048];
    static int SQL_LOGICAL_CHAR;
    int procid, paramid, paramtype, paramlength, paramxid, paramattr;
    short paramname_i;
    string  paramname[129], xtddesc[257];
    EXEC SQL END DECLARE SECTION;

    if (first) {
	snprintf( stmt2,
		  sizeof stmt2,
		  "SELECT paramid, paramname, paramtype, paramlen, paramxid, paramattr "
		  "FROM sysproccolumns "
		  "WHERE procid = ? and paramattr < 3 "
		  "ORDER BY paramattr, paramid" );
	EXEC SQL PREPARE parmlst_s FROM :stmt2;
	if (sqlca.sqlcode < 0) {
	    fprintf( stderr,
		     "Error preparing parmlst_s.  SQL: %d, ISAM: %d, errm=%s.\n",
		     sqlca.sqlcode,
		     sqlca.sqlerrd[1],
		     sqlca.sqlerrm );
	    exit(1);
	}
	EXEC SQL DECLARE parmlst CURSOR FOR parmlst_s;
	if (sqlca.sqlcode < 0) {
	    fprintf( stderr,
		     "Error decalring cursor parmlst.  SQL: %d, ISAM: %d, errm=%s.\n",
		     sqlca.sqlcode,
		     sqlca.sqlerrd[1],
		     sqlca.sqlerrm );
	    exit(1);
	}
	EXEC SQL SELECT (MOD( flags, 3) + 1) INTO :SQL_LOGICAL_CHAR FROM 'informix'.systables WHERE tabname = ' VERSION';
	first = 0;
    }

    procid = prcid;

    EXEC SQL OPEN parmlst USING :procid;
    if (sqlca.sqlcode < 0) {
	fprintf( stderr,
		 "Error opening parmlst.  SQL: %d, ISAM: %d, errm=%s.\n",
		 sqlca.sqlcode,
		 sqlca.sqlerrd[1],
		 sqlca.sqlerrm );
	exit(1);
    }

    p = procstr + sprintf( procstr, "( " );
    col = 0;
    attr = 1;

    for (;;) {
	EXEC SQL FETCH parmlst
	    INTO :paramid, :paramname indicator paramname_i, :paramtype, :paramlength, :paramxid, :paramattr;
	if (sqlca.sqlcode > 0)
	    break; 
	else if (sqlca.sqlcode < 0) {
	    fprintf( stderr,
		     "Error fetching parmlst.  SQL: %d, ISAM: %d, errm=%s.\n",
		     sqlca.sqlcode,
		     sqlca.sqlerrd[1],
		     sqlca.sqlerrm );
	    exit(1);
	}

	switch ( paramtype ) {
	case SQLCHAR:
	    sprintf( typestr, "CHAR(%d)", paramlength / SQL_LOGICAL_CHAR);
	    break;
	case SQLNCHAR:
	    sprintf( typestr, "NCHAR(%d)", paramlength / SQL_LOGICAL_CHAR );
	    break;
    EXEC SQL IFNDEF NOINT8;
	case SQLUDTFIXED:
	case SQLUDTVAR:
	case SQLROW:
	    {
		if (paramtype == SQLROW && paramxid == 0) {
		    ext = 0;
		    snprintf( typename, sizeof typename, "ROW" );
		    snprintf( typeowner, sizeof typeowner, "informix" );
		} else
		    ext = return_extended( paramxid, typeowner, typename, mode );
		if (!stripowner ) {
		    if (ext == 0)
			if (paramtype == SQLROW || (major_vers == 9 && minor_vers <= 30) || mode[0] == 'S')
			    sprintf( typestr, "\"%s\".%s",
				     typeowner,
				     typename );
			else
			    sprintf( typestr, "\"%s\".%s(%d)",
				     typeowner,
				     typename,
				     paramlength );
		    else
			sprintf( typestr, "\"%s\".%s", typeowner, typename );
		} else {
		    if (ext == 0)
			if (paramtype == SQLROW || (major_vers == 9 && minor_vers <= 30))
			    sprintf( typestr, "%s", typename );
			else
			    sprintf( typestr,
				     "%s(%d)",
				     typename,
				     paramlength );
		    else
			sprintf( typestr, "%s", typename );
		}
	    }
	    break;
	case SQLSET:
	    if (paramxid > 0)
		get_extended_desc( paramxid, xtddescription );
	    else
		strcpy( xtddescription, "SET" );
	    sprintf( typestr, "%s", xtddescription );
	    break;
	case SQLMULTISET:
	    if (paramxid > 0)
		get_extended_desc( paramxid, xtddescription );
	    else
		strcpy( xtddescription, "MULTISET" );
	    sprintf( typestr, "%s", xtddescription );
	    break;
	case SQLLIST:
	    if (paramxid > 0)
		get_extended_desc( paramxid, xtddescription );
	    else
		strcpy( xtddescription, "LIST" );
	    sprintf( typestr, "%s", xtddescription );
	    break;
    EXEC SQL ENDIF;
	case SQLSMINT:
	    sprintf( typestr, "SMALLINT" );
	    break;
	case SQLINT:
	    sprintf( typestr, "INTEGER" );
	    break;
	case SQLFLOAT:
	    sprintf( typestr, "FLOAT" );
	    break;
	case SQLSMFLOAT:
	    sprintf( typestr, "SMALLFLOAT" );
	    break;
    EXEC SQL IFNDEF NOINT8;
	case SQLINT8:
	    sprintf( typestr, "INT8" );
	    break;
    EXEC SQL ENDIF;

#if defined( SQLINFXBIGINT )
	case SQLINFXBIGINT:
	    sprintf( typestr, "BIGINT" );
	    break;
#endif
	case SQLDECIMAL:
	    if ((paramlength % 256) == 255)
		/* Floating point. */
		sprintf( typestr,
			 "DECIMAL(%d)",
			 paramlength / 256 );
	    else
		/* Fixed point. */
		sprintf( typestr,
			 "DECIMAL(%d,%.1d)",
			 paramlength / 256,
			 paramlength % 256 );
	    break;
	case SQLDATE:
	    sprintf( typestr, "DATE" );
	    break;
	case SQLMONEY:
	    sprintf( typestr,
		     "MONEY(%d,%.1d)",
		     paramlength / 256,
		     paramlength % 256 );
	    break;
	case SQLDTIME:
	    pt1 = (((paramlength / 16) % 16) <= 10) ?
		(((paramlength / 16) % 16) / 2) : 6;
	    strcpy( part1, dtparts[pt1] );
	    pt2 = ((paramlength % 16) <= 10) ?
		((paramlength % 16) / 2) : 6;
	    if (pt2 == 6) {
		fr2 = (paramlength % 16) - 10;
		sprintf( part2, "%s(%d)", dtparts[pt2], fr2 );
	    } else
		strcpy( part2, dtparts[pt2] );
	    sprintf( typestr,
		     "DATETIME %s TO %s",
		     part1,
		     part2 );
	    break;
	case SQLINTERVAL:
	    p1 = (paramlength / 16) % 16;
	    pt1 = (p1 <= 10) ? (p1 / 2) : 6;
	    fr2 = (paramlength % 16) - 10;
	    p2 = paramlength % 16;
	    pt2 = (p2 <= 10) ? (p2 / 2) : 6;
	    if (pt1 != 6) {
		fr1 = (paramlength / 256) + p1 - p2;
		sprintf( part1, "%s(%d)", dtparts[pt1], fr1 );
	    } else
		strcpy( part1, dtparts[pt1] );
	    if (pt2 == 6) {
		sprintf( part2, "%s(%d)", dtparts[pt2], fr2 );
	    } else
		strcpy( part2, dtparts[pt2] );
	    sprintf( typestr,
		     "INTERVAL %s TO %s",
		     part1,
		     part2 );
	    break;
	case SQLVCHAR:
	    sprintf( typestr,
		     "VARCHAR(%u,%.1u)",
		     ((unsigned)paramlength % (unsigned)256) / SQL_LOGICAL_CHAR,
		     ((unsigned)paramlength / (unsigned)256) % (unsigned)256 );
	    break;
	case SQLNVCHAR:
	    sprintf( typestr,
		     "NVARCHAR(%u,%.1u)",
		     ((unsigned)paramlength % (unsigned)256) / SQL_LOGICAL_CHAR,
		     ((unsigned)paramlength / (unsigned)256) % (unsigned)256 );
	    break;

	case SQLBYTES:
	    sprintf( typestr, "REFERENCES BYTE" );
	    break;
	case SQLTEXT:
	    sprintf( typestr, "REFERENCES TEXT" );
	    break;
	default:
	    sprintf( typestr, "UNKNOWN TYPE(%d, %d)",
		     paramtype,
		     paramlength );
	    break;
	}
	if (paramattr == 3 && attr == 1) {
	    p += sprintf( p, " ) RETURNING " );
	    col = 0;
	}
	if (col++) {
	    p += sprintf( p, ", " );
	}
	if (paramattr == 1)
	    p += sprintf( p, "%s", typestr );
	else if (paramname_i < 0)
	    p += sprintf( p, "%s", typestr );
	else
	    p += sprintf( p, "%s", typestr );
	
    }
    p += sprintf( p, " )" );

    EXEC SQL CLOSE parmlst;
}
