/* return_extended.ec - Return extended type name.
*/
/*
   RCS Header:
   -----------
   $Header: /home/art/GoogleDrive/Latest\040Art's\040Stuff/myschema.d/fork/RCS/return_extended.ec,v 1.18 2019/11/18 12:30:35 art Exp $

   RCS Log:
   --------
   $Log: return_extended.ec,v $
   Revision 1.18  2019/11/18 12:30:35  art
   Fixed datetime printouts and improved handling of distinct types.

   Revision 1.17  2017/04/05 14:17:15  art
   Fixed error message printout.

   Revision 1.16  2017/04/05 14:14:22  art
   Fixed timeseries printout to eliminate the 'size' clause.

   Revision 1.15  2015/11/24 17:09:46  art
   Fixed routine owner handling where there are "--" comments embedded in the parameter list.
   Fixed the column list included in UPDATE STATISTICS LOW when -u is passed.
   Fixed an issue with the DROP statement not printing ROW type parameters when --drop-proc is passed.
   Thanks to David Link and Snorri Bergmann for pointing these out.

   Revision 1.14  2015/07/07 14:29:45  art
   Added some diagnostics.  Protected extended type code from being executed on a v7 server.

   Revision 1.13  2010/01/06 13:55:18  art
   Added new option --set-owner to override all object owners with a specified owner name.

   Revision 1.12  2009/10/18 17:23:16  art
   Fixed collection type handling.  Thanks to Bruce Simms for pointing out this one.
   Fixed summary comment to include table owner if not suppressed ala dbschema.
   Removed unneeded function prototypes.

   Revision 1.11  2009/02/11 18:30:34  art
   Fixed problem with LVARCHARS on AIX-64bit.

   Revision 1.10  2009/02/11 15:55:50  art
   Fixed minor problem potential in the SQL.

   Revision 1.9  2009/01/01 17:58:25  art
   Removed RCSWhat ident string which was BB specific.

   Revision 1.8  2004/09/14 17:13:28  kagel
   Fixing ANSI database access.

   Revision 1.7  2003/12/31 20:43:29  kagel
   Fixed variable length extended type handling (ie LVARCHAR) to include the
   columns's actual length.

   Revision 1.6  2003/10/29 21:58:57  kagel
   Fixed 1.5

   Revision 1.5  2003/10/29 21:46:40  kagel
   Fixed backward compatibility problems.

   Revision 1.4  2001/10/30 14:43:32  kagel
   Removed type cast of description to varchar(255) (is varchar(256) in table).
   Expanded host var to hold description from IDBYTES (129) to 257.
   Changed colon (:) indicator separator for the indicator keyword.
   Other cosmetic.

   Revision 1.3  2001/10/29 18:37:33  kagel
   Changes implementing support for collection types.

   Revision 1.2  2001/09/26 13:33:47  kagel
   Fixed NOT NULL masking.

   Revision 1.1  2001/09/10 19:10:18  kagel
   Initial revision


*/

static const char RCSHeader[] = "@(#)$Header: /home/art/GoogleDrive/Latest\040Art's\040Stuff/myschema.d/fork/RCS/return_extended.ec,v 1.18 2019/11/18 12:30:35 art Exp $";
static const char RCSCompile[] = "@(#)$cc: " __FILE__ " " __DATE__ " " __TIME__ " $";

#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>

EXEC SQL INCLUDE sqlca;
EXEC SQL INCLUDE sqlhdr;
EXEC SQL INCLUDE sqltypes;
EXEC SQL INCLUDE "myschema.eh";

EXEC SQL INCLUDE prototypes.h;

static long rt_first_time = 1;

long return_extended(
    EXEC SQL IFNDEF NOINT8;
    EXEC SQL BEGIN DECLARE SECTION;
    PARAMETER long extended_id,
    PARAMETER string owner[UNAMELEN],
    PARAMETER string name[65535],
    PARAMETER string mode[2]
    EXEC SQL END DECLARE SECTION;
    EXEC SQL ELSE;
    extended_id,
    owner,
    name,
    mode
    EXEC SQL ENDIF;
    )
    EXEC SQL IFDEF NOINT8;
    $PARAMETER long extended_id;
    $PARAMETER string owner[UNAMELEN];
    $PARAMETER string name[65535];
    $PARAMETER string node[2];
    EXEC SQL ENDIF;
{
    EXEC SQL BEGIN DECLARE SECTION;
    static string stmt[1025];
    int seqno;
    int found = 0;
    short i_name, i_seqno;
    string desc[257];
    string lclname[IDBYTES];
    long length, lcl_length, maxlen;
    EXEC SQL END DECLARE SECTION;

    if (!Universal)
	return 0;

    if (rt_first_time == 1) {
      /**
       ** We're using outer join with sysxtddesc for unnamed collection types
       ** and some built-in extended types like LVARCHAR that have no description.
      */
      strcpy( stmt,
	      "SELECT t.owner, t.name, t.length, t.maxlen, t.mode, d.seqno, d.description "
	      "FROM   \"informix\".sysxtdtypes t, OUTER \"informix\".sysxtddesc d "
	      "WHERE  d.extended_id = t.extended_id "
	      "  AND    t.extended_id = ? "
	      "ORDER BY seqno "
              );
      
	EXEC SQL PREPARE s_xtd FROM :stmt;
	if (sqlca.sqlcode < 0) {
	   fprintf( stderr,
		    "Error PREPARING extended type query: %d, ISAM: %d.\n",
		    sqlca.sqlcode,
		    sqlca.sqlerrd[1] );
	   exit(22);
	}
	EXEC SQL DECLARE c_xtd CURSOR FOR s_xtd;
	if (sqlca.sqlcode < 0) {
	   fprintf( stderr,
		    "Error DECLARING extended type query: %d, ISAM: %d.\n",
		    sqlca.sqlcode,
		    sqlca.sqlerrd[1] );
	   exit(22);
	}
	rt_first_time = 0;
    }

    EXEC SQL OPEN c_xtd USING :extended_id;
    if (sqlca.sqlcode < 0) {
	fprintf( stderr,
		 "Error OPENING extended type query: %d, ISAM: %d.\n",
		 sqlca.sqlcode,
		 sqlca.sqlerrd[1] );
	exit(22);
    }
    
    name[0] = (char)0;
    for (;;) {

	EXEC SQL FETCH c_xtd
	    INTO :owner, :lclname indicator i_name, :lcl_length, :maxlen, :mode, :seqno indicator i_seqno, :desc;
        if (replace_owners && extended_id > 20) {
	    strcpy( owner, owner_override );
	}
	if (sqlca.sqlcode == SQLNOTFOUND) {
	  if (found == 0) length = 0;
	    break;
	} else if (sqlca.sqlcode < 0) {
	    fprintf( stderr,
		     "Error FETCHING extended type query: %d, ISAM: %d.\n",
		     sqlca.sqlcode,
		     sqlca.sqlerrd[1] );
	    exit(22);
	}

	found = 1;
	length = lcl_length;
	/**
	 * If name is null - we have an unnamed collection type.  Use the description.
	 * Note: If a collection desc exceeds 256 bytes may have to concatenate all
	 * descriptions into 'desc'.
	 */
	if (i_name == -1 && i_seqno == -1) {
	    /* Should never happen, but no control over true UDTs! */
	    fprintf( stderr, "Error: Extended type %ld has not name or description!\n", extended_id );
	    exit(22);
	} if (i_name == -1) {
	    strcat( name, desc );
	} else
	    strcpy( name, lclname );
    }
    EXEC SQL CLOSE c_xtd;
    
    return length;
}
