/* tokens.c - Functions to implement a more flexible token parser than strtok()
   This is designed for parsing ASCII delimited records.  It forgives NULL
   fields and recognizes EOL as a final delimiter.  The nl or cr is stripped.

   Version customized for myschema.

RCS Header:
----------
$Header: /home/art/GoogleDrive/Latest\040Art's\040Stuff/myschema.d/fork/RCS/tokens.c,v 1.5 2023/12/15 13:33:43 art Exp $

RCS Log
-------
$Log: tokens.c,v $
Revision 1.5  2023/12/15 13:33:43  art
Minor change. No operational difference.

Revision 1.4  2013/03/06 03:03:15  art
Fixed problem with triggers with long comma lists with not white space causing problems
when loading the schema.
Added new feature --squish-triggers to eliminate unnecessary white space formatting
to reduce problems loading the schema caused by IDS's trigger expansion making the
trigger too big to load (> 65K).  This problem goes away in 12.10 anyway, but ...

Revision 1.3  2002/10/29 18:41:10  kagel
Added new functions frstok2 and getok2 to place delimiter character in
a local pointer provided in frsttok2 instead of accessing the global
lasttok variable.  I like this better.

 * Revision 1.2  2000/04/10  15:34:40  kagel
 * Fixed parsing of VIEW definition to correct compressing out of spaces in
 * quoted strings and breaking quoted strings across line boundaries.  Main
 * fix involved including my tokens.c tokenizing code from asksort.
 *
Revision 1.1  2000/04/10 15:25:18  kagel
Initial revision


*/

#include <stdlib.h>
#include <string.h>
#include <stdio.h>

char lasttok;

static char *string, *dl;
char *delp, *pos;

/* Note pointer copy.  If the caller modifies del, dl is also altered. */
void tokdel( char *del )
{
    dl = del;
}

char *frstok( char *str )
{
    register char *p;

    string = pos = str;
    while (*pos) {
	if (strchr( dl, *pos ))
	    break;
	pos++;
    }
    if (*pos) {
	lasttok = *pos;
	*pos = (char)0;
	return string;
    } else {
	pos--; /* Make next call return NULL again. */
	lasttok = (char)0;
	if (*string)
	    return string;
	else
	    return (char *)NULL;
    }
}

char *gettok(  )
{
    register char *p;

    string = ++pos;
    while (*pos) {
	if (strchr( dl, *pos ))
	    break;
	pos++;
    }
    if (*pos) {
	lasttok = *pos;
	*pos = (char)0;
	return string;
    } else {
	pos--; /* Make next call return NULL again. */
	lasttok = (char)0;
	if (*string)
	    return string;
	else
	    return (char *)NULL;
    }
}

char *frstok2( char *str, char *del )
{
    register char *p;

    delp = del;

    string = pos = str;
    while (*pos) {
	if (strchr( dl, *pos )) {
	    *delp = *pos;
	    break;
	}
	pos++;
    }
    if (*pos) {
	lasttok = *pos;
	*pos = (char)0;
	return string;
    } else {
	pos--; /* Make next call return NULL again. */
	lasttok = (char)0;
	if (*string)
	    return string;
	else
	    return (char *)NULL;
    }
}

char *gettok2( )
{
    register char *p;

    string = ++pos;
    while (*pos) {
	if (strchr( dl, *pos )) {
	    *delp = *pos;
	    break;
	}
	pos++;
    }
    if (*pos) {
	lasttok = *pos;
	*pos = (char)0;
	return string;
    } else {
	pos--; /* Make next call return NULL again. */
	lasttok = (char)0;
	if (*string)
	    return string;
	else
	    return (char *)NULL;
    }
}


/* Find the longest string delimited by 'delim' that is shorter
   than 'wrapcol';
*/
int find_break( char *word, char delim, int wrapcol )
{
    char *start, *end, *pt;
    int wlen;

    if (word == (char *)0)
	return 0;
    if (*word == (char)0)
	return 0;
    start = end = word;
    while (pt = strchr( end, delim )) {
	pt++; /* Advance past delimiter. */
	wlen = ((long)pt - (long)start);
	if (wlen > wrapcol) {
	    return wlen;
	} else {
	    end = pt;
	}
    }
    if (end > start) {
	return ((long)end - (long)start);
    } else {
	return strlen( start );
    }
}
