/* usage.c - Help function.
*/
/*
   RCS Header:
   -----------
   $Header: /home/art/GoogleDrive/Latest\040Art's\040Stuff/myschema.d/fork/RCS/usage.c,v 2.103 2024/05/01 15:53:22 art Exp $

   RCS Log:
   --------
   $Log: usage.c,v $
   Revision 2.103  2024/05/01 15:53:22  art
   New option --suppress-audit to suppress the WITH AUDIT option
   in CREATE TABLE statements for audited tables.

   Revision 2.102  2024/05/01 15:28:49  art
   Stripped trailing spaces from the end of procedure/function
   definitions. Thanks to Doug Lawry for the suggestion.
   New option --no-sequence-init to suppress the ALTER statement
   that sets a sequence to its current value. Thanks to Doug
   Lawry for the suggestion. New option --no-er-columns to 
   suppress printing ER related special columns such as ERKEY,
   REPLCHECK, VERCOLS, & CRCOLS.
   Thanks to Paul Watson for this suggestion.

   Revision 2.101  2023/12/11 14:59:56  art
   Made the new options toggles so that this release can run correctly
   on IDS versions prior to v15.

   Revision 2.100  2023/12/11 14:21:26  art
   Added support for the LARGE and SMALL keywords in CREATE TABLE
   statements.
   Added support for INVISIBLE indexes.
   Added new options to suppress the SMALL and LARGE keywords in
   CREATE TABLE statements and the INVISIBLE keyword in CREATE
   INDEX statements to permit using myschema to port to pre-v15
   servers and to recreate tables as LARGE or SMALL depending on
   server capabilities and the v15 implicit table type specified
   in the ONCONFIG file (SMALL_TABLES=0|1).

   Revision 2.99  2022/11/06 18:58:55  art
   New option --include-sysprocs documented.

   Revision 2.98  2022/09/08 17:21:14  art
   Added new options export-binary and import-binary to generate only 
   the export or import script for using an external table in binary
   format.

   Revision 2.97  2021/11/03 14:11:14  art
   Added new option --myexport-binary to unload in INFORMIX binary
   internal format in express mode.

   Revision 2.96  2021/05/13 23:01:26  art
   Added new --myexport-dependency option to sort by dependencies
   but in reverse order to prevent inconsistent data.

   Revision 2.95  2021/05/06 11:27:56  art
   Added new --no-export-triggers - Disable select triggers in the external table 
   export scripts.

   Revision 2.94  2021/05/06 00:50:06  art
   Added new options --no-data-tables and --no-data-accesses to suppress 
   generation of unload and load scripts when --export-scripts
   --import-scripts --myexport-scripts --myexport-express is enabled.
   The former suppresses a list of tables (default: tsinstancetable)
   The latter suppresses tables built with any of a list of access 
   methods (default:ts_rtl_vtam,ts_vtam).

   Revision 2.93  2021/03/09 16:22:20  art
   New option --for-er adds WITH REPLCHECK and WITH ERKEY to each table to

   Revision 2.92  2021/02/19 17:28:15  art
   Added new options: --export-scripts & --import-scripts to create only 
   the external table files for exports or imports respectively. These
   obey the --myexport-delimiter and --myexport-escape option settings,
   but synonyms for each that start with --export- & --import were added
   for completeness. The unload filenames using these new options are 
   made with the full filename instead of the dbexport compatible 
   filenames produced by --myexport-scripts and --myexport-express.

   Revision 2.91  2021/01/12 16:42:40  art
   Added new options --no-setsessionauth and --comment-setsessionauth
   Needed because even the database owner cannot grant SETSESSIONAUTH to
   itself so some scripts will fail if these auths are included. They 
   should be processed manually. The -no suppresses and the --comment
   comments out these privs.

   Revision 2.90  2020/12/02 18:55:16  art
   Fixed -o to sort systable objects other than tables in name order within type.
   Fixed the interaction between -o and --dependency-order  to sort tables my name
   within dependency level and to sort all non-systables objects by name (was
   reverting to id order when --dependency-order is specified.

   Revision 2.89  2020/11/25 12:28:33  art
   Updated usage for --no-blade-procs.

   Revision 2.88  2020/11/23 21:18:16  art
   Added new option --alter-checks to force check constraints to be 
   generated as ALTER TABLE statements rather than inline in the 
   CREATE TABLE statement. Similar to --no-check but does not 
   include the NOVALIDATE option.

   Revision 2.87  2020/11/20 12:20:37  art
   *** empty log message ***

   Revision 2.86  2020/09/02 18:06:17  art
   Fixed missing usage text.

   Revision 2.85  2020/05/07 14:34:40  art
   Added usage for --no-private-synonyms.

   Revision 2.84  2019/11/26 20:58:54  art
   Fixed a typo in the usage for --constraint-file

   Revision 2.83  2019/04/30 10:49:36  art
   Added documentation of --views-only

   Revision 2.82  2019/04/29 23:16:42  art
   Added new option, --synonym-file

   Revision 2.81  2019/04/23 18:25:49  art
   Documented the --no-proc-cleaning option.

   Revision 2.80  2018/11/07 15:23:44  art
   Added support for the new --rename-auto-index feature to generate RENAME
   INDEX statements for auto-named indexes (ie constraint indexes).

   Revision 2.79  2018/07/16 13:27:54  art
   Added the ability to specify a file containing the password either with
   the --passwd parameter or using the $PASSWD environment variable.

   Revision 2.78  2018/06/22 16:30:05  art
   New options: --use-database-owner, --use-table-owner, --constraint-file
   Fixed some problems with the partial schema file output.
   Expanded "owner"s to all objects.

   Revision 2.77  2017/05/24 15:39:17  art
   Added new options to separate objects into different files
   by type: --procedure-file, --view-file, --index-file,
   --trigger-file. Also documented the --grant-file alternative
   to the -U file option. Thanks to Paul Watson for triggering
   the idea.

   Revision 2.76  2016/12/13 23:51:19  art
   Added support for unencrypted storage spaces when encrypted storage is enabled (12.10.xC8+).
   Added new option --no-check to output CHECK constraints with the NOVALIDATE option for
   migrating databases (effective only on a populated table created without CHECK constraints.

   Revision 2.75  2016/11/28 00:37:38  art
   Added new option, --keep-generated-names, to create constraints with the names auto-generated
   in the existing schema. The default is to not explicitely name constraints that have generated
   names.

   Revision 2.74  2016/07/29 18:34:21  art
   Fixed compiler warning.

   Revision 2.73  2016/06/30 13:56:35  art
   Fixed formatting for 80 character terminals.

   Revision 2.72  2016/06/30 13:30:07  art
   Added description of --myexport-escape.

   Revision 2.71  2015/05/13 22:21:12  art
   Added new options --set-min-extent=# and --set-min-nextent=# to control
   the extent sizing for small or empty tables.  Thanks to Fernando Oliveira
   for the suggestion.

   Revision 2.70  2015/03/25 15:41:03  art
   Added new option, --aggregates, to print out user defined aggregations.

   Revision 2.69  2014/09/21 22:35:26  art
   New option --no-validate added. When specified all FOREIGN KEY constrataints
   will be generated with the NOVALIDATE clause included.

   Revision 2.68  2014/08/31 01:51:28  art
   Documented --myexport-delimiter and fixed another.

   Revision 2.67  2013/09/11 18:09:28  art
   Added new option, --dependency-order, to order the tables based on foreign key
   dependencies with dependent tables output after the tables that they reference.

   Revision 2.66  2013/07/12 19:46:30  art
   *** empty log message ***

   Revision 2.65  2013/07/12 19:45:06  art
   Enhanced usage.

   Revision 2.64  2013/07/11 13:57:48  art
   Fixed description of -e.

   Revision 2.63  2013/03/14 03:44:21  art
   Added new option --no-storage-options which suppresses "IN <dbspace>" and
   FRAGMENT BY clauses.  This can ease some migrations.

   Revision 2.62  2013/03/11 16:55:30  art
   Added new option -K to name autonamed indexes and unnamed constraints as <table>_pk
   for primary key indexes and <table>_<type>### where type is one of "uk", "fk", "ak"
   and ### is a sequential number within the type.
   Fixed -U.
   Enhanced -v to print progress after "Analyzing schema".
   reordered index printouts to group by the type of constraint each supports if any.

   Revision 2.61  2013/03/06 03:03:15  art
   Fixed problem with triggers with long comma lists with not white space causing problems
   when loading the schema.
   Added new feature --squish-triggers to eliminate unnecessary white space formatting
   to reduce problems loading the schema caused by IDS's trigger expansion making the
   trigger too big to load (> 65K).  This problem goes away in 12.10 anyway, but ...

   Revision 2.60  2012/03/12 00:33:41  art
   Documented --index-online

   Revision 2.59  2011/08/10 16:04:33  art
   Fixed description of --infrastructure.

   Revision 2.58  2011/07/01 21:13:08  art
   Added --drop-proc to usage.

   Revision 2.57  2011/06/21 21:49:56  art
   Added new feature --infrastructure to emulate dbschema -c.

   Revision 2.56  2011/05/31 15:03:57  art
   Fixed problem outputting STATLEVEL clauses under specific conditions.

   Revision 2.55  2011/04/29 13:35:51  art
   Fixed problems handling SERIAL8 type column initialization.
   Added support for STATLEVEL and STATCHANGE in CREATE TABLE statements.
   Added support for SECURITY POLICY in CREATE TABLE statements.  Output of policy
   definitions still not supported.
   Added option --force-autostats to force the output of STATLEVEL AUTO which is
   the default and so not printed normally.

   Revision 2.54  2011/04/06 03:32:55  art
   Added new feature --no-blade-funcs/--no-blade-procs to eliminate stored
   routines associated with IBM Datablades from the output.

   Revision 2.53  2010/12/09 20:20:42  art
   Added new feature --reorg-api to use the new Admin API "defragment" function
   to reorganize selected tables.

   Revision 2.52  2010/12/09 18:35:36  art
   Added option --myexport-express.  The --myexport-scripts now generates a DELUXE mode
   import/load script, the new option creates the external tables in EXPRESS mode.

   Revision 2.51  2010/11/17 19:34:29  art
   Updated usage.

   Revision 2.50  2010/11/15 19:27:04  art
   Updated usage for new options:
   --create-conditionally
   --suppress-directives
   --directives-only

   Revision 2.49  2010/08/25 13:52:14  art
   Fixed several compile time warnings.  Thanks to John Adamski for identifying them.

   Revision 2.48  2010/06/10 03:50:51  art
   Added new option --wrap-at to specify or disable the wrap column for
   triggers.
   Fixed problem with breaking quoted strings that were not preceded by
   a space when wrapping.

   Revision 2.47  2010/05/04 23:23:32  art
   Fixed sequence output for IDS 10.00 and 11.10 (oops).
   Implemented new option --sequence, --sequence=matches-spec to print only sequence
   definitions.

   Revision 2.46  2010/03/04 14:49:37  art
   typo

   Revision 2.45  2010/03/04 14:45:27  art
   Documented --no-ags-procs

   Revision 2.44  2010/02/25 19:38:11  art
   Added text noted incompatibility between -O and --set-owner.

   Revision 2.43  2010/01/06 13:58:59  art
   Added new option --set-owner to override all object owners with a specified owner name.

   Revision 2.42  2010/01/06 13:55:18  art
   Added new option --set-owner to override all object owners with a specified owner name.

   History cleanup - 2023-12-11
*/

static const char RCSHeader[] = "@(#)$Header: /home/art/GoogleDrive/Latest\040Art's\040Stuff/myschema.d/fork/RCS/usage.c,v 2.103 2024/05/01 15:53:22 art Exp $";
static const char RCSWhat[] = "$What: <@(#) usage.c,v	 2.31 > $ $Date: 2024/05/01 15:53:22 $";
static const char RCSCompile[] = "@(#)$cc: " __FILE__ " " __DATE__ " " __TIME__ " $";

#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include <sys/types.h>
#include <string.h>
#include <limits.h>
#include <ctype.h>

void usage( char *program )
{
    char *server, *MySchema;
    char *pager;
    FILE *errout;
    int  redirected = 0;
 
    errout = fdopen( 2, "w" );
    if (isatty(2)) {
        /* fclose( errout ); -- UNIX more needs stderr open, so... */
        if (!(pager = getenv( "PAGER" )) || strlen(pager)==0)
            pager = "more";
        errout = popen( pager, "w" );
        redirected = 1;
    }
    MySchema = getenv( "MYSCHEMA" );

    fprintf(errout,
	    "Usage:\t%s -d dbase [-t table] [-f proc] -[5ACDEFILORSTVacklmoqrsuv] \n"
	    "\t     [-n nAdj] [-e eAdj] [-h host] [-U distfile] [-i serval] \n"
	    "\t     [[-N option]...] [-g authfile] [primary-file] [secondary-file]\n"
	    "\t     [-M ExtS] [--<long options>] \n"
	    "\t%s --infrastructure[=<sql|cmd>]\n\n",
	    program,
	    program
	    );
    fprintf( errout,
	     "\t%s --help\n",
	     program );
    fprintf( errout,
	     "By default myschema writes to stdout. One or two filenames may optionally be \n"
	     "given at the end of the command line. These are the primary and secodary \n"
	     "files. Output can be split so that different types of objects are written to \n"
	     "different files. See the notes at the end of this usage. The primary and \n"
	     "secondary files may also be specified using the long optons --primary=file and \n"
	     "--secondary=file respectively, as documented below.\n\n" );

    fprintf( errout,
	     "\t-d - Report on named database.  Required flag and option unless \n"
	     "\t     --infrastructure is passed.  Longopt: --database\n" );
    fprintf( errout,
	     "\t-q - Toggle quiet mode.  Eliminates extraneous output.  Needed to pipe\n"
	     "\t     output to dbaccess/isql/sqlcmd. (Default: print status)\n" );
    fprintf( errout,
	     "\t-5 - Toggle use of OL5.xx conventions.  No SMI, dummy dbspacenames,\n"
	     "\t     and 5.xx style connections.\n" );
    fprintf( errout,
	     "\t-k - Toggle creation of named indexes to replace hidden constraint \n"
	     "\t     indexes.  (Default: Create named indexes)\n" );
    fprintf( errout,
	     "\t-K - Turn on long named indexes and constraints when unnamed. (Default:\n"
	     "\t     use short names.\n" );
    fprintf( errout,
	     "\t-S - Toggle use of SE conventions.  No SMI, file paths rather than\n"
	     "\t     dbspaces, no extents, no lock mode, no fragments and 8 index keys.\n" );
    fprintf( errout,
	     "\t-a - Toggle actual page count calculations for EXTENT SIZE.  (Default: \n"
	     "\t     Set EXTENT SIZE to current value unless -m or -M is specified.)\n"
	     "\t     Also effects SERIAL number initialization as noted below.\n" );
    fprintf( errout,
	     "\t-I - Toggle print/noprint initial serial value. (Default: Print value.)\n"
	     "\t     If -a specified also report stored value in comments when not \n"
	     "\t     printing (SERIAL only).  Affects SERIAL and SERIAL8 columns.\n" );
    fprintf( errout,
	     "\t-i - Print serval as the SERIAL type initializer.  If -a specified \n"
	     "\t     report stored value in comments (SERIAL only).  Affects SERIAL \n"
	     "\t     and SERIAL8 columns. \n" );
    fprintf( errout,
	     "\t-c - Toggle using the stored -versus- actual next value for serial \n"
	     "\t     columns.  (Default: Use actual next value ie: MAX(serialcol)+1.)\n"
	     "\t     If -a specified also report stored value in comments when using \n"
	     "\t     actual.  Affects only SERIAL columns.\n" );
    fprintf( errout,
	     "\t-l - Toggle generation of a DBIMPORT compatible script.\n"
	     "\t     (Also forces -s - table fact summary output.) (Default: disabled)\n" );
    fprintf( errout,
	     "\t-m - Toggle automated extent minimization.  If on this causes myschema\n"
	     "\t     to use the actual pages plus eadj if specified instead of the \n"
	     "\t     current size for the initial extent.  Forces -a.\n"
	     "\t     (Default: Use current initial extent size as base.)\n" );
    fprintf( errout,
	     "\t-M - Strategy for determining extent sizing for fragmented tables. \n"
	     "\t     Where ExtS is one of: min, avg, max to select the actual size of \n"
	     "\t     the smallest, average, largest fragment for calculating extent \n"
	     "\t     sizes.  (Default: max).\n" );
    fprintf( errout,
	     "\t-r - Output only ALTER TABLE and ALTER FRAGMENT statements needed to \n"
	     "\t     reorganize a table(s).  Not compatible with -S, -5, -f, or -F.\n"
	     "\t     (Default: disabled); \n" );
    fprintf( errout,
	     "\t-s - Toggle output of the table facts summary dbschema includes.\n"
	     "\t     (Default: Disabled unless -l DBIMPORT compatibility specified.)\n" );
    fprintf( errout,
	     "\t-g - Write user level privileges to 'authfile'. (Default: filename2.)\n"
	     "\t     Longopt: --grant-file.\n" );
    fprintf( errout,
	     "\t-u - Replicate current levels of UPDATE STATISTICS. (Default: disabled)\n" );
    fprintf( errout,
	     "\t-U - Write distributions to distfile.  Longopt: --distributions-file\n"
	     "\t     Implies -u.  (Default filename2.)\n" );
    fprintf( errout,
	     "\t-v - Toggle verbose output.  Tell me what's happening.\n"
	     "\t     (Default: No verbose.)\n" );
    fprintf( errout,
	     "\t-V - Print source and features versions and exit.\n" );
    fprintf( errout,
	     "\t-A - Toggle suppression of 'AS owner' clause in GRANT statements.\n" 
	     "\t     (Default: Do not suppress.)\n" );
    fprintf( errout,
	     "\t-L - Toggle suppression of START VIOLATIONS statements.\n" 
	     "\t     (Default: Do not suppress.)\n" );
    fprintf( errout,
	     "\t-O - Toggle suppression of owner names, sets -A to suppress.\n"
	     "\t     (Default: Do not suppress.) Not compatible with --set-owner.\n" );
    fprintf( errout,
	     "\t-F - Toggle generation of FOREIGN KEY constraints and views referencing \n"
	     "\t     the table specified.  Ignored if -t table is not given. (Default: \n"
	     "\t     disabled)\n" );
    fprintf( errout,
	     "\t-E - Toggle SET EXPLAIN ON.  Useful to emulate what %s is doing \n"
	     "\t     and for diagnostic purposes. (Default: disabled)\n",
	     program );
    fprintf( errout,
	     "\t-o - Toggle output ordering of objects in the schema between tabid \n"
	     "\t     order and object name order.  To generate a file compatible with \n"
	     "\t     the mydbdiff script use object name ordering.  For database \n"
	     "\t     replication tabid ordering is preferred.  (Default: tabid order).\n" );
    fprintf( errout,
	     "\t     Note: -o can be used with --dependency-order to order all objects\n"
	     "\t     except tables by name while tables are order by name within \n"
	     "\t     dependencies.\n" );
    fprintf( errout,
	     "\t-C - Toggle suppression of Column level authorizations.\n" );
    fprintf( errout, "\t    (Default: Do not suppress.)\n" );
    fprintf( errout, "\t-R - Toggle suppression of REVOKE ALL statements.\n" );
    fprintf( errout, "\t    (Default: Do not suppress.)\n" );
    fprintf( errout,
	     "\t-T - Toggle suppression of Table level authorizations.\n" );
    fprintf( errout, "\t    (Default: Do not suppress.)\n" );
    fprintf( errout,
	     "\t-e - Set EXTENT SIZE to eAdj %% of EXTENT SIZE \n"
	     "\t     (Range: 0 <= eAdj <= 10000).\n"
	     "\t     (Default: 100 ==> Use calculated EXTENT SIZE - affected by -m)\n" );
    fprintf( errout,
	     "\t-n - Set NEXT SIZE to nAdj %% of adjusted EXTENT SIZE - affected by -e.\n"
	     "\t     (Range: 0 <= nAdj <= 10000).\n"
	     "\t     (Default: 0 ==> Use current NEXT SIZE)\n" );
    server = getenv( "INFORMIXSERVER" );
    fprintf( errout,
	     "\t-h - Report on schema at named host.  Longopt: --host\n"
	     "\t     Host defaults to \"%s\".\n",
	     server ? server : "default server" );
    fprintf( errout,
	     "\t-t - Report for matching tables/views/synonyms.  Longopt: --table\n"
	     "\t     Table may contain valid \"matches\" clause wildcards.\n" );
    fprintf( errout,
	     "\t    'Table' defaults to all user tables (no system catalog tables)\n"
	     "\t    if absent.  System tables will be reported if explicitly named.\n" );
    fprintf( errout,
	     "\t-f - Report for matching procedures. \n"
	     "\t    'Proc' may contain valid \"matches\" clause wildcards. (Default:\n"
	     "\t    Enabled for all procedures if no -t is passed. Disabled otherwise)\n" );
    fprintf( errout,
	     "\t-D - Report for User Defined Types only.  Cannot be used with -t.\n"
	     "\t     (Default: Enabled if no -t is passed.  Disabled otherwise.)\n" );
    fprintf( errout,
	     "\t-N - This is a new multipurpose flag for most new options.  Available \n"
	     "\t     options are:\n"
	     "\t   notypes - Do not include User Defined Types in schema, table, or \n"
	     "\t             procedure output. (Default: disabled)\n"
	     "\t   nocasts - Do not include User Defined Type Cast definitions in \n"
	     "\t             schema, table, or procedure output. (Default: disabled)\n"
	     "\t     Separate -N flags are used to specify multiple options.\n"
	     "\t     Longopt: --newopt\n" );
    fprintf( errout,
	     "Other long options:\n" );
    fprintf( errout,
	     "    --infrastructure - Print commands needed to duplicate an instance's\n"
	     "\tstorage and logging configuration.  Optionally output command line \n"
	     "\tutility lines (=cmd) or SQL API commands (=sql).  (Default: =cmd)\n" );
    fprintf( errout,
	     "    --sequence[=name-matches-spec] - Only print out sequence definitions.\n"
	     "\tThe name spec is optional and uses MATCHES wildcards. Implements \n"
	     "\tthe dbschema -seq option. To avoid confusion use '=' when including\n"
	     "\ta sequence name specification.  (Default spec: '*')\n" );
    fprintf( errout,
	     "    --no-sequence-init - Suppress setting the initial value for sequences.\n"
	     "\tThis option also avoids incrementing the current sequence value.\n" );
    fprintf( errout,
	     "    --wrap-at <column> - Affects line wrapping of trigger code.  Setting\n"
	     "\tcolumn to -1 disables the wrapping and causes myschema to print the\n"
	     "\ttrigger as it is currently wrapped. (Default: 80, Minimum: 40)\n" );
    fprintf( errout,
	     "    --squish-trigger - Removes some unneccesary spaces from trigger \n"
	     "\tdefinitions to prevent triggers that are nearly too big to create \n"
	     "\tfrom getting oversize errors when executing the schema.\n" );
    fprintf( errout,
	     "    --include-sysprocs - Generate system procedures normally ignored.\n" );
    fprintf( errout,
	     "    --no-ags-procs - Ignore any routines added by AGS's Server Studio needed \n"
	     "\tby SS to debug a routine.  Warning: All procedures with names \n"
	     "\tbeginning with \"deb_\" will be ignored and not output.\n" );
    fprintf( errout,
	     "    --no-private-synonyms - Exclude private synonyms from the output.\n" );
    fprintf( errout,
	     "    --no-proc-cleaning - Output the stored routine definitions without editing\n"
	     "\tthe text. Helps with some routines created from a file or from a string\n"
	     "\tin a host language that may have comments in odd placed. These can \n"
	     "\tconfuse the parser that does the cleaning.\n" );
    fprintf( errout,
	     "    --set-owner newowner - Override object owners with \"newowner\".\n"
	     "\tSupercedes -O with a warning if both are specified.\n" );
    fprintf( errout,
	     "    --use-database-owner - Override object owners with the owner of the \n"
	     "\tdatabase in which they reside.\n" );
    fprintf( errout,
	     "    --use-table-owner - Override the owner of dependent objects to be the same \n"
	     "\tas their parent table.\n" );
    fprintf( errout,
	     "    --simple-fragments - Disable named partitions where implicitely named.\n" );
    fprintf( errout,
	     "    --no-extent-clause - Disable printing of EXTENT SIZE and NEXT SIZE clauses.\n" );
    fprintf( errout,
	     "    --no-storage-options - Disable printing of \"IN <dbs>\" and FRAGMENT BY \n"
	     "\tclauses.\n" );
    fprintf( errout,
	     "    --dependency-order - Order tables so that dependent tables follow their \n"
	     "\tparents. Requires Informix version 11.50.xC5 or later with hierarchical\n"
	     "\tquery extensions. This option can be combined with -o to order tables by\n"
	     "\tname within their dependency level.\n" );
    fprintf( errout,
	     "    --myexport-order - like --dependency-order except that child tables precede\n"
	     "\ttheir parents so that you don't unload children that reference parents that\n"
	     "\tdidn't yet exist when the parent was unloaded. Used by myexport.\n" );
    fprintf( errout,
	     "    --primary=filename - Specify primary output file.\n" );
    fprintf( errout,
	     "    --secondary=filename - Specify secondary output file.\n" );
    fprintf( errout,
	     "    --index-file=filename - Specify an output file for indexes.\n" );
    fprintf( errout,
	     "    --procedure-file=filename - Specify an output file for stored procedures \n"
	     "\tand functions.\n" );
    fprintf( errout,
	     "    --view-file=filename - Specify an output file for views.\n" );
    fprintf( errout,
	     "    --synonym-file=filename - Specify an output file for synonyms.\n" );
    fprintf( errout,
	     "    --trigger-file=filename - Specify an output file for triggers.\n" );
    fprintf( errout,
	     "    --grant-file=filename - Specify an output file for grant and revoke statements.\n" );
    fprintf( errout,
	     "    --distributions-file=filename - Specify an output file for update statistics\n"
	     "\tcommands. Implies -u.\n" );
    fprintf( errout,
	     "    --constraint-file=filename - Specify an output file for constraints.\n" );
    fprintf( errout,
	     "    --filtering - generate code to  start violations and set\n"
	     "\tconstraints, indexes to filtering for myimport.\n" );
    fprintf( errout,
	     "    --create-conditionally - Add 'IF NOT EXISTS' clauses to all CREATE \n"
	     "\tstatements.\n" );
    fprintf( errout,
	     "    --suppress-directives - Do not print SAVE EXTERNAL DIRECTIVE statements.\n" );
    fprintf( errout,
	     "    --directives-only - Only print any SAVE EXTERNAL DIRECTIVE statements \n" 
	     "\tdefined in the database.\n" );
    fprintf( errout,
	     "    --views-only - Only print Views that match the -t pattern if given or all\n"
	     "\tviews if not given. Related triggers are included.\n" );
    fprintf( errout,
	     "    --myexport-scripts - Create load and unload scripts for the database using\n"
	     "\texternal tables. Scripts will named <database>.<table>.myexport.sql and \n"
	     "\t<database>.<table>.myimport.sql and will be placed in the path \n"
	     "\tspecified in the environment variable MYEXPORT_PATH (Default: current \n"
	     "\tdirectory). The scripts place the external table files in the same \n"
	     "\tdirectory as the scripts.  This option is only active for IDS version\n"
	     "\t11.50.xC6 and later.\n" );
    fprintf( errout,
	     "    --myexport-express - Same as --myexport-scripts but the load script will\n"
	     "\tcreate the external table to load data using EXPRESS mode rather than \n"
	     "\tDELUXE mode.  A level 0 archive must be taken after the load in order to\n"
	     "\tplace the loaded tables back into read/write mode.\n" );
    fprintf( errout,
	     "    --myexport-binary - Sets the unload data type to INFORMIX binary format. \n"
	     "\tExpress mode is assumed and myexport-delimiter and myexport-escape are \n"
	     "\tignored. Pass myexport-scripts after myexport-binary to enable DELUXE mode.\n" );
    fprintf( errout,
	     "    --myexport-delimiter - Sets the delimiter used in the dbexport header line\n"
	     "\tand in the myexport/myimport external table scripts.  (Default: '|').\n"
	     "\tAlternates: --export-delimiter, --import-delimiter\n" );
    fprintf( errout,
	     "    --myexport-escape - Sets the escape character used in the myexport and \n"
	     "\tmyimport scripts. (Default: \\) Note that v11.50 does not support \n"
	     "\talternate escape characters or the SET ENVIRONMENT variable needed to \n"
	     "\tspecify it. Alternates: --export-escape, --import-escape\n" );
    fprintf( errout,
	     "    --export-scripts - Only generate external table export scripts. Filenames\n"
	     "\tof the unload files will contain the full filename. \n" );
    fprintf( errout,
	     "    --import-scripts - Only generate external table import scripts. Filenames\n"
	     "\tof the unload files will contain the full filename.\n" );
    fprintf( errout,
	     "    --export-binary - Only generate INFORMIX format external table export \n"
	     "\tscripts. Filenames of the unload files will contain the full filename. \n" );
    fprintf( errout,
	     "    --import-binary - Only generate INFORMIX format external table import \n"
	     "\tscripts. Filenames of the unload files will contain the full filename.\n"
	     "\toptions.\n" );
    fprintf( errout,
	     "    --no-export-triggers - Disable select triggers in the external table \n"
	     "\texport scripts.\n" );
    fprintf( errout,
	     "    --no-data-tables - Do not produce export scripts for listed tables. If no\n"
	     "\ttables are listed the tsinstancetable table is auto-included. The optional\n"
	     "\targument is a comma separated list of table names. Use with export scripts\n"
	     "\toptions.\n" );
    fprintf( errout,
	     "    --no-data-access-method - Do not produce export scripts for tables created\n"
	     "\twith the listed access method names. If no access methods are listed the\n"
	     "\tts_rtl_vtam and ts_vtam access methods are auto-included. The optional\n"
	     "\targument is a comma separated list of access method names.\n" );
    fprintf( errout,
	     "    --for-er - Add WITH REPLCHECK and WITH ERKEY to each table and create the\n"
	     "\tUNIQUE index on the primary key plus the ifx_replcheck column if a primary\n"
	     "\tkey exists (this index is not created if the table has no primary key\n"
	     "\tconstraint defined). Cannot be used with --no-er-columns.\n" );
    fprintf( errout,
	     "    --no-er-columns - Do not include WITH <special columns> options in CREATE TABLE\n"
	     "\tCannot be used with --for-er.\n" );
    fprintf( errout,
	     "    --suppress-audit - Suppress printing of WITH AUDIT for audited tables.\n" );
    fprintf( errout,
	     "    --no-setsessionauth - Suppress SETSESSIONAUTH privilege generation. \n" );
    fprintf( errout,
	     "    --comment-setsessionauth - Include SETSESSIONAUTH privileges as comments. \n" );
    fprintf( errout,
	     "    --reorg-api - Generate SQL Administration API commands to defragment all\n"
	     "\tselected tables.\n" );
    fprintf( errout,
	     "    --no-blade-procs - Eliminates procedure definitions associated with external\n"
	     "\tfunctions stored under $INFORMIXDIR/extend/*. (Alias: --no-blade-funcs)\n" );
    fprintf( errout,
	     "\tAlso eliminates other objects created by blade management routines when\n"
	     "\tany blade is registered.\n" );
    fprintf( errout,
	     "    --no-validate - Generate all FOREIGN KEY constraints with the NOVALIDATE\n"
	     "\toption to avoid checking for valid parent rows.  Primarily for data \n"
	     "\tloads of prevalidated data.\n" );
    fprintf( errout,
	     "    --alter-checks - Write all check constraints as ALTER statements rather \n"
	     "\tthan as inline in the CREATE TABLE statement.\n" );
    fprintf( errout,
	     "    --no-check - Generate all CHECK costraints with the NOVALIDATE option as \n"
	     "\tseparate constraints. Most useful when adding constraints after loading\n"
	     "\tdata. \n" );
    fprintf( errout,
	     "    --index-online - Include the 'ONLINE' clause when generating CREATE INDEX\n"
	     "\tstatements.\n" );
    fprintf( errout,
	     "    --keep-generated-names - Keeps generated constraint names. <Default: no \n"
	     "\texplicit name when constraints have generated names.\n" );
    fprintf( errout,
	     "    --force-autostats - Forces the output of 'STATLEVEL AUTO' if that is the \n"
	     "\ttable's STATLEVEL setting.  This is the default setting.\n" );
    fprintf( errout,
	     "    --drop-proc - Generate DROP PROCEDURE/FUNCTION statements to preceed the \n"
	     "\tCREATE statements.  Needed after some upgrades.\n" );
    fprintf( errout,
	     "    --replace-proc - Generate procedure/function statements as CREATE OR \n"
	     "\tREPLACE.\n" );
    fprintf( errout,
	     "    --replace-views - Generate view statements as CREATE OR REPLACE.\n" );
    fprintf( errout,
	     "    --replace-triggers - Generate trigger statements as CREATE OR REPLACE.\n" );
    fprintf( errout,
	     "    --rename-auto-index=<pattern> - Generate RENAME statements for auto-\n"
	     "\tgenerated indexes. Requires a MATCHES pattern to specify tables to \n"
	     "\tprocess.\n" );
    fprintf( errout,
	     "    --user username - Connect as 'username'.\n" );
    fprintf( errout,
	     "    --passwd password - Connect using 'password'.\n" );
    fprintf( errout,
	     "\tIf one of --user or --passwd is specified without the other then an\n"
	     "\tenvironment variable is used to satisfy the  other if available.  \n"
	     "\tIf neither is present connection is attempted as a trusted user.  \n"
	     "\tUsername is extracted from $USER if available otherwise from $LOGNAME.\n"
	     "\tPassword is extracted the environment variable $PASSWD if available.\n"
	     "\tIf only one of --user or --passwd and the appropriate environment is\n"
	     "\tnot set an error is returned. If the provided password begins with an\n"
	     "\tan '@' character the following string is treated as a file path \n"
	     "\tcontaining a password on the first line. The file must be owned by the\n"
	     "\tuser executing myschema, be a regular file, and have 0600 privileges.\n" );
    fprintf( errout,
	     "    --aggregates - Print out user defined aggregates.\n" );
    fprintf( errout,
	     "    --set-min-extent - Set the minimum EXTENT SIZE setting for empty or small \n"
	     "\ttables.  NEXT SIZE is not affected unless -n is passed.\n" );
    fprintf( errout,
	     "    --set-min-nextent - Set the minimum NEXT SIZE setting for empty or small \n"
	     "\ttables.  EXTENT SIZE is not affected. The effects of -n may alter this.\n" );
    fprintf( errout,
	     "\nFor compatibility with dbschema the keyword \"all\" is accepted as a synonym for \n"
	     "\"*\" for table and procedure templates.\n\n" );
    fprintf( errout,
	     "Any or all of the flags: ACEFILORST5acklmsquv may be included in the environment\n"
	     "variable MYSCHEMA to reverse the defaults for these flags.  These flags are now\n"
	     "toggles so specifying one in MYSCHEMA and also on the command line has the\n"
	     "effect of restoring the original default.  Also naming any of these switches\n"
	     "more than once will have a net effect depending on the number of times the\n"
	     "switch is toggled.\n\n"
	);
    fprintf( errout,
	     "If a secondary file is named on the command line or using --secondary then the primary\n"
	     "file only holds cast, role, type, type privilege, sequence, and table\n"
	     "definitions. The secondary file holds all other statements not assigned\n"
	     "to separate files by one of the --*-file options: --procedure-file,\n"
	     "--index-file, --view-file, --trigger-file, --grant-file, --distributions-file.\n" );

    if (MySchema && strlen( MySchema ) > 0)
	fprintf( errout,
		 "Your current setting is: MYSCHEMA=\"%s\"\n\n",
		 MySchema );
    else
	fprintf( errout, "\nMYSCHEMA is unset or empty.\n" );
    fprintf( errout,
	     "If the environment variable FILLFACTOR exists its value will be"
	     " used to\ngenerate FILLFACTOR clauses for all indexes.\n\n"
	);

    if (redirected) {
	fflush( errout );
	pclose( errout );
    }
}

