/*  
 *   #define NOTIMINGS 1
 * Enable the #define above to eliminate references to UNIX time functions and
 * code
 *
 */
/*
  Program Id: dostats_ng

  Program Description: Next generation version of the program that generates sql
  to update statistics on a table or database;

  Author: Art S. Kagel 

  Creation Date: May 16, 1996

  Version: $Id: dostats_ng.ec,v 1.81 2023/11/24 13:51:54 art Exp $

  Usage:    dostats [-h hostname] -d databasename [-t tablename] [-f filename]
  See usage function for current usage.

  RCS Header
  ----------
  $Header: /home/art/GoogleDrive/Latest/RCS/dostats_ng.ec,v 1.81 2023/11/24 13:51:54 art Exp $

  RCS Log
  -------
  $Log: dostats_ng.ec,v $
  Revision 1.81  2023/11/24 13:51:54  art
  Added version & copyright to the verbose 2+ output.
  Thanks to Chris Karsten for the suggestion.

  Revision 1.80  2023/10/04 17:43:43  art
  Added some diagnostics.

  Revision 1.79  2023/01/18 18:36:31  art
  Updated the Usage message to properly document --include-owner
  That option affects both tables and procedures/functions. The
  original text only mentioned procedures.

  Revision 1.78  2022/07/10 17:50:27  art
  Typo in a comment.

  Revision 1.77  2021/12/21 17:12:40  art
  Updated Copyright notice.

  Revision 1.76  2021/12/21 16:43:08  art
  Fixed elapsed time accuracy.
  Added new option --display-subsec to display subsecond start 
  and end times in microseconds and expand elapsed time to 
  also display in microseconds. Enables --time-display, but,
  also works with --display-local. Made new opton --display-time
  a synony for --time-display for consistency with the other 
  time display options.

  Revision 1.75  2021/12/16 18:03:32  art
  Fixed progress output for LOW stats when writing stats to a file.

  Revision 1.74  2021/12/16 17:48:48  art
  Improved the new sort a bit.

  Revision 1.73  2021/12/16 17:21:19  art
  Fixed problem with some tables/indexes where different column
  of indexes starting with same column were not processed high.
  Thanks to David Williams for pointing out the issue.

  Revision 1.72  2020/11/11 17:42:10  art
  Added support for specifying procedure names in -i and -x with 
  the '>' character introducing the name. Thanks to Chris Karsten
  for the idea.

  Revision 1.71  2020/02/03 23:09:50  art
  Fixed problem acquiring counts for virtual tables. VTI tables are
  no longer processed.

  Revision 1.70  2019/09/11 00:58:32  art
  Fixed time display.

  Revision 1.69  2019/02/06 20:11:29  art
  Typo.

  Revision 1.68  2019/02/06 20:08:01  art
  Improved some error handling.

  Revision 1.67  2019/02/06 16:19:18  art
  Fixed problem with handling routines that have OUT and INOUT parameters.
  Thanks to Richard Fahey for pressing this issue.

  Revision 1.66  2019/01/10 23:17:04  art
  Fixed problems with long hostnames and unusual parameter types in handling
  stored procedures. Thanks to Richard Fahey for pointing these out and
  suggesting fixes.

  Revision 1.65  2018/12/28 20:09:52  art
  Expanded hostname to accept 32 characters (was 18). Thanks to Rick Fahey
  for catching this old quirk.

  Revision 1.64  2018/04/17 19:43:25  art
  Fixed handling of timeseries parameters to procedures.

  Revision 1.63  2018/04/17 18:59:19  art
  Removed debug.

  Revision 1.62  2018/04/17 18:55:26  art
  Fixed processing of procedure args that are extended types.

  Revision 1.61  2017/05/22 16:29:55  art
  Added verbose message about isolation levels ignored for non-logged databases.

  Revision 1.60  2017/05/22 16:23:23  art
  Fixed so no ISOLATION setting in non-logged databases.

  Revision 1.59  2017/04/20 19:59:12  art
  Modified readlist() to strip carriage returns from Windows files for -i & -e.
  Thanks to Randy Kennedy for illuminating this need.

  Revision 1.58  2017/02/26 19:10:58  art
  Fixed -Z to accept percentages < 1.0 for sampling by a percent of rows in addition to min row count.

  Revision 1.57  2016/12/14 00:35:06  art
  Fixed usage.

  Revision 1.56  2016/12/13 20:12:05  art
  Fixed issue with UTF8 locale databases.

  Revision 1.55  2016/12/12 21:35:38  art
  Kludge to ignore procedures named 'null'.

  Revision 1.54  2016/09/26 21:18:37  art
  Added new option --include-owner to get past weird procedure names that are a keyword.

  Revision 1.53  2016/06/29 03:15:40  art
  Fixed small table handling. Didn't like the way it was working.

  Revision 1.52  2016/06/27 21:08:14  art
  Added new options: --small-tables-high and --small-tables-threshold to
  process tables with few rows using HIGH distributions only. The size
  threshold is set by --small-tables-threshold or by --aus-thresholds and
  the AUS_SMALL_TABLES record in sysadmin.ph_thresholds.

  Revision 1.51  2016/06/27 19:30:09  art
  Replaced sysactptnhdr with sysptnhdr. Turns out sysactptnhdr is not reliable as
  it only contains tables that are active at the time it is queried. Also updated
  the copyright notice.

  Revision 1.50  2015/09/30 16:41:47  art
  Added verbose tracking of the --distributions-high option.

  Revision 1.49  2015/09/30 16:21:22  art
  Added new option: --distributions-high. Allows the user to provide a
  list of columns that must have HIGH distributions.  This overrides
  the default distributions for those columns.

  Revision 1.48  2015/03/26 13:46:59  art
  Minor enhancements around --aus-thresholds.  Added verbose display for these
  settings.

  Revision 1.47  2015/03/26 13:04:19  art
  FIxed problem using the AUS threshold values.
  Thanks to Javier Cortes Vila for spotting this one.

  Revision 1.46  2014/11/03 23:37:06  art
  Fixed the problem with handling VERY long procedure argument lists.

  Revision 1.45  2014/10/23 20:44:53  art
  Fixed problem handling VERY large procedure argument lists (>2000 bytes).  Was depending on esql to allocate
  the string to hold the lvarchar list but it was only allocating up to 2K.  Now doint the allocation myself
  using ifx_var_alloc( &var, 65560 ) so it should now hold a longer list than it is actually possible to create.

  Revision 1.44  2014/10/02 00:15:17  art
  Modified code to ignore BYTE, TEXT, BLOB, & CLOB type columns since the engine
  does not calculate distributions for these types anyway (except when they are
  embedded in a UDT that defines it's own function for calculating distributions
  but then the coltypes and/or xtdtype will be different from these).

  Revision 1.43  2014/08/01 01:21:55  art
  Updated copyright notice.

  Revision 1.42  2014/08/01 01:19:42  art
  fixed usage.

  Revision 1.41  2014/08/01 01:14:21  art
  Added new option, --display-local, to display the start time of each step
  in local timezone instead of the UTC produced by --time-display.
  Thanks to Hugo Tellez for the suggestion.

  Revision 1.40  2014/06/27 16:48:48  art
  Added new option --execute-local to set the tk_dbs field in ph_task so that
  the task executes connected to the target database.  Implies --proc-local.

  Revision 1.39  2013/12/24 18:13:01  art
  Fixed a typo.

  Revision 1.38  2013/12/24 17:12:57  art
  Fixed compile time warnings on HPUX.  Updated copyright notice.

  Revision 1.37  2013/11/26 20:23:52  art
  Fixed usage help display format problem.

  Revision 1.36  2013/07/10 19:47:57  art
  Fixed typo.

  Revision 1.35  2013/07/10 16:50:03  art
  Changed --drop-distributions and --clean-distributions to use LOW DROP DISTRIBUTIONS ONLY
  instead for v11+

  Revision 1.34  2013/05/01 15:40:49  art
  Changed --aus-threshold to override the global AUS setting for browsing with the table's statchange
  value if -b is specified as well.

  Revision 1.33  2013/02/10 22:18:19  art
  Updated Copyright

  Revision 1.32  2012/08/09 20:17:19  art
  Expanded some strings to eliminate overflows.

  Revision 1.31  2012/08/09 20:05:21  art
  Fixed LOW stats progress trace to separate column names with commas properly.

  Revision 1.30  2012/03/26 22:39:50  art
  Added new option --error-count to return the number of errors encountered
  if -n is specified.  -n now behaves as it originally did without this
  flag and returns zero if the number of errors was not exceeded.

  Revision 1.29  2012/03/25 19:09:32  art
  If -n is specified, now returns the number of errors encountered to the
  point of exit.  If that equals the value passed to -n, then the run did
  not complete successfully.

  Revision 1.28  2012/03/09 14:30:45  art
  Improved error output of the a1 cursor fails.

  Revision 1.27  2012/01/24 20:38:29  art
  Fixed bugs in SET ISOLATION when executing immediately.
  Fxed a bug in --drop-distributions.
  Added --clean-distributions for cleaning out old distributions when
  only specific tables are being processed.  Can be used globally, but
  --drop-distributions will tend to be faster.
  Restricted --drop-distributions from being used with -t, -x or -i
  since it would drop distributions from all tables and then only
  rebuild the distributions for the specified tables.
  Thanks to Ashok Sharma for pointing out these bugs.

  Revision 1.26  2011/12/08 16:40:59  art
  Fixed to output SET ISOLATION commands to the command file or procedure if not executing directly.

  Revision 1.25  2011/12/05 20:28:47  art
  Fixed problem with having an old v7.xx procedure, systdistold, lying around in later
  releases due to upgrading in-place.  This had been fixed long ago in one code path,
  but missed the other.  Got it now.

  Revision 1.24  2011/11/17 20:33:53  art
  Removed extraneous COMMIT WORK statements for ANSI mode databases.  Fixed a missing CLOSE <cursor>
  when a loop made a shortcut exit.

  Revision 1.23  2011/11/17 16:41:23  art
  Fixed problem in the manual row counting function.

  Revision 1.22  2011/09/01 22:40:48  art
  Removed redundant printout.

  Revision 1.21  2011/09/01 22:39:14  art
  Fixed -v<num>.  -v with no args deprecated.

  Revision 1.20  2011/09/01 22:30:44  art
  Trace somehow went missing.  Fixed.

  Revision 1.19  2011/09/01 22:13:59  art
  Updated usage.  Restricted -x"!SELECT..." since -i can be used be
  negating the filters in the query.  That simplifies the code and
  permits processing the query as a derived table.
  Added printing the full query that processes tables to verbose=3+ output.

  Revision 1.18  2011/09/01 14:48:36  art
  Fixed -i '!<query>' processing to allow ORDER BY clauses in the query for version
  11 and higher that support derived tables.

  Revision 1.17  2011/08/03 17:48:35  art
  Fixed bug when running against an ANSI mode database.  Periods missing from
  some statements between the owner and object names.

  Revision 1.16  2011/06/24 17:18:21  art
  cosmetics.

  Revision 1.15  2011/06/24 17:15:14  art
  Made --procedure=procname mandatory when using --proc-local

  Revision 1.14  2011/06/24 16:45:52  art
  Added new feature --proc_local.  Creates the table of commands for the procedure to execute in
  each database rather than in sysadmin.  This is to support earlier Informix versions that do
  not have a task manager (so no sysadmin) and to support non-logged databases.

  Revision 1.13  2011/05/23 16:04:57  art
  Updated the copyright statement.

  Revision 1.12  2011/05/13 14:25:10  art
  Added new options --force-run and --auto-run to enforce FORCE mode or
  AUTO mode disregarding the AUTO_STAT_MODE setting in the ONCONFIG
  file and the environment.

  Revision 1.11  2011/05/11 22:13:19  art
  *** empty log message ***

  Revision 1.10  2011/05/11 20:24:45  art
  *** empty log message ***

  Revision 1.9  2011/05/11 19:53:24  art
  Fixed procedure so it will not error out recompiling itself.

  Revision 1.8  2011/05/09 19:25:59  art
  Fixed problem with creating the stored procedure.

  Revision 1.7  2010/07/13 19:27:05  art
  Changed type of tabinfo.nrows from bigint to double.  Using bigint seems to occassionally cause
  conversion errors (SQLCODE = -9971).  Copying nrows to a local bigint for calculations for
  comparisons also, for precision.

  Revision 1.7  2010/07/13 19:22:19  art
  Changed type of tabinfo.nrows from bigint to double.  Using bigint seems to occassionally cause
  conversion errors (SQLCODE = -9971).  Copying nrows to a local bigint for calculations for
  comparisons also, for precision.

  Revision 1.6  2010/04/02 16:12:15  art
  Added new feature --isolation allows one to specify the isolation level
  dostats uses when querying sysmaster and the target database.

  Revision 1.5  2010/01/20 22:06:10  art
  Fixed usage.

  Revision 1.4  2010/01/20 22:03:41  art
  Added new option --drop-distributions to begin by deleting all data distributions
  from the sysdistrib catalog table.  Useful after an IDS version upgrade.

  Revision 1.3  2009/10/13 17:02:11  art
  Added new --time-display command to display the starting time of each operation
  and group when executing immediately.
  Thanks to Neil Truby for the suggestion.

  Revision 1.2  2009/08/24 03:08:31  art
  Fixed versioning.

*/

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>
#include <ctype.h>
#include <errno.h>
#if !defined(NOTIMINGS)
#include <time.h>
#include <sys/time.h>
#endif
#if defined(i386) || defined(linux) || defined(__hpux)
#include <string.h>
#else
#include <strings.h>
#endif
#include <signal.h>

#if defined(_WINDOWS_)
#include <windows.h>
#define SIGQUIT SIGBREAK
#endif

#include "getopt.h"  /* Not <getopt.h> need the delivered header. */

#if !defined( FALSE )
#define FALSE (0)
#endif
#if !defined( TRUE )
#define TRUE (!FALSE)
#endif
#define Version "7.00 - NG"
#define Revision "$Revision: 1.81 $ - NG"

EXEC SQL INCLUDE sqlhdr;
EXEC SQL INCLUDE sqlca;
EXEC SQL INCLUDE sqltypes;
EXEC SQL INCLUDE datetime;
EXEC SQL INCLUDE locator;

static char rcs_header[] = "$Header: /home/art/GoogleDrive/Latest/RCS/dostats_ng.ec,v 1.81 2023/11/24 13:51:54 art Exp $";

static struct sqlca_s local_sqlca;

int return_coltypes( int prcid, char *procstr );

EXEC SQL BEGIN DECLARE SECTION;
struct {
    int tabid, partnum;
    double nrows;
    int days;
    short statchange;
    short colno[16], colind, daysind, statchgind;
    string tabname[129];
    string tabowner[33];
    string idxowner[33];
    string idxname[129];
    string keystring[2049];
} tabinfo, lastidx;  

typedef struct rowcounts_s {
    int partnum;
    bigint nrows;
} rowcounts_t;

typedef struct S_BLOBBUFF {
    char *buff, *base;
    int bbsize, global_length, full_length;
} blobbuff_t;

typedef struct high_list_s {
    char dbsname[129];
    char tabname[129];
    char colname[129];
} high_list_t;

rowcounts_t *rowcounts = (rowcounts_t *)NULL;
string databasename[129], host[1000], connhost[1000], username[1000], passwd[1000];
string *statement = (char *)NULL;
string statement2[65536], waitstr_dostats[200];
string tablename[129], columnnames[3278][129], procname[129], *dbstmt;
string specificname[129];
high_list_t *force_high_cols;
string procowner[33];
short arg_ind, specific_ind;
loc_t exprtext;
int evalpos, procid;
blobbuff_t tblobbuff, *blobbuff;
int stmt_len = 0, dbstmt_len = 0;
char tempstr[65600], pdqstmt_1[200], pdqstmt_2[200];
char procstmt[65535], tempstr2[65600], arglist_temp[65600];
short colno, columntypes[32768], xtdtypes[32768];
int NoDistOnly = 0;  /* If TRUE do not include 'DISTRIBUTIONS ONLY' clause. */
int NoLevel = 0;     /* If TRUE do not include statistics level keywords. */
int ForceDistOnly = 0; /* If TRUE use HIGH ... DISTRIBUTIONS ONLY and LOW even
			  for single column indexes. */
int doBrowse = 0;		 /* Find tables with row counts changed more */
double BrowseThreshHold = 15.00; /* than BrowseThreshHold % since last run */
int haveBrowseThreshHold = 0;
int doPhysicalCount = 0;
int doDistAging = 0;	/* Find tables with distributions older than DistAgingLimit days. */
int DistAgingLimit = 30;
int haveDistAgingLimit = 0;
int tablePDQ = 0;
int procedurePDQ = 0;
int QuotedIdents = 0;
int aus_small_table = 0;
int first = 1;
double nrecs;
int numparts;
int major_vers, minor_vers;
string level_vers[256], server_type[256];
int waitlen = 10;
char schedprocname[129], schedprocfilename[1025];
char schedule_stmt[65535];
int tbl_chk, OK;
bigint nrows;
EXEC SQL END DECLARE SECTION;

int conn = 0, processed = 0;
char revision[1024];
FILE *outfile = NULL, *highfile = NULL;
int dotable = 1, doindex = 1, docolumn = 1, domedium = 1, tabmedium = 1, dropdists = 0,
    is_se = 0, doprocs = 0, proconly = 0, ExpandReport = 0, display_time = 0,
    once = 0, verbose = 0, singledb = 0, dosystem = 0, doexplain = 0, doPDQ = 0,
    doFragExpr = 0, useShared = 0, DoSpecific = 1, have_waitlen = 0, drop_distributions = 0;
int dumpargs = 0, display_local = 0, display_fract = 0;
char isolation_level[2] = "c";
int secs, msecs;
int proc_local = 0, wipe_table = 0;
char dtparts[7][10];

#if defined(NOTIMINGS)
struct timeval { int left; };
#endif

struct timeval totstart, totend, totdiff;
struct timeval dbsstart, dbsend, dbsdiff;
struct timeval tabstart, tabend, tabdiff;
struct timeval stmtstart, stmtend, stmtdiff;

/* FUNCTION PROTOTYPES */
#if !defined(NOTIMINGS)
void timediff( struct timeval, struct timeval, int *, int * );
#endif
void usage( char *program );
void exit( int stat );
void dostats_shutdown( int sig );
#define STR_STEP 1024
void readlist( char **, int *, char **, int *, char * );
void appendStr( char **, int *, char * );
double minres( int nrecs );
void InitKeyStr( char *keystr );
short ParseKeyStr( );
long long getrowcount( char *tabowner, char *tabname, int tabid, int partnum );
int openit( loc_t *loc, int flags, int bsize );
int writeit( loc_t *loc, char *buf, int nbytes );
int closeit( loc_t *loc );
void StripTypeList( char *list );
int cfind( char *item, char collist[][129], int ncols );
void Store_Statement( char *statement );
void Write_Statement( char *statement );
void TimeStart( struct timeval *start, int ExpandReport );
void TimeMark( struct timeval start, int ExpandReport, char *intro, int display, int strt );
void Create_Process_Table( char *database );

void (*Out_Func)( char * );

#if !defined( __GNUC__ )
#if defined( max )
#undef max
#endif

#if defined( strlower )
#undef strlower
#endif

#define max(a,b) ((a) > (b) ? (a) : (b) )
#define strlower( t, s, m ) {					\
	char *target = (t), *source = (s); int max = (m);	\
	while (max-- && *source) {				\
	    *target++ = tolower(*source++);			\
	}							\
	*target = 0;						\
    }
#else
#ifndef max
inline int max( int a, int b)
{
    return (a > b ? a : b);
}
#endif

static inline void strlower( char *target, char *source, int max )
{
    while (max-- && *source) {
        *target++ = tolower(*source++);
    }
    *target = 0;
}
#endif

int listcomp( const void *left, const void *right )
{
    high_list_t *lft, *rht;
    int res;

    lft = (high_list_t *)left;
    rht = (high_list_t *)right;
    res = strcmp( lft->dbsname, rht->dbsname );
    if (res < 0)
	return -1;
    else if (res > 0)
	return 1;
    else {
	res = strcmp( lft->tabname, rht->tabname );
	if (res < 0)
	    return -1;
	else if (res > 0)
	    return 1;
	else {
	    res = strcmp( lft->colname, rht->colname );
	    if (res < 0)
		return -1;
	    else if (res > 0)
		return 1;
	    else
		return 0;
	}
    }
}

void dostats_shutdown( int sig )
{
    sqlbreak();
    if (conn == 1) {
	EXEC SQL SET CONNECTION 'WORK';
	sqlbreak();
    } else {
	EXEC SQL SET CONNECTION 'DBASES';
	sqlbreak();
    }
    EXEC SQL DISCONNECT ALL;
    if (outfile != (FILE *)NULL) {
	fflush( outfile );
	fclose( outfile );
    }
    exit( 1 );
}

void version( ) {
    snprintf( revision, sizeof revision, "%s", Revision );
    fprintf( stderr, "\ndostats_ng: Features Version %s, Source %s!\n",
	    Version,
	    strtok( revision, "$" ) );
    fprintf( stderr,
	     "\tCopyright 2009 - 2023 by Art S. Kagel.\n" );
    fprintf( stderr,
	     "\tLicense for private usage (including use by "
	     "commercial enterprise\n"
	     "\tfor its own needs) is granted.  Commercial and "
	     "exploitation rights\n"
	     "\tare reserved.\n\n" );
}

int main(int argc, char *argv[])
{
    int i, c, errflag=0, dflag=0, tflag=0, hflag=0, pflag=0, mflag=0,
	ANSIMode=0, InclOwner=0, Logging=0;
    char lasttbl[129], filename[1024], *h, *pdqenv, *dumb;
    static short columns[32768];
    int ncols, column, column2, leadsame, jj;
    int tableset = 0, indexset = 0, columnset = 0, mediumset = 0, samplesize = 0;
    double resolution = 0.0, mresolution = 0.0, confidence = 0.0, samplefloat=0.0;
    int resset = 0,  mresset = 0, confset = 0, sampleset = 0, err = 0;
    int do_scheduling = 0, do_proconly = 0, do_procedure = 0, do_err_cnt = 0;
    int run_locally = 0, have_high_list = 0, num_high = 0;
    char pwd[1025];
    FILE *schedfile = (FILE *)NULL;
    char sch_starttime[50], sch_lasttime[50], sch_frequency[50];
    char sch_sunday[4], sch_monday[4], sch_tuesday[4], sch_wednesday[4];
    char sch_thursday[4], sch_friday[4], sch_saturday[4];
    char confstring[30], resstring[30], mresstring[30], samplestring[100];
    int  sch_have_freq = 0, sch_have_start = 0, sch_have_end = 0, sch_onetime = 0;
    char sch_freq_type, sch_dbs[19];
    char ownerstr[500], lastowner[500], lowcols[65536];
    char *includes = (char *)NULL, *excludes = (char *)NULL;
    char *subquery = (char *)NULL, *exquery = (char *)NULL;
    char *dbincludes = (char *)NULL, *dbexcludes = (char *)NULL;
    char *princludes = (char *)NULL, *prexcludes = (char *)NULL;
    int incl_len = 0, excl_len = 0, dbexcl_len = 0, princl_len = 0, prexcl_len = 0, qry_len = 0, xqry_len = 0;
    int sawlistincl = 0, sawlistexcl = 0, sawsubincl = 0, sawsubexcl = 0;
    int noexit = 0, error_ret = 1, use_aus_thresholds = 0, force_run = 0, auto_run = 0;
    char *at, *pstemp_env, high_list_file[1025];
    long long rowcnt, l_rowcnt;
    int small_tables_high = 0, small_tables_count = 0, have_small_threshold = 0;
    struct option longopts[] = {
				{ "schedule", no_argument, (int *)NULL, 0x100 },
				{ "first-time", required_argument, (int *)NULL, 0x101 },
				{ "last-time", required_argument, (int *)NULL, 0x102 },
				{ "frequency", required_argument, (int *)NULL, 0x103 },
				{ "procedure", required_argument, (int *)NULL, 0x104 },
				{ "one-time", no_argument, (int *)NULL, 0x105 },
				{ "to-file", required_argument, (int *)NULL, 'f' },
				{ "aus-thresholds", no_argument, (int *)NULL, 0x106 },
				{ "time-display", no_argument, (int *)NULL, 0x107 },
				{ "display-time", no_argument, (int *)NULL, 0x107 },
				{ "drop-distributions", no_argument, (int *)NULL, 0x108 },
				{ "isolation", required_argument, (int *)NULL, 0x109 },
				{ "force-run", no_argument, (int *)NULL, 0x110 },
				{ "auto-run", no_argument, (int *)NULL, 0x111 },
				{ "proc-local", no_argument, (int *)NULL, 0x112 },
				{ "clean-distributions", no_argument, (int *)NULL, 0x113 },
				{ "error-count", no_argument, (int *)NULL, 0x114 },
				{ "execute-local", no_argument, (int *)NULL, 0x115 },
				{ "display-local", no_argument, (int *)NULL, 0x116 },
				{ "distributions-high", required_argument, (int *)NULL, 0x117 },
				{ "small-tables-high", no_argument, (int *)NULL, 0x118 },
				{ "small-tables-threshold", required_argument, (int *)NULL, 0x119 },
				{ "include-owner", no_argument, (int *)NULL, 0x120 },
				{ "display-subsec", no_argument, (int *)NULL, 0x121 },
				{ (char *)NULL, no_argument, (int *)NULL, -1 }
    };
    int longint;

    signal( SIGINT, dostats_shutdown );
    signal( SIGQUIT, dostats_shutdown );

    gettimeofday( &totstart, NULL );

    if (getenv( "DELIMIDENT" ) != (char *)NULL) {
	QuotedIdents = 1;
    }

    snprintf( tablename, sizeof tablename, "%s", "*" );
    host[0] = (char)0;
    memset( ownerstr, 0, sizeof ownerstr );
    strcpy( sch_dbs, "sysadmin" );
    strcpy( sch_starttime, "'00:00:01'" );
    strcpy( sch_lasttime, "'00:00:05'" );
    sch_frequency[0] = (char)0;
    strcpy( sch_sunday, "'f'" );
    strcpy( sch_monday, "'f'" );
    strcpy( sch_tuesday, "'f'" );
    strcpy( sch_wednesday, "'f'" );
    strcpy( sch_thursday, "'f'" );
    strcpy( sch_friday, "'f'" );
    strcpy( sch_saturday, "'f'" );
    while ((c = getopt_long( argc,
			     argv,
			     "c:d:f:h:i:n:r:t:x:A:B:M:P:Q:R:CEFGILNSTVXabeglmpv:w:zZ:",
			     longopts,
			     &longint )) != EOF )
	{
	    switch(c) {
	    case 0x100:
		/* Enable writing commands to the IDS 11 scheduler. */
		do_scheduling = 1;
		break;
	    case 0x104:
		do_proconly = 1;
		snprintf( schedprocname, sizeof schedprocname, "%s", optarg );
		break;
	    case 0x101:
		/* First time to execute scheduled commands. */
		snprintf( sch_starttime, sizeof sch_starttime, "'%s'", optarg );
		sch_have_start = 1;
		break;
	    case 0x102:
		/* Last time to execute scheduled commands. */
		snprintf( sch_lasttime, sizeof sch_lasttime, "'%s'", optarg );
		sch_have_end = 1;
		break;
	    case 0x105:
		/* Set procedure scheduled to run once only then disable
		   itself. */
		sch_onetime = 1;
		break;
	    case 0x106:
		/* Set aging, browse, and PDQPRIORITY using the Auto
		   Update Statistics configuration. */
		use_aus_thresholds = 1;
		break;
	    case 0x116:
		display_local = 1;
		display_time = 1;
		break;
	    case 0x117:
		snprintf( high_list_file, sizeof high_list_file, "%s", optarg );
		have_high_list = 1;
		break;
	    case 0x107:
		display_time = 1;
		break;
	    case 0x108:
		drop_distributions = 1;
		break;
	    case 0x109:
		/* Set isolation level. */
		if (strlen( optarg ) != 1) {
		    fprintf( stderr, "Invalid isolation level = %s\n", optarg );
		    usage( argv[0] );
		    return 1;
		}
		strcpy( isolation_level, optarg );
		isolation_level[0] = tolower( isolation_level[0] );
		if (isolation_level[0] != 'c' &&
		    isolation_level[0] != 'd' &&
		    isolation_level[0] != 'l')
		    {
			fprintf( stderr, "Invalid isolation level = %s\n", optarg );
			usage( argv[0] );
			return 1;
		    }
		break;
	    case 0x110:
		/* Force stats to run even if programmed thresholds are not met.  11.70+ */
		force_run = 1;
		break;
	    case 0x111:
		/* Obey thresholds even if force is default.  11.70+ */
		auto_run = 1;
		break;
	    case 0x112:
		proc_local = 1;
		break;
	    case 0x113:
		wipe_table = 1;
		break;
	    case 0x114:
		do_err_cnt = 1;
		break;
	    case 0x115:
		run_locally = 1;
		proc_local = 1;
		break;
	    case 0x103:
		/* Frequency at which to execute scheduled commands. */
		sch_have_freq = 1;
		sch_freq_type = optarg[0];
		switch (sch_freq_type) {
		case 'd':
		case 'D':
		    snprintf( sch_frequency, sizeof sch_frequency, "'1 00:00:00'" );
		    strcpy( sch_sunday, "'t'" );
		    strcpy( sch_monday, "'t'" );
		    strcpy( sch_tuesday, "'t'" );
		    strcpy( sch_wednesday, "'t'" );
		    strcpy( sch_thursday, "'t'" );
		    strcpy( sch_friday, "'t'" );
		    strcpy( sch_saturday, "'t'" );
		    break;
		case 'w':
		case 'W':
		    snprintf( sch_frequency, sizeof sch_frequency, "'7 00:00:00'" );
		    switch (optarg[1]) {
		    case '0':
			strcpy( sch_sunday, "'t'" );
			break;
		    case '1':
			strcpy( sch_monday, "'t'" );
			break;
		    case '2':
			strcpy( sch_tuesday, "'t'" );
			break;
		    case '3':
			strcpy( sch_wednesday, "'t'" );
			break;
		    case '4':
			strcpy( sch_thursday, "'t'" );
			break;
		    case '5':
			strcpy( sch_friday, "'t'" );
			break;
		    case '6':
			strcpy( sch_saturday, "'t'" );
			break;
		    }
		    break;
		case 'm':
		case 'M':
		    snprintf( sch_frequency, sizeof sch_frequency, "'30 00:00:00'" );
		    switch (optarg[1]) {
		    case '0':
			strcpy( sch_sunday, "'t'" );
			break;
		    case '1':
			strcpy( sch_monday, "'t'" );
			break;
		    case '2':
			strcpy( sch_tuesday, "'t'" );
			break;
		    case '3':
			strcpy( sch_wednesday, "'t'" );
			break;
		    case '4':
			strcpy( sch_thursday, "'t'" );
			break;
		    case '5':
			strcpy( sch_friday, "'t'" );
			break;
		    case '6':
			strcpy( sch_saturday, "'t'" );
			break;
		    }
		    break;
		}
		break;
	    case 'z':
		dumpargs = 1;
		break;
	    case 'G':
		doPhysicalCount = 1;
		break;
	    case 'N':
		DoSpecific = 0;
		break;
	    case 'm':
		mflag = 1;
		break;
	    case 'n':
		noexit = atoi( optarg );
		break;
	    case 'p':
		pflag++;
		break;
	    case 'S':
		singledb = 1;
		useShared = 1;
		break;
	    case 'v':
		verbose = atoi( optarg );
		break;
	    case 'b':
		doBrowse = 1;
		break;
	    case 'g':
		doFragExpr = 1;
		break;
	    case 'e':
		ExpandReport = 1;
		break;
	    case 'a':
		doDistAging = 1;
		break;
	    case 'E':
		ExpandReport = 2;
		break;
	    case 'F':
		ForceDistOnly = 1;
		break;
	    case 'X':
		doexplain = 1;
		break;
	    case 'B':
		BrowseThreshHold = atof( optarg );
		haveBrowseThreshHold = 1;
		break;
	    case 'A':
		DistAgingLimit = atoi( optarg );
		haveDistAgingLimit = 1;
		break;
	    case 'P':
		procedurePDQ = atol( optarg );
		break;
	    case 'Q':
		tablePDQ = atol( optarg );
		doPDQ = 1;
		break;
	    case 'd':
		if (dflag)
		    errflag++;
		else{
		    dflag++;
		    if (QuotedIdents) {
			strncpy(databasename, optarg, 128);
			databasename[128] = (char)0;
		    } else {
			strlower(databasename, optarg, 128);
		    }
		}
		break;
	    case 't':
		if (tflag)
		    errflag++;
		else{
		    tflag++;
		    if (QuotedIdents) {
			strncpy(tablename, optarg, 128);
			tablename[128] = (char)0;
		    } else {
			strlower(tablename, optarg, 128);
		    }
		}
		break;
	    case 'f':
		if (strcmp( "-", optarg ) == 0) {
		    snprintf( filename, sizeof filename, "%s", "stdout" );
		    outfile = stdout;
		} else {
		    snprintf( filename, sizeof filename, "%s", optarg );
		    outfile = fopen( filename, "w" );
		}
		fprintf( stderr, "Writing commands to: %s\n", filename );
		if (!outfile) {
		    fprintf( stderr,
			     "Cannot open output: %s, errno=%d.\n",
			     filename,
			     errno );
		    exit(3);
		}
		break;
	    case 'h':
		if (hflag)
		    errflag++;
		else{
		    hflag++;
		    snprintf( host, sizeof host, "%s", optarg );
		}
		break;
	    case 'i':
		/* Form table include list. */
		if (optarg[0] == '@') {
		    readlist( &includes,
			      &incl_len,
			      (char **)NULL,
			      (int *) NULL,
			      optarg );
		    sawlistincl = 1;
		} else if (optarg[0] == '!') {
		    if (strncasecmp( &(optarg[1]), "select", 6 ) != 0) 
			appendStr( &subquery, &qry_len, "select tabname from systables where " );
		    appendStr( &subquery, &qry_len, &(optarg[1]) );
		    sawsubincl = 1;
		} else if (optarg[0] == '>' ) {
		    if (princludes)
			appendStr( &princludes, &princl_len, "','" );
		    else
			appendStr( &princludes, &princl_len, " OR procname IN ('" );
		    appendStr( &princludes, &princl_len, strtok( optarg, ">" ) );
		} else {
		    if (includes)
			appendStr( &includes, &incl_len, "','" );
		    else
			appendStr( &includes, &incl_len, "('" );
		    appendStr( &includes, &incl_len, optarg );
		    sawlistincl = 1;
		}
		break;
	    case 'x':
		/* Form table exclude list. */
		if (optarg[0] == '@') {
		    readlist( &excludes,
			      &excl_len,
			      &dbexcludes,
			      &dbexcl_len,
			      optarg );
		    sawlistexcl = 1;
		} else if (optarg[0] == '!') {
		    if (strncasecmp( &(optarg[1]), "select", 6 ) == 0) {
			fprintf( stderr, "-x '!select...' has been deprecated. Invert your filters and use -i!\n" );
			usage( argv[0] );
			exit( 1 );
		    }
		    appendStr( &exquery, &xqry_len, " AND a.tabname NOT IN (" );
		    appendStr( &exquery, &xqry_len, "select tabname from systables where " );
		    appendStr( &exquery, &xqry_len, &(optarg[1]) );
		    appendStr( &exquery, &xqry_len, ") " );
		    sawsubexcl = 1;
		} else if (strchr( optarg, ':' )) {
		    /* Database exclusion. */
		    if (dbexcludes)
			appendStr( &dbexcludes, &dbexcl_len, "','" );
		    else
			appendStr( &dbexcludes, &dbexcl_len, "AND name NOT IN ('" );
		    appendStr( &dbexcludes, &dbexcl_len, strtok( optarg, ":" ) );
		} else if (strchr( optarg, '>' )) {
		    /* Procedure exclusion. */
		    if (prexcludes)
			appendStr( &prexcludes, &prexcl_len, "','" );
		    else
			appendStr( &prexcludes, &prexcl_len, "AND procname NOT IN ('" );
		    appendStr( &prexcludes, &prexcl_len, strtok( optarg, ">" ) );
		  
		} else {
		    /* Table exclusion. */
		    if (excludes)
			appendStr( &excludes, &excl_len, "','" );
		    else
			appendStr( &excludes, &excl_len, "('" );
		    appendStr( &excludes, &excl_len, optarg );
		    sawlistexcl = 1;
		}
		break;
	    case 'T':
		tableset = 1;
		tabmedium = 1;
		break;
	    case 'I':
		indexset = 1;
		tabmedium = 0;
		break;
	    case 'r':
		mresolution = atof( optarg );
		mresset = 1;
		break;
	    case 'R':
		resolution = atof( optarg );
		resset = 1;
		break;
	    case 'c':
		confidence = atof( optarg );
		confset = 1;
		break;
	    case 'Z':
		samplesize = atoi( optarg );
		samplefloat = atof( optarg );
		sampleset = 1;
		break;
	    case 'C':
		columnset = 1;
		tabmedium = 0;
		break;
	    case 'L':
		tabmedium = 0;
		tableset = 1;
		indexset = 0;
		dropdists = 1;
		break;
	    case 'l':
		tabmedium = 0;
		tableset = 1;
		indexset = 0;
		dropdists = 0;
		break;
	    case 'M':
		mediumset = 1;
		tabmedium = 0;
		break;
	    case 'w':
		waitlen = atoi( optarg );
		have_waitlen = 1;
		break;
	    case 0x118:
		small_tables_high = 1;
		break;
	    case 0x119:
		small_tables_count = atoi( optarg );
		have_small_threshold = 1;
		break;
	    case 0x120:
		InclOwner = 1;
		break;
	    case 0x121:
		display_fract = 1;
		display_time = 1;
		break;
	    case 'V':
		version( );
		exit(0);
	    case '?':
	    default:
		errflag++;
		break;
	    }
	}

    if (errflag || dflag == 0){ usage( argv[0] ); exit( 1 ); }

    if (optind < argc) {
	fprintf( stderr, "Extra arguments ignored!\n" );
    }

    if (pflag && !tflag && !includes && !excludes && !subquery) {
	proconly = 1;
	doprocs = 1;
    } else if (pflag || (dflag && !tflag && !includes && !excludes && !subquery && !exquery))
	doprocs = 1;

    if (tableset || indexset || columnset || mediumset)
	dotable = doindex = docolumn = domedium = tabmedium = 0;

    if (tableset) {
	dotable = 1;
    }

    if (indexset) {
	doindex = 1;
    }

    if (columnset) {
	docolumn = 1;
    }

    if (mediumset) {
	domedium = 1;
	tabmedium = 1;
    }

    if (mflag) {
	dosystem = 1;
    }

    if ((tflag || excludes || includes) && drop_distributions) {
	fprintf( stderr,
		 "Cannot use -t, -x, or -i with --drop-distributions.  Use --clean-distributions with these\n"
		 "instead to remove old distributions from specific tables.\n" );
	usage( argv[0] );
	exit(1);
    }

    if (confset && !tabmedium) {
	fprintf( stderr,
		 "Confidence option included but MEDIUM level stats have not "
		 "been requested!  Ignoring.\n" );
	confset = 0;
    }

    if (sampleset && !tabmedium) {
	fprintf( stderr,
		 "Sampling size option included but MEDIUM level stats have not "
		 "been requested!  Ignoring.\n" );
	sampleset = 0;
    }

    if (sampleset && ((samplesize <= 0 && samplefloat <= 0.0) || samplefloat > 1.0)) {
	fprintf( stderr,
		 "Invalid minimum sample size (%d | %f) is out of range (0 < MinSamp < nrows)!\n"
		 "  or (0.0 < SampPct < 1.0)!\n",
		 samplesize,
		 samplefloat );
	usage( argv[0] );
	exit(1);
    } else if (sampleset) {
        if (samplesize == 0 && samplefloat > 0.0 && samplefloat < 1.0)
	    snprintf( samplestring, sizeof samplestring, " SAMPLING SIZE %6.2f ", samplefloat );
	else
	    snprintf( samplestring, sizeof samplestring, " SAMPLING SIZE %d ", samplesize );
    }
    
    if (resset && !doindex) {
	fprintf( stderr,
		 "HIGH Resolution option included but HIGH stats"
		 " have not been requested!  Ignoring.\n" );
	resset = 0;
    }

    if (mresset && !tabmedium) {
	fprintf( stderr,
		 "MEDIUM Resolution option included but MEDIUM stats"
		 " have not been requested!  Ignoring.\n" );
	mresset = 0;
    }

    if (confset && (confidence > 0.9900000000 || confidence <= 0.80)) {
	fprintf( stderr,
		 "Confidence level specified is out of range(0.80 -> 0.99)!\n" );
	usage( argv[0] );
	exit(1);
    } else if (confset) {
	snprintf( confstring, sizeof confstring, " %15.10f", confidence );
	if (!mresset) {
	    mresset = 1;
	    mresolution = 2.5;
	}
    }

    if (resset && (resolution > 10.0 || resolution < 0.005)) {
	fprintf( stderr,
		 "HIGH Resolution specified is out of "
		 "range(max(0.005,(1/nrows)) -> 10.0%%)!\n" );
	usage( argv[0] );
	exit(1);
    } else if (resset)
	snprintf( resstring, sizeof resstring, " RESOLUTION %15.10f", resolution );

    if (mresset && (mresolution > 10.0 || mresolution < 0.005)) {
	fprintf( stderr,
		 "MEDIUM Resolution specified is out of "
		 "range((max(0.005,(1/nrows)) -> 10.0%%)!\n" );
	usage( argv[0] );
	exit(1);
    } else if (mresset)
	snprintf( mresstring, sizeof mresstring, " RESOLUTION %15.10f", mresolution );

    if (tflag && strcmp( tablename, "none" ) == 0 && !includes && !subquery ) {
	fprintf( stderr,
		 "Includes required if tablename is \"none\"!\n" );
	usage( argv[0] );
	exit( 1 );
    } else if (tflag && strcmp( tablename, "none" ) == 0 && excludes && exquery && !includes && !subquery) {
	fprintf( stderr,
		 "Excludes prohibited if tablename is \"none\"!\n" );
	usage( argv[0] );
	exit( 1 );
    } else if (strcmp( tablename, "*" ) == 0 && includes && subquery && !excludes && !exquery) {
	fprintf( stderr,
		 "Tablename template other than '*' required with includes!\n" );
	usage( argv[0] );
	exit( 1 );
    }

    if (procedurePDQ < 0 || procedurePDQ > 100) {
	fprintf( stderr,
		 "Store procedure PDQ priority out-of-range(0-100): %d!\n",
		 procedurePDQ );
	usage( argv[0] );
	exit( 1 );
    }

    if (useShared && doBrowse) {
	fprintf( stderr,
		 "-b option not possible with a shared memory connection!\n"
		 "Remove -S and change to a networked or pipe connection.\n" );
	usage( argv[0] );
	exit( 1 );
    }

    if (haveBrowseThreshHold && !doBrowse) {
	fprintf( stderr,
		 "Browse threshhold (-B) entered without browse request(-b)!\n" );
	usage( argv[0] );
	exit( 1 );
    }

    if (BrowseThreshHold < 0.0001 || BrowseThreshHold > 10000.00 ) {
	fprintf( stderr,
		 "Browse threshhold out-of-range(0.0001-10000.0): %f!\n",
		 BrowseThreshHold );
	usage( argv[0] );
	exit( 1 );
    }

    if (haveDistAgingLimit && !doDistAging) {
	fprintf( stderr,
		 "Distribution Aging Limit (-A) entered without Aging request(-a)!\n" );
	usage( argv[0] );
	exit( 1 );
    }

    if (DistAgingLimit < 0) {
	fprintf( stderr,
		 "Distribution Aging Limit out-of-range(>= 0): %d!\n",
		 DistAgingLimit );
	usage( argv[0] );
	exit( 1 );
    }

    if (!doPDQ) {
	pdqenv = getenv( "PDQPRIORITY" );
	if (pdqenv) {
	    tablePDQ = atol( pdqenv );
	} else
	    tablePDQ = 0;
    }

    if (tablePDQ < 0 || tablePDQ > 100) {
	fprintf( stderr,
		 "Table PDQPRIORITY out-of-range(0-100): %d!",
		 tablePDQ );
	usage( argv[0] );
	exit( 1 );
    }

    do_procedure = do_scheduling + do_proconly;

    if (do_scheduling && proc_local) {
	fprintf( stderr, "Cannot use --proc-local with --schedule!\n" );
	usage( argv[0] );
	exit(1);
    }

    if (proc_local && !do_proconly) {
	fprintf( stderr, "--proc-local requires --procedure=procname!\n" );
	usage( argv[0] );
	exit(1);
    }

    if (!(do_scheduling || do_proconly) && sch_onetime) {
	fprintf( stderr, "--one-time passed without --schedule!\n" );
	usage( argv[0] );
	exit( 1 );
    }

    if (!do_scheduling && sch_have_start) {
	fprintf( stderr, "--first-time passed without --schedule!\n" );
	usage( argv[0] );
	exit( 1 );
    }

    if (!do_scheduling && sch_have_end) {
	fprintf( stderr, "--last-time passed without --schedule!\n" );
	usage( argv[0] );
	exit( 1 );
    }

    if (!do_scheduling && sch_have_freq) {
	fprintf( stderr, "--frequency passed without --schedule!\n" );
	usage( argv[0] );
	exit( 1 );
    }

    if (!do_scheduling && run_locally) {
	fprintf( stderr, "--execute-local passed without --schedule!\n" );
	usage( argv[0] );
	exit( 1 );
    }

    if (do_scheduling && outfile != (FILE *)NULL) {
	fprintf( stderr, "--schedule is not compatible with -f/--to-file!\n" );
	usage( argv[0] );
	exit( 1 );
    }

    if (do_scheduling && singledb) {
	fprintf( stderr, "--schedule requires a non-shared-memory connection type!\n" );
	usage( argv[0] );
	exit( 1 );
    }

    if (have_high_list == 1 && dotable == 0) {
	fprintf( stderr,
		 "--distributions-high requires table processing!\n" );
	usage( argv[0] );
	exit( 1 );
    }

    if (display_local && (do_scheduling || do_proconly || outfile != (FILE *)NULL)) {
	fprintf( stderr,
		 "--time-local not sensible with -f, --schedule,, or --procedure\n" );
	usage( argv[0] );
	exit( 1 );
    } else if (display_time && (do_scheduling || do_proconly || outfile != (FILE *)NULL)) {
	fprintf( stderr,
		 "--time-display not sensible with -f, --schedule,, or --procedure\n" );
	usage( argv[0] );
	exit( 1 );
    }

    if (auto_run && force_run) {
	fprintf( stderr,
		 "--auto-run and --force-run are mutually exclusive!\n" );
	usage( argv[0] );
	exit( 1 );
    }

    snprintf( pdqstmt_1, sizeof pdqstmt_1, "SET PDQPRIORITY %d;", procedurePDQ );
    snprintf( pdqstmt_2, sizeof pdqstmt_2, "SET PDQPRIORITY %d;", tablePDQ );
     
    if (strcmp( databasename, "" ) == 0
	|| ( tflag == 1 && strcmp( tablename, "" ) == 0 ))
	{
	    usage( argv[0] );
	    exit( 1 );
	}

    if (run_locally) {
	strcpy(sch_dbs, databasename );
    }
    
    if (includes && sawlistincl)
	appendStr( &includes, &incl_len, "') " );
    else if (includes)
	appendStr( &includes, &incl_len, ") " );
    
    if (excludes && sawlistexcl)
	appendStr( &excludes, &excl_len, "') " );
    else if (excludes)
	appendStr( &excludes, &excl_len, ") " );

    if (dbexcludes)
	appendStr( &dbexcludes, &dbexcl_len, "') " );

    if (prexcludes)
        appendStr( &prexcludes, &prexcl_len, "') " );

    if (princludes)
        appendStr( &princludes, &princl_len, "') " );

    if (do_scheduling && !sch_have_freq) {
	/* Scheduling enabled but no frequency set.  Default to Weekly
	   on Sunday. */
	snprintf( sch_frequency, sizeof sch_frequency, "'7 00:00:00'" );
	strcpy( sch_sunday, "'t'" );
	sch_have_freq = 1;
    }

    if (do_scheduling && sch_have_freq && !sch_have_start) {
	sprintf( sch_starttime, "'00:00:00'" );
    }

    if (do_scheduling && sch_have_freq  && !sch_have_end) {
	sprintf( sch_lasttime, "'00:00:05'" );
    }

    
    if (verbose > 0) {
	if (verbose > 1)
	    version( );
	fprintf( stderr, "Verbose mode level %d               ENABLED....\n", verbose );
	if (QuotedIdents)
	    fprintf( stderr,
		     "Delimited identifiers               ENABLED....\n" );
	switch (isolation_level[0]) {
	case 'c':
	    fprintf( stderr, "ISOLATION LEVEL: COMMITTED READ.\n" );
	    break;
	case 'd':
	    fprintf( stderr, "ISOLATION LEVEL: DIRTY READ.\n" );
	    break;
	case 'l':
	    fprintf( stderr, "ISOLATION LEVEL: COMMITTED READ LAST COMMITTED.\n" );
	    break;
	}
	fprintf( stderr, "Isolation settings ignored for non-logged databases!\n" );
	if (auto_run)
	    fprintf( stderr,
		     "AUTO mode enforced.\n" );
	else if (force_run)
	    fprintf( stderr,
		     "FORCE mode enforced.\n" );
	if (wipe_table)
	    fprintf( stderr,
		     "Initially drop distributions for specified tables   ENABLED.\n" );
	else
	    fprintf( stderr,
		     "Initially drop distributions         %s...\n",
		     drop_distributions ? "ENABLED." : "DISABLED" );
	if (doprocs) {
	    fprintf( stderr,
		     "Stored Procedure processing         ENABLED....\n" );
	    fprintf( stderr,
		     "   Default PDQ priority for procedures set to: %d.\n",
		     procedurePDQ );
	} else
	    fprintf( stderr,
		     "Stored Procedure processing         DISABLED...\n" );
	if (singledb)
	    fprintf( stderr,
		     "Shared Memory Connection handling   ENABLED....\n" );
	else
	    fprintf( stderr,
		     "Shared Memory Connection handling   DISABLED...\n" );
	if (display_local)
	    fprintf( stderr,
		     "Execution start local%s time display  ENABLED....\n",
		     display_fract ? " fractional" : "" );
	else if (display_time)
	    fprintf( stderr,
		     "Execution start%s time display        ENABLED....\n",
		     (display_fract == 1 ? " fractional" : "") );
	if (ExpandReport == 1)
	    fprintf( stderr,
		     "Expanded%s Reporting                  ENABLED....\n",
		     (display_fract == 1 ? " fractional" : "") );
	else if (ExpandReport == 2)
	    fprintf( stderr,
		     "Expanded%s Time Reporting             ENABLED....\n",
		     (display_fract == 1 ? " fractional" : "") );
	else 
	    fprintf( stderr,
		     "Expanded Reporting                  DISABLED...\n" );
	if (ForceDistOnly)
	    fprintf( stderr,
		     "Force Distributions Only            ENABLED....\n" );
	else
	    fprintf( stderr,
		     "Force Distributions Only            DISABLED...\n" );
	if (NoDistOnly != TRUE)
	    fprintf( stderr,
		     "Distributions Only clauses          ENABLED....\n" );
	else
	    fprintf( stderr,
		     "Distributions Only clauses          DISABLED...\n");
	if (dosystem)
	    fprintf( stderr,
		     "System catalog processing           ENABLED....\n");
	else
	    fprintf( stderr,
		     "System catalog processing           DISABLED...\n");
	if (tflag)
	    fprintf( stderr,
		     "Table Name Template                 SUPPLIED (%s)...\n",
		     tablename );
	else
	    fprintf( stderr,
		     "Table Name Template                 NOT SUPPLIED...\n");
	if (proconly)
	    fprintf( stderr, "Only Stored Procedures will be processed.\n" );
	if (hflag)
	    fprintf( stderr,
		     "Host Name                           SUPPLIED (%s)...\n",
		     host );
	else
	    fprintf( stderr,
		     "Host Name                           NOT SUPPLIED...\n");
	if (do_scheduling) {
	    fprintf( stderr,
		     "Command Execution                   SCHEDULED...\n" );
	    if (do_proconly)
		fprintf( stderr,
			 "    Command procedure: %s\n",
			 schedprocname );
	} else if (!outfile)
	    fprintf( stderr,
		     "Command Execution                   IMMEDIATE...\n" );
	else
	    fprintf( stderr,
		     "Command Execution                   DEFERRED...\n" );
	
	if (subquery != (char *)NULL)
	    fprintf( stderr,
		     "Table Inclusion SubQuery Processed:\n\t%s\n",
		     subquery );
	if (includes != (char *)NULL)
	    fprintf( stderr,
		     "Table Inclusion List Processed:\n\ttabname IN %s\n",
		     includes );
	if (exquery != (char *)NULL)
	    fprintf( stderr,
		     "Table Exclusion SubQuery Processed:\n\t%s\n",
		     exquery );
	if (excludes != (char *)NULL)
	    fprintf( stderr,
		     "Table Exclusion List Processed:\n\ttabname NOT IN %s\n",
		     excludes );
	if (dbexcludes != (char *)NULL)
	    fprintf( stderr,
		     "Database Exclusion List Processed:\n\t%s\n",
		     (dbexcludes + 4) );
	if (tableset || indexset || columnset || mediumset) {
	    if (tabmedium)
		fprintf( stderr,
			 "Table Level Medium Statistics       ENABLED...\n" );
	    else
		fprintf( stderr,
			 "Table Level Low Statistics          ENABLED...\n" );
	    if (dropdists)
		fprintf( stderr, "Distributions will be dropped.\n" );
	    if (!indexset)
		fprintf( stderr,
			 "Index Lead High Statistics          DISABLED...\n" );
	    if (!columnset)
		fprintf( stderr,
			 "Index Key LOW Statistics            DISABLED...\n" );
	}
	if (mresset)
	    fprintf( stderr,
		     "Medium Statistics Resolution set to: %f\n",
		     mresolution );
	if (resset)
	    fprintf( stderr,
		     "High Statistics Resolution set to: %f\n",
		     resolution );
	if (confset)
	    fprintf( stderr,
		     "Medium Statistics Confidence set to: %f\n",
		     confidence );
	if (doexplain)
	    fprintf( stderr,
		     "SET EXPLAIN                         ENABLED...\n" );
	if (doBrowse)
	    fprintf( stderr,
		     "Table Browsing                      ENABLED...\n" );
	if (doDistAging)
	    fprintf( stderr,
		     "Distribution Aging                  ENABLED...\n" );
	if (doFragExpr)
	    fprintf( stderr,
		     "Fragment expression parsing enabled\n" );
	if (waitlen > 0)
	    fprintf( stderr, "Lock mode set to wait %d seconds.\n", waitlen );
	else if (waitlen == 0)
	    fprintf( stderr, "Lock mode set to not wait.\n" );
	else  /* (waitlen < 0) */
	    fprintf( stderr, "Lock mode set to wait forever.\n" );
	if (have_high_list == 1) {
	    fprintf( stderr,
		     "Processing HIGH distribution preferences from file: %s.\n",
		     high_list_file );
	}
    }
    
    /* Access requested database */
    
    if ((at=strchr( databasename, '@' )) != (char *)0) {
	/* Database name contains embedded server name.  Extract it. */
	if (host[0] == (char)0) {
	    *at++ = (char)0;
	    strncpy( host, at, 129 );
	} else {
	    fprintf( stderr, "Host specified in -h and -d!  Choose one.\n" );
	    usage( argv[0] );
	    exit( 1 );
	}
    }

    if (host[0] == (char)0) {
	/* No host specified.  Connect to default. */
	h = getenv( "INFORMIXSERVER" );
	if (h)
	    snprintf( host, sizeof host, "%s", h );
	else {
	    fprintf( stderr,
		     "Cannot open database connection.  "
		     "INFORMIXSERVER not set.\n"
		     "Set INFORMIXSERVER environment variable or include "
		     "'-h hostname' option.\n" );
	    usage( argv[0] );
	    exit( 1 );
	}
    }
    
    if (username[0] == (char)0) {
	/* No username specified.  Connect to default. */
	h = getenv( "INFORMIXUSER" );
	if (h)
	    snprintf( username, sizeof username, "%s", h );
    }
    
    
    if (passwd[0] == (char)0) {
	/* No username specified.  Connect to default. */
	h = getenv( "INFORMIXPASS" );
	if (h)
	    snprintf( passwd, sizeof passwd, "%s", h );
    }

    snprintf( connhost, sizeof connhost, "@%s", host );
    
    if (!singledb || doBrowse || doDistAging) {
	if (username[0] != (char)0) {
	    if (passwd[0] != (char)0)
		EXEC SQL CONNECT TO :connhost AS 'DBASES'
		    USER :username USING :passwd
		    WITH CONCURRENT TRANSACTION;
	    else {
		fprintf( stderr,
			 "Cannot specify INFORMIXUSER without INFORMIXPASS!\n" );
		usage( argv[0] );
		exit( 1 );
	    }
	} else
	    EXEC SQL CONNECT TO :connhost AS 'DBASES'
		WITH CONCURRENT TRANSACTION;
	if (sqlca.sqlcode) {
	    fprintf( stderr,
		     "FAILED: Cannot connect to %s (@%s); error=%d, isam=%d\n\n",
		     "SMI DATABASE",
		     host,
		     (int)sqlca.sqlcode,
		     (int)sqlca.sqlerrd[1] );
	    if (sqlca.sqlcode == -27000) {
		if (doBrowse) {
		    fprintf( stderr,
			     "-b option not possible with a shared memory connection!\n"
			     "Remove -S and change to a networked or pipe connection.\n" );
		} else {
		    fprintf( stderr,
			     "You cannot use a shared memory connection in "
			     "$INFORMIXSERVER or the -h option \nwithout the -S "
			     "option.  Specify a pipe or network connection or -S "
			     "and rerun.\n\n" );
		}
	    }
	    exit(1);
	}
	EXEC SQL DATABASE "sysmaster";
	if (sqlca.sqlcode < 0) {
	    /* No sysmaster database.  Must be SE or 5.xx engine.  Quit now! */
	    fprintf( stderr, "dostats_ng only supports IDS version 10.00 and later!\n" );
	    exit(1);
	} else if (sqlca.sqlcode == 0) {
	    if (!singledb) {
		/* Found sysmaster.  Continue. */
		switch (isolation_level[0]) {
		case 'd':
		    EXEC SQL SET ISOLATION DIRTY READ;
		    break;
		case 'c':
		    EXEC SQL SET ISOLATION COMMITTED READ;
		    break;
		case 'l':
		    EXEC SQL SET ISOLATION COMMITTED READ LAST COMMITTED;
		    break;
		}

		if (strlen( databasename ) == 3 &&
		    (strcmp( databasename, "ALL" ) == 0 ||
		     strcmp( databasename, "all" ) == 0))
		    snprintf( databasename, sizeof databasename, "%s", "*" );
		appendStr( &dbstmt, &dbstmt_len,
			   "SELECT name FROM \"informix\".sysdatabases "
			   "WHERE name != 'sysmaster' "
			   "AND name matches ? " );
		if (dbexcludes)
		    appendStr( &dbstmt, &dbstmt_len, dbexcludes );
		appendStr( &dbstmt, &dbstmt_len, " ORDER BY name");
		EXEC SQL PREPARE getdb
		    FROM :dbstmt;
		if (sqlca.sqlcode) {
		    fprintf( stderr,
			     "FAILED: Cannot PREPARE select statement for databases, error: %d.\n",
			     (int)sqlca.sqlcode );
		    fprintf( stderr, "Error at character %d.\nStatement is:\n",
			     (int)sqlca.sqlerrd[4] );
		    fprintf( stderr, "SELECT name FROM sysdatabases" );
		    exit(2);
		}
		EXEC SQL DECLARE dbc CURSOR WITH HOLD FOR getdb;
		if (sqlca.sqlcode) {
		    fprintf( stderr,
			     "FAILED: Cannot declare dbc cursor; Error = %d\n",
			     (int)sqlca.sqlcode );
		    exit(1);
		}
		EXEC SQL OPEN dbc using :databasename;
		if (sqlca.sqlcode) {
		    fprintf( stderr,
			     "FAILED: Cannot open dbc cursor; Error = %d\n",
			     (int)sqlca.sqlcode );
		    exit(1);
		}
	    }
	    if ((doBrowse || doDistAging) && !doPhysicalCount) {
		int part;

		EXEC SQL SELECT count(*)
		    INTO :numparts
		    FROM "informix".sysptnhdr;
		rowcounts =
		    (rowcounts_t *)malloc( numparts * sizeof (rowcounts_t) );

		EXEC SQL DECLARE rowcount_c CURSOR WITH HOLD FOR
		    SELECT partnum, nrows
		    FROM "informix".sysptnhdr
		    ORDER BY partnum;
		if (sqlca.sqlcode) {
		    fprintf( stderr,
			     "FAILED: Cannot declare rowcount_c cursor. "
			     "Error = %d, isam=%d\n",
			     (int)sqlca.sqlcode,
			     (int)sqlca.sqlerrd[1] );
		    exit(1);
		}
		EXEC SQL OPEN rowcount_c;
		if (sqlca.sqlcode) {
		    fprintf( stderr,
			     "FAILED: Cannot open rowcount_c cursor. "
			     "Error = %d, isam=%d\n",
			     (int)sqlca.sqlcode,
			     (int)sqlca.sqlerrd[1] );
		    exit(1);
		}
		for (part = 0; part < numparts; part++) {
		    EXEC SQL FETCH rowcount_c INTO :rowcounts[part];
		    if (sqlca.sqlcode)
			break;
		}
		if (sqlca.sqlcode < 0) {
		    fprintf( stderr,
			     "FAILED: Error fetching rowcounts.  Error=%d, isam=%d\n\n",
			     (int)sqlca.sqlcode,
			     (int)sqlca.sqlerrd[1] );
		    exit( 1 );
		}
		EXEC SQL CLOSE rowcount_c;
		EXEC SQL FREE rowcount_c;
	    } else
		numparts = 0;
	    if (singledb) {
		EXEC SQL DISCONNECT 'DBASES';
		if (sqlca.sqlcode) {
		    fprintf( stderr,
			     "FAILED: Cannot safely disconnect from sysmaster! "
			     "Error=%d, isam=%d\n\n",
			     (int)sqlca.sqlcode,
			     (int)sqlca.sqlerrd[1] );
		    exit( 1 );
		}
	    }
	}
    }

    if (username[0] != (char)0) {
	if (passwd[0] != (char)0)
	    EXEC SQL CONNECT TO :connhost AS 'WORK'
		USER :username USING :passwd
		WITH CONCURRENT TRANSACTION;
	else {
	    fprintf( stderr,
		     "Cannot specify INFORMIXUSER without INFORMIXPASS!\n" );
	    usage( argv[0] );
	    exit( 1 );
	}
    } else
	EXEC SQL CONNECT TO :connhost AS 'WORK'
	    WITH CONCURRENT TRANSACTION;
    if (sqlca.sqlcode) {
	fprintf( stderr,
		 "FAILED: Cannot connect to %s (@%s); error=%d, isam=%d\n\n",
		 "WORK DATABASE",
		 host,
		 (int)sqlca.sqlcode,
		 (int)sqlca.sqlerrd[1] );
	if (sqlca.sqlcode == -27000)
	    fprintf( stderr,
		     "FAILED: You cannot use a shared memory connection in "
		     "$INFORMIXSERVER or the -h option \nwithout the -S "
		     "option.  Specify a pipe or network connection or -S "
		     "and rerun.\n\n" );
	exit(1);
    }
    if (waitlen == 0)
	strcpy( waitstr_dostats, "SET LOCK MODE TO NOT WAIT" );
    else if (waitlen < 0)
	strcpy( waitstr_dostats, "SET LOCK MODE TO WAIT" );
    else /* (waitlen > 0) */
	sprintf( waitstr_dostats, "SET LOCK MODE TO WAIT %d", waitlen );
    EXEC SQL EXECUTE IMMEDIATE :waitstr_dostats;
    if (sqlca.sqlcode < 0) {
	fprintf( stderr,
		 "Warning: error setting lock mode to%s wait; "
		 "error=%d, isam=%d.\n",
		 ((waitlen == 0) ? " not" : ""),
		 (int)sqlca.sqlcode,
		 (int)sqlca.sqlerrd[1] );
    }

    /* Version detection. */
    EXEC SQL DATABASE 'sysmaster';
    if (sqlca.sqlcode == -387) {
	fprintf( stderr,
		 "FAILED: No permissions to database(sysmaster@%s).  "
		 "Rerun as: informix.\n",
		 host );
	exit(1);
    } else if (sqlca.sqlcode) {
	fprintf( stderr,
		 "FAILED: Cannot access database (sysmaster); error=%d, isam=%d\n",
		 (int)sqlca.sqlcode,
		 (int)sqlca.sqlerrd[1] );
	exit(1);
    }
    EXEC SQL SELECT DBINFO( 'version', 'major' )
	INTO :major_vers
	FROM "informix".systables where tabid = 1;
    if (sqlca.sqlcode == -728) {
	/* No sysmaster database.  Must be SE or 5.xx engine.  Quit now! */
	fprintf( stderr, "dostats_ng only supports IDS version 10.00 and later!\n" );
	exit(1);
    } else if (sqlca.sqlcode < 0) {
	fprintf( stderr,
		 "FAILED: Error fetching major version id.  SQL=%d, ISAM=%d.\n",
		 (int)sqlca.sqlcode,
		 (int)sqlca.sqlerrd[1] );
	exit( 1 );
    } else {
	if (major_vers < 10) {
	    fprintf( stderr,
		     "dostats_ng does not support IDS versions prior to 10.00!\n" );
	    exit( 1 );
	}

	EXEC SQL SELECT DBINFO( 'version', 'minor' )
	    INTO :minor_vers
	    FROM "informix".systables where tabid = 1;
	if (sqlca.sqlcode < 0) {
	    fprintf( stderr,
		     "FAILED: Error fetching minor version id.  SQL=%d, ISAM=%d.\n",
		     (int)sqlca.sqlcode,
		     (int)sqlca.sqlerrd[1] );
	    exit( 1 );
	}
	EXEC SQL SELECT DBINFO( 'version', 'level' )
	    INTO :level_vers
	    FROM "informix".systables where tabid = 1;
	if (sqlca.sqlcode < 0) {
	    fprintf( stderr,
		     "FAILED: Error fetching patch level string.  SQL=%d, ISAM=%d.\n",
		     (int)sqlca.sqlcode,
		     (int)sqlca.sqlerrd[1] );
	    exit( 1 );
	}
	EXEC SQL SELECT DBINFO( 'version', 'server-type' )
	    INTO :server_type
	    FROM "informix".systables where tabid = 1;
	if (sqlca.sqlcode < 0) {
	    fprintf( stderr,
		     "FAILED: Error fetching server type string.  SQL=%d, ISAM=%d.\n",
		     (int)sqlca.sqlcode,
		     (int)sqlca.sqlerrd[1] );
	    exit( 1 );
	}
    }

    if (major_vers < 11 || (major_vers == 11 && minor_vers < 70)) {
	if (force_run) {
	    fprintf( stderr,
		     "--force_run not supported for this Informix version (%d.%d).  Ignoring.\n",
		     major_vers,
		     minor_vers );
	    force_run = 0;
	}
	if (auto_run) {
	    fprintf( stderr,
		     "--auto_run not supported for this Informix version (%d.%d).  Ignoring.\n",
		     major_vers,
		     minor_vers );
	    auto_run = 0;
	}
    }

    if (do_scheduling && major_vers < 11) {
	/* No scheduler available! */
	fprintf( stderr,
		 "Task scheduling not supported by server version: %d.%d.%s\n",
		 major_vers,
		 minor_vers,
		 level_vers );
	exit( 1 );
    }

    if (use_aus_thresholds && major_vers < 11) {
	/* No scheduler available! */
	fprintf( stderr,
		 "Auto Update Statistics thresholds not supported by server version: %d.%d.%s\n",
		 major_vers,
		 minor_vers,
		 level_vers );
	exit( 1 );
    } else if (use_aus_thresholds) {
	if (!doPDQ) {
	    EXEC SQL SELECT value
		INTO :tablePDQ
		FROM sysadmin:"informix".ph_threshold
		WHERE name = 'AUS_PDQ';
	    if (sqlca.sqlcode < 0) {
		fprintf( stderr,
			 "FAILED: Cannot fetch AUS PDQPRIORIT setting; error=%d, isam=%d\n\n",
			 (int)sqlca.sqlcode,
			 (int)sqlca.sqlerrd[1] );
		exit( 1 );
	    } else if (sqlca.sqlcode == SQLNOTFOUND) {
		fprintf( stderr,
			 "WARNING: AUS PDQPRIORITY setting is not available.  Using default.\n\n" );
	    } else
		doPDQ = 1; /* Force this to make sure an explicit setting of "0" isn't overridden below. */
	}
	if (!haveDistAgingLimit) {
	    EXEC SQL SELECT value
		INTO :DistAgingLimit
		FROM sysadmin:"informix".ph_threshold
		WHERE name = 'AUS_AGE';
	    if (sqlca.sqlcode < 0) {
		fprintf( stderr,
			 "FAILED: Cannot fetch AUS Aging setting; error=%d, isam=%d\n\n",
			 (int)sqlca.sqlcode,
			 (int)sqlca.sqlerrd[1] );
		exit( 1 );
	    } else if (sqlca.sqlcode == SQLNOTFOUND) {
		fprintf( stderr,
			 "WARNING: AUS Aging setting is not available.  Using default.\n\n" );
	    }
	}
	if (!haveBrowseThreshHold) {
	    EXEC SQL SELECT value
		INTO :BrowseThreshHold
		FROM sysadmin:"informix".ph_threshold
		WHERE name = 'AUS_CHANGE';
	    if (sqlca.sqlcode < 0) {
		fprintf( stderr,
			 "FAILED: Cannot fetch Change Browsing setting; error=%d, isam=%d\n\n",
			 (int)sqlca.sqlcode,
			 (int)sqlca.sqlerrd[1] );
		exit( 1 );
	    } else if (sqlca.sqlcode == SQLNOTFOUND) {
		fprintf( stderr,
			 "WARNING: AUS Change Browsing setting is not available.  Using default.\n\n" );
	    }
	}
	if (!have_small_threshold) {
	    EXEC SQL SELECT value
		INTO :aus_small_table
		FROM sysadmin:"informix".ph_threshold
		WHERE name = 'AUS_SMALL_TABLES';
	    if (sqlca.sqlcode < 0) {
		fprintf( stderr,
			 "FAILED: Cannot fetch Small Table Threshold setting; error=%d, isam=%d\n\n",
			 (int)sqlca.sqlcode,
			 (int)sqlca.sqlerrd[1] );
		exit( 1 );
	    } else if (sqlca.sqlcode == SQLNOTFOUND) {
		fprintf( stderr,
			 "WARNING: AUS Small Table Threshold setting is not available.  Using default.\n\n" );
		small_tables_count = 0;
		have_small_threshold = 2;
	    } else
		small_tables_count = aus_small_table;
	}
    }
    if (tablePDQ == 0 && !doPDQ) {
	tablePDQ = 10;
    }
    if (verbose > 0) {
	fprintf( stderr,
		 "PDQPRIORITY for table processing set to: %d.\n",
		 tablePDQ );
	fprintf( stderr,
		 "Distribution Aging limit set to: %d days\n",
		 DistAgingLimit );
	fprintf( stderr,
		 "Table browse threshhold set to: %f%%\n",
		 BrowseThreshHold );
	if (small_tables_high) {
	    fprintf( stderr,
		     "Small table processing enabled.\n" );
	    if (use_aus_thresholds == 1 && have_small_threshold == 0) {
		fprintf( stderr,
			 "\tUsing AUS_SMALL_TABLES threshold: %d.\n",
			 small_tables_count );
	    } else {
		fprintf( stderr,
			 "\tSmall table threshold set to: %d.\n",
			 small_tables_count );
	    }
	}
    }
    pstemp_env = getenv( "PSORT_NPROCS" );
    if (pstemp_env == (char *)NULL || strlen( pstemp_env ) == 0) {
	fprintf( stderr,
		 "   Warning!  Optimized servers produce statistics fastest with PSORT_NPROCS set.\n" );
    }
    pstemp_env = getenv( "PSORT_DBTEMP" );
    if (pstemp_env == (char *)NULL || strlen( pstemp_env ) == 0) {
	fprintf( stderr,
		 "   Warning!  Optimized servers produce statistics fastest with PSORT_DBTEMP set.\n" );
    }
    pstemp_env = getenv( "DBUPSPACE" );
    if (pstemp_env == (char *)NULL || strlen( pstemp_env ) == 0) {
	fprintf( stderr,
		 "   Warning!  Optimized servers produce statistics fastest with DBUPSPACE set.\n" );
    }

    if (sampleset && major_vers < 11) {
	fprintf( stderr, "   Minimum sample size specified but unsupported.  Ignoring.\n" );
	sampleset = 0;
    }

    if (do_procedure) {
	if (do_scheduling || !proc_local) {
	    /* Connection to sysadmin for scheduling if needed. */
	    if (username[0] != (char)0) {
		if (passwd[0] != (char)0)
		    EXEC SQL CONNECT TO :connhost AS 'SCHEDULER'
			USER :username USING :passwd
			WITH CONCURRENT TRANSACTION;
		else {
		    fprintf( stderr,
			     "Cannot specify INFORMIXUSER without INFORMIXPASS!\n" );
		    usage( argv[0] );
		    exit( 1 );
		}
	    } else
		EXEC SQL CONNECT TO :connhost AS 'SCHEDULER'
		    WITH CONCURRENT TRANSACTION;
	    if (sqlca.sqlcode) {
		fprintf( stderr,
			 "FAILED: Cannot connect to %s (@%s); error=%d, isam=%d\n\n",
			 "sysadmin",
			 host,
			 (int)sqlca.sqlcode,
			 (int)sqlca.sqlerrd[1] );
		if (sqlca.sqlcode == -27000) {
		    fprintf( stderr,
			     "Task scheduling is not possible with a shared memory connection!\n"
			     "Change to a networked or pipe connection type.\n" );
		}
		exit(1);
	    }
	    EXEC SQL DATABASE "sysadmin";
	    if (sqlca.sqlcode == -387) {
		fprintf( stderr,
			 "Cannot open sysadmin database SQL: %d, ISAM: %d.\n"
			 "  You must be user informix or a DBSA user to use task scheduling!\n",
			 sqlca.sqlcode,
			 sqlca.sqlerrd[1] );
		exit( 1 );
	    } else if (sqlca.sqlcode == -329 && sqlca.sqlerrd[1] == -111) {
		if (do_scheduling)
		    fprintf( stderr,
			     "Sysadmin database does not exist!\nSysadmin must exist to use --schedule\n" );
		else if (do_proconly)
		    fprintf( stderr,
			     "Sysadmin database does not exist!\nYou can only use --procedure with --proc-local!\n" );
		exit( 1 );
	    } else if (sqlca.sqlcode < 0) {
		fprintf( stderr,
			 "Cannot open sysadmin database.  SQL: %d  ISAM: %d.\n",
			 sqlca.sqlcode,
			 sqlca.sqlerrd[1] );
		exit( 1 );
	    }
	}

	if (!proc_local)
	    Create_Process_Table( "sysadmin" );

	Out_Func = Store_Statement;
    } else {
	Out_Func = Write_Statement;
    }

    if (!singledb) {
	EXEC SQL SET CONNECTION 'DBASES';
	if (sqlca.sqlcode) {
	    fprintf( stderr,
		     "FAILED %d: Cannot set connection to (%s); error=%d, isam=%d\n",
		     __LINE__,
		     "DBASES",
		     (int)sqlca.sqlcode,
		     (int)sqlca.sqlerrd[1] );
	    exit(1);
	}
    }

    /* Loop on all <selected> databases. */
    for (;;) {
	fprintf( stderr, "\n" );

	/* Switch to connection for database work. */
	if (singledb) {
	    /* This is shared memory connection.  Simulate the fetch. */
	    if (once) {
		sqlca.sqlcode = 100;
		break;
	    } else {
		sqlca.sqlcode = 0;
		once = 1;
	    }
	} else {
	    EXEC SQL SET CONNECTION 'DBASES';
	    conn = 1;
	    if (sqlca.sqlcode) {
		fprintf( stderr,
			 "FAILED %d: Cannot set connection to (%s); "
			 "error=%d, isam=%d\n",
			 __LINE__,
			 "DBASES",
			 (int)sqlca.sqlcode,
			 (int)sqlca.sqlerrd[1] );
		exit(1);
	    }
	    EXEC SQL FETCH dbc INTO :databasename;
	    if (sqlca.sqlcode == SQLNOTFOUND) {
		if (!processed)
		    fprintf( stderr,
			     "FAILED\nNo databases selected!.\n" );
		break;
	    } else if (sqlca.sqlcode == -329) {
		fprintf( stderr,
			 "FAILED: No permissions to database.  Rerun as: informix.  "
			 "Continuing.\n" );
		continue;
	    } else if (sqlca.sqlcode < 0) {
		fprintf( stderr,
			 "FAILED: Cannot fetch dbc cursor, SQLerr=%d, ISAM=%d.\n",
			 (int)sqlca.sqlcode,
			 (int)sqlca.sqlerrd[1] );
		exit(1);
	    }
	    processed++;
	}
	TimeStart( &dbsstart, ExpandReport );

	if (verbose >= 0)
	    fprintf( stderr, "Working on database: %s.\n", databasename );

	if (do_procedure) {

	    if (schedfile != (FILE *)NULL) {
		/* Finish up: insert scheduler record. */

		if (do_proconly) {
		    if (verbose >= 2)
			fprintf( stderr, "Dropping procedure(1): %s.\n", schedprocname );
		    snprintf( schedule_stmt,
			      sizeof schedule_stmt,
			      "DROP PROCEDURE %s;",
			      schedprocname );
		    EXEC SQL EXECUTE IMMEDIATE :schedule_stmt;
		    if (sqlca.sqlcode != 0 && sqlca.sqlcode != -674) {
			fprintf( stderr,
				 "Error dropping old copy of procedure!  SQL: %d  ISAM: %d\n",
				 sqlca.sqlcode,
				 sqlca.sqlerrd[1] );
			exit( 1 );
		    }
		}

		if (do_scheduling)
		    fprintf( stderr,
			     "Scheduling stats update for database: %s.\n",
			     databasename );
		else
		    fprintf( stderr,
			     "Creating stats procedure %s in database: %s.\n",
			     schedprocname,
			     databasename );

		EXEC SQL CREATE ROUTINE FROM :schedprocfilename;
		if (sqlca.sqlcode != 0) {
		    fprintf( stderr,
			     "Error creating scheduling procedure: %s!  SQL: %d  ISAM: %d\n",
			     schedprocname,
			     sqlca.sqlcode,
			     sqlca.sqlerrd[1] );
		    exit( 1 );
		}
		unlink( schedprocfilename );
		EXEC SQL SET CONNECTION 'SCHEDULER';
		if (sqlca.sqlcode != 0) {
		    fprintf( stderr,
			     "Error changing connections to SCHEDULER!  SQL: %d  ISAM: %d\n",
			     sqlca.sqlcode,
			     sqlca.sqlerrd[1] );
		    exit( 1 );
		}
		if (do_scheduling) {
		    EXEC SQL SELECT count(*) INTO :OK FROM ph_group WHERE group_name = 'DOSTATS';
		    if (sqlca.sqlcode != 0) {
			fprintf( stderr,
				 "Error verifying DOSTATS task group record!  SQL: %d  ISAM: %d\n",
				 sqlca.sqlcode,
				 sqlca.sqlerrd[1] );
			exit( 1 );
		    }
		    if (OK == 0) {
			EXEC SQL INSERT INTO ph_group (group_id, group_name, group_description)
			    VALUES ( 0, "DOSTATS", "Update statistics generated by dostats(c)." );
			if (sqlca.sqlcode < 0 && sqlca.sqlcode != -268) {
			    fprintf( stderr,
				     "Error inserting DOSTATS task group record!  SQL: %d  ISAM: %d\n",
				     sqlca.sqlcode,
				     sqlca.sqlerrd[1] );
			    exit( 1 );
			}
		    }
		    
		    /* Add scheduling record here. --ASK-- */
		    snprintf( schedule_stmt,
			      sizeof schedule_stmt,
			      "INSERT INTO ph_task( tk_id, "
			      "       tk_name, tk_type, tk_group, tk_description, tk_execute, tk_dbs "
			      "       tk_start_time, tk_stop_time, tk_frequency, tk_Sunday, tk_Monday, "
			      "       tk_Tuesday, tk_Wednesday, tk_Thursday, tk_Friday, tk_Saturday ) "
			      "VALUES ( 0, '%s_task', 'TASK', 'DOSTATS', 'dostats run for %s', "
			      "   'EXECUTE PROCEDURE %s:%s($DATA_TASK_ID, $DATA_SEQ_ID, %s);', "
			      "   %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s );",
			      schedprocname,
			      schedprocname,
			      databasename,
			      schedprocname,
			      sch_onetime ? "\"t\"" : "\"f\"",
			      sch_dbs,
			      sch_starttime[0] == (char)(0) ? "NULL" : sch_starttime,
			      sch_lasttime[0] == (char)(0) ? "NULL" : sch_lasttime,
			      sch_frequency[0] == (char)(0) ? "NULL" : sch_frequency,
			      sch_sunday,
			      sch_monday,
			      sch_tuesday,
			      sch_wednesday,
			      sch_thursday,
			      sch_friday,
			      sch_saturday );
		    EXEC SQL EXECUTE IMMEDIATE :schedule_stmt;
		    if (sqlca.sqlcode != 0) {
			fprintf( stderr,
				 "Error inserting task scheduler record!  SQL: %d  ISAM: %d\n%s\n%*.*s\n",
				 sqlca.sqlcode,
				 sqlca.sqlerrd[1],
				 schedule_stmt,
				 sqlca.sqlerrd[4],
				 sqlca.sqlerrd[4],
				 "^" );
			exit( 1 );
		    }
		}
	    }

	    if (!do_proconly) {
		snprintf( schedprocname,
			  sizeof schedprocname,
			  "updstats_%s_%s_%d",
			  databasename,
			  tflag ? tablename : "all",
			  getpid() );
	    }

	    dumb = getcwd( pwd, sizeof pwd );

	    snprintf( schedprocfilename,
		      sizeof schedprocfilename,
		      "%s/%s.sql",
		      pwd,
		      schedprocname );
	    schedfile = fopen( schedprocfilename, "wt" );
	    outfile = schedfile;

	    fprintf( schedfile,
		     "CREATE PROCEDURE %s( taskid int, seqid int, clearit boolean );\n",
		     schedprocname );
	    fprintf( schedfile,
		     "    DEFINE cont, trailing boolean;\n" );
	    fprintf( schedfile,
		     "    DEFINE part char(800);\n" );
	    fprintf( schedfile,
		     "    DEFINE l_statement char(32767);\n" );
	    fprintf( schedfile,
		     "    DEFINE seq int;\n\n" );
	    fprintf( schedfile,
		     "    ON EXCEPTION IN (-705) \n    END EXCEPTION WITH RESUME;\n\n" );
	    if (verbose > 1) {
		fprintf( schedfile,
			 "    SET DEBUG FILE TO '%s.txt'; \n", schedprocname );
		fprintf( schedfile,
			 "    TRACE ON; \n" );
	    }
	    if (waitlen < 0)
		fprintf( schedfile,
			 "    SET LOCK MODE TO WAIT; \n" );
	    else if (waitlen == 0)
		fprintf( schedfile,
			 "    SET LOCK MODE TO NOT WAIT; \n" );
	    else 
		fprintf( schedfile,
			 "    SET LOCK MODE TO WAIT %d; \n", waitlen );
	    fprintf( schedfile,
		     "    LET l_statement = ''; \n" );
	    fprintf( schedfile,
		     "    LET trailing = 'f'; \n" );
	    fprintf( schedfile,
		     "    FOREACH SELECT sequence, continued, statement \n" );
	    fprintf( schedfile,
		     "            INTO seq, cont, part \n" );
	    fprintf( schedfile,
		     "            FROM %s:dostats_stmts \n",
		     proc_local ? databasename : "sysadmin" );
	    fprintf( schedfile,
		     "            WHERE function = '%s' \n", schedprocname );
	    fprintf( schedfile,
		     "            ORDER BY 1, 2  desc \n" );
	    if (verbose > 1) {
		fprintf( schedfile, "    TRACE part;\n" );
		fprintf( schedfile, "    TRACE l_statement;\n" );
	    }
	    fprintf( schedfile,
		     "        IF (cont) THEN \n" );
	    fprintf( schedfile,
		     "            IF (trailing) THEN \n" );
	    fprintf( schedfile,
		     "                LET part = ' ' || part; \n" );
	    fprintf( schedfile,
		     "            END IF \n" );
	    fprintf( schedfile,
		     "            LET l_statement = rtrim( l_statement ) || rtrim( part ); \n" );
	    fprintf( schedfile,
		     "            IF (substr( part, 768 ) = ' ') THEN \n" );
	    fprintf( schedfile,
		     "               LET trailing = 't'; \n" );
	    fprintf( schedfile,
		     "            ELSE \n" );
	    fprintf( schedfile,
		     "               LET trailing = 'f'; \n" );
	    fprintf( schedfile,
		     "            END IF \n" );
	    fprintf( schedfile,
		     "        ELSE \n" );
	    fprintf( schedfile,
		     "            IF (trailing) THEN \n" );
	    fprintf( schedfile,
		     "                LET part = ' ' || part; \n" );
	    fprintf( schedfile,
		     "            END IF \n" );
	    fprintf( schedfile,
		     "            LET l_statement = rtrim( l_statement ) || rtrim( part ); \n" );
	    fprintf( schedfile,
		     "            EXECUTE IMMEDIATE l_statement; \n" );
	    fprintf( schedfile,
		     "            IF SQLCODE = -705 THEN \n" );
	    fprintf( schedfile,
		     "                CONTINUE FOREACH; \n" );
	    fprintf( schedfile,
		     "            END IF \n" );
	    fprintf( schedfile,
		     "            LET trailing = 'f'; \n" );
	    fprintf( schedfile,
		     "            LET l_statement = ''; \n" );
	    fprintf( schedfile,
		     "        END IF \n" );
	    fprintf( schedfile,
		     "    END FOREACH; \n" );
	    if (do_scheduling)
		fprintf( schedfile,
			 "    IF (clearit = 't') THEN \n"
			 "        UPDATE sysadmin:ph_task SET tk_enable = 'f' WHERE tk_id = taskid; \n"
			 "        DELETE FROM sysadmin:dostats_stmts WHERE function = '%s'; \n "
			 "    END IF\n",
			 schedprocname );
	    if (verbose > 1)
		fprintf( schedfile, "    TRACE OFF; \n" );
	    fprintf( schedfile, "end procedure \n" );
	    fprintf( schedfile,
		     "DOCUMENT \n" );
	    fprintf( schedfile,
		     "    'Procedure created by dostats!'; \n" );
	    fflush( schedfile );
	    fclose( schedfile );
	}

	if (!singledb) {
	    /* Switch to connection for table work. */
	    EXEC SQL SET CONNECTION 'WORK';
	    conn = 2;
	    if (sqlca.sqlcode) {
		fprintf( stderr,
			 "FAILED: %d: Cannot set connection to (%s); "
			 "error=%d, isam=%d\n",
			 __LINE__,
			 "WORK",
			 (int)sqlca.sqlcode,
			 (int)sqlca.sqlerrd[1] );
		exit(1);
	    }
	}

	EXEC SQL DATABASE :databasename;
	if (sqlca.sqlcode == -387) {
	    fprintf( stderr,
		     "FAILED: No permissions to database(%s@%s).  "
		     "Rerun as: informix.  "
		     "Continuing.\n",
		     databasename,
		     host );
	    continue;
	} else if (sqlca.sqlcode) {
	    fprintf( stderr,
		     "FAILED: Cannot access database (%s); error=%d, isam=%d\n",
		     databasename,
		     (int)sqlca.sqlcode,
		     (int)sqlca.sqlerrd[1] );
	    exit(1);
	}
	if (sqlca.sqlwarn.sqlwarn1 == 'W' && sqlca.sqlwarn.sqlwarn2 == 'W') {
	    ANSIMode = 1;
	    Logging = 1;
	} else if (sqlca.sqlwarn.sqlwarn1 == 'W') {
	    ANSIMode = 0;
	    Logging = 1;
	} else {
	    ANSIMode = 0;
	    Logging = 0;
	}

	if (proc_local)
	    Create_Process_Table( databasename );  /* Create the statement list table in each database. */

	if (!do_procedure && outfile != (FILE *)NULL) {
	    fprintf( outfile, "DATABASE %s;\n", databasename );
	}

	if (Logging == 1) {
	    switch (isolation_level[0]) {
	    case 'd':
		EXEC SQL SET ISOLATION DIRTY READ;
		if (outfile != (FILE *)NULL)
		    Out_Func( "SET ISOLATION DIRTY READ;" );
		break;
	    case 'c':
		EXEC SQL SET ISOLATION COMMITTED READ;
		if (outfile != (FILE *)NULL)
		    Out_Func( "SET ISOLATION COMMITTED READ;" );
		break;
	    case 'l':
		EXEC SQL SET ISOLATION COMMITTED READ LAST COMMITTED;
		if (outfile != (FILE *)NULL)
		    Out_Func( "SET ISOLATION COMMITTED READ LAST COMMITTED;" );
		break;
	    }
	}
	if (!proconly) {
	    if (doexplain) {
	        /* Send SET EXPLAIN to the runtime script if not executing. */
		if (outfile != (FILE *)NULL)
		    Out_Func( "SET EXPLAIN ON;" );
		/* Always run set explain on dostats itself, if requested. */
		EXEC SQL SET EXPLAIN ON;
	    }
	    if (outfile == (FILE *)NULL)
		EXEC SQL EXECUTE IMMEDIATE :pdqstmt_2;
	    else
		Out_Func( pdqstmt_2 );

	    if (drop_distributions) {
		if (verbose > 0) {
		    fprintf( stderr,
			     "Clearing all Distributions):..." );
		}
		TimeStart( &stmtstart, ExpandReport );
		if (outfile == (FILE *)NULL) {
		    if (verbose > 0) {
			fprintf( stderr,
				 "    %s%s%s(Clearing Old Distributions):...",
				 QuotedIdents ? "\"" : "",
				 tabinfo.tabname,
				 QuotedIdents ? "\"" : "" );
		    }
		    if (major_vers >= 11) {
			snprintf( statement2, sizeof statement2,
				  "UPDATE STATISTICS LOW DROP DISTRIBUTIONS ONLY;" );
			EXEC SQL EXECUTE IMMEDIATE :statement2;
			if (sqlca.sqlcode < 0) {
			    fprintf( stderr,
				     "Error dropping distributions!  SQL: %d, ISAM: %d.\n",
				     (int)sqlca.sqlcode,
				     (int)sqlca.sqlerrd[1] );
			    exit(1);
			}
		    } else {
			EXEC SQL DELETE FROM sysdistrib;
			if (sqlca.sqlcode < 0) {
			    fprintf( stderr,
				     "Error dropping table distributions!  SQL: %d, ISAM: %d.\n",
				     (int)sqlca.sqlcode,
				     (int)sqlca.sqlerrd[1] );
			    exit(1);
			} else if (verbose > 0)
			    fprintf( stderr,
				     "%d sysdistrib rows deleted.\n",
				     sqlca.sqlerrd[2] );
		    }
		} else {
		    if (major_vers >= 11) {
			Out_Func( "UPDATE STATISTICS LOW DROP DISTRIBUTIONS ONLY;" );
		    } else {
			Out_Func( "DELETE FROM sysdistrib;" );
		    }
		}
		if (verbose > 0) {
		    TimeMark( stmtstart, ExpandReport, "SUCCESSFUL", 0, 0 );
		} else {
		    fprintf( stderr, "\n" );
		}
	    }

	    if (have_high_list) {
		char line[2048];
		int highsz = 1000;

		highfile = fopen( high_list_file, "rt" );
		if (highfile == (FILE *)NULL) {
		    fprintf( stderr, "Error opening high column list: %d.\n", errno );
		    exit( 1 );
		}
		force_high_cols = malloc( highsz * sizeof (high_list_t) );
		/* Read in the list of columns to force high stats */
		for (num_high = 0; ; num_high++ ) {
		    char *s;

		    if (num_high >= highsz) {
			highsz += 1000;
			force_high_cols = realloc( force_high_cols, highsz );
		    }
		    s = fgets( line, sizeof line, highfile );
		    if (feof( highfile ) == 1)
			break;
		    sscanf( line,
			    "%s %s %s\n",
			    force_high_cols[num_high].dbsname,
			    force_high_cols[num_high].tabname,
			    force_high_cols[num_high].colname );
		}
		/* Sort the list be databasename, tabname, colname */
		qsort( force_high_cols, num_high, sizeof (high_list_t), listcomp );
	    }

	    if (!statement) { /* First database, build statement. */
		/* Select for column names. */
		snprintf( statement2, sizeof statement2,
			  "SELECT colno, colname, coltype "
			  "    FROM \"informix\".syscolumns "
			  "    WHERE tabid = ? "
			  "    ORDER BY colno" );
		/* Select for tables and index leading columns. */
		appendStr( &statement,
			   &stmt_len,
			   "SELECT a.tabid, a.partnum, a.nrows, a.tabname, a.owner, b.owner, b.idxname, "
			   "indexkeyarray_out(b.indexkeys) AS allparts, " );

		if (doDistAging)
		    appendStr( &statement,
			       &stmt_len,
			       "(  SELECT (today - MIN(d.constructed)) "
			       "   FROM \"informix\".sysdistrib d "
			       "   WHERE d.tabid = a.tabid ) " );
		else
		    appendStr( &statement,
			       &stmt_len,
			       "0 " );

		if (major_vers > 11 || (major_vers == 11 && minor_vers >= 70)) {
		    appendStr( &statement,
			       &stmt_len,
			       ", a.statchange " );
		} else {
		    appendStr( &statement,
			       &stmt_len,
			       ", NULL::INT " );
		}

		appendStr( &statement,
			   &stmt_len,
			   "FROM \"informix\".systables a, outer (\"informix\".sysindices b, outer "
			   "\"informix\".sysobjstate s) " );

		if (major_vers >= 11 && subquery) {
		    /* Use a dynamic table so ORDER BY is supported. */
		    appendStr( &statement,
			       &stmt_len,
			       ", ( " );
		    appendStr( &statement,
			       &stmt_len,
			       subquery );
		    appendStr( &statement,
			       &stmt_len,
			       ") as flt " );
		}

		appendStr( &statement,
			   &stmt_len,
			   "WHERE a.tabid = b.tabid AND tabtype = 'T' " );

		if (!dosystem)
		    appendStr( &statement, &stmt_len, "AND a.tabid >= 100 " );

		appendStr( &statement, &stmt_len,
			   "AND a.tabid = s.tabid "
			   "AND a.am_id = 0 "
			   "AND s.objtype = 'I' "
			   "AND b.idxname = s.name "
			   "AND s.state != 'D' " );
		if (tflag && strcmp( tablename, "none" ) != 0) {
		    if (includes || (subquery && major_vers < 11))
			appendStr( &statement,
				   &stmt_len,
				   "AND (a.tabname MATCHES '" );
		    else
			appendStr( &statement,
				   &stmt_len,
				   "AND a.tabname MATCHES '" );

		    appendStr( &statement, &stmt_len, tablename );
		    appendStr( &statement, &stmt_len, "' " );
		} else if (includes || (subquery && major_vers < 11)) {
		    appendStr( &statement, &stmt_len, "AND (a.tabname IS NULL " );
		}
		if (subquery && major_vers < 11) {
		    appendStr( &statement, &stmt_len, " OR a.tabname IN (" );
		    appendStr( &statement, &stmt_len, subquery );
		    appendStr( &statement, &stmt_len, ") " );
		    appendStr( &statement, &stmt_len, ") " );
		} else if (subquery) {
		    appendStr( &statement, &stmt_len, "AND a.tabname = flt.tabname " );
		}
		if (includes ) {
		    appendStr( &statement, &stmt_len, "OR a.tabname IN " );
		    appendStr( &statement, &stmt_len, includes );
		    appendStr( &statement, &stmt_len, ") " );
		}
		if (exquery) {
		    appendStr( &statement, &stmt_len, exquery );
		}
		if (excludes) {
		    appendStr( &statement,
			       &stmt_len,
			       "AND a.tabname NOT IN " );
		    appendStr( &statement, &stmt_len, excludes );
		}
		/*		    appendStr( &statement,
				    &stmt_len,
				    "AND indexkeyarray_out(b.indexkeys) NOT LIKE '<%' " ); 
				    -- Enable to skip functional keys without columns -- */
		appendStr( &statement,
			   &stmt_len,
			   "ORDER BY 1, 3, 8, 7; " );
	    }
	    EXEC SQL PREPARE stmt2 FROM :statement2;
	    if (sqlca.sqlcode != 0) {
		fprintf( stderr,
			 "FAILED\nCannot PREPARE stmt2.  SQL=%d, ISAM=%d, "
			 "POS=%d.\n",
			 (int)sqlca.sqlcode,
			 (int)sqlca.sqlerrd[1],
			 (int)sqlca.sqlerrd[4] );
		exit(1);
	    }
	    EXEC SQL DECLARE curs2 CURSOR WITH HOLD FOR stmt2;
	    if (sqlca.sqlcode != 0) {
		fprintf( stderr,
			 "FAILED\nCannot DECLARE curs2.  "
			 "SQL=%d, ISAM=%d, POS=%d.\n",
			 (int)sqlca.sqlcode,
			 (int)sqlca.sqlerrd[1],
			 (int)sqlca.sqlerrd[4] );
		exit(1);
	    }
	    EXEC SQL
		PREPARE stmt1 FROM :statement;
	    if (sqlca.sqlcode != 0) {
		fprintf( stderr,
			 "FAILED: Cannot PREPARE select statement for table data, "
			 "error: %d.\n",
			 (int)sqlca.sqlcode );
		fprintf( stderr, "Error at character %d.\nStatement is:\n",
			 (int)sqlca.sqlerrd[4] );
		fprintf( stderr, "%s\n\n",statement );
		exit(2);
	    } else if (verbose > 2) {
		fprintf( stderr,
			 "Tables selected with:\n%s;\n",
			 statement );
	    }
	
	    EXEC SQL
		DECLARE a1 CURSOR WITH HOLD FOR stmt1;
	    if (sqlca.sqlcode != 0) {
		fprintf( stderr,
			 "FAILED: Cannot declare a1 cursor; Error = %d, %d\n",
			 (int)sqlca.sqlcode,
			 (int)sqlca.sqlerrd[1] );
		exit(1);
	    }
	
	    EXEC SQL
		OPEN a1;
	    if (sqlca.sqlcode == SQLNOTFOUND) {
		fprintf( stderr, "FAILED: No table matching \"%s\".\n", tablename );
		exit(3);
	    } else if (sqlca.sqlcode != 0) {
		fprintf( stderr,
			 "FAILED: Cannot open a1 cursor; Error = %d, %d (%s, %s)\n",
			 (int)sqlca.sqlcode,
			 (int)sqlca.sqlerrd[1],
			 (char *)sqlca.sqlerrm,
			 (char *)sqlca.sqlerrp );
		exit(1);
	    }

	    lasttbl[0] = (char)0;
	    i = 0;
	    /* Loop on tables/indexes. */
	    memset( &lastidx, 0, sizeof lastidx );
		
	    for (;;) {
		EXEC SQL FETCH a1
		    INTO :tabinfo.tabid,
		    :tabinfo.partnum,
		    :tabinfo.nrows,
		    :tabinfo.tabname,
		    :tabinfo.tabowner,
		    :tabinfo.idxowner,
		    :tabinfo.idxname,
		    :tabinfo.keystring : tabinfo.colind,
		    :tabinfo.days : tabinfo.daysind,
		    :tabinfo.statchange : tabinfo.statchgind;

		if (sqlca.sqlcode ) {
		    if (sqlca.sqlcode == SQLNOTFOUND)
			break;
		    else {
			fprintf( stderr,
				 "FAILED: Cannot FETCH table data (a1).  SQLCODE = %d, ISAM = %d.\nStatement is:\n",
				 (int)sqlca.sqlcode,
				 (int)sqlca.sqlerrd[1] );
			fputs( statement, stderr );
			fputs( "\n\n", stderr );
			exit(1);
		    }
		} else {
		    int skip = 0;

		    rowcnt = getrowcount( tabinfo.tabowner, tabinfo.tabname, tabinfo.tabid, tabinfo.partnum );
		    
		    if (ANSIMode == 1 || InclOwner == 1)
		        snprintf( ownerstr, sizeof ownerstr, "\"%s\".", tabinfo.tabowner );
		    else
		        memset( ownerstr, 0, sizeof ownerstr );

		    if (tabinfo.colind > 0)
			fprintf( stderr,
				 "WARNING: Key information too long (%d)!  "
				 "Some keys may not be processed.\n",
				 tabinfo.colind );
		    /* Need to decode the keystring into colno's. */
		    memset( tabinfo.colno, 0, sizeof tabinfo.colno );
		    InitKeyStr( tabinfo.keystring );
		    for (i=0; i < 16; i++) {
			tabinfo.colno[i] = ParseKeyStr();
			if (tabinfo.colno[i] == 0)
			    break;
		    }

		    /* Ignore multiple indexes on a table with identical keys
                       except for the direction of each (ASC/DESC). */
		    if (tabinfo.tabid == lastidx.tabid &&
			memcmp( &(tabinfo.colno), &(lastidx.colno), sizeof tabinfo.colno ) == 0)
			continue;

		    if (doBrowse || doDistAging) {
			
			if (doBrowse) {
			    /* Verify rowcount out of range, else continue. */
			    double diff;

			    nrows = tabinfo.nrows;
			    if (nrows > 0 || rowcnt != nrows) {
				if (nrows > 0) {
				    diff = (((double)abs( nrows - rowcnt )
					     / (double)nrows)
					    * (double)100.00);
				    if ((tabinfo.statchgind == -1 || !use_aus_thresholds) && diff < BrowseThreshHold) {
					skip = 1; /* Skip this table */
				    } else if (use_aus_thresholds && tabinfo.statchgind == 0 && diff < (double)tabinfo.statchange ) {
					skip = 1;
				    }
				} else /* No rows before.  Force stats to update! */
				    skip = 0;
			    } else /* No rows, no change. */
				skip = 1;
			}
			if (doDistAging && skip) {
			    /* Use Distribution Aging to include tables. */ 
			    if ((tabinfo.daysind < 0 && rowcnt > 0)
				|| (tabinfo.daysind == 0 && tabinfo.days > DistAgingLimit))
				skip = 0;
			} else if (doDistAging && !doBrowse) {
			    /* Use Distribution Aging to eliminate tables. */
			    if ((tabinfo.daysind == 0 && tabinfo.days <= DistAgingLimit)
				|| (tabinfo.daysind < 0 && rowcnt == 0))
				skip = 1;
			}
			if (skip) /* Skip this table it does not pass Browse and Aging tests. */
			    continue;
		    }
		    if (strcmp( lasttbl, tabinfo.tabname ) != 0) {

			/* Unless this is the first table, if optimizations followed process last table. */
			if (lasttbl[0] != (char)0) {
			    int col, highlen, medlen;
			    int medfirst = 1, highfirst = 1;
			    char highcols[32768], medcols[32768];

			    medlen = 32767 - (56 + strlen( lasttbl ));
			    highlen = medlen + 2;
			    highcols[0] = (char)0;
			    medcols[0] = (char)0;
			    for (col=0; col < ncols; col++) {
				if ((strlen( highcols ) + strlen( columnnames[col] ) + 1 ) > highlen) {
				    /* Name list is getting too long.  Issue command now and start over. */
				    snprintf( tempstr, sizeof tempstr,
					      "UPDATE STATISTICS HIGH FOR TABLE %s%s%s%s (%s)%s%s%s%s;",
					      lastowner,
					      QuotedIdents ? "\"" : "",
					      lasttbl,
					      QuotedIdents ? "\"" : "",
					      highcols,
					      resset ? resstring : "",
					      (NoDistOnly) ? "" : " DISTRIBUTIONS ONLY",
					      (force_run) ? " FORCE" : "",
					      (auto_run) ? " AUTO" : "" );
				    if (doindex) {
					if (outfile == (FILE *)NULL) {
					    if (verbose >= 0) {
						TimeStart( &stmtstart, ExpandReport );
						fprintf( stderr, "        %s(HIGH)...", highcols );
					    }

					    EXEC SQL EXECUTE IMMEDIATE :tempstr;
					    if (sqlca.sqlcode == -313) {
						fprintf( stderr,
							 "FAILED\nNo permissions to update statistics in"
							 " database: %s@%s.\nRerun as informix.  "
							 "Continuing with stored procedures anyway.\n",
							 databasename,
							 host );
						break;
					    } else if (sqlca.sqlcode == -740) {
						fprintf( stderr,
							 "FAILED\nMinimum resolution for this table is: %f\n",
							 minres( l_rowcnt ) );
						exit(3);
					    } else if (sqlca.sqlcode) {
						fprintf( stderr,
							 "FAILED(1)\nCannot Update statistics for table: "
							 "%s%s%s%s, column %s; "
							 "\n\tSQLCODE = %d, ISAM = %d, POS = %d\n",
							 ownerstr,
							 QuotedIdents ? "\"" : "",
							 lasttbl,
							 QuotedIdents ? "\"" : "",
							 columnnames[column],
							 (int)sqlca.sqlcode,
							 (int)sqlca.sqlerrd[1],
							 (int)sqlca.sqlerrd[4] );
						if (sqlca.sqlcode == -201 || sqlca.sqlcode == -352)
						    fprintf( stderr, "%s\n", tempstr );
						if (!noexit) {
						    exit(do_err_cnt ? error_ret : 0);
						} else {
						    noexit--;
						    error_ret++;
						}
					    } else
						TimeMark( stmtstart, ExpandReport, "SUCCESSFUL", 0, 0 );
					} else {
					    if (verbose >= 0)
						fprintf( stderr, "\t%s(HIGH)", highcols );
					    Out_Func( tempstr );
					    if (verbose >= 0)
						fprintf( stderr, "\n" );
					}
				    }
				    if (ANSIMode)
					if (!outfile) {
					    EXEC SQL COMMIT WORK;
					} else if (Logging) {
					    Out_Func( "COMMIT WORK;" );
					}
				    highfirst = 1;
				    highcols[0] = (char)0;
				}
				if (tabmedium && (strlen( medcols ) + strlen( columnnames[col] ) + 1) > medlen) {
				    /* Name list is getting too long.  Issue command now and start over. */
				    snprintf( tempstr, sizeof tempstr,
					      "UPDATE STATISTICS MEDIUM FOR TABLE %s%s%s%s (%s)%s%s%s%s%s;",
					      lastowner,
					      QuotedIdents ? "\"" : "",
					      lasttbl,
					      QuotedIdents ? "\"" : "",
					      medcols,
					      sampleset ? samplestring : "",
					      mresset ? mresstring : "",
					      (mresset && confset) ? confstring : "",
					      (force_run) ? " FORCE" : "",
					      (auto_run) ? " AUTO" : "" );
				    if (doindex) {
					if (outfile == (FILE *)NULL) {
					    if (verbose >= 0) {
						TimeStart( &stmtstart, ExpandReport );
						fprintf( stderr,
							 "        %s(MEDIUM)...", medcols );
					    }
					    EXEC SQL EXECUTE IMMEDIATE :tempstr;
					    if (sqlca.sqlcode == -313) {
						fprintf( stderr,
							 "FAILED\nNo permissions to update statistics in"
							 " database: %s@%s.\nRerun as informix.  "
							 "Continuing with stored procedures anyway.\n",
							 databasename,
							 host );
						break;
					    } else if (sqlca.sqlcode == -740) {
						fprintf( stderr,
							 "FAILED\nMinimum resolution for this table is: %f\n",
							 minres( l_rowcnt ) );
						exit(3);
					    } else if (sqlca.sqlcode) {
						fprintf( stderr,
							 "FAILED(2)\nCannot Update statistics for table: "
							 "%s%s%s%s, column %s; "
							 "\n\tSQLCODE = %d, ISAM = %d, POS = %d\n",
							 ownerstr,
							 QuotedIdents ? "\"" : "",
							 lasttbl,
							 QuotedIdents ? "\"" : "",
							 columnnames[column],
							 (int)sqlca.sqlcode,
							 (int)sqlca.sqlerrd[1],
							 (int)sqlca.sqlerrd[4] );
						if (sqlca.sqlcode == -201 || sqlca.sqlcode == -352)
						    fprintf( stderr, "%s\n", tempstr );
						if (!noexit) {
						    exit(do_err_cnt ? error_ret : 0);
						} else {
						    noexit--;
						    error_ret++;
						}
					    } else
						TimeMark( stmtstart, ExpandReport, "SUCCESSFUL", 0, 0 );
					} else {
					    if (verbose >= 0)
						fprintf( stderr, "\t%s(MEDIUM)", medcols );
					    Out_Func( tempstr );
					    if (verbose >= 0)
						fprintf( stderr, "\n" );
					}
					if (ANSIMode)
					    if (!outfile) {
						if (verbose >= 2)
						    fprintf( stderr, "CW at: %d.\n", __LINE__ );
						EXEC SQL COMMIT WORK;
					    } else {
						Out_Func( "COMMIT WORK;" );
						if (verbose >= 2)
						    fprintf( stderr, "CW at: %d.\n", __LINE__ );
					    }
				    }
				    medfirst = 1;
				    medcols[0] = (char)0;
				}
				if ((columns[col] & 1) || (small_tables_high == 1 && l_rowcnt <= small_tables_count)) {
				    /* Include column in HIGH. */
				    if (highfirst) {
					highfirst = 0;
				    } else {
					strcat( highcols, ", " );
				    }
				    strcat( highcols, columnnames[col] );
				} else {
				    if (columntypes[col] != 11 && columntypes[col] != 12 && !(columntypes[col] == 41 && (xtdtypes[col] == 11 || xtdtypes[col] == 12))) {
					/* Skip all TEXT, BYTE, BLOB, & CLOB type columns. */
					if (medfirst) {
					    medfirst = 0;
					} else {
					    strcat( medcols, ", " );
					}
					strcat( medcols, columnnames[col] );
				    }
				}
			    }

			    /* Columns lists built. Process. */
			    if (strlen( highcols )) {
				snprintf( tempstr, sizeof tempstr,
					  "UPDATE STATISTICS HIGH FOR TABLE %s%s%s%s (%s)%s%s%s%s;",
					  lastowner,
					  QuotedIdents ? "\"" : "",
					  lasttbl,
					  QuotedIdents ? "\"" : "",
					  highcols,
					  resset ? resstring : "",
					  (NoDistOnly) ? "" : " DISTRIBUTIONS ONLY",
					  (force_run) ? " FORCE" : "",
					  (auto_run) ? " AUTO" : "" );
				if (doindex) {
				    if (outfile == (FILE *)NULL) {
					if (verbose >= 0) {
					    TimeStart( &stmtstart, ExpandReport );
					    fprintf( stderr, "        %s(HIGH)...", highcols );
					}

					EXEC SQL EXECUTE IMMEDIATE :tempstr;
					if (sqlca.sqlcode == -313) {
					    fprintf( stderr,
						     "FAILED\nNo permissions to update statistics in"
						     " database: %s@%s.\nRerun as informix.  "
						     "Continuing with stored procedures anyway.\n",
						     databasename,
						     host );
					    break;
					} else if (sqlca.sqlcode == -740) {
					    fprintf( stderr,
						     "FAILED\nMinimum resolution for this table is: %f\n",
						     minres( l_rowcnt ) );
					    exit(3);
					} else if (sqlca.sqlcode) {
					    fprintf( stderr,
						     "FAILED(3)\nCannot Update statistics for table: "
						     "%s%s%s%s, column %s; "
						     "\n\tSQLCODE = %d, ISAM = %d, POS = %d\n",
						     ownerstr,
						     QuotedIdents ? "\"" : "",
						     lasttbl,
						     QuotedIdents ? "\"" : "",
						     columnnames[column],
						     (int)sqlca.sqlcode,
						     (int)sqlca.sqlerrd[1],
						     (int)sqlca.sqlerrd[4] );
					    if (sqlca.sqlcode == -201 || sqlca.sqlcode == -352)
						fprintf( stderr, "%s\n", tempstr );
					    if (!noexit) {
						exit(do_err_cnt ? error_ret : 0);
					    } else {
						noexit--;
						error_ret++;
					    }
					} else
					    TimeMark( stmtstart, ExpandReport, "SUCCESSFUL", 0, 0 );

				    } else {
					if (verbose >= 0)
					    fprintf( stderr, "\t%s(HIGH)", highcols );
					Out_Func( tempstr );
					if (verbose >= 0)
					    fprintf( stderr, "\n" );
				    }
				    if (ANSIMode)
					if (!outfile) {
					    if (verbose >= 2)
						fprintf( stderr, "CW at: %d.\n", __LINE__ );
					    EXEC SQL COMMIT WORK;
					} else {
					    Out_Func( "COMMIT WORK;" );
					    if (verbose >= 2)
						fprintf( stderr, "CW at: %d.\n", __LINE__ );
					}
				}
				highcols[0] = (char)0;
			    }
			    if (tabmedium && strlen( medcols )) {
				snprintf( tempstr, sizeof tempstr,
					  "UPDATE STATISTICS MEDIUM FOR TABLE %s%s%s%s (%s)%s%s%s%s%s;",
					  lastowner,
					  QuotedIdents ? "\"" : "",
					  lasttbl,
					  QuotedIdents ? "\"" : "",
					  medcols,
					  sampleset ? samplestring : "",
					  mresset ? mresstring : "",
					  (mresset && confset) ? confstring : "",
					  (force_run) ? " FORCE" : "",
					  (auto_run) ? " AUTO" : "" );
				if (doindex) {
				    if (outfile == (FILE *)NULL) {
					if (verbose >= 0) {
					    TimeStart( &stmtstart, ExpandReport );
					    fprintf( stderr, "        %s(MEDIUM)...", medcols );
					}

					EXEC SQL EXECUTE IMMEDIATE :tempstr;
					if (sqlca.sqlcode == -313) {
					    fprintf( stderr,
						     "FAILED\nNo permissions to update statistics in"
						     " database: %s@%s.\nRerun as informix.  "
						     "Continuing with stored procedures anyway.\n",
						     databasename,
						     host );
					    break;
					} else if (sqlca.sqlcode == -740) {
					    fprintf( stderr,
						     "FAILED\nMinimum resolution for this table is: %f\n",
						     minres( l_rowcnt ) );
					    exit(3);
					} else if (sqlca.sqlcode) {
					    fprintf( stderr,
						     "FAILED(4)\nCannot Update statistics for table: "
						     "%s%s%s%s, column %s; "
						     "\n\tSQLCODE = %d, ISAM = %d, POS = %d\n",
						     ownerstr,
						     QuotedIdents ? "\"" : "",
						     lasttbl,
						     QuotedIdents ? "\"" : "",
						     columnnames[column],
						     (int)sqlca.sqlcode,
						     (int)sqlca.sqlerrd[1],
						     (int)sqlca.sqlerrd[4] );
					    if (sqlca.sqlcode == -201 || sqlca.sqlcode == -352)
						fprintf( stderr, "%s\n", tempstr );
					    if (!noexit) {
						exit(do_err_cnt ? error_ret : 0);
					    } else {
						noexit--;
						error_ret++;
					    }
					} else
					    TimeMark( stmtstart, ExpandReport, "SUCCESSFUL", 0, 0 );
				    } else {
					if (verbose >= 0)
					    fprintf( stderr, "\t%s(MEDIUM)", medcols );
					Out_Func( tempstr );
					if (verbose >= 0)
					    fprintf( stderr, "\n" );
				    }
				    if (ANSIMode)
					if (!outfile) {
					    if (verbose >= 2)
						fprintf( stderr, "CW at: %d.\n", __LINE__ );
					    EXEC SQL COMMIT WORK;
					} else {
					    Out_Func( "COMMIT WORK;" );
					    if (verbose >= 2)
						fprintf( stderr, "CW at: %d.\n", __LINE__ );
					}
				}
				medcols[0] = (char)0;
			    }
			}

			if (outfile == (FILE *)NULL && lasttbl[0] != (char)0) {
			    char tblstr[512];
			    snprintf( tblstr,
				      sizeof tblstr,
				      "      Table %s%s%s%s completed ",
				      lastowner,
				      QuotedIdents ? "\"" : "",
				      lasttbl,
				      QuotedIdents ? "\"" : "" );

			    TimeMark( tabstart, ExpandReport, tblstr, 1, 0 );
			}

			if (tabinfo.tabid != lastidx.tabid) {
			    if (verbose >= 0) {
				TimeStart( &tabstart, ExpandReport );
				fprintf( stderr,
					 "    Working on table %s%s%s%s:\n",
					 ownerstr,
					 QuotedIdents ? "\"" : "",
					 tabinfo.tabname,
					 QuotedIdents ? "\"" : "" );
			    }
			}

			/* New table.  Update stats to medium for the whole table. */
			memset( &lastidx, 0, sizeof lastidx );
			if (wipe_table) {
			    if (verbose > 0) {
				fprintf( stderr,
					 "    %s%s%s(Clearing Old Distributions):...",
					 QuotedIdents ? "\"" : "",
					 tabinfo.tabname,
					 QuotedIdents ? "\"" : "" );
			    }
                            if (major_vers >= 11) {
                                snprintf( tempstr, sizeof tempstr,
					  "UPDATE STATISTICS LOW FOR TABLE %s DROP DISTRIBUTIONS ONLY;",
					  tabinfo.tabname );
                            } else {
                                snprintf( tempstr, sizeof tempstr,
					  "DELETE FROM sysdistrib WHERE tabid = %d;",
					  tabinfo.tabid );
                            }
                            TimeStart( &stmtstart, ExpandReport );
                            if (outfile == (FILE *)NULL) {
                                EXEC SQL EXECUTE IMMEDIATE :tempstr;
                                if (sqlca.sqlcode != 0) {
                                    fprintf( stderr,
                                             "Error clearing old stats for table %s. Continuing.\n",
                                             tabinfo.tabname );
                                } else if (verbose > 0) {
                                    TimeMark( stmtstart, ExpandReport, "SUCCESSFUL", 0, 0 );
				} else {
				    fprintf( stderr, "\n" );
                                }
                            } else {
                                Out_Func( tempstr );
				if (verbose > 0)
				    fprintf( stderr, "\n" );
                            }
			}
			if (dotable) {
			    if (!tabmedium) {
				snprintf( tempstr, sizeof tempstr,
					  "UPDATE STATISTICS %s FOR TABLE %s%s%s%s%s%s%s;",
					  NoLevel ? "" : "LOW",
					  ownerstr,
					  QuotedIdents ? "\"" : "",
					  tabinfo.tabname,
					  QuotedIdents ? "\"" : "",
					  (NoDistOnly || !dropdists)
					  ? "" : " DROP DISTRIBUTIONS",
					  (force_run) ? " FORCE" : "",
					  (auto_run) ? " AUTO" : "");

				if (outfile == (FILE *)NULL) {
				    if (verbose >= 0) {
					TimeStart( &stmtstart, ExpandReport );
					fprintf( stderr,
						 "        %s%s%s(%s)...",
						 QuotedIdents ? "\"" : "",
						 tabinfo.tabname,
						 QuotedIdents ? "\"" : "",
						 tabmedium ? "MEDIUM" : "LOW" );
				    }

			    
				    EXEC SQL EXECUTE IMMEDIATE :tempstr;
				    if (sqlca.sqlcode == -313) {
					fprintf( stderr,
						 "FAILED\nNo permissions to update statistics in"
						 " database: %s@%s.\nRerun as informix.  "
						 "Continuing with stored procedures anyway.\n",
						 databasename,
						 host );
					break;
				    } else if (sqlca.sqlcode == -740) {
					fprintf( stderr,
						 "FAILED\nMinimum resolution for this table is: %f\n",
						 minres( rowcnt ) );
					exit(3);
				    } else if (sqlca.sqlcode) {
					fprintf( stderr,
						 "FAILED(5)\nCannot Update statistics for table: %s%s%s%s; "
						 "\n\tSQLCODE = %d, ISAM = %d, POS = %d\n",
						 ownerstr,
						 QuotedIdents ? "\"" : "",
						 tabinfo.tabname,
						 QuotedIdents ? "\"" : "",
						 (int)sqlca.sqlcode,
						 (int)sqlca.sqlerrd[1],
						 (int)sqlca.sqlerrd[4] );
					if (sqlca.sqlcode == -201 || sqlca.sqlcode == -352)
					    fprintf( stderr, "%s\n", tempstr );
					if (!noexit) {
					    exit(do_err_cnt ? error_ret : 0);
					} else {
					    noexit--;
					    error_ret++;
					}
				    } else {
					TimeMark( stmtstart, ExpandReport, "SUCCESSFUL", 0, 0 );
				    }
				} else {
				    if (verbose >= 0)
					fprintf( stderr,
						 "%s%s%s(%s)",
						 QuotedIdents ? "\"" : "",
						 tabinfo.tabname,
						 QuotedIdents ? "\"" : "",
						 tabmedium ? "MEDIUM" : "LOW" );
				    Out_Func( tempstr );
				    if (verbose >= 0)
					fprintf( stderr, "\n" );
				}
				if (ANSIMode)
				    if (!outfile) {
					if (verbose >= 2)
					    fprintf( stderr, "CW at: %d.\n", __LINE__ );
					EXEC SQL COMMIT WORK;
				    } else {
					Out_Func( "COMMIT WORK;" );
					if (verbose >= 2)
					    fprintf( stderr, "CW at: %d.\n", __LINE__ );
				    }
			    }
			}
			snprintf( lasttbl, sizeof lasttbl, "%s", tabinfo.tabname );
			l_rowcnt = rowcnt;
			if (ANSIMode == 1 || InclOwner == 1)
			    snprintf( lastowner, sizeof lastowner, "\"%s\".", tabinfo.tabowner );
			else
			    memset( lastowner, 0, sizeof lastowner );

			ncols = 0;
			EXEC SQL OPEN curs2 USING :tabinfo.tabid;
			if (sqlca.sqlcode != 0) {
			    fprintf( stderr,
				     "FAILED\nCannot OPEN curs2.  SQL=%d, ISAM=%d.\n",
				     (int)sqlca.sqlcode,
				     (int)sqlca.sqlerrd[1] );
			    exit(1);
			}
			/* Initialize flags. */
			if (small_tables_high == 1 && rowcnt <= small_tables_count) 
			    memset( columns, 1, sizeof columns );  
			else
			    memset( columns, 0, sizeof columns );  
			for (ncols=0; ncols < 32768; ncols++) {
			    EXEC SQL FETCH curs2
				INTO :colno, :columnnames[ncols], :columntypes[ncols], :xtdtypes[ncols];
			    if (sqlca.sqlcode < 0) {
				fprintf( stderr,
					 "FAILED\nCannot FETCH curs2.  "
					 "SQL=%d, ISAM=%d.\n",
					 (int)sqlca.sqlcode,
					 (int)sqlca.sqlerrd[1] );
				exit(1);
			    } else if (sqlca.sqlcode == SQLNOTFOUND)
				break;
			}
			EXEC SQL CLOSE curs2;
		    }
		    if (ANSIMode == 1 || InclOwner == 1)
		        snprintf( ownerstr, sizeof ownerstr, "\"%s\".", tabinfo.tabowner );
		    else
		        memset( ownerstr, 0, sizeof ownerstr );
		    if (tabinfo.colind >= 0) {
			/* This table has index(es).  Update statistics high for the
			   first column in each index and low for the whole key. */
			column = ((tabinfo.colno[0] > 0) ?
				  tabinfo.colno[0] :
				  (-1 * tabinfo.colno[0])) - 1;
			/* Make certain we did not do this columns already. */
			if ((columns[column] % 2) == 0) { 
			    columns[column] |= 1;  /* Set flag 'HIGH DONE' for this column. */
			    columns[column] |= 4; /* Set 'DISTRIBUTIONS ONLY DONE' flag for this column. */
			}
			/* For multi-column indexes starting with common column
			   lists do update statistics high for first different
			   column. */
			if (lastidx.tabname[0] != (char)0) {
			    /* Don't need to do this for the first index. */
			    leadsame = 0;
			    for (jj=0; lastidx.colno[jj] && tabinfo.colno[jj]; jj++) {
				/* Compare first columns of index to last index. */
				if (lastidx.colno[jj] == tabinfo.colno[jj])
				    leadsame = 1;
				else
				    break;
			    }
			    /* If there is work to do make sure we did not do this
			       column already! */
			    column = ((tabinfo.colno[jj] > 0) ?
				      tabinfo.colno[jj] :
				      (-1 * tabinfo.colno[jj])) - 1;
			    if (doindex &&
				!NoLevel &&
				leadsame &&
				column >= 0 &&
				(columns[column] % 2) == 0)
				{
				    columns[column] |= 1;  /* Set 'HIGH DONE' flag for this column. */
				    columns[column] |= 4; /* Set 'DISTRIBUTIONS ONLY DONE' flag for this column. */
				}
			    /* Look backwards at lastidx also because that column
			       may not have been done yet. */
			    if (lastidx.colno[jj] != 0) {
				column2 = ((lastidx.colno[jj] > 0) ?
					   lastidx.colno[jj] :
					   (-1 * lastidx.colno[jj])) - 1;
				if (doindex && !NoLevel && leadsame && (columns[column2] % 2) == 0)
				    {
					if (ANSIMode == 1 || InclOwner == 1)
					    snprintf( lastowner, sizeof lastowner, "\"%s\".", lastidx.tabowner );
					else
					    memset( lastowner, 0, sizeof lastowner );
					columns[column2] |= 1;  /* Set flag 'HIGH DONE' for this column. */
					columns[column2] |= 4; /* Set 'DISTRIBUTIONS ONLY DONE' flag for this column. */
				    }
			    }
			}
			if (docolumn && ((tabinfo.colno[1] != 0) ||
					 ((tabinfo.colno[1] == 0) && (columns[column]/4)%2) == 1))
			    {
				snprintf( tempstr, sizeof tempstr,
					  "UPDATE STATISTICS %s FOR TABLE %s%s%s%s (",
					  NoLevel ? "" : "LOW",
					  ownerstr,
					  QuotedIdents ? "\"" : "",
					  tabinfo.tabname,
					  QuotedIdents ? "\"" : ""
					  );
				lowcols[0] = (char)0;

				for (c = 0; c < 16; c++) {
				    if (tabinfo.colno[c] != 0) {
					if (c != 0) {
					    strcat( tempstr, ", " );
					    if (verbose >= 0)
						strcat( lowcols, ", " );
					}
					column2 = ((tabinfo.colno[c] > 0) ?
						   tabinfo.colno[c] :
						   (-1 * tabinfo.colno[c])) - 1;
					strcat( tempstr, columnnames[column2] );
					strcat( lowcols, columnnames[column2] );
				    }
				}
				snprintf( tempstr2,
					  sizeof tempstr2,
					  ")%s%s;",
					  (force_run) ? " FORCE" : "",
					  (auto_run) ? " AUTO" : "" );
				strcat( tempstr, tempstr2 );
				if (outfile == (FILE *)NULL) {
				    if (verbose >= 0) {
					TimeStart( &stmtstart, ExpandReport );
					fprintf( stderr, "        %s(LOW)...", lowcols );
				    }
			    
				    EXEC SQL EXECUTE IMMEDIATE :tempstr;
				    if (sqlca.sqlcode == -313) {
					fprintf( stderr,
						 "FAILED\nNo permissions to update statistics in"
						 " database: %s@%s.\nRerun as informix.  "
						 "Continuing with stored procedures anyway.\n",
						 databasename,
						 host );
					break;
				    } else if (sqlca.sqlcode == -740) {
					fprintf( stderr,
						 "FAILED\nMinimum resolution for this table is: %f\n",
						 minres( rowcnt ) );
					exit(3);
				    } else if (sqlca.sqlcode) {
					fprintf( stderr,
						 "FAILED(6)\nCannot Update statistics for table: "
						 "%s%s%s%s, column %s; "
						 "\n\tSQLCODE = %d, ISAM = %d, "
						 "POS = %d\n",
						 ownerstr,
						 QuotedIdents ? "\"" : "",
						 tabinfo.tabname,
						 QuotedIdents ? "\"" : "",
						 columnnames[column2] ,
						 (int)sqlca.sqlcode,
						 (int)sqlca.sqlerrd[1],
						 (int)sqlca.sqlerrd[4] );
					if (sqlca.sqlcode == -201 || sqlca.sqlcode == -352)
					    fprintf( stderr, "%s\n", tempstr );
					if (!noexit) {
					    exit(do_err_cnt ? error_ret : 0);
					} else {
					    noexit--;
					    error_ret++;
					}
				    } else
					TimeMark( stmtstart, ExpandReport, "SUCCESSFUL", 0, 0 );

				} else {
				    if (verbose >= 0)
					fprintf( stderr, "        %s(LOW)...", lowcols );
				    Out_Func( tempstr );
				    if (verbose >= 0)
					fprintf( stderr, "\n" );
				}
			    }
		    }

		    /* Do we need to look at fragmentation expressions? */
		    if (doFragExpr && !tabinfo.partnum) {
			char *tok;

			/* This is a fragmented table.  Process fragment
			   expression columns if any. */
			exprtext.loc_loctype = LOCUSER;
			exprtext.loc_buffer = (char *)malloc( 4096 );
			if (!exprtext.loc_buffer) {
			    fprintf( stderr, 
				     "FAILED\nCould not allocate memory for fragment expressions. "
				     "Errno=%d.\n", 
				     errno );
			    exit(1);
			}
			exprtext.loc_bufsize = 4096;
			exprtext.loc_write = writeit;
			exprtext.loc_open = openit;
			exprtext.loc_close = closeit;

			tblobbuff.bbsize = 4096;
			tblobbuff.base = (char *)malloc( tblobbuff.bbsize );
			if (!tblobbuff.base) {
			    fprintf( stderr, 
				     "FAILED\nCould not allocate memory for tblob buffer. "
				     "Errno=%d.\n", 
				     errno );
			    exit(1);
			}
			tblobbuff.buff = tblobbuff.base;
			tblobbuff.full_length = tblobbuff.global_length = 0;
			blobbuff = &tblobbuff;

			/* Perhaps prepare this above. --ASK */
			EXEC SQL DECLARE c_fragexpr CURSOR WITH HOLD FOR
			    SELECT evalpos, exprtext
			    FROM "informix".sysfragments
			    WHERE tabid = :tabinfo.tabid
			    AND fragtype = 'T'
			    AND strategy = 'E'
			    ORDER BY evalpos;
			if (sqlca.sqlcode != 0) {
			    fprintf( stderr,
				     "FAILED DECLARING cursor for fragexpr.  Code=%d, ISAM=%d.\n",
				     (int)sqlca.sqlcode,
				     (int)sqlca.sqlerrd[1] );
			    exit(1);
			}
			EXEC SQL OPEN c_fragexpr;
			if (sqlca.sqlcode != 0) {
			    fprintf( stderr,
				     "FAILED OPENING cursor for fragexpr.  Code=%d, ISAM=%d.\n",
				     (int)sqlca.sqlcode,
				     (int)sqlca.sqlerrd[1] );
			    exit(1);
			}
			for(;;) {
			    EXEC SQL FETCH c_fragexpr
				INTO :evalpos, :exprtext;
			    if (sqlca.sqlcode != 0)
				break;
			    tblobbuff.buff[tblobbuff.global_length] = (char)0;
			    strcat( tblobbuff.buff, " " );
			    tblobbuff.buff += strlen( tblobbuff.buff );
			    tblobbuff.full_length += tblobbuff.global_length;
			}
			if (sqlca.sqlcode < 0) {
			    fprintf( stderr,
				     "FAILED FETCHING fragexpr. Code=%d, ISAM=%d.\n",
				     (int)sqlca.sqlcode,
				     (int)sqlca.sqlerrd[1] );
			    exit(1);
			}
			EXEC SQL CLOSE c_fragexpr;
			if (tblobbuff.full_length) {
			    /* OK, got the expression, now parse for column names! */
			    tok = strtok( tblobbuff.base, "( )<>=!\t|" );
			    while (tok) {
				column = cfind( tok, columnnames, ncols );
				if (column >= 0) {
				    /* OK this token at least looks like a column name. */
				    /* Make certain we did not do this column already. */
				    if ((columns[column] % 2) == 0) { 
					columns[column] |= 1;  /* Set flag 'HIGH DONE' for this column. */
					columns[column] |= 4; /* Set 'DISTRIBUTIONS ONLY DONE' flag for this column. */
				    }
				}
				tok = strtok( (char *)NULL, "( )<>=!\t|" );
			    }
			}
		    }
		}
		/* ASK - Apply forced HIGH columns here! */
		if (have_high_list == 1) {
		    char dbs[129], tab[129], col[129];
		    char highcols[32768];
		    int pos, res, again, highfirst;
		    EXEC SQL BEGIN DECLARE SECTION;
		    short colno;
		    string colname[129];
		    EXEC SQL END DECLARE SECTION;

		    EXEC SQL PREPARE fh_col_s FROM
			"SELECT (colno - 1) FROM syscolumns WHERE tabid = ? AND colname = ?";
		    if (sqlca.sqlcode < 0) {
			fprintf( stderr,
				 "ERROR PREPARING fh_col_s. Code=%d, ISAM=%d.\n",
				 (int)sqlca.sqlcode,
				 (int)sqlca.sqlerrd[1] );
			exit(1);
		    }

		    /* Find current database/table in list? */
		    highcols[0] = (char)0;
		    highfirst = 1;
		    pos = 0;
		looptop:
		    again = 0;
		    for (; pos < num_high; pos++) {
			if ((res = strcmp( force_high_cols[pos].dbsname, databasename )) < 0)
			    continue;
			else if (res > 0)
			    break;
			else {
			    if((res = strcmp( force_high_cols[pos].tabname, tabinfo.tabname )) < 0)
				continue;
			    else if (res > 0)
				continue;
			    else {
				/* find colno and see if this column has already been done. */
				strcpy( colname, force_high_cols[pos].colname );
				EXEC SQL EXECUTE fh_col_s INTO :colno USING :tabinfo.tabid, :colname;
				if (sqlca.sqlcode < 0) {
				    fprintf( stderr,
					     "ERROR FETCHING fh_col_s. Code=%d, ISAM=%d.\n",
					     (int)sqlca.sqlcode,
					     (int)sqlca.sqlerrd[1] );
				    exit(1);
				}
				if ((columns[colno] % 2) == 0) { 
				    columns[colno] |= 1;  /* Set flag 'HIGH DONE' for this column. */
				    /* Include column in HIGH. */
				    if (highfirst) {
					highfirst = 0;
				    } else {
					strcat( highcols, ", " );
				    }
				    /* If the list is getting too long, print the command and start a new list. */
				    if ((strlen( highcols ) + strlen( columnnames[colno] )) >= sizeof highcols) {
					again = 1;
					break;
				    }
				    strcat( highcols, columnnames[colno] );
				}
			    }
			}
		    }
		    if (again && strlen( highcols ) > 0) {
			snprintf( tempstr, sizeof tempstr,
				  "UPDATE STATISTICS HIGH FOR TABLE %s%s%s%s (%s)%s%s%s%s;",
				  lastowner,
				  QuotedIdents ? "\"" : "",
				  lasttbl,
				  QuotedIdents ? "\"" : "",
				  highcols,
				  resset ? resstring : "",
				  " DISTRIBUTIONS ONLY",
				  (force_run) ? " FORCE" : "",
				  (auto_run) ? " AUTO" : "" );
			if (doindex) {
			    if (outfile == (FILE *)NULL) {
				if (verbose >= 0) {
				    TimeStart( &stmtstart, ExpandReport );
				    fprintf( stderr, "\nFORCING HIGH FOR COLUMNS  %s...", highcols );
				}

				EXEC SQL EXECUTE IMMEDIATE :tempstr;
				if (sqlca.sqlcode == -313) {
				    fprintf( stderr,
					     "FAILED\nNo permissions to update statistics in"
					     " database: %s@%s.\nRerun as informix.  "
					     "Continuing with stored procedures anyway.\n",
					     databasename,
					     host );
				    break;
				} else if (sqlca.sqlcode == -740) {
				    fprintf( stderr,
					     "FAILED\nMinimum resolution for this table is: %f\n",
					     minres( l_rowcnt ) );
				    exit(3);
				} else if (sqlca.sqlcode) {
				    fprintf( stderr,
					     "FAILED(1)\nCannot Update statistics for table: "
					     "%s%s%s%s, column %s; "
					     "\n\tSQLCODE = %d, ISAM = %d, POS = %d\n",
					     ownerstr,
					     QuotedIdents ? "\"" : "",
					     lasttbl,
					     QuotedIdents ? "\"" : "",
					     columnnames[column],
					     (int)sqlca.sqlcode,
					     (int)sqlca.sqlerrd[1],
					     (int)sqlca.sqlerrd[4] );
				    if (sqlca.sqlcode == -201 || sqlca.sqlcode == -352)
					fprintf( stderr, "%s\n", tempstr );
				    if (!noexit) {
					exit(do_err_cnt ? error_ret : 0);
				    } else {
					noexit--;
					error_ret++;
				    }
				} else
				    TimeMark( stmtstart, ExpandReport, "SUCCESSFUL", 0, 0 );
			    } else {
				if (verbose >= 0)
				    fprintf( stderr, "\t%s(HIGH)", highcols );
				Out_Func( tempstr );
				if (verbose >= 0)
				    fprintf( stderr, "\n" );
			    }
			}
			if (ANSIMode)
			    if (!outfile) {
				if (verbose >= 2)
				    fprintf( stderr, "CW at: %d.\n", __LINE__ );
				EXEC SQL COMMIT WORK;
			    } else {
				Out_Func( "COMMIT WORK;" );
				if (verbose >= 2)
				    fprintf( stderr, "CW at: %d.\n", __LINE__ );
			    }
			highfirst = 1;
			highcols[0] = (char)0;
			if (again == 1)
			    goto looptop;
		    }
		}

		memcpy( &lastidx, &tabinfo, sizeof tabinfo );
	    }
	    EXEC SQL CLOSE a1;
	    if (lasttbl[0] != (char)0) {
		int col, highlen, medlen;
		int medfirst = 1, highfirst = 1;
		char highcols[32768], medcols[32768];

		medlen = 32767 - (56 + strlen( lasttbl ));
		highlen = medlen + 2;
		highcols[0] = (char)0;
		medcols[0] = (char)0;
		for (col=0; col < ncols; col++) {
		    if ((strlen( highcols ) + strlen( columnnames[col] ) + 1) > highlen) {
			/* Name list is getting too long.  Issue command now and start over. */
			snprintf( tempstr, sizeof tempstr,
				  "UPDATE STATISTICS HIGH FOR TABLE %s%s%s%s (%s)%s%s%s%s;",
				  lastowner,
				  QuotedIdents ? "\"" : "",
				  lasttbl,
				  QuotedIdents ? "\"" : "",
				  highcols,
				  resset ? resstring : "",
				  (NoDistOnly) ?
				  "" : " DISTRIBUTIONS ONLY",
				  (force_run) ? " FORCE" : "",
				  (auto_run) ? " AUTO" : "");
			if (doindex) {
			    if (outfile == (FILE *)NULL) {
				if (verbose >= 0) {
				    TimeStart( &stmtstart, ExpandReport );
				    fprintf( stderr, "\t%s(HIGH)...", highcols );
				}

				EXEC SQL EXECUTE IMMEDIATE :tempstr;
				if (sqlca.sqlcode == -313) {
				    fprintf( stderr,
					     "FAILED\nNo permissions to update statistics in"
					     " database: %s@%s.\nRerun as informix.  "
					     "Continuing with stored procedures anyway.\n",
					     databasename,
					     host );
				    break;
				} else if (sqlca.sqlcode == -740) {
				    fprintf( stderr,
					     "FAILED\nMinimum resolution for this table is: %f\n",
					     minres( l_rowcnt ) );
				    exit(3);
				} else if (sqlca.sqlcode) {
				    fprintf( stderr,
					     "FAILED(7)\nCannot Update statistics for table: "
					     "%s%s%s%s, column %s; "
					     "\n\tSQLCODE = %d, ISAM = %d, POS = %d\n",
					     ownerstr,
					     QuotedIdents ? "\"" : "",
					     lasttbl,
					     QuotedIdents ? "\"" : "",
					     columnnames[column],
					     (int)sqlca.sqlcode,
					     (int)sqlca.sqlerrd[1],
					     (int)sqlca.sqlerrd[4] );
				    if (sqlca.sqlcode == -201 || sqlca.sqlcode == -352)
					fprintf( stderr, "%s\n", tempstr );
				    if (!noexit) {
					exit(do_err_cnt ? error_ret : 0);
				    } else {
					noexit--;
					error_ret++;
				    }
				} else
				    TimeMark( stmtstart, ExpandReport, "SUCCESSFUL", 0, 0 );

			    } else {
				if (verbose >= 0)
				    fprintf( stderr,
					     "\t%s:\n",
					     columnnames[column]  );
				Out_Func( tempstr );
			    }
			    if (ANSIMode)
				if (!outfile) {
				    if (verbose >= 2)
					fprintf( stderr, "CW at: %d.\n", __LINE__ );
				    EXEC SQL COMMIT WORK;
				} else {
				    Out_Func( "COMMIT WORK;" );
				    if (verbose >= 2)
					fprintf( stderr, "CW at: %d.\n", __LINE__ );
				}
			}
			highfirst = 1;
			highcols[0] = (char)0;
		    }
		    if (tabmedium && (strlen( medcols ) + strlen( columnnames[col] ) + 1) > medlen) {
			/* Name list is getting too long.  Issue command now and start over. */
			snprintf( tempstr, sizeof tempstr,
				  "UPDATE STATISTICS MEDIUM FOR TABLE %s%s%s%s (%s)%s%s%s%s%s;",
				  lastowner,
				  QuotedIdents ? "\"" : "",
				  lasttbl,
				  QuotedIdents ? "\"" : "",
				  medcols,
				  sampleset ? samplestring : "",
				  mresset ? mresstring : "",
				  (mresset && confset) ? confstring : "",
				  (force_run) ? " FORCE" : "",
				  (auto_run) ? " AUTO" : "" );
			if (doindex) {
			    if (outfile == (FILE *)NULL) {
				if (verbose >= 0) {
				    TimeStart( &stmtstart, ExpandReport );
				    fprintf( stderr, "\t%s(MEDIUM)...", medcols );
				}

				EXEC SQL EXECUTE IMMEDIATE :tempstr;
				if (sqlca.sqlcode == -313) {
				    fprintf( stderr,
					     "FAILED\nNo permissions to update statistics in"
					     " database: %s@%s.\nRerun as informix.  "
					     "Continuing with stored procedures anyway.\n",
					     databasename,
					     host );
				    break;
				} else if (sqlca.sqlcode == -740) {
				    fprintf( stderr,
					     "FAILED\nMinimum resolution for this table is: %f\n",
					     minres( l_rowcnt ) );
				    exit(3);
				} else if (sqlca.sqlcode) {
				    fprintf( stderr,
					     "FAILED(8)\nCannot Update statistics for table: "
					     "%s%s%s%s, column %s; "
					     "\n\tSQLCODE = %d, ISAM = %d, POS = %d\n",
					     ownerstr,
					     QuotedIdents ? "\"" : "",
					     lasttbl,
					     QuotedIdents ? "\"" : "",
					     columnnames[column],
					     (int)sqlca.sqlcode,
					     (int)sqlca.sqlerrd[1],
					     (int)sqlca.sqlerrd[4] );
				    if (sqlca.sqlcode == -201 || sqlca.sqlcode == -352)
					fprintf( stderr, "%s\n", tempstr );
				    if (!noexit) {
					exit(do_err_cnt ? error_ret : 0);
				    } else {
					noexit--;
					error_ret++;
				    }
				} else
				    TimeMark( stmtstart, ExpandReport, "SUCCESSFUL", 0, 0 );

			    } else {
				if (verbose >= 0)
				    fprintf( stderr,
					     "\t%s:\n",
					     columnnames[column]  );
				Out_Func( tempstr );
			    }
			    if (ANSIMode)
				if (!outfile) {
				    if (verbose >= 2)
					fprintf( stderr, "CW at: %d.\n", __LINE__ );
				    EXEC SQL COMMIT WORK;
				} else {
				    Out_Func( "COMMIT WORK;" );
				    if (verbose >= 2)
					fprintf( stderr, "CW at: %d.\n", __LINE__ );
				}
			}
			medfirst = 1;
			medcols[0] = (char)0;
		    }
		    if (columns[col] & 1) {
			/* Include column in HIGH. */
			if (highfirst) {
			    highfirst = 0;
			} else {
			    strcat( highcols, ", " );
			}
			strcat( highcols, columnnames[col] );
		    } else {
			/* Include column in MEDIUM. */
			if (columntypes[col] != 11 && columntypes[col] != 12 && !(columntypes[col] == 41 && (xtdtypes[col] == 11 || xtdtypes[col] == 12))) {
			    /* Skip all TEXT, BYTE, BLOB, & CLOB type columns. */
			    if (medfirst) {
				medfirst = 0;
			    } else {
				strcat( medcols, ", " );
			    }
			    strcat( medcols, columnnames[col] );
			}
		    }
		}

		/* Process outputs for the last index. */
		if (strlen( highcols )) {
		    snprintf( tempstr, sizeof tempstr,
			      "UPDATE STATISTICS HIGH FOR TABLE %s%s%s%s (%s)%s%s%s%s;",
			      lastowner,
			      QuotedIdents ? "\"" : "",
			      lasttbl,
			      QuotedIdents ? "\"" : "",
			      highcols,
			      resset ? resstring : "",
			      (NoDistOnly) ?
			      "" : " DISTRIBUTIONS ONLY",
			      (force_run) ? " FORCE" : "",
			      (auto_run) ? " AUTO" : "");
		    if (doindex) {
			if (outfile == (FILE *)NULL) {
			    if (verbose >= 0) {
				TimeStart( &stmtstart, ExpandReport );
				fprintf( stderr, "\t%s(HIGH)...", highcols );
			    }

			    EXEC SQL EXECUTE IMMEDIATE :tempstr;
			    if (sqlca.sqlcode == -313) {
				fprintf( stderr,
					 "FAILED\nNo permissions to update statistics in"
					 " database: %s@%s.\nRerun as informix.  "
					 "Continuing with stored procedures anyway.\n",
					 databasename,
					 host );
				break;
			    } else if (sqlca.sqlcode == -740) {
				fprintf( stderr,
					 "FAILED\nMinimum resolution for this table is: %f\n",
					 minres( l_rowcnt ) );
				exit(3);
			    } else if (sqlca.sqlcode) {
				fprintf( stderr,
					 "FAILED(9)\nCannot Update statistics for table: "
					 "%s%s%s%s, column %s; "
					 "\n\tSQLCODE = %d, ISAM = %d, POS = %d\n",
					 ownerstr,
					 QuotedIdents ? "\"" : "",
					 lasttbl,
					 QuotedIdents ? "\"" : "",
					 columnnames[column],
					 (int)sqlca.sqlcode,
					 (int)sqlca.sqlerrd[1],
					 (int)sqlca.sqlerrd[4] );
				if (sqlca.sqlcode == -201 || sqlca.sqlcode == -352)
				    fprintf( stderr, "%s\n", tempstr );
				if (!noexit) {
				    exit(do_err_cnt ? error_ret : 0);
				} else {
				    noexit--;
				    error_ret++;
				}
			    } else
				TimeMark( stmtstart, ExpandReport, "SUCCESSFUL", 0, 0 );

			} else {
			    if (verbose >= 0)
				fprintf( stderr, "\t%s(HIGH)...", highcols );
			    Out_Func( tempstr );
			    if (verbose >= 0)
				fprintf( stderr, "\n" );
			}
			if (ANSIMode)
			    if (!outfile) {
				if (verbose >= 2)
				    fprintf( stderr, "CW at: %d.\n", __LINE__ );
				EXEC SQL COMMIT WORK;
			    } else {
				Out_Func( "COMMIT WORK;" );
				if (verbose >= 2)
				    fprintf( stderr, "CW at: %d.\n", __LINE__ );
			    }
		    }
		    highcols[0] = (char)0;
		}
		if (tabmedium && strlen( medcols )) {
		    snprintf( tempstr, sizeof tempstr,
			      "UPDATE STATISTICS MEDIUM FOR TABLE %s%s%s%s (%s)%s%s%s%s%s;",
			      lastowner,
			      QuotedIdents ? "\"" : "",
			      lasttbl,
			      QuotedIdents ? "\"" : "",
			      medcols,
			      sampleset ? samplestring : "",
			      mresset ? mresstring : "",
			      (mresset && confset) ? confstring : "",
			      (force_run) ? " FORCE" : "",
			      (auto_run) ? " AUTO" : "" );
		    if (doindex) {
			if (outfile == (FILE *)NULL) {
			    if (verbose >= 0) {
				TimeStart( &stmtstart, ExpandReport );
				fprintf( stderr, "\t%s(MEDIUM)...", medcols );
			    }

			    EXEC SQL EXECUTE IMMEDIATE :tempstr;
			    if (sqlca.sqlcode == -313) {
				fprintf( stderr,
					 "FAILED\nNo permissions to update statistics in"
					 " database: %s@%s.\nRerun as informix.  "
					 "Continuing with stored procedures anyway.\n",
					 databasename,
					 host );
				break;
			    } else if (sqlca.sqlcode == -740) {
				fprintf( stderr,
					 "FAILED\nMinimum resolution for this table is: %f\n",
					 minres( l_rowcnt ) );
				exit(3);
			    } else if (sqlca.sqlcode) {
				fprintf( stderr,
					 "FAILED(10)\nCannot Update statistics for table: "
					 "%s%s%s%s, column %s; "
					 "\n\tSQLCODE = %d, ISAM = %d, POS = %d\n",
					 ownerstr,
					 QuotedIdents ? "\"" : "",
					 lasttbl,
					 QuotedIdents ? "\"" : "",
					 columnnames[column],
					 (int)sqlca.sqlcode,
					 (int)sqlca.sqlerrd[1],
					 (int)sqlca.sqlerrd[4] );
				if (sqlca.sqlcode == -201 || sqlca.sqlcode == -352)
				    fprintf( stderr, "%s\n", tempstr );
				if (!noexit) {
				    exit(do_err_cnt ? error_ret : 0);
				} else {
				    noexit--;
				    error_ret++;
				}
			    } else
				TimeMark( stmtstart, ExpandReport, "SUCCESSFUL", 0, 0 );

			} else {
			    if (verbose >= 0)
				fprintf( stderr, "\t%s(MEDIUM)", medcols );
			    Out_Func( tempstr );
			    if (verbose >= 0)
				fprintf( stderr, "\n" );
			}
			if (ANSIMode)
			    if (!outfile) {
				if (verbose >= 2)
				    fprintf( stderr, "CW at: %d.\n", __LINE__ );
				EXEC SQL COMMIT WORK;
			    } else {
				Out_Func( "COMMIT WORK;" );
				if (verbose >= 2)
				    fprintf( stderr, "CW at: %d.\n", __LINE__ );
			    }
		    }
		    medcols[0] = (char)0;
		}
	    }

	    if (outfile == (FILE *)NULL) {
		char tblstr[512];
		snprintf( tblstr,
			  sizeof tblstr,
			  "      Table %s%s%s%s completed ",
			  lastowner,
			  QuotedIdents ? "\"" : "",
			  lasttbl,
			  QuotedIdents ? "\"" : "" );

		TimeMark( tabstart, ExpandReport, tblstr, 1, 0 );
	    }

	    if (doexplain) {
		if (outfile == (FILE *)NULL)
		    EXEC SQL SET EXPLAIN OFF;
		else
		    Out_Func( "SET EXPLAIN OFF;" );
	    }
	}

	if (doprocs) {
	    if (outfile == (FILE *)NULL)
		EXEC SQL EXECUTE IMMEDIATE :pdqstmt_1;
	    else
		Out_Func( pdqstmt_1 );
	    
	    if (major_vers < 11) {
		/*
		  Expected problem with external routines finally reared its ugly
		  head and had to be fixed.  Limiting processing to ONLY SPL routines.
		*/
		snprintf( procstmt,
			  sizeof procstmt,
			  "SELECT UNIQUE sp.procid, sp.owner, sp.procname, sp.specificname ) "
			  "FROM \"informix\".sysprocedures sp, \"informix\".sysroutinelangs sr "
			  "WHERE sp.langid = sr.langid and sr.langname = 'spl' and procname != ' oldsystdist' "
			  "  AND procname != 'systdistold' %s %s "
			  "ORDER BY sp.procid ",
			  prexcludes ? prexcludes : "",
			  princludes ? princludes : "" );
	    } else { /*
		       This block courtesy of Douglas Palmer.  Need to validate why the AND procname...
		     */ /*  
			    Expected problem with external routines finally reared its ugly
			    head and had to be fixed.  Limiting processing to ONLY SPL routines.
			*/
		snprintf( procstmt,
			  sizeof procstmt,
			  "SELECT UNIQUE sp.procid, sp.owner, sp.procname, sp.specificname "
			  "FROM \"informix\".sysprocedures sp, \"informix\".sysroutinelangs sr "
			  "WHERE sp.langid = sr.langid and sr.langname = 'spl' and procname != ' oldsystdist' "
			  "  AND procname NOT MATCHES '*seclabel_*' "
		          "  AND procname != ' systdistold' AND procname != 'null' %s %s "
			  "ORDER BY sp.procname ",
			  prexcludes ? prexcludes : "",
			  princludes ? princludes : "" );
	    }

	    if (doexplain) EXEC SQL SET EXPLAIN ON;
	    
	    EXEC SQL PREPARE proc_s from :procstmt;
	    if (sqlca.sqlcode < 0) {
		fprintf( stderr,
			 "FAILED PREPARING STATEMENT procs_s.  SQL=%d, ISAM=%d (%s-%s).\n",
			 (int)sqlca.sqlcode,
			 (int)sqlca.sqlerrd[1],
			 sqlca.sqlerrm,
			 sqlca.sqlerrp );
		fprintf( stderr,
			 "%s\n",
			 procstmt );
		exit(9);
	    }
	    EXEC SQL DECLARE procs CURSOR WITH HOLD FOR proc_s;
	    if (sqlca.sqlcode < 0) {
		fprintf( stderr,
			 "FAILED DECLARING CURSOR procs.  SQL=%d, ISAM=%d.\n",
			 (int)sqlca.sqlcode,
			 (int)sqlca.sqlerrd[1] );
		exit(9);
	    }
	    EXEC SQL OPEN procs;
	    if (sqlca.sqlcode < 0) {
		fprintf( stderr,
			 "FAILED OPENING CURSOR procs.  SQL=%d, ISAM=%d.\n",
			 (int)sqlca.sqlcode,
			 (int)sqlca.sqlerrd[1] );
		exit(9);
	    }
	    fprintf( stderr, "\n" );
	    TimeStart( &tabstart, ExpandReport );

	    if (verbose >= 0)
		fprintf( stderr, "Procedures:\n" );

	    for (;;) {
		EXEC SQL FETCH procs
		    INTO :procid,
		    :procowner,
		    :procname,
		    :specificname INDICATOR specific_ind;
		if (sqlca.sqlcode < 0) {
		    fprintf( stderr,
			     "FAILED FETCHING CURSOR procs.  SQL=%d, ISAM=%d.\n",
			     (int)sqlca.sqlcode,
			     (int)sqlca.sqlerrd[1] );
		    exit(9);
		} else if (sqlca.sqlcode == SQLNOTFOUND)
		    break;

		TimeStart( &stmtstart, ExpandReport );

		if (ANSIMode == 1 || InclOwner == 1)
		    snprintf( ownerstr, sizeof ownerstr, "\"%s\".", procowner );
		else
		    memset( ownerstr, 0, sizeof ownerstr );

		arglist_temp[0] = 0;
		if (return_coltypes( procid, arglist_temp ) != 0) {
		    if (dumpargs) {
			fprintf( stderr,
				 "Diag(RAW):    %s(%s).\n",
				 procname,
				 arglist_temp );
			fflush( stderr );
		    }
		    if (strlen( arglist_temp ) > 0)
			StripTypeList( arglist_temp );
		    if (dumpargs) {
			fprintf( stderr,
				 "Diag(COOKED): %s(%s).\n\n",
				 procname,
				 arglist_temp );
			fflush( stderr );
		    }
		    snprintf( tempstr, sizeof tempstr,
			      "UPDATE STATISTICS FOR PROCEDURE %s%s%s%s(%s);",
			      ownerstr,
			      QuotedIdents ? "\"" : "",
			      procname,
			      QuotedIdents ? "\"" : "",
			      arglist_temp );
		} else if (DoSpecific && specific_ind != -1) {
		    snprintf( tempstr, sizeof tempstr,
			      "UPDATE STATISTICS FOR SPECIFIC PROCEDURE %s%s%s%s;",
			      ownerstr,
			      QuotedIdents ? "\"" : "",
			      specificname,
			      QuotedIdents ? "\"" : "" );
		} else
		    snprintf( tempstr, sizeof tempstr,
			      "UPDATE STATISTICS FOR PROCEDURE %s%s%s%s;",
			      ownerstr,
			      QuotedIdents ? "\"" : "",
			      procname,
			      QuotedIdents ? "\"" : "" );
		if (outfile == (FILE *)NULL) {
		    if (verbose >= 0)
			fprintf( stderr,
				 "\t%s%s%s...",
				 QuotedIdents ? "\"" : "",
				 procname,
				 QuotedIdents ? "\"" : "" );
		    EXEC SQL EXECUTE IMMEDIATE :tempstr;
		    if (sqlca.sqlcode < 0) {
			fprintf( stderr,
				 "FAILED\nCannot Update statistics for procedure: %s%s%s. "
				 "SQL=%d, ISAM=%d.\n",
				 QuotedIdents ? "\"" : "",
				 procname,
				 QuotedIdents ? "\"" : "",
				 (int)sqlca.sqlcode,
				 (int)sqlca.sqlerrd[1] );
			if (!noexit) {
			    exit(do_err_cnt ? error_ret : 0);
			} else {
			    noexit--;
			    error_ret++;
			}
		    } else
			TimeMark( stmtstart, ExpandReport, "SUCCESSFUL", 0, 0 );
		} else {
		    if (verbose >= 0)
			fprintf( stderr,
				 "\t%s%s%s:\n",
				 QuotedIdents ? "\"" : "",
				 procname,
				 QuotedIdents ? "\"" : "" );
		    Out_Func( tempstr );
		}

		/* In ANSI Mode there are no singleton transactions and the
		   entire dostats session would be a single HUGE transaction
		   possibly tying up resources and locking sysprocedures
		   records.  Therefore we must commit each statement
		   manually. */
		if (ANSIMode)
		    if (!outfile) {
			if (verbose >= 2)
			    fprintf( stderr, "CW at: %d.\n", __LINE__ );
			EXEC SQL COMMIT WORK;
		    } else {
			Out_Func( "COMMIT WORK;" );
			if (verbose >= 2)
			    fprintf( stderr, "CW at: %d.\n", __LINE__ );
		    }
	    }
	    EXEC SQL CLOSE procs;
	    TimeMark( tabstart, ExpandReport, "  Procedures completed ", 1, 0 );
	}

	if (outfile == (FILE *)NULL) {
	    char dbstr[512];

	    snprintf( dbstr,
		      sizeof dbstr,
		      "  Database %s completed ",
		      databasename );
	    TimeMark( dbsstart, ExpandReport, dbstr, 1, 0 );
	}
	if (outfile != (FILE *)NULL && !do_procedure)
	    fprintf( outfile, "\n" );
    }

    if (verbose >= 2)
	fprintf( stderr, "CW at: %d.\n", __LINE__ );

    /* In ANSI Mode there are no singleton transactions and the entire dostats
       session would be a single HUGE transaction possibly tying up resources
       and locking sysprocedures records.  Therefore we must commit each
       statement manually. */
    if (ANSIMode)
	if (!outfile) {
	    if (verbose >= 2)
		fprintf( stderr, "CW at: %d.\n", __LINE__ );
	    EXEC SQL COMMIT WORK;
	} else {
	    Out_Func( "COMMIT WORK;" );
	    if (verbose >= 2)
		fprintf( stderr, "CW at: %d.\n", __LINE__ );
	}

    if (outfile == (FILE *)NULL) {
	TimeMark( totstart, ExpandReport, "Processing completed ", 1, 0 );
    }

    if (do_procedure && schedfile != (FILE *)NULL) {
	/* Finish up: insert scheduler record. */

	EXEC SQL SET CONNECTION 'WORK';
	if (sqlca.sqlcode) {
	    fprintf( stderr,
		     "FAILED %d: Cannot set connection to (%s); error=%d, isam=%d\n",
		     __LINE__,
		     "WORK",
		     (int)sqlca.sqlcode,
		     (int)sqlca.sqlerrd[1] );
	    exit(1);
	}

	if (do_proconly) {
	    if (verbose >= 2)
		fprintf( stderr, "Dropping procedure(2): %s.\n", schedprocname );
	    snprintf( schedule_stmt,
		      sizeof schedule_stmt,
		      "DROP PROCEDURE %s;",
		      schedprocname );
	    EXEC SQL EXECUTE IMMEDIATE :schedule_stmt;
	    if (sqlca.sqlcode != 0 && sqlca.sqlcode != -674) {
		fprintf( stderr,
			 "Error dropping old copy of procedure!  SQL: %d  ISAM: %d\n",
			 sqlca.sqlcode,
			 sqlca.sqlerrd[1] );
		exit( 1 );
	    }
	}
	
	if (do_scheduling)
	    fprintf( stderr,
		     "Scheduling stats update for database: %s.\n",
		     databasename );
	else
	    fprintf( stderr,
		     "Creating stats procedure %s in database: %s.\n",
		     schedprocname,
		     databasename );

	EXEC SQL CREATE ROUTINE FROM :schedprocfilename;
	if (sqlca.sqlcode != 0) {
	    fprintf( stderr,
		     "Error creating scheduling procedure: %s!  SQL: %d  ISAM: %d\n",
		     schedprocname,
		     sqlca.sqlcode,
		     sqlca.sqlerrd[1] );
	    exit( 1 );
	}
	unlink( schedprocfilename );

	if (do_scheduling) {
	    EXEC SQL SET CONNECTION 'SCHEDULER';
	    if (sqlca.sqlcode != 0) {
		fprintf( stderr,
			 "Error changing connections to SCHEDULER!  SQL: %d  ISAM: %d\n",
			 sqlca.sqlcode,
			 sqlca.sqlerrd[1] );
		exit( 1 );
	    }

	    EXEC SQL SELECT count(*) INTO :OK FROM ph_group WHERE group_name = 'DOSTATS';
	    if (sqlca.sqlcode != 0) {
		fprintf( stderr,
			 "Error verifying DOSTATS task group record!  SQL: %d  ISAM: %d\n",
			 sqlca.sqlcode,
			 sqlca.sqlerrd[1] );
		exit( 1 );
	    }
	    if (OK == 0) {
		EXEC SQL INSERT INTO ph_group (group_id, group_name, group_description)
		    VALUES ( 0, "DOSTATS", "Update statistics generated by dostats(c)." );
		if (sqlca.sqlcode < 0 && sqlca.sqlcode != -268) {
		    fprintf( stderr,
			     "Error inserting DOSTATS task group record!  SQL: %d  ISAM: %d\n",
			     sqlca.sqlcode,
			     sqlca.sqlerrd[1] );
		    exit( 1 );
		}
	    }
	    /* Add scheduler record here. --ASK-- */
	    snprintf( schedule_stmt,
		      sizeof schedule_stmt,
		      "INSERT INTO ph_task( tk_id, "
		      "       tk_name, tk_type, tk_group, tk_description, tk_execute, "
		      "       tk_start_time, tk_stop_time, tk_frequency, tk_Sunday, tk_Monday, "
		      "       tk_Tuesday, tk_Wednesday, tk_Thursday, tk_Friday, tk_Saturday ) "
		      "VALUES ( 0, '%s_task', 'TASK', 'DOSTATS', 'dostats run for %s', "
		      "   'EXECUTE PROCEDURE %s:%s($DATA_TASK_ID, $DATA_SEQ_ID, %s);', %s, %s, %s, %s, %s, %s, %s, %s, %s, %s );",
		      schedprocname,
		      schedprocname,
		      databasename,
		      schedprocname,
		      sch_onetime ? "\"t\"" : "\"f\"",
		      sch_starttime[0] == (char)(0) ? "NULL" : sch_starttime,
		      sch_lasttime[0] == (char)(0) ? "NULL" : sch_lasttime,
		      sch_frequency[0] == (char)(0) ? "NULL" : sch_frequency,
		      sch_sunday,
		      sch_monday,
		      sch_tuesday,
		      sch_wednesday,
		      sch_thursday,
		      sch_friday,
		      sch_saturday );
	    EXEC SQL EXECUTE IMMEDIATE :schedule_stmt;
	    if (sqlca.sqlcode != 0) {
		fprintf( stderr,
			 "Error inserting task scheduler record!  SQL: %d  ISAM: %d\n%s\n%*.*s\n",
			 sqlca.sqlcode,
			 sqlca.sqlerrd[1],
			 schedule_stmt,
			 sqlca.sqlerrd[4],
			 sqlca.sqlerrd[4],
			 "^" );
	    }
	}
    }
    return (do_err_cnt ? (error_ret - 1) : 0);
}


void readlist( char **list, int *len, char **dblist, int *dblen, char *file )
{
    FILE *fil;
    char string[1025];

    if ((int)strlen( file ) == 1)
	fil = stdin;
    else {
	fil = fopen( &file[1], "rt" );
	if (fil == (FILE *)NULL) {
	    fprintf( stderr, "Cannot open list file: %s.\n", &file[1] );
	    exit( 19 );
	}
    }

    if (fil == stdin) {
	fputs( "table> ", stderr );
	fflush( stderr );
    }
    while (fgets( string, 1025, fil )) {
	char *ch;

	/* remove trailing newline */
	string[(int)strlen( string ) - 1] = (char)0;
	/* strip off any carriage return in the string (Windows files). */
	if ((ch = strchr( string, '\r' )) != (char *)NULL)
	    *ch = (char)0;

	/* new feature: skip blank lines and comments */
	/* TODO: skip blank lines with blanks and allow leading space? */
	if (! *string || *string == '#')
	    continue;

	if (strchr( string, ':' )) {
	    /* database exclude. */
	    if (dblist == (char **) NULL) {
		fprintf( stderr,
			 "Databases not supported in include lists!\n" );
		exit(20);
	    }

	    if (*dblist)
		appendStr( dblist, dblen, "','" );
	    else
		appendStr( dblist, dblen, " AND name NOT IN ('" );

	    appendStr( dblist, dblen, strtok( string, ":" ) );
	} else {
	    /* Table include/exclude. */
	    if (*list)
		appendStr( list, len, "','" );
	    else
		appendStr( list, len, "('" );

	    appendStr( list, len, string );
	}
	if (fil == stdin) {
	    fputs( "table> ", stderr );
	    fflush( stderr );
	}
    }

    if ((int)strlen( file ) > 1)
	fclose( fil );
}

void appendStr( char **str, int *str_size, char *newstr )
{
    int sum_len;

    if (*str == NULL) {
	*str_size = STR_STEP;
	*str = (char *) malloc( STR_STEP );
	**str = 0;
    }

    sum_len = strlen(*str) + strlen(newstr) + 1;
    if (sum_len > *str_size) {
	/* round up sum_len to multiple of STR_STEP */
	/* optimiser won't eliminate this - it would change the meaning */
	*str_size = (sum_len + STR_STEP - 1) / STR_STEP * STR_STEP;

	*str = (char *)realloc( *str, *str_size );

	if (! *str) {
	    fprintf( stderr, "FAILED\nError allocating memory errno = %d.\n", errno );
	    exit( 255 );
	}
    }
    strcat(*str, newstr);
}

#if !defined(NOTIMINGS)
void timediff( struct timeval start,
	       struct timeval end,
	       int *secs,
	       int *msec )
{
    *secs = end.tv_sec - start.tv_sec;
    *msec = end.tv_usec - start.tv_usec;
    if (*msec < 0) {
	*secs--;
	*msec += 1000000;
    }
}
#endif

double minres( int nrecs )
{
    double res;

    if (nrecs > 1) {
	res = (1.0 / (double)nrecs);
	if (res < 0.005)
	    res = 0.005;
	else if (res > 10.0)
	    res = 10.0;
    } else
	res = 10.0;

    return res;
}

int cmpcnts( const void *one, const void *two )
{
    if (((rowcounts_t *)one)->partnum < ((rowcounts_t *)two)->partnum)
	return -1;
    else if (((rowcounts_t *)one)->partnum > ((rowcounts_t *)two)->partnum)
	return 1;
    else
	return 0;
}


long long just_count_it( 
			EXEC SQL BEGIN DECLARE SECTION;
			PARAMETER char *tabowner,
			PARAMETER char *tabname
			EXEC SQL END DECLARE SECTION;
			 )
{
    EXEC SQL BEGIN DECLARE SECTION;
    static bigint nrows;
    static char altsql[2048];
    EXEC SQL END DECLARE SECTION;
    static char lastowner[33] = "", lasttable[129] = "";

    if (strcmp( lastowner, tabowner ) == 0 && strcmp( lasttable, tabname ) == 0) {
	return nrows;
    } else {
        snprintf( lastowner, sizeof lastowner, "%s", tabowner );
	snprintf( lasttable, sizeof lasttable, "%s", tabname );
    }

    if (QuotedIdents == 1) 
	snprintf( altsql,
		  sizeof altsql,
		  "SELECT count(*) FROM \"%s\".\"%s\"",
		  tabowner,
		  tabname );
    else
	snprintf( altsql,
		  sizeof altsql,
		  "SELECT count(*) FROM \"%s\".%s",
		  tabowner,
		  tabname );
    EXEC SQL PREPARE altcnt1 FROM :altsql;
    if (sqlca.sqlcode != 0) {
	fprintf( stderr,
		 "FAILED preparing alt table rowcount! Err=%d, isam=%d\n\n",
		 (int)sqlca.sqlcode,
		 (int)sqlca.sqlerrd[1] );
	fprintf( stderr, "Query: %s;\n", altsql );
	exit( 1 );
    }

    EXEC SQL EXECUTE altcnt1 INTO :nrows;
    if (sqlca.sqlcode != 0) {
	fprintf( stderr,
		 "FAILED executing alt table rowcount! Err=%d, isam=%d for table: %s\n\n",
		 (int)sqlca.sqlcode,
		 (int)sqlca.sqlerrd[1],
		 tabname );
	fprintf( stderr,
		 "Statement was: %s;\n",
		 altsql );
	exit( 1 );
    }

    return nrows;
}


long long getrowcount(
		      EXEC SQL BEGIN DECLARE SECTION;
		      PARAMETER char *tabowner,
		      PARAMETER char *tabname,
		      PARAMETER int tabid,
		      PARAMETER int partnum
		      EXEC SQL END DECLARE SECTION;
		      )
{
    EXEC SQL BEGIN DECLARE SECTION;
    bigint nrows = 0LL;
    int partn;
    EXEC SQL END DECLARE SECTION;
    rowcounts_t *rc, lcl;
    static int once = 1;

    if (!numparts) {
	/* No counts retrieved from sysptnhdr.  Just count the table. */
        return just_count_it( tabowner, tabname );
    } else if (partnum) {
	lcl.partnum = partnum;
	rc = (rowcounts_t *)bsearch( &lcl,
				     rowcounts,
				     numparts,
				     sizeof (rowcounts_t),
				     cmpcnts );
	if (rc)
	    nrows = rc->nrows;
	else {
	    fprintf( stderr,
		     "Error looking up partnum to match rowcount.  "
		     "Getting counts the hard way (non-fragmented).\n" );
	    nrows = just_count_it( tabowner, tabname );
	}
    } else {
	/* Fragmented table.  Get fragment partnums and look them up. */
        if (once) {
	    EXEC SQL DECLARE partn_c CURSOR WITH HOLD FOR
		SELECT partn
		INTO :partn
		FROM "informix".sysfragments
		WHERE tabid = :tabid
		AND fragtype = 'T';
	    if (sqlca.sqlcode != 0) {
		fprintf( stderr,
			 "FAILED declaring cursor for fragment rowcounts! Err=%d, isam=%d\n\n",
			 (int)sqlca.sqlcode,
			 (int)sqlca.sqlerrd[1] );
		exit( 1 );
	    }
	    once = 0;
	}
	EXEC SQL OPEN partn_c;
	if (sqlca.sqlcode != 0) {
	    fprintf( stderr,
		     "FAILED opening cursor for fragment rowcounts! Err=%d, isam=%d\n\n",
		     (int)sqlca.sqlcode,
		     (int)sqlca.sqlerrd[1] );
	    exit( 1 );
	}
	nrows = 0;
	for (;;) {
	    EXEC SQL FETCH partn_c
		INTO :partn;
	    if (sqlca.sqlcode < 0) {
		fprintf( stderr,
			 "FAILED fetching fragment rowcounts! Err=%d, isam=%d\n\n",
			 (int)sqlca.sqlcode,
			 (int)sqlca.sqlerrd[1] );
		exit( 1 );
	    } else if (sqlca.sqlcode == 100)
		break;
	    lcl.partnum = partn;
	    rc = (rowcounts_t *)bsearch( &lcl,
					 rowcounts,
					 numparts,
					 sizeof (rowcounts_t),
					 cmpcnts );
	    if (rc)
		nrows += rc->nrows;
	    else {
		fprintf( stderr,
			 "Error looking up partnum to match rowcount.  "
			 "Getting counts the hard way (fragmented table).\n" );
		EXEC SQL CLOSE partn_c;
		return just_count_it( tabowner, tabname );
	    }
	}
	EXEC SQL CLOSE partn_c;
    }

    return nrows;
}

void usage( char *program )
{
    char *pager;
    FILE *errout;
    int  redirected = 0;
 
    errout = fdopen( 2, "w" );
    if (isatty(2)) {
        /* fclose( errout ); -- UNIX more needs stderr open, so... */
        if (!(pager = getenv( "PAGER" )) || (int)strlen(pager)==0)
            pager = "more";
        errout = popen( pager, "w" );
        redirected = 1;
    }

    fprintf(errout,
	    "Usage:\t%s [-h host] -d database [-t table] [-p] [-f filename] [-g] [-N]\n"
	    "\t\t[-i table_1 [-i table_2 [...]]] [-x [table_a|dbase_a:] [-x...]] \n"
	    "\t\t[-i @[file1] [-i @[file2]]] [-x @[file1] [-x @[file2]]] [-G] \n"
	    "\t\t[-i >[proc1] [-i >[proc2]]] [-x >[proc1] [-x >[proc2]]] \n"
	    "\t\t[-i '!where clause'|subquery] [-x '!where clause'] \n"
	    "\t\t[-S] [-v<-1|0|1>] [-e|-E] [-s] [-V] [-F] [-T] [-I]\n"
	    "\t\t [-C] [-M] [-L] [-P procPDQ] [-Q tablePDQ] [-w Nsecs] [-X]\n"
	    "\t\t[-R HighRes] [-r MedRes] [-c MedConf] [-Z MinSamp|MinPct] [-m] \n"
	    "\t\t[-b[-B BrowseThreshHold]] [-a [-A AgingLimit]] [-n errlimit] \n"
	    "\t\t[--procedure=procname] [--proc-local] [--execute-local] \n"
	    "\t\t[--schedule [--frequency [d|w[0-6]|m[0-6]]]\n"
	    "\t\t    [--first-time <datetime>] --distributions-high <list file>\n"
	    "\t\t    [--one-time]] [--aus-thresholds] [--clean-distributions]\n"
	    "\t\t[--time-display] [--display-local] [--display-subsec]\n"
	    "\t\t[--drop-distributions] [--force_run] [--include-owner]\n"
	    "\t\t[--small-tables-high [--small-tables-threshold=<val>]]\n"
	    "\t\t[--isolation <c|d|l>] [--force-run] [--auto-run]\n",
	    program );
    fprintf( errout,
	     "\t-h Host defaults to $INFORMIXSERVER.  (The program uses multiple \n"
	     "\t   connections to the host.  This requires a non-shared memory \n"
	     "\t   connection.) \n" );
    fprintf( errout,
	     "\t-d Database may contain valid \"matches\" clause wildcards.\n" );
    fprintf( errout,
	     "\t   The string 'ALL' is equivalent to '*' and will cause %s to \n"
	     "\t   execute against all databases except sysmaster.\n",
	     program ); 
    fprintf( errout,
	     "\t-t Table may contain valid \"matches\" clause wildcards.\n"
	     "\t   Table defaults to '*', all user tables (No system catalog tables.)\n"
	     "\t   The special name \"none\" can be used with the -i option to specify\n"
	     "\t   a limited list of tables with dissimilar names.\n" );
    fprintf( errout,
	     "\t-G Get physical row count using COUNT(*).  On servers with a very large\n"
	     "\t   number of tables this option can improve runtime as sysptnhdr has no\n"
	     "\t   indices and must be scanned.\n" );
    fprintf( errout,
	     "\t-b Browse mode.  Select only tables which have had their row count \n"
	     "\t   change by a percentage specified by the -B, browse threshhold, \n"
	     "\t   option since statistics were last updated.  Overrides inclusions. \n"
	     "\t   This option requires a non-shared memory connection and so is not \n"
	     "\t   compatible with -S. \n" );
    fprintf( errout,
	     "\t-B Specify browse threshhold.  Specify percent change used by the \n"
	     "\t   browse mode option (-b) to filter tables on which to operate. \n"
	     "\t   Range: 0.0001<->10000.0  (Default: 15.0%%) A table's statchange \n"
	     "\t   percent will override.\n" );
    fprintf( errout,
	     "\t-a Distribution Aging.  Select only tables whose distributions are \n"
	     "\t   more than AgingLimit days old.  If specified with -b adds tables \n"
	     "\t   rejected by browsing that satisfy the AgingLimit.  Otherwise Aging \n"
	     "\t   eliminates tables otherwise included if they do no exceed AgingLimit.\n" );
    fprintf( errout,
	     "\t-A Specify distribution AgingLimit.  Range: >= 0. (Default: 30 days) \n" );
    fprintf( errout,
	     "\t-p Update statistics for stored procedures.  If -t is not also\n"
	     "\t   specified only stored procedures are updated.  (Default: true\n"
	     "\t   if no -t option provided, false if -t, -x, or -i are provided.)\n" );
    fprintf( errout,
	     "\t-Q Set PDQPRIORITY for updating statistics on tables.  \n"
	     "\t   (Default: Environment value of PDQPRIORITY or zero.)\n");
    fprintf( errout,
	     "\t-w Specify wait setting for lock mode.  >0 wait N seconds, <0 wait \n"
	     "\t   forever, ==0 set to not wait.\n" );
    fprintf( errout,
	     "\t-P Set PDQPRIORITY for compiling stored procedures.  A PDQPRIORITY \n"
	     "\t   value for stored procedure compilation is now always produced.\n"
	     "\t   (Default: zero.)\n" );
    fprintf( errout,
	     "\t-f filename writes statistics commands to the named file and does not \n"
	     "\t   execute (default: Execute commands immediately).  If filename is '-' \n"
	     "\t   output is written to stdout (errout should be redirected). \n"
	     "\t   (Long option: --to-file).\n" );
    fprintf( errout,
	     "\t-X Enable SET EXPLAIN ON during the statistics run.  You can review\n"
	     "\t   the sqexplain.out file do see if other options and environment \n"
	     "\t   variables may improve the performance of dostats.  See Informix \n"
	     "\t   manuals about using PDQPRIORITY, PSORT_NPROCS, PSORT_DBTEMP, \n"
	     "\t   DBSPACETEMP, and DBUPSPACE with UPDATE STATISTICS.\n" ); 
    fprintf( errout,
	     "\t-v Verbose mode.  There are 4 modes: \n"
             "\t   -1 - No progress output.  Only print error messages. \n"
             "\t    0 - Normal progress output. \n"
             "\t    1 - Expands all explicit and inferred options in addition to normal \n"
             "\t        progress output. \n"
	     "\t    2 - Adds even more verbosity. \n" );
    fprintf( errout,
	     "\t-S Shared memory connection handling.  Only permits a single database\n"
	     "\t   to be processed.  Does not support browsing mode!\n" );
    fprintf( errout,
	     "\t-i Specify tables to forceably include in the tables processed.  One\n"
	     "\t   may enter a single table, the @ sign followed by the name of a file\n"
	     "\t   containing a list of tables one per line, or an exclamation point (!)\n"
	     "\t   followed by either a WHERE filter to be applied to systables to \n"
	     "\t   select tables for inclusion, or a complete sub-query beginning with \n"
	     "\t   the keyword 'select' which will return a list of valid table names \n"
	     "\t   to include.  To aid script use if the argument to -i is simply '@' \n"
	     "\t   stdin is read.  Multiple -i options are concatenated to form a net \n"
	     "\t   list.  Note that all -i options can be used together.  (NB - Database\n"
	     "\t   includes are not implemented.)  NOTE: The included SELECT statement\n"
	     "\t   is now processed as a derived table with v11.xx.  ORDER BY and other\n"
	     "\t   SQL features which are restricted in the sub-queries used in earlier\n"
	     "\t   releases (still used when communicating with Informix v10.00) are now\n"
	     "\t   supported!\n" );
    fprintf( errout,
	     "\t     To specify procedures to include in procedure processing, precede\n"
	     "\t   the procedure name with a greater-than sign (>). Multiple entries are\n"
	     "\t   supported.\n" );
    fprintf( errout,
	     "\t-x Specify tables or databases to forceably exclude from those\n"
	     "\t   processed.  One may enter a single table or database, the @ sign\n"
	     "\t   followed by the name of a file containing a list of tables and/or\n"
	     "\t   databases, one per line, or an exclamation point (!) followed by a\n"
	     "\t   WHERE filter to be applied to systables to select tables to be\n"
	     "\t   excluded.  To aid script use if the argument to -x is simply '@' \n"
	     "\t   stdin is read.  Multiple -x options are concatenated to form a net \n"
	     "\t   list.  Note that all -x options can be used together.  Database \n"
	     "\t   names are specified by appending a colon (:) to the name (ex: \n"
	     "\t   \"sysutils:\" to exclude the sysutils database).\n" );
    fprintf( errout,
	     "\t     To specify procedures to exclude from procedure processing, precede\n"
	     "\t   the procedure name with a greater-than sign (>). Multiple entries are\n"
	     "\t   supported.\n" );
    fprintf( errout,
	     "\t     NOTE: passing a complete SELECT to -x is deprecated.  Improvements\n"
	     "\t   in the handling of -i '!SELECT...' make it better to simply invert\n"
	     "\t   the filters one would pass to -x and use -i instead.\n" );
    fprintf( errout,
	     "\t-e Expand reporting to output elapsed time of each stage in seconds \n"
	     "\t   and fractions. \n" );
    fprintf( errout,
	     "\t-E Expand reporting to output elapsed time of each stage in hours,\n"
	     "\t   minutes, seconds and fractions. \n" );
#if defined(NOTIMINGS)
    fprintf( errout,
	     "\t   THE TIMING REPORT OPTIONS (-e, -E) have been disabled in this \n"
	     "\t   executable.\n" );
#endif
    fprintf( errout,
	     "\t-V Print out dostats version and source code revision and exit.\n" );
    fprintf( errout,
	     "\t-F Disable single column index optimization.  Use HIGH...DISTRIBUTIONS \n"
	     "\t   ONLY and LOW which is faster on some systems/databases.\n" );
    fprintf( errout,
	     "\t-N No specific keyword for updating stored procedures/functions. \n" );
    fprintf( errout, 
             "\t-n Do not exit on error until errlimit errors are encountered.  Only \n"
             "\t   This only affects errors running update statistics commands. \n" );
    fprintf( errout,
	     "\t--proc-local Create the table containing commands in each database\n"
	     "\t\trather than in sysadmin.  Needed to support servers earlier than\n"
	     "\t\t11.50 and non-logged databases.  Requires --procedure to provide\n"
	     "\t\ta known procedure name.  Cannot be used with --schedule.\n" );
    fprintf( errout,
	     "\t--execute-local  Set the procedure to execute in the target database.\n"
	     "\t\tThis option or proc-local is required for unlogged and ANSI mode\n"
	     "\t\tdatabases.  Stores the procedure in the target database like\n"
	     "\t\t--proc-local but also executes it there. Can be used with\n"
	     "\t\t--schedule.\n" );
    fprintf( errout,
	     "\t--procedure Create the named procedure with the UPDATE STATISTICS \n"
	     "\t\tcommands to run.  Can be used to replace a previously scheduled \n"
	     "\t\tcommand procedure, to create a new named procedure for \n"
	     "\t\tscheduling or just to create a procedure to be manually executed\n"
	     "\t\tlater. If --procedure is not specified, and --schedule is, a \n"
	     "\t\tdefault name is generated for the procedure that is scheduled. \n" );
    fprintf( errout,
	     "\t--schedule Create a stored procedure with the commands and schedule the\n"
	     "\t\tprocedure to run in the future.  Defaults to running once at\n"
	     "\t\tthe following midnight.  Adjust run time and frequency with \n"
	     "\t\tthe following options:\n"
	     "\t\t--frequency How often to run the update statistics \n"
	     "\t\t    procedure created: \n"
	     "\t\t      d - run daily.\n"
	     "\t\t      w - run weekly on day: 0-Sunday, 1-Monday, etc...\n"
	     "\t\t      m - run monthly on the first: 0-Sunday, 1-Monday,\n"
	     "\t\t\tetc.. of month\n"
	     "\t\t--first-time Time of day to run (or to first run) the \n"
	     "\t\t    procedure. \n"
	     "\t\t--last-time  Last time of day to run (if frequency is\n"
	     "\t\t    less than 1 day).\n"
	     "\t\t--one-time  Run the procedure as scheduled then disable the run.\n\n"
	     "\t  You can see your dostats scheduled tasks by querying the task group\n"
	     "\t  (ph_task.tk_group = 'DOSTATS'\n" );
    fprintf( errout,
	     "\t--aus-thresholds Retrieve the Auto Update Statistics settings for \n"
	     "\t   Aging, Browsing (Change), and PDQPRIORITY.  -A, -B, & -Q override \n"
	     "\t   the corresponding individual AUS settings. Aging and Browsing are \n"
	     "\t   only enabled if -a and/or -b are also specified. Also sets the \n"
	     "\t   small table row count for the --small-tables-high option.\n" );
    fprintf( errout,
	     "\t--time-display Display starting time for each command and block of \n"
	     "\t   commands in UTC timezone.  Not sensible if -f, --schedule, or \n"
	     "\t   --procedure are specified.\n" );
    fprintf( errout,
	     "\t--display-local Display starting time for each command and block of \n"
	     "\t   commands in the local timezone.  Not sensible if -f, --schedule, \n"
	     "\t   or --procedure are specified. Overrides --time-display.\n" );
    fprintf( errout,
	     "\t--display-subsec display start times to 1/1000000th second. Enables \n"
	     "\t   --display-time. --display-local is valid with --display-subsec.\n" );
    fprintf( errout,
	     "\t--drop-distributions Begin by dropping all existing data distributions\n"
	     "\t   from the sysdistrib catalog table.  Must be executed by user \n"
	     "\t   'informix' as the database owner does not have privileges on catalog\n"
	     "\t   tables.  Most useful after an IDS version upgrade. Cannot be used \n"
	     "\t   with -t, -x, or -i -- use --clean-distributions instead.\n" );
    fprintf( errout,
	     "\t--clean-distributions Begin processing each table by deleting existing \n"
	     "\t   distributions.  Similar to --drop-distributions but operates on\n"
	     "\t   single tables.  Can be used with -t, -x & -i.\n" );
    fprintf( errout,
	     "\t--isolation Set isolation level for sysmaster and the affected database.\n"
	     "\t   Levels are: c - Committed Read, d - Dirty Read, or l - Committed Read\n"
	     "\t   with the LAST COMMITTED option. When sysadmin is accessed to schedule\n"
	     "\t   runs default isolation is always used. (Default: Committed Read).\n" );
    fprintf( errout,
	     "\t--force-run Force statistics to be updated even if the statchange level\n"
	     "\t   has not been achieved.  Overrides AUTO_STAT_MODE configuration and \n"
	     "\t   environment settings. Supported by Informix v11.70 and later.\n" );
    fprintf( errout,
	     "\t--auto-run Cause statistics to take consideration for the statchange \n"
	     "\t   level.  Overrides AUTO_STAT_MODE configuration and environment \n"
	     "\t   settings. Supported by Informix v11.70 and later.\n" );
    fprintf( errout,
	     "\t--error-count If -n is also specified then dostats returns the number\n"
	     "\t   of errors encountered to the point of exit.\n" );
    fprintf( errout,
	     "\t--small-tables-high - process tables with low row counts HIGH on all\n"
	     "\t   columns. The default 'small' table row count is zero (0). Can be \n"
	     "\t   set using --small-tables-threshold to pass in the value or by \n"
	     "\t   setting the AUS_SMALL_TABLES threshold in sysadmin.ph_thresholds\n"
	     "\t   and passing the --aus-thresholds option.\n" );
    fprintf( errout,
	     "\t--small-tables-threshold - set the threshold to determine small table\n"
	     "\t   processing when --small-tables-high is passed. This option takes \n"
	     "\t   precedence over the AUS thresholds even if --aus-thresholds is \n"
	     "\t   passed.\n" );
    fprintf( errout,
	     "\t--distributions-high - Provides the name of a file containing a list\n"
	     "\t   of columns that should have HIGH distributions regardless of the\n"
	     "\t   usual level.  File lines contain three space separated fields:\n"
	     "\t   databasename tablename columnname. The order of lines is not\n"
	     "\t   important, dostats sorts the list after reading it.\n" );
    fprintf( errout,
	     "\t--include-owner - Include table and procedure owner names to get \n"
	     "\t   around problems with tables and procedures named after keywords \n"
	     "\t   (null, hour, distributions, etc.\n" );
    fprintf( errout,
	     "\t-R HighRes sets resolution for HIGH Level distributions.\n" );
    fprintf( errout,
	     "\t   Calculate distributions by dividing keys into (100 / HighRes)\n" );
    fprintf( errout,
	     "\t   bins each with approximately the same number of keys.\n" );
    fprintf( errout,
	     "\t   Valid range: max(0.005,1/nrows) <= HighRes <= 10.00  \n"
	     "\t   Default: 0.5 (200 bins)\n" );
    fprintf( errout,
	     "\t-r MedRes sets resolution for MEDIUM Level distributions.\n" );
    fprintf( errout,
	     "\t   Calculate distributions by dividing keys into (100 / MedRes)\n" );
    fprintf( errout,
	     "\t   bins each with approximately the same number of keys.\n" );
    fprintf( errout,
	     "\t   Valid range: max(0.005,1/nrows) <= MedRes <= 10.00  \n"
	     "\t   Default: 2.5 (40 bins)\n" );
    fprintf( errout,
	     "\t-c MedConf set confidence level for MEDIUM Level distributions.\n" );
    fprintf( errout,
	     "\t   Valid range: 0.80 <= MedConf <= 0.99, Default: 0.95\n" );
    fprintf( errout,
	     "\t-m Enable processing of system catalog metadata tables.\n"
	     "\t   (Default: disabled)\n" );
    fprintf( errout,
	     "\t-Z Set minimum sample size for MEDIUM resolution. (0 < MinSamp < nrows)\n" 
	     "\t   or (0.0 < MinPct < 1.0) Values less than 1.0 are treated as a \n"
	     "\t   percentage of rows to sample. Values greater than 1 are treated\n"
	     "\t   as the minimum number of rows to sample. A value of 1 evaluates\n"
	     "\t   all rows!\n" );
    fprintf( errout,
	     " \n"
	     "\t   By default all of -T, -I, & -C flags are set.  Manually specifying \n"
	     "\t   any of -T, -L, -I, or -C disables any of the others not also \n"
	     "\t   explicitly specified.  Enabling -T -I emulates dostats version 1. \n"
	     "\t   Options -T & -L are mutually exclusive if both are given the last \n"
	     "\t   prevails. \n" );
    fprintf( errout,
	     "\t    -T  Enable table level statistics update to MEDIUM. \n" );
    fprintf( errout,
	     "\t    -L  Set table level statistics update to LOW and drop distributions. \n" );
    fprintf( errout,
	     "\t    -l  Set table level statistics update to LOW keeping existing \n"
	     "\t\tdistributions. \n" );
    fprintf( errout,
	     "\t    -I  Enable statistics update of first column of each index to HIGH. \n" );
    fprintf( errout,
	     "\t    -C  Enable statistics update indexed columns to LOW. \n" );
    fprintf( errout,
	     "\t    -M  Enable statistics update MEDIUM for non-indexed columns. \n" );
    fprintf( errout,
	     "\t    -g  Enable statistics update fragmentation expression columns to \n"
	     "\t        HIGH (ie treat like first index key columns). \n" );
    fprintf( errout,
	     "\n\tSetting PDQPRIORITY in the %s environment causes an effective \n"
	     "\targument of '-P 0' to be added the commandline to protect procedures. \n",
	     program);
    fprintf( errout,
	     "\n\tThe environment variables INFORMIXUSER and INFORMIXPASS can to used\n"
	     "\tto specify a username and password (respectively) other than the \n"
	     "\tcurrent user's id and password.\n\n" );

    if (redirected) {
        fflush( errout );
        pclose( errout );
    }
}


#if !defined(MAXWORD)
#define MAXWORD 65536
#endif

/* Remove 'out' & 'inout' parameter type entries from parameter type lists. */
void StripTypeList( char *list )
{
    char *out, word[MAXWORD], *wptr;
    int cnt = 0;

    out = list;
    word[0] = (char)0;
    wptr = word;
    while (1) {
	switch (*list) {
	case 0:
	case ' ':
	case ',':
	    *wptr = (char)0;
	    if (strcmp( word, "out" ) == 0 || strcmp( word, "inout" ) == 0 || 
		strcmp( word, "OUT" ) == 0 || strcmp( word, "INOUT" ) == 0)
		{
		    /* Toss this word. */
		    wptr = word;
		    if (*list == (char)0) {
			return;				/* Yes - go back, we're done. */
		    }
		    list++; /* Skip the trailing space also. */
		} else {
		/* insert this word in the output. */
		strncpy( out, word, strlen( word ) );	/* Append the word to the output. */
		wptr = word;				/* Set up for the next word. */
		out += strlen( word );			/* Adjust the output pointer. */
		*out = *list;				/* Copy the separator char also. */
		out++;					/* Adjust pointer again. */
		if (*list != (char)0)			/* Done? */
		    list++;				/* No - skip separator. */
		else
		    return;				/* Yes - go back, we're done. */
	    }
	    break;
	default:					/* Build each word here. */
	    *wptr = *list;
	    list++;
	    wptr++;
	    cnt++;
	    if (cnt >= MAXWORD) {
		fprintf( stderr,
			 "Word size (%d) exceeded.  Word is: <%s> \n",
			 MAXWORD,
			 word );
		exit( 1 );
	    }
	    break;
	}
    }
}


static short col;
static char *colstr, *opclstr, *tok;
static char *tmp;

void InitKeyStr( char *keystr )
{
    if (keystr[0] == '<') {
	/* Functional key.  2nd token is a list of columns. */
      
	tok = strtok( keystr, "(" ); /* Strip procid */
	if(tok) {
	    tmp = tok + strlen(tok) + 1;
	}
	colstr = strtok( (char *)NULL, "), [" ); /* Get collist */
	tmp = colstr + strlen( colstr ) + 1;

	if (tmp[0] == '[' || tmp[1] == '[') {
	    opclstr = strtok( (char *)NULL, "], " ); /* Strip opclass */
	    tmp = opclstr + strlen( opclstr ) + 1;
	}
	col = (short)atol( colstr );
    } else {
	/* Columns key.  First token we encounter is the colno. */
	tok = strtok( keystr, "[ " );
	if(tok) {
	    tmp = tok + strlen(tok) + 1;
	}
	if (tok) {
	    col = (short)atol( tok );

	    /* Next token is the op class id */
	    tok = strtok( (char *)NULL, "], [" ); /* Strip opclass */
	    if(tok) {
		tmp = tok + strlen(tok) + 1;
	    }
	} else {
	    col = 0;
	}
    }
}

short ParseKeyStr( )
{
    short ret;
    
    ret = abs( col );

    if (ret) {
	tok = strtok( (char *)NULL, "], [()" );

	if(tok) {
	    tmp = tok + strlen(tok) + 1;
	}

	if (tok && tok[0] == '<') {
	    /* Start of another functional key. */
	    colstr = strtok( (char *)NULL, "), [" ); /* Get collist */
	    tmp = colstr + strlen( colstr ) + 1;
	    if (tmp[0] == '[' || tmp[1] == '[') {
		opclstr = strtok( (char *)NULL, "], " ); /* Strip opclass */
		tmp = opclstr + strlen( opclstr ) + 1;
	    }
	    col = (short)atol( colstr );
	} else if (tok) {
	    if(tok) {
		tmp = tok + strlen(tok) + 1;
	    }
	    col = (short)atol( tok );

	    if(tok) {
		tmp = tok + strlen(tok) + 1;
	    }
	    /* Next token is the op class id */
	    tok = strtok( (char *)NULL, "], [()" ); /* Strip opclass */
	    if(tok) {
		tmp = tok + strlen(tok) + 1;
	    }
	} else {
	    col = 0;
	}
    }
    return ret;
}

static off_t pos = 0;
static int inblob = 0;
 
int openit( loc_t *loc, int flags, int bsize )
{
    if (inblob == 1) {
        fprintf( stderr, "Open called for new blob without close.\n"
		 "length = %d.!\n",
		 (int)loc->loc_xfercount );
    }
    loc->loc_status = 0;
    loc->loc_xfercount = 0L;
    inblob = 0;
    blobbuff->global_length = 0 ;
    pos = 0;
    /* Make sure blob buffer is large enough to hold BLOB. */
    if (bsize > blobbuff->bbsize) {
        blobbuff->bbsize = bsize + (1024 - (bsize % 1024));
	blobbuff->buff = (char *)realloc( blobbuff->buff, blobbuff->bbsize );
    }
    memset( blobbuff->buff, 0, blobbuff->bbsize );
 
    return 0;
}
 
int writeit( loc_t *loc, char *buf, int nbytes )
{
    if ((pos + nbytes) > blobbuff->bbsize) {
        blobbuff->bbsize += (2 * (nbytes < 1024 ? 1024 : nbytes));
        blobbuff->buff = (char *)realloc( blobbuff->buff, blobbuff->bbsize );
    }
    if (!inblob)
        inblob = 1;
    memcpy( &(blobbuff->buff[pos]), buf, nbytes );
    pos += nbytes;
 
    loc->loc_xfercount += nbytes;
    blobbuff->global_length += nbytes;
 
    return 0;
}
 
int closeit( loc_t *loc )
{
    unsigned int count;
 
    inblob = 0;
    loc->loc_status = 0;
    if (loc->loc_oflags & LOC_WONLY) {
        /* Fetching, cleanup. */
        loc->loc_indicator = 0;
        loc->loc_size = loc->loc_xfercount;
    }
    count = blobbuff->global_length ;
    if( blobbuff->global_length != loc->loc_xfercount)
        fprintf( stderr,
		 "GLOBAL_LENGTH !=  loc->loc_xfercount <%d, %d> \n",
		 blobbuff->global_length,
		 (int)loc->loc_xfercount );
    pos = 0;
 
    return 0;
}

/*
 * Find a column name in the list of names.  Returns index if found,
 * else -1.  Simple linear search, the list is short.
 */
int cfind( char *item, char collist[][129], int ncols )
{
    int col;

    for (col = 0; col < ncols; col++)
	if (strcmp( item, collist[col] ) == 0)
	    break;
    if (col >= ncols)
	return -1;
    else
	return col;
}

/* Store the statistics statements in the dostats_stmts table either in sysadmin or the local database for later execution. */
void Store_Statement(
		     EXEC SQL BEGIN DECLARE SECTION;
		     PARAMETER char *statement
		     EXEC SQL END DECLARE SECTION;
		     )
{
    static char last_proc[129] = "";
    int left;

    EXEC SQL BEGIN DECLARE SECTION;
    static int sequence = 0;
    string part[769];
    char cont_char[2];
    EXEC SQL END DECLARE SECTION;

    if (strcmp( last_proc, schedprocname ) != 0) {
	/* First call for this procedure.  Initialize. */
	sequence = 1;
	strcpy( last_proc, schedprocname );
	if (proc_local)
	    EXEC SQL DELETE FROM dostats_stmts WHERE function = :schedprocname;
	else
	    EXEC SQL DELETE FROM sysadmin:dostats_stmts WHERE function = :schedprocname;
    }

    if (verbose > 1)
	fprintf( stderr, "In Store_Statement sequence = %d with:\n\t%s\n", sequence, statement );

    cont_char[1] = (char)0;
    left = strlen( statement );
    while (left > 0) {
	snprintf( part, sizeof part, "%s", statement );

	left -= 768;
	statement += 768;

	if (left > 0) {
	    cont_char[0] = 't';
	} else {
	    cont_char[0] = 'f';
	}

	if (proc_local)
	    EXEC SQL INSERT INTO dostats_stmts
		(
		 function,
		 sequence,
		 continued,
		 statement
		 ) VALUES
		(
		 :schedprocname,
		 :sequence,
		 :cont_char,
		 :part );
	else
	    EXEC SQL INSERT INTO sysadmin:dostats_stmts
		(
		 function,
		 sequence,
		 continued,
		 statement
		 ) VALUES
		(
		 :schedprocname,
		 :sequence,
		 :cont_char,
		 :part );
	if (sqlca.sqlcode < 0) {
	    fprintf( stderr,
		     "Error inserting statements to dostats_stmts%s.  SQL: %d, ISAM: %d.\n",
		     proc_local ? "local" : "",
		     sqlca.sqlcode,
		     sqlca.sqlerrd[1] );
	}
	sequence++;
    }
    if (verbose > 1)
	fprintf( stderr, "Bottom of Store_Statement.\n" );
}

/* Write the statistics statement to the output stream for later execution. */
void Write_Statement( char *statement )
{
    fprintf( outfile, "%s\n", statement );
    fflush( outfile );
}

void TimeStart( struct timeval *start, int ExpandReport ) {
    char dstr[30];
    struct tm timep;

#if !defined(NOTIMINGS)
    if (ExpandReport || display_time)
	gettimeofday( start, NULL );
    if (display_time) {
	if (display_local)
	    localtime_r( &(start->tv_sec), &timep );
	else
	    gmtime_r( &(start->tv_sec), &timep );
	strftime( dstr, sizeof dstr, "%F %T", &timep );
	if (display_fract)
	    fprintf( stderr, "%s.%06d: ", dstr, start->tv_usec );
	else
	    fprintf( stderr, "%s: ", dstr );
    }
#endif
}

void TimeMark( struct timeval start, int ExpandReport, char *intro, int display, int strt ) {
#if !defined(NOTIMINGS)
    struct timeval end;
    int secs, msecs;
    char dstr[30];
    struct tm timep;

    if (ExpandReport || display_time || strt == 0)
	gettimeofday( &end, NULL );
    if (display_time && display) {
        if (display_local)
            localtime_r( &(end.tv_sec), &timep );
        else
            gmtime_r( &(end.tv_sec), &timep );
	strftime( dstr, sizeof dstr, "%F %T", &timep );
	if (display_fract)
	    fprintf( stderr, "%s.%06d: ", dstr, end.tv_usec );
	else
	    fprintf( stderr, "%s: ", dstr );
    }
    
    if (ExpandReport) {
	timediff( start,
		  end,
		  &secs,
		  &msecs );
	if (ExpandReport == 2) {
	    fprintf( stderr,
		     "%s\t(%d hrs %d mins %d.%*.*d Secs)\n",
		     intro,
		     (secs / 3600),
		     ((secs % 3600) / 60),
		     (secs % 60),
		     (display_fract == 1) ? 6 : 3,
		     (display_fract == 1) ? 6 : 3,
		     (display_fract == 1) ? msecs : ((msecs + 500)/1000) );
	} else {
	    fprintf( stderr,
		     "%s\t(%d.%*.*d Secs)\n",
		     intro,
		     secs,
		     (display_fract == 1) ? 6 : 3,
		     (display_fract == 1) ? 6 : 3,
		     (display_fract == 1) ? msecs : ((msecs + 500)/1000) );
	}
    } else if (verbose >= 0)
	fprintf( stderr, "%s\n", intro );
#else
    if (verbose >= 0)
	fprintf( stderr, "%s\n", intro );
#endif
}


void Create_Process_Table( char *database )
{

    EXEC SQL SELECT count(*) INTO :tbl_chk FROM systables WHERE tabname = 'dostats_stmts';
    if (sqlca.sqlcode < 0) {
	fprintf( stderr,
		 "Cannot verify dostats_stmts table.  SQL: %d  ISAM: %d.\n",
		 sqlca.sqlcode,
		 sqlca.sqlerrd[1] );
	exit( 1 );
    }
    if (tbl_chk == 0) {
	/* Table does not already exist.  Create it. */
	EXEC SQL CREATE TABLE dostats_stmts (
					     function char(128),
					     sequence int,
					     continued boolean,
					     statement lvarchar(768) );
	if (sqlca.sqlcode < 0) {
	    fprintf( stderr,
		     "Cannot create dostats_stmts table in <%s>.  SQL: %d  ISAM: %d.\n",
		     database,
		     sqlca.sqlcode,
		     sqlca.sqlerrd[1] );
	    exit( 1 );
	}
	EXEC SQL CREATE INDEX dostats_stmts_pk on dostats_stmts (function, sequence, continued desc);
	if (sqlca.sqlcode < 0) {
	    fprintf( stderr,
		     "Cannot index dostats_stmts table in <%s>.  SQL: %d  ISAM: %d.\n",
		     database,
		     sqlca.sqlcode,
		     sqlca.sqlerrd[1] );
	    exit( 1 );
	}
    }
}
 
void get_extended_desc ( int xid, char *xtddescription )
{
    EXEC SQL BEGIN DECLARE SECTION;
    int paramxid;
    string xtddesc[257];
    EXEC SQL END DECLARE SECTION;

    paramxid = xid;
    xtddescription[0] = (char)0;

    EXEC SQL DECLARE c_set_desc CURSOR FOR
	SELECT description
	FROM sysxtddesc
	WHERE extended_id = :paramxid
	ORDER BY seqno;
    if (sqlca.sqlcode < 0) {
	fprintf( stderr, 
		 "Error declaring extended description. SQL=%d, ISAM=%d.\n",
		 sqlca.sqlcode,
		 sqlca.sqlerrd[1] );
	exit(1);
    }
    EXEC SQL OPEN c_set_desc;
    if (sqlca.sqlcode < 0) {
	fprintf( stderr, 
		 "Error opening extended description. SQL=%d, ISAM=%d.\n",
		 sqlca.sqlcode,
		 sqlca.sqlerrd[1] );
	exit(1);
    }
    for (;;) {
	EXEC SQL FETCH c_set_desc INTO :xtddesc;
	if (sqlca.sqlcode == 100)
	    break;
	else if (sqlca.sqlcode < 0) {
	    fprintf( stderr, 
		     "Error fetching extended description. SQL=%d, ISAM=%d.\n",
		     sqlca.sqlcode,
		     sqlca.sqlerrd[1] );
	    exit(1);
	}
	strcat( xtddescription, xtddesc );
    }
}

int return_coltypes( int prcid, char *procstr )
{
    char part1[30], part2[30], typename[65536];
    int p1, p2, pt1, pt2, fr1, fr2;
    int ext, col, attr;
    char *p;
    char xtddescription[65536], mode[2];

    EXEC SQL BEGIN DECLARE SECTION;
    static string stmt3[2048];
    int procid, paramid, paramtype, paramlength, paramxid, paramattr;
    short paramname_i;
    string typestr[32764], paramname[129], xtddesc[257];
    EXEC SQL END DECLARE SECTION;

 redo:
    if (first == 1) {
	snprintf( stmt3,
		  sizeof stmt3,
		  "SELECT paramtypes "
		  "FROM sysprocedures "
		  "WHERE procid = ? " );
	EXEC SQL PREPARE parmtype_s FROM :stmt3;
	if (sqlca.sqlcode < 0) {
	    fprintf( stderr,
		     "Error preparing parmtype_s.  SQL: %d, ISAM: %d, errm=%s.\n",
		     sqlca.sqlcode,
		     sqlca.sqlerrd[1],
		     sqlca.sqlerrm );
	    exit(1);
	}
	EXEC SQL DECLARE parmtype CURSOR FOR parmtype_s;
	if (sqlca.sqlcode == -404 ||
	    sqlca.sqlcode == -408 ||
	    sqlca.sqlcode == -259 ||
	    sqlca.sqlcode == -267) {
	    first = 1;
	    EXEC SQL FREE parmtype_s;
	    goto redo;
	} else if (sqlca.sqlcode < 0) {
	    fprintf( stderr,
		     "Error declaring cursor parmtype.  SQL: %d, ISAM: %d, errm=%s.\n",
		     sqlca.sqlcode,
		     sqlca.sqlerrd[1],
		     sqlca.sqlerrm );
	    exit(1);
	}
	first = 0;
    }

    procid = prcid;

    EXEC SQL OPEN parmtype USING :procid;
    if (sqlca.sqlcode == -404 ||
	sqlca.sqlcode == -408 ||
	sqlca.sqlcode == -259 ||
	sqlca.sqlcode == -276 ||
	sqlca.sqlcode == -285)
	{
	    first = 1;
	    EXEC SQL FREE parmtype_s;
	    EXEC SQL FREE parmtype;
	    goto redo;
	} else if (sqlca.sqlcode < 0) {
	fprintf( stderr,
		 "Error opening parmtype.  SQL: %d, ISAM: %d, errm=%s.\n",
		 sqlca.sqlcode,
		 sqlca.sqlerrd[1],
		 sqlca.sqlerrm );
	exit(1);
    }

    p = procstr;
    col = 0;
    attr = 1;

    EXEC SQL FETCH parmtype
	INTO :typestr;
    if (sqlca.sqlcode == -404) {
        first = 1;
	EXEC SQL FREE parmtype_s;
	EXEC SQL FREE parmtype;
	goto redo;
    } else if (sqlca.sqlcode < 0) {
	fprintf( stderr,
	         "Error fetching parmtype.  SQL: %d, ISAM: %d, errm=%s.\n",
		 sqlca.sqlcode,
		 sqlca.sqlerrd[1],
		 sqlca.sqlerrm );
	exit(1);
    }

    EXEC SQL CLOSE parmtype;

    strcpy( procstr, typestr );

    if (strlen( typestr ) == 0)
        return 0;
    else
        return 1;
}
